# Macro definition
macro(BSP_COMPILE_KERNEL_MODULE obj source_path)
	set(source_files)
	file(GLOB_RECURSE source_files ${source_path}/k_*.c)
	set(objects)
    set(source_file_exist FALSE)
	foreach(file_path ${source_files})
		string(REPLACE "/" ";" temp_str ${file_path})
		list(GET temp_str -1 temp_file)
		string(REPLACE "." ";" temp_str2 ${temp_file})
		list(GET temp_str2 0 file_name)
		string(APPEND objects " ")
		string(APPEND objects ${file_name}.o)
        set(source_file_exist TRUE)
	endforeach()
    
    if(source_file_exist)
        # Module info
        add_definitions(-D__KERNEL__ -DMODULE)

        set(OUTDIR ${BSP_KERNEL_LIBRARY_OUTPUT_DIRECTORY})
        set(PWD ${CMAKE_CURRENT_SOURCE_DIR}/source)
        
        set(makefile_info)
        string(APPEND makefile_info "KERNEL_VER = $(shell uname -r)\n")
        string(APPEND makefile_info "obj-m += ${obj}.o\n")
        string(APPEND makefile_info "${obj}-objs := ${objects}\n")
        string(APPEND makefile_info "EXTRA_CFLAGS = -g -O0\n\n")
        string(APPEND makefile_info "build:kernel_modules\n\n")

        string(APPEND makefile_info "kernel_modules:\n")
        string(APPEND makefile_info "	cp ${BSP_PUBH_DIR}/*.h .\n")
        string(APPEND makefile_info "	cp ${BSP_HOME_DIR}/code/type/pubh/kernelh/*.h .\n")
        string(APPEND makefile_info "	make -C /lib/modules/$(KERNEL_VER)/build M=$(PWD) modules\n")
        string(APPEND makefile_info "	mv ${obj}.ko ${OUTDIR}\n")
        string(APPEND makefile_info "clean:\n")
        string(APPEND makefile_info "	rm -f *.o *.mod.o *.mod.c *.symvers *.order *.mod .*.cmd *.h Makefile\n\n")

        file(WRITE ${CMAKE_CURRENT_SOURCE_DIR}/source/Makefile ${makefile_info})
        
        set(script_info)
        string(APPEND script_info "cd source\n")
        string(APPEND script_info "make\n")
        string(APPEND script_info "make clean\n")
        file(WRITE ${CMAKE_CURRENT_SOURCE_DIR}/compile.sh ${script_info})
    endif()		
endmacro(BSP_COMPILE_KERNEL_MODULE)

# Set the output path
macro(BSP_CONFIG_OUTPUT_PATHS targetname)
  set_target_properties( ${targetname}
	PROPERTIES
		RUNTIME_OUTPUT_DIRECTORY ${BSP_RUNTIME_OUTPUT_DIRECTORY}
		ARCHIVE_OUTPUT_DIRECTORY ${BSP_STATIC_LIBRARY_OUTPUT_DIRECTORY}
		LIBRARY_OUTPUT_DIRECTORY ${BSP_LIBRARY_OUTPUT_DIRECTORY}
  )
endmacro(BSP_CONFIG_OUTPUT_PATHS)

# Get all of the subdirectories' name
# subdirnamelist the output parameter,means the list of the subdirectories，can not add the ${}
# targetdir      the target path, should be the whole path
macro(BSP_SUBDIRNAMELIST_MACRO subdirnamelist targetdir)
    file(GLOB children ${targetdir}/*) # get targetdir's content
    set(dirlist "")
    foreach(child ${children})
        file(RELATIVE_PATH child_name ${targetdir} ${child})
        if(IS_DIRECTORY ${targetdir}/${child_name})
            list(APPEND dirlist ${child_name})
        endif()
    endforeach()
    list(APPEND ${subdirnamelist} ${dirlist})
endmacro()
 
# Check if the directore has CMakeLists
# hascmakelist the output parameter,means if the targetdir has CMakeLists.txt，can not add the ${}
# targetdir    the target path, should be the whole path
macro(BSP_CHECK_DIR_HAS_CMAKELIST hascmakelist targetdir)
    set(${hascmakelist} FALSE)
    if(IS_DIRECTORY ${targetdir})
        if(EXISTS ${targetdir}/CMakeLists.txt)
            set(${hascmakelist} TRUE)
        endif()
    else()
        message(FATAL_ERROR "Invalid dir para: targetdir = ${targetdir}")
    endif()
endmacro()
 
# add the add_subdirectory() call for all of the subdirectories who have the CMakeList file
# targetdir    the target path, should be the whole path
macro(BSP_ADD_SUBDIRECTORY targetdir)
    set(subdirnamelisttemp "")
    BSP_SUBDIRNAMELIST_MACRO(subdirnamelisttemp ${targetdir})
    foreach(subdir ${subdirnamelisttemp})
        set(hascmakelisttemp FALSE)
        BSP_CHECK_DIR_HAS_CMAKELIST(hascmakelisttemp ${targetdir}/${subdir})
        if(${hascmakelisttemp})
            message(STATUS "macro add_subdir go to add_subdir = ${targetdir}/${subdir}")
            add_subdirectory(${targetdir}/${subdir})
        endif()
    endforeach()
endmacro()


# Find the kernel release
execute_process(
    COMMAND uname -r
    OUTPUT_VARIABLE KERNEL_RELEASE
    OUTPUT_STRIP_TRAILING_WHITESPACE
)

# Find the headers
find_path(
    KERNELHEADERS_DIR
    include/linux/user.h
    PATHS /usr/src/linux-headers-${KERNEL_RELEASE}
)

message(STATUS "Kernel release: ${KERNEL_RELEASE}")
message(STATUS "Kernel headers: ${KERNELHEADERS_DIR}")

#[[
include_directories(
    ${KERNELHEADERS_DIR}/include
)
]]