# CBB Project Build指南
-----------------------------------
## Build步骤
- 在cbb/build/路径下，执行build.sh，参数只有一个，为编译链相关配置的cmake文件路径
  ```c
  ./build.sh [the toolchain.cmake file's path]
  ```
- toolchain.cmake文件编译器，编译链路径等编译相关参数定义，一个例子如下:
  ```c
    # this is required
    SET(CMAKE_SYSTEM_NAME Linux)
    
    SET(CMAKE_PRODUCT_NAME ER8411)
    
    # specify the cross compiler
    SET(CMAKE_C_COMPILER   /usr/bin/cc)
    SET(CMAKE_C_FLAGS "-DNDEBUG -O2")
    # SET(CMAKE_CXX_COMPILER /opt/arm/usr/bin/ppc_74xx-g++)
    # SET(CMAKE_CXX_FLAGS "")
    
    set(TOOLCHAIN_PATH /usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games:/usr/local/games:/snap/bin)
    # where is the target environment 
    # SET(CMAKE_FIND_ROOT_PATH  /opt/arm/ppc_74xx /home/rickk/arm_inst)
    
    # search for programs in the build host directories (not necessary)
    # SET(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
    # for libraries and headers in the target directories
    # SET(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
    # SET(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
    
    # configure Boost and Qt
    # SET(QT_QMAKE_EXECUTABLE /opt/qt-embedded/qmake)
    # SET(BOOST_ROOT /opt/boost_arm)
  ```
-----------------------------------------------------------------------
## 编译产物
- 编译的中间产物和库文件等均在build/路径下生成，.so,.ko等编译库生成路径在：
  - .so文件路径在${CMAKE_HOME_DIRECTORY}/build/output/so/
  - .ko文件路径在${CMAKE_HOME_DIRECTORY}/build/output/ko/
------------------------------------------------------------
## CBB新建模块步骤
- 在cbb/code/路径下为CBB各功能模块目录，新建模块的CMakeLists.txt只需要修改lib库的名字为新建模块名即可，该代码行如下：
  ```c
  set(LIB_NAME [lib名])
  ```
--------------------------------------------------------------------------
##Unit Test模块搭建流程
  - 单元测试模块用的C++ boost，安装依赖
  ```c
    sudo aptitude install mpi-default-dev libicu-dev python-dev libz2-dev g++
  ```
  - 下载boost库，
-------------------------------------------------------------
## 注意点
  - cbb，usp 各模块.c文件的路径source/下不要建立子目录，全部.c都放在source/下，头文件也是一样的
  - 对于cbb，usp的.c文件，如果为用户态的，命名规则为**cbb_xxx.c/cbb_xxx.h/usp_xxx.c/usp_xxx.h**。为内核态的，命名规则为**k_cbb_xxx.c/k_cbb_xxx.h/k_usp_xxx.c/k_usp_xxx.h**
  - CBB,USP代码模块，用户态代码禁止添加编译宏，kernel态代码编译宏由build.sh的toolChain.cmake导入
  - USP功能模块的私有.h放在privh/下，公有暴露的.h放在pubh/下，公有.h命名需要以usp_xxx.h格式。
<<<<<<< HEAD
-------------------------------------------------------------
# 数据库使用指南
## 1 数据结构介绍
```c
typedef struct _DATA_INFO{
    union _data
    {
        void*         vptr;     \\ for Blob
        char*         cptr;     \\ for Text  
        int           num;      \\ for Int
        double        real;     \\ for Double
    }data;
    int   len;
}DATAINFO;

typedef enum _DATATYPE
{
    BLOB,
    DOUBLE,
    INT,
    TEXT,
    TEXT16
}DATATYPE;
```
本库支持的字段包括INT(整型)、DOUBLE(浮点数)、BLOB(二进制数据)、TEXT(utf-8 字符串)以及
TEXT16(utf-16 字符串)。其中，BLOB、TEXT和TEXT16这三种类型的数据需要指定数据的长度，保存
在len字段中。 

每个DATAINFO表示了一张表中的一项数据，那么对表中的一行数据需要使用一维数组表示，而对
表中的若干行数据也就需要使用相应的二维数组表示。示例如下：

| id | name | age | score |
|:---:|:---:|:---:|:-----:|
|1|abc|10|70.5|
|2|bcd|11|80.5|

- 这里假设 id 是 INT 类型，name 是 TEXT 类型， age 是 BLOB 类型，score 是 DOUBLE 类型。
那么，对于将id = 1转换为 DATAINFO 类型的变量并指定其类型
```c
  /* id = 1 */
  DATAINFO di;
  DATATYPE dt;
  di.data.num = 1;
  di.len = 0;   //对于int和double类型的数据，len设置为0
  dt = INT;
```
- 如果需要将转换一行数据，则需要调用者自行维护一个一维数组，该数组的长度即为数据的列数。
```c
  /* id = 1, name = abd, age = 10, score = 70.5 */
  char *name = "abc";
  int age = 10;
  void *age_ptr = (void *)(&age);  // 为了测试二进制数据，强制转换
  int column = 4;
  DATATYPE*  dt[] = {INT, TEXT, BLOB, DOUBLE};
  DATAINFO* di_row = (DATAINFO*)malloc(sizeof(DATAINFO) * column);
  
  di_row[0].data.num = 1;
  di_row[0].len = 0;   
  
  di_row[1].data.cptr = name;
  di_row[1].len = strlen(name);   
  
  di_row[2].data.vptr = age_ptr;
  di_row[2].len = sizeof(int);   
  
  di_row[3].data.real = 70.5;
  di_row[3].len = 0;   
  
  free(di_row);
```
- 如果需要将一张表或者若干行数据转换成DATAINFO结构，同样需要调用者自行维护一个二维数组，
其尺寸为：数据的行数 * 数据的列数。
```c
  /* 
   * id1 = 1, name1 = abd, age1 = 10, score1 = 70.5
   * id2 = 2, name2 = bcd, age2 = 11, score2 = 80.5
   */
  char *name1 = "abc";
  char *name2 = "bcd";
  int age1 = 10;
  int age2 = 11;
  void *age_ptr1 = (void *)(&age1); 
  void *age_ptr2 = (void *)(&age2); 
  
  int i = 0;
  int row = 2;
  int column = 4;
  DATATYPE*  dt[] = {INT, TEXT, BLOB, DOUBLE};
  DATAINFO** di_table = (DATAINFO**)malloc(sizeof(DATAINFO*) * row);
  for(i = 0; i < row; i++)
  {
      di_table[i] = (DATAINFO*)malloc(sizeof(DATAINFO) * column);
  }
  di_table[0][0].data.num = 1;
  di_table[0][0].len = 0;   
  
  di_table[0][1].data.cptr = name1;
  di_table[0][1].len = strlen(name1);   
  
  di_table[0][2].data.vptr = age_ptr1;
  di_table[0][2].len = sizeof(int);   
  
  di_table[0][3].data.real = 70.5;
  di_table[0][3].len = 0;
  
  di_table[1][0].data.num = 2;
  di_table[1][0].len = 0;   
  
  di_table[1][1].data.cptr = name2;
  di_table[1][1].len = strlen(name2);   
  
  di_table[1][2].data.vptr = age_ptr2;
  di_table[1][2].len = sizeof(int);   
  
  di_table[1][3].data.real = 80.5;
  di_table[1][3].len = 0;
  
  for(i = 0; i < row; i++)
  {
      free(di_table[i]);
  }
  free(di_table);
```
-------------------------------------------------------------
## 2 接口介绍

接口一览：
  - [config_write](#1)
  - [config_write_multi](#2)
  - [config_read](#3)
  - [log_write](#4)
  - [log_read](#5)

数据库和sql语句是分不开的，sqlite3中提供了一种便捷的变量绑定方法，其形式如下：
```c
  char *sql_select = "SELECT id, score FROM dpi WHERE age > ?;";
  
  char *sql_update = "UPDATE dpi \
                        SET score = ?, age = ?  \
                        WHERE id = ? OR name LIKE ?;";
```
- 在如上所示的sql语句中，利用？来替代所有需要绑定变量的地方，其中常见的sql语句中，
变量绑定的位置要么在前半段（WHERE前），通常是需要修改或查询的值；要么在后半段
（WHERE后），通常是条件的值。所有？处的变量都将用DATAINFO结构的数据进行传参。
- 值得注意的是select语句，待查询的值虽然没有用？替代，但是返回的结果依旧是通过DATAINFO
结构返回。

综上所述，在usp_db.h中提供的接口如下： 
- <div id="1">config_write</div>
```c
  int config_write(OPTYPE op, char* sql_cmd,
                    DATAINFO* datainfo_new, DATATYPE* dt_new, int column_new, 
                    DATAINFO* datainfo_old, DATATYPE* dt_old, int column_old);
```
config_write适用于不频繁的配置类写操作，可以完成单行数据的增/删/改操作。此接口直接和
数据库交互，而不经过缓存队列。此外，内存的维护由调用者完成。

|参数|类型|说明|
|:---:|:----:|:----|
|op|OPTYPE|操作类型，取值包括INSERT/UPDATE/DELETE|
|sql_cmd|char *|待执行的sql语句，支持增删改操作|
|datainfo_new|DATAINFO *|对应前文提到的前半段的参数|
|dt_new|DATATYPE *|上述参数所对应的类型|
|column_new|int|上述参数所对应的数量|
|datainfo_old|DATAINFO *|对应前文提到的后半段的参数，主要是条件值|
|dt_old|DATATYPE *|上述参数所对应的类型|
|column_old|int|上述参数所对应的数量|
|返回值|int|0代表成功，-1代表失败|
使用示例：
```c
  /* 将 id = 3 或者 name LIKE y% 的记录，更新为score = 60.0， age = 12 */
  char *sql_update = "UPDATE dpi \
                        SET score = ?, age = ?  \
                        WHERE id = ? OR name LIKE ?;";
  int id = 3;
  char* name = "y%";
  double score = 60.0;
  int age = 12;
  void* age_ptr = (void *)(&age);
  
  int column_new = 2;
  DATATYPE* dt_new[] = {DOUBLE, BLOB};
  DATAINFO* di_new = (DATAINFO *)malloc(sizeof(DATAINFO) * column_new);
  
  di_new[0].data.real = 60.0;
  di_new[0].len = 0;
  di_new[1].data.vptr = age_ptr;
  di_new[1].len = sizeof(int);
  
  int column_old = 2;
  DATATYPE dt_old[column_old] = {INT, TEXT};
  di_old[0].data.num = id;
  di_old[0].len = 0;
  di_old[1].data.cptr = name;
  di_old[1].len = strlen(name);
  
  config_write(UPDATE, sql_update, di_new, dt_new, column_new, 
                                      di_old, dt_old, column_old);
  free(di_new);
```

- <div id="2">config_write_multi</div>
```c
int config_write_multi(OPTYPE op, char* sql_cmd, int row, 
                        DATAINFO** datainfo_new, DATATYPE* dt_new, int column_new, 
                        DATAINFO** datainfo_old, DATATYPE* dt_old, int column_old);
```
config_write_multi是config_write批量写的接口，可以完成多行数据的增/删/改操作。内存的维护
同样由调用者完成。

|参数|类型|说明|
|:---:|:----:|:----|
|op|OPTYPE|操作类型，取值包括INSERT/UPDATE/DELETE|
|sql_cmd|char *|待执行的sql语句，支持增删改操作|
|row|int|需要执行的行数|
|datainfo_new|DATAINFO **|对应前文提到的前半段的参数|
|dt_new|DATATYPE *|上述参数所对应的类型|
|column_new|int|上述参数所对应的数量|
|datainfo_old|DATAINFO **|对应前文提到的后半段的参数，主要是条件值|
|dt_old|DATATYPE *|上述参数所对应的类型|
|column_old|int|上述参数所对应的数量|
|返回值|int|0代表成功，-1代表失败|
使用示例：
```c
  /* 删除 name = abc，score = 70.5 和 name = bcd，score = 80.5的行 */
  char *sql_delete = "DELETE FROM dpi \
                        WHERE name = ? AND score = ?;";
  char* name1 = "abc";
  char* name2 = "bcd";
  
  double score1 = 70.5;
  double score2 = 80.5;
  
  int row = 2;
  int column_old = 2;
  DATAINFO** di_old = (DATAINFO**)malloc(sizeof(DATAINFO*) * row);
  for(i = 0; i < row; i++)
  {
      di_old[i] = (DATAINFO*)malloc(sizeof(DATAINFO) * column);
  }
  
  DATATYPE* dt_old[] = {TEXT, DOUBLE};
  di_old[0][0].data.cptr = name1;
  di_old[0][0].len = strlen(name1);
  di_old[0][1].data.real = score1;
  di_old[0][1].len = 0;
  
  di_old[1][0].data.cptr = name2;
  di_old[1][0].len = strlen(name2);
  di_old[1][1].data.real = score2;
  di_old[1][1].len = 0;
  
  config_write_multi(DELETE, sql_delete, row, NULL, NULL, 0, 
                                        di_old, dt_old, column_old);
                                        
  for(i = 0; i < row; i++)
  {
      free(di_old[i]);
  }
  free(di_old);
```

- <div id="3">config_read</div>
```c
int config_read(char* sql_cmd,  
                    DATAINFO* datainfo_in, DATATYPE* dt_in, int column_in,
                    DATAINFO*** datainfo_out, DATATYPE* dt_out, int column_out, int offset, int* row_out);
```
config_read 将从表中获取对应的查询结果。限制了最大查询数量 MAX_SELECT_COUNT，如果超过
该值需要配合offset字段多次查询。该函数中不对内存进行操作，需要调用者自行申请和释放相关内存

|参数|类型|说明|
|:---:|:----:|:----|
|sql_cmd|char *|待执行的select语句|
|datainfo_in|DATAINFO *|对应前文提到的后半段的参数，主要是条件值|
|dt_in|DATATYPE *|上述参数所对应的类型|
|column_in|int|上述参数所对应的数量|
|datainfo_out|DATAINFO ***|查询结果可能有多条，同时需要从函数传出，所以为三级指针|
|dt_out|DATATYPE *|上述参数所对应的类型|
|column_out|int|上述参数所对应的数量|
|offset|int|偏移量，对应sql中的OFFSET字段|
|row_out|int *|入参时代表希望查询的行数，出参时代表实际查询到的行数|
|返回值|int|0代表成功，-1代表失败|
使用示例：
```c
  /* 查询 id > 10， id < 100 同时 name 中包含 Y 的记录*/
  char *sql_select = "SELECT * FROM dpi \
                        WHERE id > ? AND id < ? AND name LIKE %Y%;";
  int id1 = 10;
  int id2 = 100;
  char* name = "%Y%";
  
  
  /* 构建查询条件 */
  int column_in = 3;
  DATAINFO* di_in = (DATAINFO*)malloc(sizeof(DATAINFO) * column_in);
  DATATYPE* dt_in[] = {INT, INT, TEXT};
  di_in[0].data.num = id1;
  di_in[0].len = 0;
  di_in[1].data.num = id2;
  di_in[1].len = 0;
  di_in[2].data.cptr = name;
  di_in[2].len = strlen(name);
  
  int row = 100; 
  int column_out = 4; /* 从select * 可知结果有4列 */
  DATAINFO** di_out = (DATAINFO**)malloc(sizeof(DATAINFO*) * row);
  for(i = 0; i < row; i++)
  {
    di_new[i] = (DATAINFO*)malloc(sizeof(DATAINFO) * column_out);
  }
  
  int offset = 0;
  /* 循环是为了查询到全部结果，如果大概知道会有多少条结果，可以把row值设大一些，以避免循环 */
  while(row != 0)
  {
    config_read(sql_select, di_in, dt_in, column_in, 
                        &di_out, column_out, offset, &row_out);
    /* 处理查询到的结果 */
    ...
    offset += row;
  }
                                        
  for(i = 0; i < row; i++)
  {
    free(di_out[i]);
  }
  free(di_out);
  free(di_in);
```
- <div id="4">log_write</div>
```c
int log_write(MODULETYPE mod, DATAINFO* datainfo, int column);
```
log_write写方法使用缓冲，适用于频繁的日志类数据库写，仅支持对单条日志的插入操作。函数中不对内
存进行操作，需要调用者自行申请和释放相关内存

值得说明的是，日志的数据库操作只有新增和查询，不允许修改和删除操作，因此写日志的操作被简化许多

|参数|类型|说明|
|:---:|:----:|:----|
|mod|MODULETYPE|模块名，例如当前有DPI和IPS两个模块|
|datainfo|DATAINFO *|对应插入操作的数据|
|column|int|上述参数所对应的数量|
|返回值|int|0代表成功，-1代表失败|

- <div id="5">log_read</div>
```c
int log_read(char* sql_cmd,  
                    DATAINFO* datainfo_in, DATATYPE* dt_in, int column_in,
                    DATAINFO*** datainfo_out, DATATYPE* dt_out, int column_out, int offset, int* row_out);
```
log_read和config_read函数是一致的，因为都涉及到条件查询，无法将sql语句固定下来，详细用法见
[config_read](#3)

|参数|类型|说明|
|:---:|:----:|:----|
|sql_cmd|char *|待执行的select语句|
|datainfo_in|DATAINFO *|对应前文提到的后半段的参数，主要是条件值|
|dt_in|DATATYPE *|上述参数所对应的类型|
|column_in|int|上述参数所对应的数量|
|datainfo_out|DATAINFO ***|查询结果可能有多条，同时需要从函数传出，所以为三级指针|
|dt_out|DATATYPE *|上述参数所对应的类型|
|column_out|int|上述参数所对应的数量|
|offset|int|偏移量，对应sql中的OFFSET字段|
|row_out|int *|入参时代表希望查询的行数，出参时代表实际查询到的行数|
|返回值|int|0代表成功，-1代表失败|
-------------------------------------------------------------

