
/*! Copyright(c) 2008-2015 Shenzhen TP-LINK Technologies Co.Ltd.
 *
 *\file		i2c.c
 *\brief	i2c get and set 
 *\details	i2c get and set 
 *
 *\author	marujun
 *\version	1.0
 *\date		20022-5-21
 *
 *\warning	
 *
 *\history \20022-5-21,	create file
 *						 
 */

/**************************************************************************************************/
/*                                      INCLUDE_FILES                                             */
/**************************************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>

#include "i2c.h"
#include "i2cbusses.h"
#include "i2c-dev.h"

#include "usp_pal_sync.h"/*信号量*/
#include "usp_pal_thread.h" /*pal_usleep*/
/**************************************************************************************************/
/*                                      DEFINES                                                   */
/**************************************************************************************************/
#define I2C_DEVICE "/dev/i2c-1"
#define I2C_GPIO_DEVICE "/dev/i2c-2"
#define I2C_BUS0_ID 1
#define I2C_BUS1_ID 2


/**************************************************************************************************/
/*                                      VARIABLES                                                 */
/**************************************************************************************************/
int	 gi2c_drv_mode = 0;

static pal_mutex_t sem_i2c_bus = NULL;


/**************************************************************************************************/
/*                                      FUNCTIONS                                                 */
/**************************************************************************************************/
int ad_i2c_init(void)
{
    /* I2C bus sem init */
	sem_i2c_bus = pal_mutex_create("sem_i2c_bus");/* 使用pal层接口创建信号量消息队列 */
	if(sem_i2c_bus == NULL)
	{
		return (ERR_HW_OP_FAIL);/* return需统一调用此宏 */
	}

	/* I2C SDA/SCL的模拟GPIO已经在设备树中设置好了，无需再指定&初始化 */ 
	return (ERR_NO_ERROR);	 
}


static void help(void)
{
	fprintf(stderr, "Syntax: i2cget [-f] [-y] I2CBUS CHIP-ADDRESS "
	        "[DATA-ADDRESS [MODE]]\n"
	        "        i2cget -V\n"
	        "  MODE can be: 'b' (read byte data, default)\n"
	        "               'w' (read word data)\n"
	        "               'c' (write byte/read byte)\n"
		"  If DATA-ADDRESS is omitted, a single read byte command is "
		"issued\n"
	        "  Append 'p' to MODE for PEC checking\n"
	        "  I2CBUS is an integer\n");
	print_i2c_busses(0);
}

static int check_funcs(int file, int i2cbus, int size, int daddress, int pec)
{
	unsigned long funcs;

	/* check adapter functionality */
	if (ioctl(file, I2C_FUNCS, &funcs) < 0) {
		fprintf(stderr, "Error: Could not get the adapter ");
		return -1;
	}

	switch (size) {
	case I2C_SMBUS_BYTE:
		if (!(funcs & I2C_FUNC_SMBUS_READ_BYTE)) {
			fprintf(stderr, "Error: Adapter for i2c bus %d does "
			        "not have read byte capability\n", i2cbus);
			return -1;
		}
		if (daddress >= 0
		 && !(funcs & I2C_FUNC_SMBUS_WRITE_BYTE)) {
			fprintf(stderr, "Error: Adapter for i2c bus %d does "
			        "not have write byte capability\n", i2cbus);
			return -1;
		}
		break;

	case I2C_SMBUS_BYTE_DATA:
		if (!(funcs & I2C_FUNC_SMBUS_READ_BYTE_DATA)) {
			fprintf(stderr, "Error: Adapter for i2c bus %d does "
			        "not have read byte data capability\n", i2cbus);
			return -1;
		}
		break;

	case I2C_SMBUS_WORD_DATA:
		if (!(funcs & I2C_FUNC_SMBUS_READ_WORD_DATA)) {
			fprintf(stderr, "Error: Adapter for i2c bus %d does "
			        "not have read word data capability\n", i2cbus);
			return -1;
		}
		break;
	}

	if (pec
	 && !(funcs & (I2C_FUNC_SMBUS_PEC | I2C_FUNC_I2C))) {
		fprintf(stderr, "Warning: Adapter for i2c bus %d does "
		        "not seem to support PEC\n", i2cbus);
	}

	return 0;
}

int i2c_get(int i2cbus, int address, int daddress, usp_uint8_t *buffer)
{
	char *end;
	int res, file;
	int size = I2C_SMBUS_BYTE_DATA;
	char filename[20];

	if (i2cbus < 0 || i2cbus > 0x3f) {
		fprintf(stderr, "Error: I2CBUS argument invalid!\n");
		fprintf(stderr, "Error: Read failed i2cbus:%d, address: 0x%x, daddress:0x%x\n", i2cbus, address, daddress);
	}

	if (address < 3 || address > 0x77) {
		fprintf(stderr, "Error: Chip address invalid!\n");
		fprintf(stderr, "Error: Read failed i2cbus:%d, address: 0x%x, daddress:0x%x\n", i2cbus, address, daddress);
	}

	if (daddress < 0 || daddress > 0xFf) {
		fprintf(stderr, "Error: Data address invalid!\n");
		fprintf(stderr, "Error: Read failed i2cbus:%d, address: 0x%x, daddress:0x%x\n", i2cbus, address, daddress);
	}

	file = open_i2c_dev(i2cbus, filename, 0);
	if (file < 0 || set_slave_addr(file, address, 0))
		return ERR_HW_OP_FAIL;
		//exit(1);
#if 0
	if (pec && ioctl(file, I2C_PEC, 1) < 0) {
		fprintf(stderr, "Error: Could not set PEC: %s\n",
		        strerror(errno));
		return ERR_HW_OP_FAIL;
		//exit(1);
	}
#endif

	res = i2c_smbus_read_byte_data(file, daddress);

	close(file);

	if (res < 0) {
		fprintf(stderr, "Error: Read failed i2cbus:%d, address: 0x%x, daddress:0x%x\n", i2cbus, address, daddress);
		return ERR_HW_OP_FAIL;
		//exit(2);
	}

	*buffer = res;

	return ERR_NO_ERROR;
}
#if 0
static void help(void)
{
	fprintf(stderr, "Syntax: i2cset [-f] [-y] I2CBUS CHIP-ADDRESS DATA-ADDRESS "
	        "VALUE [MODE] [MASK]\n"
	        "        i2cset -V\n"
	        "  MODE is 'b[yte]' or 'w[ord]' (default b)\n"
	        "  Append 'p' to MODE for PEC checking\n"
	        "  I2CBUS is an integer\n");
	print_i2c_busses(0);
	return ERR_HW_OP_FAIL;
	//exit(1);
}
#endif
int i2c_set(int i2cbus, int address, int daddress, usp_uint8_t *buffer)
{
	char *end;
	int res, size, file, value;
	int vmask = 0;
	int e1;
	char filename[20];
	unsigned long funcs;

	if (i2cbus < 0 || i2cbus > 0x3f) {
		fprintf(stderr, "Error: I2CBUS argument invalid!\n");
		fprintf(stderr, "Error: write failed i2cbus:%d, address: 0x%x, daddress:0x%x, value:0x%x\n", i2cbus, address, daddress, value);
	}

	if (address < 3 || address > 0x77) {
		fprintf(stderr, "Error: Chip address invalid!\n");
		fprintf(stderr, "Error: write failed i2cbus:%d, address: 0x%x, daddress:0x%x, value:0x%x\n", i2cbus, address, daddress, value);
	}

	if (daddress < 0 || daddress > 0xFf) {
		fprintf(stderr, "Error: Data address invalid!\n");
		fprintf(stderr, "Error: write failed i2cbus:%d, address: 0x%x, daddress:0x%x, value:0x%x\n", i2cbus, address, daddress, value);
	}

	size = I2C_SMBUS_BYTE_DATA;

	value = *buffer;
	if (value < 0 || (size == I2C_SMBUS_BYTE_DATA && value > 0xFf)) {
		fprintf(stderr, "Error: Data value out of range!\n");
	}

	file = open_i2c_dev(i2cbus, filename, 0);
	if (file < 0) {
		return ERR_HW_OP_FAIL;
		//exit(1);
	}

	/* check adapter functionality */
	if (ioctl(file, I2C_FUNCS, &funcs) < 0) {
		fprintf(stderr, "Error: Could not get the adapter ");
		return ERR_HW_OP_FAIL;
		//exit(1);
	}

	if (!(funcs & I2C_FUNC_SMBUS_WRITE_BYTE_DATA)) {
		fprintf(stderr, "Error: Adapter for i2c bus %d does "
				"not have byte write capability\n", i2cbus);
		return ERR_HW_OP_FAIL;
		//exit(1);
	}

	if (set_slave_addr(file, address, 0) < 0)
		return ERR_HW_OP_FAIL;
		//exit(1);

	e1 = 0;
	if (size == I2C_SMBUS_WORD_DATA) {
		res = i2c_smbus_write_word_data(file, daddress, value);
	} else {
		res = i2c_smbus_write_byte_data(file, daddress, value);
	}
	if (res < 0) {
		fprintf(stderr, "Error: write failed i2cbus:%d, address: 0x%x, daddress:0x%x, value:0x%x\n", i2cbus, address, daddress, value);
		fprintf(stderr, "Warning - write failed\n");
		e1++;
	}

	if (size == I2C_SMBUS_WORD_DATA) {
		res = i2c_smbus_read_word_data(file, daddress);
	} else {
		res = i2c_smbus_read_byte_data(file, daddress);
	}
	close(file);

	if (res < 0) {
		//fprintf(stderr, "Error: write failed i2cbus:%d, address: 0x%x, daddress:0x%x, value:0x%x\n", i2cbus, address, daddress, value);
		e1++;
	} else
	if (res != value) {
		e1++;
		/*
		fprintf(stderr, "Error: write mismatch i2cbus:%d, address: 0x%x, daddress:0x%x, value:0x%x\n", i2cbus, address, daddress, value);
		fprintf(stderr, "Warning - data mismatch - wrote "
		        "0x%0*x, read back 0x%0*x\n",
		        size == I2C_SMBUS_WORD_DATA ? 4 : 2, value,
		        size == I2C_SMBUS_WORD_DATA ? 4 : 2, res);
		*/
	} else {
		//fprintf(stderr, "Value 0x%0*x written, readback matched\n", size == I2C_SMBUS_WORD_DATA ? 4 : 2, value);
	}

	return ERR_NO_ERROR;
}


/*!
 *\fn		ad_misc_i2c_data_get
 *\brief	I2C data get function
 *
 *\param[in]	i2c_id	i2c id
 *\param[in]	addr	i2c slave device address
 *\param[in]	offset	data offset
 *\param[in]	buff_size read buffer size
  *\param[out]	buff	buff ptr
 *
 *\return
 *\retval	ERR_NO_ERROR/ERR_HW_OP_FAIL
 *
 *\note
 */
int ad_misc_i2c_data_get(usp_uint8_t i2c_id, usp_uint8_t addr, usp_uint32_t offset, usp_uint32_t buff_size, OUT usp_uint8_t *buff)
{
	int rv = 0;

	pal_mutex_take(sem_i2c_bus, pal_mutex_FOREVER);
	//if(gi2c_drv_mode ==  I2C_USE_CPSS_INTF) 
	{
		rv = i2c_get(i2c_id, addr, offset, buff);
	}
	pal_mutex_give(sem_i2c_bus);

	if(rv > 0)
	{
		return ERR_HW_OP_FAIL;
	}
	return ERR_NO_ERROR;
}

/*!
 *\fn		ad_misc_i2c_data_set
 *\brief	I2C data set function
 *
 *\param[in]	i2c_id	i2c id
 *\param[in]	addr	i2c slave device address
 *\param[in]	offset	data offset
 *\param[in]	buff_size write buffer size
  *\param[out]	buff	buff ptr
 *
 *\return
 *\retval	ERR_NO_ERROR/ERR_HW_OP_FAIL
 *
 *\note
 */
int ad_misc_i2c_data_set(usp_uint8_t i2c_id, usp_uint8_t addr, usp_uint32_t offset, usp_uint32_t buff_size, usp_uint8_t *buff)
{
	int rv = 0;

	pal_mutex_take(sem_i2c_bus, pal_mutex_FOREVER);
	//if(gi2c_drv_mode ==  I2C_USE_CPSS_INTF) 
	{
		rv = i2c_set(i2c_id, addr, offset, buff);
	}
	pal_mutex_give(sem_i2c_bus);
	
	if(rv > 0)
	{
		return ERR_HW_OP_FAIL;
	}
	return ERR_NO_ERROR;


}

