/*! Copyright(c) 2008-2015 Shenzhen TP-LINK Technologies Co.Ltd.
 *
 *\file		ad_poe_config.h
 *\brief	ad head file for config
 *\details	sw head file for config
 *
 *\author	Cui_binglin
 *\version	1.0
 *\date		2015-01-06
 *
 *\warning	
 *
 *\history \2015-01-06,	create file
 *						 
 */

 
/**************************************************************************************************/
/*                                      CONFIGURATIONS                                            */
/**************************************************************************************************/

#ifndef _AD_POE_CONFIG_20150106_H
#define _AD_POE_CONFIG_20150106_H

/**************************************************************************************************/
/*                                      INCLUDE_FILES                                             */
/**************************************************************************************************/
#include "common_sys.h"

/**************************************************************************************************/
/*                                      DEFINES                                                   */
/**************************************************************************************************/

#define POE_MAIN_TASK_MS    100 /* Poe主任务轮询周期，单位毫秒 */

/* 定义该宏，将使用虚拟POE */
/*#define POE_USE_VIRTUAL_CHIP*/
/*所有机型中支持的最大POE端口数，数组定义时使用*/
#define MAX_POE_PORT_NUM               48

#define PSE_CHIP_MAX_NUMS        		12

#define PORTS_PER_PSE_CHIP 				(4)

/* 剩余功率小于阈值时, Po_e MAX 灯亮起 */
#define POWER_REMAIN_MAX_LED_ON    70/*0.1w*/

/*时间段或者功率管理状态*/ 
#define POE_RELEASE     0   /*非管理状态*/
#define POE_OCCUPY      -1  /*设置为负方便flag manage的定时器计时管理，定时器将不对此状态进行计时处理*/

/* 隔一段时间重新对因为功率限制而被断电的PD上电, 单位秒 */
#define POWER_RECONNECT_TIME       7

/* 隔一段时间重新对因为start fault而被断电的PD上电, 单位秒 */
#define POWER_RECONNECT_TIME_BY_FAULT       2

/* 隔一段时间重新对因为端口过载而被断电的PD上电, 单位秒 */
#define POWER_RECONNECT_TIME_BY_ICUT        2


/*POE timer 粒度 单位 秒*/
#define POE_FLAG_TIMER   1

/*各个定时器时间，单位秒*/
#define POE_TIMER_PWR_OVER_30w   180
#define POE_TIMER_PWR_OVERLOAD   180
#define POE_TIMER_SHORT_CIRCUIT  180
#define POE_TIMER_I2C_RW_FAILED   7200
#define POE_TIMER_TIMESEG_DIS    POWER_RECONNECT_TIME


/*POE user config enable or disable*/
#define POE_DISABLE (0)
#define POE_ENABLE 	(1)

/*功率管理时，判断端口是否满足掉电条件*/
#define POE_MANAGE_EN   (1)
#define POE_MANAGE_DIS  (0)


/*支持的标准，用于区分是否支持class4/30w*/
#define IEEE802_3_AF 0
#define IEEE802_3_AT 1
#define IEEE802_3_BT 2


/**************************************************************************************************/
/*                                      TYPES                                                     */
/**************************************************************************************************/

typedef enum{
	DET_UNKNOWN = 0,
	DET_SHORT_CIRCUIT = 1,
	DET_RES_1 = 2,
	DET_REG_LOW = 3,
	DET_OK = 4,
	DET_REG_HIGH = 5,
	DET_OPEN_CIRCUIT = 6,
	DET_RES_2 = 7,
	DET_FET_FAULT = 8
}PSE_DETECTION_STATUS_ENUM;


typedef enum
{
	POE_PRI_HIGH_AD = 0,
	POE_PRI_MIDDLE_AD,
	POE_PRI_LOW_AD
} POE_PRIORITY_STATE_AD;

typedef enum
{
	POE_OFF= 0,
	PORT_TURNING_ON,/*对于需手动控制上电的PSE芯片，准备就绪，可以上电的状态*/	
	POE_ON ,

	/*Bad State*/
	POE_OVERLOAD,
	POE_SHORT,
	POE_NOSTANDARD_PD,
	POE_VOLTAGE_HIGH,
	POE_VOLTAGE_LOW,
	POE_HW_FAULT,
	POE_OVERTEMPERATURE,
	POE_START_FAULT
} POE_POWER_STATE;

typedef enum
{   /*	uint(0.1w)*/
	POE_CLASS_CLASS1_AD = 40,
	POE_CLASS_CLASS2_AD = 70,
	POE_CLASS_CLASS3_AD = 154,
	POE_CLASS_CLASS4_AD = 300,
	POE_CLASS_CLASS0_AD = 330,
	POE_CLASS_CLASS5_AD = 450,
	POE_CLASS_CLASS6_AD = 600,
	POE_CLASS_CLASS7_AD = 750,
	POE_CLASS_CLASS8_AD = 900,
	POE_CLASS_CLASSUN_AD
} POE_CLASS_STATE_AD;

typedef enum
{
	POE_POWER_OFF = 0,
	POE_POWER_ON = 1
}POE_POWER_ON_CTRL;

typedef enum
{
	POE_LED_OFF = 0,      /* 熄灭     */
	POE_LED_GREEN_ON = 1,     /* 绿色恒亮 */
	POE_LED_GREEN_BLINK = 2,  /* 绿色闪烁 */
	POE_LED_YELLOW_ON =3,    /* 黄色恒亮 */
	POE_LED_YELLOW_BLINk = 4, /* 黄色闪烁 */
	POE_LED_RED_ON = 5,       /* 红色恒亮 */
	POE_LED_RED_BLINK= 6,     /* 红色闪烁 */
	POE_LED_STATE_END
} POE_LED_STATE;


typedef enum
{
	ERR_POE_CONFIG_BEGIN_NUM = 0,
	ERR_POE_CONFIG_PD_INIT_FAIL,						//PD's initialization failed
	ERR_POE_CONFIG_ENTER_CONFIG_MODE_FAILED,		//Entering Config Mode failed
	ERR_POE_CONFIG_RETURN_FROM_CONFIG_MODE_FAIL,	//Return from Config Mode failed
	ERR_POE_CONFIG_PORT_POW_LIMIT_OUT_OF_RANGE,
	ERR_POE_CONFIG_PD_CONFLICTS,					//PD's configuration conflicts
	ERR_POE_CONFIG_SYS_COM_OUT_OF_RANGE,
	ERR_POE_CONFIG_PKT_IS_NOT_REPORT,
	ERR_POE_CONFIG_WRONG_CSUM,
	ERR_POE_CONFIG_CONFLICT_SUBJ,
	ERR_POE_CONFIG_WRONG_DATA,
	ERR_POE_CONFIG_WRONG_KEY_FIELD,
	ERR_POE_CONFIG_TIME_OUT,
	ERR_POE_CONFIG_UNKNOW_ERROR,
	ERR_POE_CONFIG_END
}err_code_poe_config_t;


typedef enum
{
	/*port base*/

	/*event for snmp*/
	POE_MANAGE_PWR_CHANGE,     /* 端口状态变化 */
	POE_MANAGE_PWR_DENY,  /* 端口功率管理掉电 */
	POE_MANAGE_PWR_OVER_30w,    /* 端口功率超过30W */
	POE_MANAGE_PWR_OVER_60w,    /* 端口功率超过60W */
	POE_MANAGE_PWR_OVERLOAD, /* 端口功率过限 */
	POE_MANAGE_SHORT_CIRCUIT,       /* 端口短路 */

	/*power manage*/
	POE_MANAGE_TIMESEG_DIS, /* 因为时间片管理掉电 */
	POE_MANAGE_POWER_DIS,   /* 因为总功率超限掉电 */
	POE_MANAGE_RECOVERY_DIS,/* 因为Po_e Recovery掉电 */

	/*led*/
	POE_MANAGE_LED_FORCE,/*LED处于强制模式*/

	/* i2c */
	POE_I2C_RW_FAILED,	/* I2C操作失败 */
	/*所有port base flag需在此之前*/
	POE_MANAGE_PORT_FLAG_COUNT,

	
} POE_MANAGE_PORT_FLAG_TYPE;

typedef enum
{

	/*system base*/
	POE_MANAGE_THERMAL_SHUTDOWN,     /* 过温保护 */
	POE_MANAGE_MAX_PWR_BUDGET,      /* 超过总功率    */

	/*所有sys base flag需在此之前*/
	POE_MANAGE_SYS_FLAG_COUNT
} POE_MANAGE_SYS_FLAG_TYPE;


typedef struct
{
	int actual_state;   /* 端口实际上是否在供电      */
	int actual_cur;     /* 端口实际电流 unit: m_a     */
	int actual_vol;     /* 端口实际电压 unit: 0.1V  -- 更改为0.1V ,by zengjunjun 2015-04-23 */
	int actual_cons;    /* 端口实际功率 unit: 0.1W   */
		
	POE_CLASS_STATE_AD pd_class;       /* 端口连接的PD设备class等级 */
	POE_POWER_STATE power_state;   /* 端口状态: on off overload short */
	POE_POWER_STATE power_state_backup;   /* 端口之前状态: on off overload short */
	
} power_info_t;



typedef struct
{
	/*user config*/
	int port_no;        /* 端口号                    */
	int poe_enable;     /* 端口Po_e是否开启               */
	int power_limit;    /* 端口功率限制 unit: 0.1W   */
	POE_PRIORITY_STATE_AD priority;       /* 端口优先级                */
	
	/*power*/
	power_info_t power_info;

	/*physical*/
		/* 端口事件 */
	usp_uint8_t     ev_pe_chg    : 1; /* event: power enable change      */
	usp_uint8_t     ev_pg_chg    : 1; /* event: power good change        */
	usp_uint8_t     ev_det_end   : 1; /* event: detection end            */
	usp_uint8_t     ev_cla_end   : 1; /* event: classification end       */
	usp_uint8_t     ev_i_cut     : 1; /* event: i cut fault occurred                  */
	usp_uint8_t     ev_ld_disc   : 1; /* event: load disconnect          */
	usp_uint8_t     ev_start_flt : 1; /* event: start fault              */
	usp_uint8_t     ev_i_lim      : 1; /* event: i lim fault occurred    */
	usp_uint8_t     ev_over_temp : 1; /* event: over temperature         */
	usp_uint8_t     ev_vdd_uvlo  : 1; /* event: vdd undervoltage-lockout */
	usp_uint8_t     ev_vpwr_uvlo  : 1; /* event: vpwr undervoltage-lockout */
		/* 端口状态 */
	usp_uint8_t     st_pe        : 1; /* status: power enable            */
	usp_uint8_t     st_pg        : 1; /* status: power good              */
	usp_uint8_t     st_det;           /* status: detection               */
	usp_uint8_t     st_cla;           /* stauts: classification          */
	usp_uint8_t     cc_res;           /* Connection Check Result : Single or Dual Signature */
	
	usp_uint32_t physical_event;/*对于非管理型PSE芯片，侦测、分级等事件，用于判断power state，预留32bit，根据芯片不同自己定义*/
	usp_uint32_t physical_status;/*对于非管理型PSE芯片，侦测、分级等状态，用于判断power state，预留32bit，根据芯片不同自己定义*/

	/* stm state*/
	int stm_state;      /* 状态机目前的状态 */
	
	/*led*/
	POE_LED_STATE led_state_cur;/*led 当前显示状态*/
	POE_LED_STATE led_state_backup;/*当led处于强制显示状态时，存储led退出强制模式后的显示状态*/

	int fault_count;    /*异常次数*/
	
} poe_port_info_t;

typedef struct
{
	int port_flag[POE_MANAGE_PORT_FLAG_COUNT][MAX_POE_PORT_NUM];
	int sys_flag[POE_MANAGE_SYS_FLAG_COUNT];
} poe_manage_flag_t;


typedef struct
{
	
	/*sys base*/
	int poe_port_num;		/*poe port num*/
	int poe_port_per_pse;		/*port per pse dev*/
	int poe_line_per_port;
	int poe_uport_min;
	int poe_uport_max;

	int sys_poe_enable;     /* Po_e是否开启           */
	int sys_power_limit;    /* 功率限制 unit: 0.1W   */
	int sys_actual_cons;    /* 实际功率 unit: 0.1W   */

	/*添加对支持标准的定义，用以区分是否支持class4*/
	/* IEEE802_3_AF or IEEE802_3_AT*/
	int standard;


	/*port base*/
	poe_port_info_t poe_port_info[MAX_POE_PORT_NUM]; 

	/*manage flag*/
	poe_manage_flag_t poe_manage_flag;

	/*led base*/
	POE_LED_STATE max_led_state;
	POE_LED_STATE fan_led_state;
} poe_control_t;

extern poe_control_t g_poe_control;

/*global*/
#define POE_CONTROL 			    g_poe_control

#define POE_PORT_NUM  				g_poe_control.poe_port_num
#define POE_UPORT_MIN  				g_poe_control.poe_uport_min
#define POE_UPORT_MAX  				g_poe_control.poe_uport_max

#define POE_STANDARD                g_poe_control.standard

/*port info*/
#define POE_PORT_INFO(_PORT_NO)  	g_poe_control.poe_port_info[_PORT_NO-1]


#define POE_PORT_POWER_INFO(_PORT_NO)  				POE_PORT_INFO(_PORT_NO).power_info
#define POE_PORT_POWER_STATE(_PORT_NO)				POE_PORT_POWER_INFO(_PORT_NO).power_state
#define POE_PORT_POWER_STATE_BACKUP(_PORT_NO)		POE_PORT_POWER_INFO(_PORT_NO).power_state_backup


/*flag*/
#define POE_MANAGE_FLAG    g_poe_control.poe_manage_flag

#define POE_MANAGE_PORT_FLAG(_TYPE,_PORT_NO) g_poe_control.poe_manage_flag.port_flag[_TYPE][_PORT_NO-1]
#define POE_MANAGE_SYS_FLAG(_TYPE)  g_poe_control.poe_manage_flag.sys_flag[_TYPE]


#define POE_MANAGE_PWR_DENY_FLAG(_PORT_NO) 		POE_MANAGE_PORT_FLAG(POE_MANAGE_PWR_DENY,_PORT_NO)
#define POE_MANAGE_TIMESEG_DIS_FLAG(_PORT_NO) 	POE_MANAGE_PORT_FLAG(POE_MANAGE_TIMESEG_DIS,_PORT_NO)
#define POE_MANAGE_RECOVERY_DIS_FLAG(_PORT_NO) 	POE_MANAGE_PORT_FLAG(POE_MANAGE_RECOVERY_DIS,_PORT_NO)
#define POE_MANAGE_POWER_DIS_FLAG(_PORT_NO) 	POE_MANAGE_PORT_FLAG(POE_MANAGE_POWER_DIS,_PORT_NO)


#define POE_MANAGE_LED_FORCE_FLAG(_PORT_NO) 	POE_MANAGE_PORT_FLAG(POE_MANAGE_LED_FORCE,_PORT_NO)


#define CLASS_UNKNOWN      			0
#define CLASS_1                    	1
#define CLASS_2                    	2
#define CLASS_3                    	3
#define CLASS_4                    	4
#define CLASS_5                		5
#define CLASS_0                    	6
#define CLASS_OVER_CURR   			7

/* state machine of poe port */
typedef enum
{
    STM_BEGIN = 0,
    STM_PORT_OFF,
    STM_PORT_TURNING_ON,
    STM_PORT_ON,
    STM_PORT_BAD,
    STM_PORT_START_FAULT
} poe_stm_state_t;


/**************************************************************************************************/
/*                                      VARIABLES                                                 */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      FUNCTIONS                                                 */
/**************************************************************************************************/

/*!
 *\Function:
 *      ad_poe_get_port_power_limt
 *\Description:
 *      Get the Po_e port power limit.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *     power_limit   - port power limit
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_port_power_limt(int user_port, int *power_limit);

/*!
 *\Function:
 *      ad_poe_get_poe_port_led_state
 *\Description:
 *      Get Po_e port LED state.
 *\Input:
 *      user_port      - user_port, start from 1  
 *\Output:
 *      state         - led state
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_poe_port_led_state(int user_port, POE_LED_STATE *state);

/*!
 *\Function:
 *      ad_poe_set_sys_power_limt
 *\Description:
 *      Set the system power limit.
 *\Input:
 *      power_limit    - power limit be set, 单位为0.1W
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_sys_power_limt(IN  int  power_limit);

/*!
 *\Function:
 *      ad_poe_get_sys_power_limt
 *\Description:
 *      Get the system power limit.
 *\Input:
 *      n/a
 *\Output:
 *      power_limit    - power limit, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_sys_power_limt
(
    OUT int  *power_limit
);

/*!
 *\Function:
 *      ad_poe_get_sys_power_cons
 *\Description:
 *      Get the system actual power consumption.
 *\Input:
 *      n/a
 *\Output:
 *      power_cons     - actual power consumption, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_sys_power_cons
(
    OUT int  *power_cons
);

/*!
 *\Function:
 *      ad_poe_get_sys_power_remain_percent
 *\Description:
 *      Get the system remain power and percent.
 *\Input:
 *      n/a
 *\Output:
 *      power_cons     - actual power consumption, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_sys_power_remain_percent
(
    OUT float  *power_remain,
    OUT float  *power_percent
);


/*!
 *\Function:
 *      ad_poe_set_port_power_limt
 *\Description:
 *      Set the Po_e port power limit.
 *\Input:
 *      user_port      - user_port, start from 1
 *      power_limit   - port power limit
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_port_power_limt(int user_port, int power_limit);

/*!
 *\Function:
 *      ad_poe_set_port_user_status
 *\Description:
 *      Set the Po_e port user status.
 *\Input:
 *      user_port      - user_port, start from 1
 *      status        - POE func enabled or not
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_port_user_status
(
    IN  int            user_port,
    IN  int  status
);

/*!
 *\Function:
 *      ad_poe_get_port_status
 *\Description:
 *      Get the Po_e port POE_DISABLE_AD or POE_ENABLE_AD
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      status        - POE func enabled or not
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_port_status(int user_port, int *status);


/*!
 *\Function:
 *      ad_poe_set_port_priority
 *\Description:
 *      Set the Po_e port priority.
 *\Input:
 *      user_port      - user_port, start from 1
 *      priority      - port priority
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_port_priority(int user_port, POE_PRIORITY_STATE_AD priority);


/*!
 *\Function:
 *      ad_poe_get_port_power_cons
 *\Description:
 *      Get the system actual power consumption.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      power_cons     - actual power consumption, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_port_power_cons
(
	IN  int user_port,
    OUT int  *power_cons
);

/*!
 *\Function:
 *      ad_poe_get_port_power_info
 *\Description:
 *      get the poe port power supply information
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      power_info     - power info you get
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_port_power_info
(
	IN  int user_port,
    OUT power_info_t  *power_info
);

/*!
 *\Function:
 *      ad_poe_get_port_power_state
 *\Description:
 *      get the port power supply state
 *\Input:
 *      user_port            - user_port, start from 1
 *\Output:
 *      power_state         - current power state
 *      power_state_backup  - last power state
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_port_power_state(int user_port, POE_POWER_STATE *power_state,POE_POWER_STATE *power_state_backup);

/*!
 *\Function:
 *      ad_poe_set_port_state
 *\Description:
 *      Set the Po_e port Status.
 *\Input:
 *      user_port      - user_port, start from 1
 *      status        - POE func enabled or not
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_port_state
(
    IN  int            user_port,
    IN  int  status
);

/*!
 *\Function:
 *      ad_poe_get_standard
 *\Description:
 *      Get the pd standard.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      standard      - pd standard
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_standard
(
    IN  int            user_port,
    OUT  int  *standard
);

/*!
 *\Function:
 *      ad_poe_set_port_power_state
 *\Description:
 *      Get the pd standard.
 *\Input:
 *      user_port      - user_port, start from 1
 *      power_state    - power supply state to set
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_port_power_state
(
	IN  int            user_port,
	IN POE_POWER_STATE power_state
);

/*!
 *\Function:
 *      ad_poe_port_info_updt
 *\Description:
 *      read the newest power information per port, and calculate the total power
 *	   of all ports to the g_poe_control.sys_actual_cons
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
void ad_poe_clear_poe_port_power_info(int user_port);


#if 0
#endif

/*!
 *\Function:
 *      ad_poe_manage_check
 *\Description:
 *      check whether the port can be disconnected due to the power manage algorithm
 *\Input:
 *      user_port      - user_port, start from 1
 *      lowest        - port' priority must be lower than this argument 
 *\Output:
 *      n/a
 *\Return:
 *      POE_MANAGE_EN  - the port can be disconnected
 *      (Others)       - the port cannot be disconnected
 *\Note:
 *      n/a
 */
int ad_poe_manage_check(int user_port,int lowest);

#if 0
#endif

/*!
 *\Function:
 *      ad_poe_flag_manage
 *\Description:
 *      deal with the flag for power manage
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      n/a
 *\Note:
 *      n/a
 */
void ad_poe_flag_manage();

/*!
 *\Function:
 *      ad_poe_sys_flag_get
 *\Description:
 *      get the system level flag value
 *\Input:
 *      sys_flag - flag type
 *\Output:
 *      value    - flag value
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_sys_flag_get(POE_MANAGE_SYS_FLAG_TYPE sys_flag,int *value);

/*!
 *\Function:
 *      ad_poe_sys_flag_set
 *\Description:
 *      set the system level flag value
 *\Input:
 *      sys_flag - flag type
 *      value    - flag value to set
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_sys_flag_set(POE_MANAGE_SYS_FLAG_TYPE sys_flag,int value);

/*!
 *\Function:
 *      ad_poe_port_flag_get
 *\Description:
 *      get the port level flag value
 *\Input:
 *      user_port   - user_port, start from 1
 *      port_flag  - flag type
 *\Output:
 *      value      - flag value		
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_port_flag_get(int user_port,POE_MANAGE_PORT_FLAG_TYPE port_flag,int *value);

/*!
 *\Function:
 *      ad_poe_port_flag_set
 *\Description:
 *      set the port level flag value
 *\Input:
 *      user_port   - user_port, start from 1
 *      port_flag  - flag type
 *      value      - flag value to set
 *\Output:
 *      n/a		
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_port_flag_set(int user_port,POE_MANAGE_PORT_FLAG_TYPE port_flag,int value);

/*!
 *\Function:
 *      ad_poe_enable
 *\Description:
 *      enabe the pse power supply
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_enable(void);



 
#ifdef POE_USE_VIRTUAL_CHIP

typedef enum 
{
	POE_NOT_INSERT_PD = 0,
	POE_INSERT_PD = 1
}POE_INSERT_PD_STATE;

typedef struct
{
	int enable;
	POE_INSERT_PD_STATE insert_pd;
	POE_POWER_STATE power_state;
}V_POE_PSE;

typedef struct
{
	int current;
	int voltage;
	int power;
	POE_CLASS_STATE_AD pd_class;
}V_POE_PD;

/*!
 *\Function:
 *      ad_poe_get_PSE
 *\Description:
 *      get the pse configuration
 *\Input:
 *      user_port    - user_port, start from 1
 *\Output:
 *      p_pse		- pse configuration
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
STATUS ad_poe_get_PSE( int user_port, V_POE_PSE *p_pse);
/*!
 *\Function:
 *      ad_poe_set_PSE
 *\Description:
 *      config the pse
 *\Input:
 *      user_port    - user_port, start from 1
 *      p_pse		- pse configure 
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
STATUS ad_poe_set_PSE( int user_port, V_POE_PSE *p_pse);

/*!
 *\Function:
 *      ad_poe_get_PD
 *\Description:
 *      get the pd configuration
 *\Input:
 *      user_port    - user_port, start from 1
 *\Output:
 *      p_pse		- pd configuration
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
STATUS ad_poe_get_PD( int user_port, V_POE_PD *p_pd);

/*!
 *\Function:
 *      ad_poe_set_PD
 *\Description:
 *      config the pd
 *\Input:
 *      user_port    - user_port, start from 1
 *      p_pd		- pd power argument configure 
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
STATUS ad_poe_set_PD( int user_port, V_POE_PD *p_pd);


#endif


/**************************************************************************************************/
/*                                      GLOBAL_FUNCTIONS                                          */
/**************************************************************************************************/

/*!
 *\Function:
 *      ad_poe_init
 *\Description:
 *      ad layer init entry of the poe module 
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_init(void);

int get_pse_port_line_num();
int ad_poe_user_port_to_act_user_port(int user_port,int ad_port_index);
int ad_poe_act_user_port_to_user_port(int ad_port_num);
int ad_poe_port_check_power_allowed(IN int user_port, IN int sys_remain_power);
int ad_poe_port_power_on_control(int user_port, POE_POWER_ON_CTRL status);
int ad_poe_set_poe_port_led_state(int user_port, POE_LED_STATE state);
int ad_poe_power_info_updt(void);
int ad_poe_led_ctrl(int enable);
void ad_poe_update_poe_max_led(void);
int ad_poe_get_sys_power_cons_cali(OUT int  *power_cons_cali);

#define PSE_PORT_LINE_NUM       									get_pse_port_line_num()
#define PSE_USERPORT_TO_ADPORT(user_port,ad_port_index)        		ad_poe_user_port_to_act_user_port(user_port,ad_port_index)
#define PSE_ADPORT_TO_USERPORT(ad_port_num)       					ad_poe_act_user_port_to_user_port(ad_port_num)
#define PSE_CHECK_PORT_POWER_ALLOWED(user_port,sys_remain_power)       (ad_poe_port_check_power_allowed(user_port,sys_remain_power))



/* 判断当前端口是否有效 */
#define POE_PORT_CHK(port_no)		if (port_no < POE_UPORT_MIN || port_no > POE_UPORT_MAX){ return (ERR_BAD_PARAM);} 

/* 循环 */
#define POE_PORT_ITER(port_no)		for (port_no = POE_UPORT_MIN; port_no <= POE_UPORT_MAX; port_no++)
		

#endif

