/**************************************************************************************************/
/*                                      INCLUDE_FILES                                             */
/**************************************************************************************************/

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>





#include "timer.h"
#include "i2c.h"
#include "gpio.h"
//#include "usp_type.h"
#include "ad_poe_misc_ti.h"
//#include "ad_poe_config.h"
#include "common_sys.h"
#include "usp_pal_thread.h"
#include "usp_pal_sync.h"


/**************************************************************************************************/
/*                                      DEFINES                                                   */
/**************************************************************************************************/


typedef struct _MISC_LIB_STRUCT_
{
	int (*ad_poe_led_update)();
} MISC_LIB_STRUCT;


#define POE_RESET_ON		0
#define POE_RESET_OFF		1

#define FAN_STATE_GOOD		1
#define FAN_STATE_BAD		0

#define FAN_LED_ON			1
#define FAN_LED_OFF			0

#define POE_MAX_LED_ON		1
#define POE_MAX_LED_OFF		0

#define POE_PORT_LED_OFF	0
#define POE_PORT_LED_ON		1

#define POE_MODE_LED_OFF	0
#define POE_MODE_LED_ON		1

#define I2C_BUS0_ID 0
#define I2C_BUS1_ID 1



#define POE_LED_BLINK_INTERVAL		   500	


GPIO_CTRL_STRUCT *g_port_led = NULL;
GPIO_CTRL_STRUCT *g_poe_led = NULL;
GPIO_CTRL_STRUCT *g_poe_max_led = NULL;
GPIO_CTRL_STRUCT *g_poe_rst = NULL;

usp_uint8_t *g_poe_i2c_id = NULL;

static pal_mutex_t mutex_led = NULL;
static pal_mutex_t sem_poe_bus = NULL;
static pal_thread_t ad_led_update_task_id = PAL_THREAD_ERROR;

static int global_poe_led_flag = TRUE;

/*以下定义机型相关变量*/

/*er7212pc*/
GPIO_CTRL_STRUCT g_poe_led_er7212pc[] = {
	{PIN_TYPE_GPIO,43,0},
	{PIN_TYPE_GPIO,44,0},
	{PIN_TYPE_GPIO,45,0},
	{PIN_TYPE_GPIO,46,0},
	{PIN_TYPE_GPIO,47,0},
	{PIN_TYPE_GPIO,50,0},
	{PIN_TYPE_GPIO,51,0},
	{PIN_TYPE_GPIO,52,0},
};

GPIO_CTRL_STRUCT g_poe_max_led_er7212pc = {PIN_TYPE_GPIO, 53, 0};
GPIO_CTRL_STRUCT g_poe_rst_er7212pc = {PIN_TYPE_GPIO, 11, 1};

usp_uint8_t g_poe_i2c_id_er7212pc = 0;

/**************************************************************************************************/
/*                                      LOCAL_FUNCTIONS                                           */
/**************************************************************************************************/

int ad_poe_bus_init(void)
{
	int rv=0;
    /* I2C bus sem init */
	sem_poe_bus = pal_mutex_create("sem_poe_bus");/* 使用pal层接口创建信号量消息队列 */
	if(sem_poe_bus == NULL)
	{
		return (ERR_HW_OP_FAIL);/* return需统一调用此宏 */
	}

	rv = ad_i2c_init();

	/* I2C SDA/SCL的模拟GPIO已经在设备树中设置好了，无需再指定&初始化 */ 
	return (ERR_NO_ERROR);	 
}

int ad_poe_bus_reset()
{
	/* I2C Bus由Marvel sdk自己控制，不需要再从外部RESET */	
	return (ERR_NO_ERROR);
}

int ad_poe_chip_reset(void)
{
	/* POE_RESET_GPIO在adMiscInit已经初始化，不用再初始化 */
	if(g_poe_rst == NULL)
	{
		return (ERR_NO_ERROR);
	}
	ad_misc_gpio_set(0, g_poe_rst->type, g_poe_rst->pin, POE_RESET_ON);
	pal_usleep(500000);
	ad_misc_gpio_set(0, g_poe_rst->type, g_poe_rst->pin, POE_RESET_OFF);
	pal_usleep(2000000);
 
	return (ERR_NO_ERROR);
}

/*!
 *\fn		 int ad_poe_bus_write(int offset,unsigned char *buffer,unsigned int length)
 *\brief	 通过I2C Bus向对应地址（offset）的TPS23861的寄存器（buffer[0]）写入内容(buffer[1])
 *
 *\param[in]	offset	 	- PSE Slave Address
 * 		 		buffer[0]   - Reg Address
 *				buffer[1]   - Data
 *				length	 	- must be 2
 *
 *\return	 N/A
 *
 *\note	   
 */
int ad_poe_bus_write(unsigned int offset,unsigned char *buffer,unsigned int length)
{
	int rv=0;
 
	if( buffer == NULL)
	{
		return (ERR_BAD_PARAM);
	}
 
	pal_mutex_take(sem_poe_bus, pal_mutex_FOREVER); 
	rv = ad_misc_i2c_data_set(*g_poe_i2c_id, offset, buffer[0], 1, &(buffer[1]));
	pal_mutex_give(sem_poe_bus);
	if(rv == 0)
	{
		return (ERR_NO_ERROR);
	}
	else
	{
		return (ERR_POE_CONFIG_UNKNOW_ERROR);
	}
	 
}
 
/*!
 *\fn		 int ad_poe_bus_read(int offset, unsigned char *buffer, unsigned int *length)
 *\brief	 通过I2C Bus读取对应地址（offset）的TPS23861的寄存器（buffer[0]）的内容(buffer[1])
 *
 *\param[in]	offset	 	- PSE Slave Address
 * 		 		buffer[0]   - Reg Address
 				length	 	- must be 2
 *\param[out]	buffer[1]   - Data
 *
 *\Return:
 * 	 ERR_NO_ERROR  - on success
 * 	 (Others)	   - on error
 *
 *\note	   
 */
int ad_poe_bus_read(unsigned int offset, unsigned char *buffer, unsigned int *length)
{
	int rv = 0;
	usp_uint8_t temp_buff[2] = {0};
 
	if(length == NULL || buffer == NULL)
	{
		 return (ERR_BAD_PARAM);
	}
	temp_buff[0]=buffer[0];
	pal_mutex_take(sem_poe_bus, pal_mutex_FOREVER); 
	rv = ad_misc_i2c_data_get(*g_poe_i2c_id, offset, temp_buff[0], 1, &(temp_buff[1]));
	pal_mutex_give(sem_poe_bus);
	buffer[1] = temp_buff[1];
	if(rv == 0)
	{
		return (ERR_NO_ERROR);
	}
	else
	{
		return (ERR_POE_CONFIG_UNKNOW_ERROR);
	}
}


int ad_poe_led_ctrl(int enable)
{
	global_poe_led_flag = enable;
	return ERR_NO_ERROR;
}


/* PoE MAX LED: Green Off, Yellow Off */
void ad_led_poe_max_led_off(void)
{
	usp_uint32_t dev = 0;

	if(g_poe_max_led != NULL)
	{
		ad_misc_gpio_set(dev, g_poe_max_led->type, g_poe_max_led->pin, 0^(g_poe_max_led->invrt));
	}

}

/* PoE MAX LED: Green On, Yellow Off */
void ad_led_poe_max_led_green_on(void)
{
	usp_uint32_t dev = 0;
	
	if (!global_poe_led_flag)
	{
		ad_led_poe_max_led_off();
		return;
	}

	if(g_poe_max_led != NULL)
	{
		ad_misc_gpio_set(dev, g_poe_max_led->type, g_poe_max_led->pin, 1^(g_poe_max_led->invrt));
	}
}


/* PoE MAX LED: Green Blink */
void ad_led_poe_max_led_green_blink(void)
{
	usp_uint32_t dev = 0;
	static int flag = 0;

	if (!global_poe_led_flag)
	{
		ad_led_poe_max_led_off();
		return;
	}
	if(g_poe_max_led != NULL)
	{
		flag++;
		if (flag <= POE_LED_BLINK_INTERVAL / POE_MAIN_TASK_MS)
		{
			ad_misc_gpio_set(dev, g_poe_max_led->type, g_poe_max_led->pin, 0^(g_poe_max_led->invrt));
		}
		else
		{
			ad_misc_gpio_set(dev, g_poe_max_led->type, g_poe_max_led->pin, 1^(g_poe_max_led->invrt));
		}
		
		if (flag >= 2 * POE_LED_BLINK_INTERVAL / POE_MAIN_TASK_MS)
		{
			flag = 0;
		}
	}
}



void ad_update_poe_led()
{
	int user_port = 0;
	POE_LED_STATE led_state;
	usp_uint32_t dev = 0;
	static usp_int8_t blink_flag = 0;

	blink_flag ^= 1;

	for( user_port = POE_UPORT_MIN; user_port <= POE_UPORT_MAX; user_port++)	
	{
		ad_poe_get_poe_port_led_state(user_port, &led_state);

		if (!global_poe_led_flag)
		{
			led_state = POE_LED_OFF;
		}
		
		switch(led_state)
		{
			case POE_LED_OFF:
				ad_misc_gpio_set(dev, g_poe_led[user_port-1].type, g_poe_led[user_port-1].pin, 0^(g_poe_led[user_port-1].invrt));
				break;
			case POE_LED_GREEN_ON:
				ad_misc_gpio_set(dev, g_poe_led[user_port-1].type, g_poe_led[user_port-1].pin, 1^(g_poe_led[user_port-1].invrt));
				//ad_led_force(dev, &g_poe_led[user_port-1], LED_COLOR_G, POE_PORT_LED_ON);
				break;
			case POE_LED_GREEN_BLINK:
				ad_misc_gpio_set(dev, g_poe_led[user_port-1].type, g_poe_led[user_port-1].pin, blink_flag^(g_poe_led[user_port-1].invrt));
				//blink_port_array[user_port-1] = led_state;
				break;
			case POE_LED_YELLOW_ON:
				//ad_led_force(dev, &g_poe_led[user_port-1], LED_COLOR_Y, POE_PORT_LED_ON);
				break;
			case POE_LED_YELLOW_BLINk:
				//blink_port_array[user_port-1] = led_state;
				break;
			default:
				break;
		}
	}
	return ;
}


static void ad_led_update_task()
{
	while(1)
	{
		//if (!is_switch_init_finished())
		if(0)
		{
		    pal_sleep(1);
		    continue;
		}

		pal_mutex_take(mutex_led, pal_mutex_FOREVER);
		ad_update_poe_led();
		pal_mutex_give(mutex_led);
		
		pal_usleep(100000); /* 100 ms */
	}
}


int ad_poe_board_init(void)
{
	int rv = 0;
	int dev = 0;
	int port_no = 0;
	
	rv = ad_poe_bus_init();
	if (rv > 0)
	{
		DBG("ad_poe_bus_init error\n");
	}

	/*PSE Reset GPIO init*/
	if(g_poe_rst != NULL)
	{
		if(g_poe_rst->type == PIN_TYPE_GPIO)
		{
			ad_misc_gpio_init(dev, g_poe_rst->pin);
			ad_misc_gpio_dir_set(dev, g_poe_rst->pin, GPIO_DIRECTION_OUTPUT, 1);
		}
	}

	/*PSE LED GPIO init */
	if (g_poe_led != NULL)
	{
		POE_PORT_ITER(port_no)
		{
			if(g_poe_led[port_no-1].type == PIN_TYPE_GPIO)
			{
				ad_misc_gpio_init(dev, g_poe_led[port_no-1].pin);
				ad_misc_gpio_dir_set(dev, g_poe_led[port_no-1].pin, GPIO_DIRECTION_OUTPUT, 1);
				ad_misc_gpio_set(dev, g_poe_led[port_no-1].type, g_poe_led[port_no-1].pin, g_poe_led[port_no-1].invrt);
			}
		}
	}
	/*PSE MAX LED GPIO init */
	if(g_poe_max_led != NULL)
	{
		if(g_poe_max_led->type == PIN_TYPE_GPIO)
		{
			ad_misc_gpio_init(dev, g_poe_max_led->pin);
			ad_misc_gpio_dir_set(dev, g_poe_max_led->pin, GPIO_DIRECTION_OUTPUT, 1);
			ad_misc_gpio_set(dev, g_poe_max_led->type, g_poe_max_led->pin, g_poe_max_led->invrt);
		}
	}

	ad_led_update_task_id = pal_thread_create("Led_update_task", 4096, 50, ad_led_update_task, NULL); 
	if (PAL_THREAD_ERROR == ad_led_update_task_id)   
	{       
	      return (ERR_NO_MEMORY);
	}

	return (ERR_NO_ERROR);
}


int ad_poe_misc_init()
{

	switch (get_model_info())
	{
		case AD_MARVELL_ER7212PC:
			g_poe_led = g_poe_led_er7212pc;
			g_poe_max_led = &g_poe_max_led_er7212pc;
			g_poe_rst = &g_poe_rst_er7212pc;
			g_poe_i2c_id = &g_poe_i2c_id_er7212pc;
			break;
		default:
			g_poe_led = g_poe_led_er7212pc;
			g_poe_max_led = &g_poe_max_led_er7212pc;
			g_poe_rst = &g_poe_rst_er7212pc;
			g_poe_i2c_id = &g_poe_i2c_id_er7212pc;
			//g_poe_fan_led = &g_poe_fan_led_tl_sg2210hp_m2;
			//g_poe_mode_led = &g_poe_mode_led_tl_sg2210hp_m2;
			//g_speed_mode_led = &g_speed_mode_led_tl_sg2210hp_m2;

			/*Board HW reset*/
			//gsoft_rst = &gsoft_rst_tl_sg2210hp_m2;
			/*Factory default pin*/
			//gfactory_rst = &gfactory_rst_tl_sg2210hp_m2;

			//gpoe_i2c_sda = &gpoe_i2c_sda_tl_sg2210hp_m2;
			//gpoe_i2c_scl = &gpoe_i2c_scl_tl_sg2210hp_m2;
			//g_poe_rst = &g_poe_rst_tl_sg2210hp_m2;
			//gfan_state = &gfan_state_tl_sg2210hp_m2;
			//gled_mode = &gled_mode_flag_tl_sg2210hp_m2;
			//gfan_speed1 = &gfan_speed1_tl_sg2210hp_m2;
			//gfan_speed2 = &gfan_speed2_tl_sg2210hp_m2;
			//gfan_speed3 = &gfan_speed3_tl_sg2210hp_m2;
			//misc_lib.ad_poe_led_update = ad_poe_led_update_ac3;
			
			break;
	}


	mutex_led = pal_mutex_create("mutex_led");
	if(mutex_led == NULL)
	{
		return (ERR_HW_OP_FAIL);
	}
	

	return 1;
}
