/*! Copyright(c) 2008-2015 Shenzhen TP-LINK Technologies Co.Ltd.
 *
 *\file		ad_poe_ti.c
 *\brief	
 *\details	
 *
 *\author		zengjunjun
 *\version	1.0
 *\date		2015-04-16
 *
 *\warning	
 *
 *\history \2015-04-16,	create the file
 *						 
 */


/**************************************************************************************************/
/*                                      CONFIGURATIONS                                            */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      INCLUDE_FILES                                             */
/**************************************************************************************************/
/* 标准C库头文件 */
#include <stdio.h>
#include <string.h>

/* TPLINK通用头文件 */
////#include "shr_error.h"/*err id*/


/* 操作系统接口头文件 */
#include "usp_pal_sync.h"/*信号量*/
#include "usp_pal_thread.h" /*pal_usleep*/
/* 各模块头文件 */
#include "i2c.h"
#include "gpio.h"
#include "ad_poe_config.h"
#include "common_sys.h"
#include "ad_poe_misc_ti.h"
#include "mp3924.h"
#include "timer.h"


/* sdk头文件 */

/**************************************************************************************************/
/*                                    SYS  DEFINES                                                */
/**************************************************************************************************/
/*定义err id的lid mid secmid字段*/

#ifdef ERR_LAYER_ID
#undef ERR_LAYER_ID
#endif
#define ERR_LAYER_ID LAYER_ID_ADPT

#ifdef ERR_MODULE_ID
#undef ERR_MODULE_ID
#endif
#define ERR_MODULE_ID MODULE_ID_POE

#ifdef ERR_SEC_MODULE_ID
#undef ERR_SEC_MODULE_ID
#endif
#define ERR_SEC_MODULE_ID SEC_MODULE_ID_POE_CONFIG


#define FUNC_CALL(func, rv) \
    do {\
        rv = func;\
        if (rv<0)\
        {\
            DBG(" @@ Func:%s Line:%d --> Cite \"%s\" error: rv=%d\n", __FUNCTION__, __LINE__, #func, rv);\
            return rv;\
        }\
    } while(0)

#define FUNC_CALL_DONE(func, rv) \
		do {\
			rv = func;\
			if (rv<0)\
			{\
				DBG(" @@ Func:%s Line:%d --> Cite \"%s\" error: rv=%d\n", __FUNCTION__, __LINE__, #func, rv);\
				goto _done;\
			}\
		} while(0)

#define PARAM_CHK(expr, rv) \
    do {\
        if ((int)(expr))\
        {\
            DBG(" @@ Func:%s Line:%d --> Check \"%s\" error!\n", __FUNCTION__, __LINE__, #expr);\
            return rv;\
        }\
    } while (0)

#define STM_STEP_CALL(func, para1, para2) \
    do\
    {\
        int old_state;\
        do\
        {\
            old_state = *para2;\
            func(para1, para2);\
        } while (old_state != *para2);\
    } while(0)


/**************************************************************************************************/
/*									   DEFINES													  */
/**************************************************************************************************/

/* MAX灯状态 */
#define PWR_MAX_LED_STATE_OFF       0
#define PWR_MAX_LED_STATE_ON        1
#define PWR_MAX_LED_STATE_BLINK     2
TIMER_ID ad_poe_max_led_timer_id = ERROR;

#define POE_RESEND_MAX_CNT       	2		/* I2C读写失败重试次数，大于等于0 */
#define I2C_BUS_RETRY_TIMES		  	2		/* I2C编址失败重试次数，大于等于1 */

#define PSE_I2C_ADDR_CFG_INFO_FILE "/tplink/usrapp/data/i2c_addr.cfg" /* 保存I2C地址配置的历史次数文件 */

#define I2C_MP3924_SADDR0_NEW		0x20
#define I2C_MP3924_SADDR1_NEW		0x21
#define I2C_MP3924_SADDR2_NEW		0x22
#define I2C_MP3924_SADDR3_NEW		0x23
#define I2C_MP3924_SADDR4_NEW		0x24
#define I2C_MP3924_SADDR5_NEW		0x25

#define MP3924_PMAX     0x15E // 0x15E * 0.4 = 140W
#define MP3924_PMAX_MSB (MP3924_PMAX >> 1)
#define MP3924_PMAX_LSB (MP3924_PMAX & 0x01)
#define MP3924_RESET_DETECT   0x83

#define PSE_REG_SADDR		      0x11 /* PSE从机地址寄存器 */
#define PSE_REG_UNLOCK		      0xAA
#define PSE_TEMP_ADDR             0x20 /* 临时地址，用于配置I2C地址 */
#define GLOBAL_I2C_ADDRESS		  0x7F


#define PSE_CHIP_NUMS        	(g_poe_control.poe_port_num/g_poe_control.poe_port_per_pse)
#define PSE_CHIP_ITER(dev)		for(dev = 0; dev < PSE_CHIP_NUMS; dev++)
#define PSE_CHIP_CHK(dev) 		if(dev < 0 || dev >= PSE_CHIP_NUMS){ return (ERR_BAD_PARAM);} 


#define CON_ADDR_CFG_MAX_CNT	2
#define ALL_ADDR_CFG_MAX_CNT	5

/**
  *  对class4类型PD使用AUTO方式进行POWER ON 
  *  解决TPS23861 对class4 PD上电的芯片bug
  *  2016-03-18 zengjunjun
  */

#define USE_AUTO_TO_PWR_ON_CLASS4

/**************************************************************************************************/
/*                                     TYPES                                                      */
/**************************************************************************************************/
typedef struct _TI_LIB_STRUCT_
{
    int (*ad_poe_chip_init)();
    int (*ad_poe_get_port_current_from_reg_val)(int user_port, float *current);
	int (*ad_poe_get_port_voltage_from_reg_val)(int user_port, float *voltage);
	int (*ad_poe_get_port_class_from_reg_val)(int user_port, int *pd_class);
	int (*ad_poe_set_port_state)( int user_port,  int status);
	int (*ad_poe_port_power_on_control)(int user_port, POE_POWER_ON_CTRL status);
	int (*ad_poe_updt_event_status)();
	int (*ad_poe_check_class)(usp_uint8_t class_value);
	int (*ad_poe_power_info_updt)();
} TI_LIB_STRUCT;


/**************************************************************************************************/
/*                                      EXTERN_PROTOTYPES                                         */
/**************************************************************************************************/


/**************************************************************************************************/
/*                                      LOCAL_PROTOTYPES                                          */
/**************************************************************************************************/
//static int ad_poe_set_op_mode_to_semi_auto(int user_port);

int ad_poe_chip_init_mp3924(void);
int ad_poe_get_port_current_from_reg_val_mp3924(int user_port, float *current);
int ad_poe_get_port_voltage_from_reg_val_mp3924(int user_port, float *voltage);
int ad_poe_get_port_class_from_reg_val_mp3924(int user_port, int *pd_class);
int ad_poe_set_port_state_mp3924( int user_port,  int status);
int ad_poe_port_power_on_control_mp3924(int user_port, POE_POWER_ON_CTRL status);
int ad_poe_updt_event_status_mp3924(void);
int ad_poe_check_class_mp3924(usp_uint8_t class_value);
int ad_poe_power_info_updt_mp3924(void);
void ad_poe_re_config_mp3924();

static int ad_poe_port_flag_set_one_line(int user_port, POE_MANAGE_PORT_FLAG_TYPE port_flag, int value);


/**************************************************************************************************/
/*                                      VARIABLES                                                 */
/**************************************************************************************************/

int start_flt_cnt = 0;
int start_flt_flag = FALSE;
static int poe_max_led_blink_flag=0;

/* i2c addr autofix开关 */
static char i2c_addr_autofix = TRUE;

/* Po_e control 结构 */
poe_control_t g_poe_control= {0};

/* sem */
static pal_mutex_t sem_poe_control = NULL;
char *sem_poe_control_des = "sem_poe_control";


#define POE_CONTROL_SEM_TAKE() pal_mutex_take(sem_poe_control, pal_mutex_FOREVER)
#define POE_CONTROL_SEM_GIVE() pal_mutex_give(sem_poe_control)


static pal_sem_t sem_poe_time = NULL;


/*!<关闭poe功能之前要先关闭探测和分级(Detect/Class Enable Register)，
    保证不被其他Detect/Class Enable Register设置打断
    每个端口一个信号量
    */
static pal_sem_t p_sem_poe_off[MAX_POE_PORT_NUM];
static char sem_poe_off_desc[MAX_POE_PORT_NUM][20];


static unsigned char pse_init_data_mp3924[][2]=
{
/* SEMI_AUTO */	 
	{MP3924_POE_DISCONNECT_ENABLE, 	0x0F},
	{MP3924_POE_2EVENT_AND_CLASS5_ENABLE, 	0x0F},
	{MP3924_POE_PMAX_POWER_SETTING_MSB, 		MP3924_PMAX_MSB},  
	{MP3924_POE_PMAX_POWER_SETTING_LSB, 		MP3924_PMAX_LSB},  	
	{MP3924_POE_GENERAL_ENABLE_CONTROL, 		0x06},
	{MP3924_POE_OPERATION_MODE_SETING, 		0xAA}, /* 0xAA semi_auto ,0xFF auto */
	{MP3924_POE_DET_CLS_ENABLE, 0xFF},
	{MP3924_POE_INTERRUPT_MASK, 		MP3924_RESET_DETECT},
	{MP3924_POE_ICUT1_THRESHOLD, 		0x04},
	{MP3924_POE_ILIM1_THRESHOLD, 		0x01},
	{MP3924_POE_ICUT2_THRESHOLD, 		0x04},
	{MP3924_POE_ILIM2_THRESHOLD, 		0x01},
	{MP3924_POE_ICUT3_THRESHOLD, 		0x04},
	{MP3924_POE_ILIM3_THRESHOLD, 		0x01},
	{MP3924_POE_ICUT4_THRESHOLD, 		0x04},
	{MP3924_POE_ILIM4_THRESHOLD, 		0x01},	

};

static unsigned char pse_addr_mp3924_new[PSE_CHIP_MAX_NUMS] = {
	I2C_MP3924_SADDR0_NEW,
	I2C_MP3924_SADDR1_NEW,
	I2C_MP3924_SADDR2_NEW,
	I2C_MP3924_SADDR3_NEW,
	I2C_MP3924_SADDR4_NEW,
	I2C_MP3924_SADDR5_NEW,
};

/* state machine string of poe port, for debugging */
char *poe_stm_state_str[] = 
{
    "STM_BEGIN",
    "STM_PORT_OFF",
    "STM_PORT_TURNING_ON",
    "STM_PORT_ON",
    "STM_PORT_BAD"
};

//static unsigned char pse_temp_addr[PSE_CHIP_MAX_NUMS] = {0x20,0x28,0x20,0x28,0x20,0x28};


static unsigned char *pse_addr = NULL;

static int has_config_addr = FALSE;
static int con_cfg_addr_times = 0;
static int all_cfg_addr_times = 0;
static int config_addr_times = 0;

#define ADDR_CFG_CNT_EXCEED (con_cfg_addr_times >= CON_ADDR_CFG_MAX_CNT || all_cfg_addr_times >= ALL_ADDR_CFG_MAX_CNT)

/* 如果通过auto模式进行power on的，则置为TRUE */
static unsigned char use_auto_pwr_on[MAX_POE_PORT_NUM];


/*pse operating mode */
#define OFF_MODE                    0x00
#define MANUAL_MODE                 0x01
#define SEMI_AUTO_MODE              0x02
#define AUTO_MODE                   0x03
 
#define POE_PLUS_RST_VAL 	 0x00            /* POE Plus寄存器复位值 */
#define POE_PLUS_MASK        0xF0
 
/*复位检测功能使能情况*/
static int rst_det_enable = TRUE;
//static int rst_det_enable = FALSE;


TI_LIB_STRUCT ti_lib;

/**************************************************************************************************/
/*                                      LOCAL_FUNCTIONS                                           */
/**************************************************************************************************/
#if 0
#endif

/*!
 *\Function:
 *      ad_poe_user_port2Pse_port
 *\Description:
 *      user port to pse chip port.
 *\Input:
 *      user_port  - 1,2,...24
 *\Output:
 *      pse_num     - pse number, 0,1,2,3,4,5
 *      pse_port    - 1,2,3,4
 *\Return:
 *      n/a
 *\Note:
 *      n/a
 */

/*	正常顺序:
 *	[user_port]		[pse_num]		[pse_port]
 *	1				0				1
 *	2				0				2
 *	3				0				3
 *	4				0				4
 *	...				...				...
 *	21				5				1
 *	22				5				2
 *	23				5				3
 *	24				5				4
 */

int get_pse_type_info()
{
	int type;
	switch (get_model_info())
	{
		case AD_MARVELL_ER7212PC:
			type = PSE_CHIP_TYPE_MP3924;
			break;
		default:
			type = PSE_CHIP_TYPE_MP3924;
			break;
	}

	return type;
}

int get_pse_port_line_num()
{

	return g_poe_control.poe_line_per_port;
	
}

int ad_poe_user_port_to_act_user_port(int user_port,int ad_port_index)	
{
	int index=0;

	if(g_poe_control.poe_line_per_port == POE_PORT_DUAL_LINE)
	{
		index = (user_port * 2 - 1 + ad_port_index);
	}
	else
	{
		index = user_port;
	}

	return index;
	
}

int ad_poe_act_user_port_to_user_port(int ad_port_num)
{
	int index=0;

	if(g_poe_control.poe_line_per_port == POE_PORT_DUAL_LINE)
	{
		index = ((ad_port_num + 1) / 2);
	}
	else
	{
		index = ad_port_num;
	}

	return index;
	
}


static int ad_poe_user_port2Pse_port_mp3924(int user_port, usp_uint8_t *pse_num, usp_uint8_t *pse_port)
{
	if( NULL == pse_num || NULL == pse_port )
	{
		return (ERR_BAD_PARAM);
	}

	*pse_num = (user_port - 1) / PORTS_PER_PSE_CHIP;
	*pse_port = ((user_port % PORTS_PER_PSE_CHIP) == 0) ? PORTS_PER_PSE_CHIP : (user_port % PORTS_PER_PSE_CHIP);
	//*pse_port = 5-(*pse_port);

	return (ERR_NO_ERROR);
}

static int ad_poe_pse_port2User_port_mp3924(usp_uint8_t pse_num, usp_uint8_t pse_port, int *user_port)
{
	if( NULL == user_port )
	{
		return (ERR_BAD_PARAM);
	}

	//*user_port = (pse_num * PORTS_PER_PSE_CHIP) + (5-pse_port);
	*user_port = (pse_num * PORTS_PER_PSE_CHIP) + pse_port;

	return (ERR_NO_ERROR);
}

int ad_poe_port_check_power_allowed(IN int user_port, IN int sys_remain_power)
{
	/*
	if(g_poe_control.poe_line_per_port == POE_PORT_DUAL_LINE)
	{
		return ad_poe_port_check_power_allowed_2388x(user_port,sys_remain_power);
	}
	else
	{
		return TRUE;
	}
	*/
	return TRUE;
	
}


static int ad_poe_user_port2Pse_port_default(int user_port, usp_uint8_t *pse_num, usp_uint8_t *pse_port)
{

	//POINTER_IS_VALID(pse_num);
	//POINTER_IS_VALID(pse_port);

	*pse_num = (user_port - 1) / g_poe_control.poe_port_per_pse;
	*pse_port = ((user_port % g_poe_control.poe_port_per_pse) == 0) ? g_poe_control.poe_port_per_pse : (user_port % g_poe_control.poe_port_per_pse);	

	return (ERR_NO_ERROR);
}

static int ad_poe_pse_port2User_port_default(usp_uint8_t pse_num, usp_uint8_t pse_port, int *user_port)
{
	//POINTER_IS_VALID(user_port);

	*user_port = (pse_num * g_poe_control.poe_port_per_pse) + (pse_port);

	return (ERR_NO_ERROR);
}

static int ad_poe_user_port2Pse_port(int user_port, usp_uint8_t *pse_num, usp_uint8_t *pse_port)
{
	switch (get_model_info())
	{	
		case AD_MARVELL_ER7212PC:
			return ad_poe_user_port2Pse_port_mp3924(user_port, pse_num, pse_port);
		default:
			return ad_poe_user_port2Pse_port_default(user_port, pse_num, pse_port);
	}
}


static int ad_poe_pse_port2User_port(usp_uint8_t pse_num, usp_uint8_t pse_port, int *user_port)
{
	switch (get_model_info())
	{
		case AD_MARVELL_ER7212PC:
			return ad_poe_pse_port2User_port_mp3924(pse_num, pse_port, user_port);
		default:
			return ad_poe_pse_port2User_port_default(pse_num, pse_port, user_port);
	}
}

/*!
 *\Function:
 *      ad_poe_reconnect_port
 *\Description:
 *      由于功率管理而关闭的端口, 每隔一段时间对其重新尝试上电.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_reconnect_port(int port_no)
{
    power_info_t *power_info;
	poe_port_info_t *port_info;
	int timeseg_dis = 0;
	int recovery_dis = 0;

	POE_PORT_CHK(port_no);

	port_info = &(POE_PORT_INFO(port_no));
	power_info=&(POE_PORT_POWER_INFO(port_no));
	timeseg_dis = POE_MANAGE_TIMESEG_DIS_FLAG(port_no);
	recovery_dis = POE_MANAGE_RECOVERY_DIS_FLAG(port_no);

    /* 重新打开因为功率管理而关闭的端口 */
    if (power_info->actual_state == POE_DISABLE \
        && port_info->poe_enable == POE_ENABLE \
        && timeseg_dis == POE_RELEASE
        && recovery_dis == POE_RELEASE)
    {
		ad_poe_set_port_state(PSE_ADPORT_TO_USERPORT(port_no), POE_ENABLE);
    }
   
	return (ERR_NO_ERROR);
}


/*!
 *\Function:
 *      ad_poe_set_poe_port_led_state
 *\Description:
 *      Set Po_e port LED state.
 *\Input:
 *      user_port      - user_port, start from 1
 *      state         - led state
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_set_poe_port_led_state_one_line(int user_port, POE_LED_STATE state)
{
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
    POE_PORT_INFO(user_port).led_state_backup = state;
    if (POE_MANAGE_LED_FORCE_FLAG(user_port) == 0)
    {
        POE_PORT_INFO(user_port).led_state_cur = state;
    }
    POE_CONTROL_SEM_GIVE();
    
    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_set_poe_port_led_state
 *\Description:
 *      Set Po_e port LED state.
 *\Input:
 *      user_port      - user_port, start from 1
 *      state         - led state
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_poe_port_led_state(int user_port, POE_LED_STATE state)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
	int sys_power_limit = 0;
    
    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport())
    {
        return ERR_BAD_PARAM;
    }
    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_set_poe_port_led_state_one_line(ad_port_num, state);
    }
    
    return ERR_NO_ERROR;
}


/*!
 *\Function:
 *      ad_poe_get_poe_port_led_state
 *\Description:
 *      Get Po_e port LED state.
 *\Input:
 *      user_port      - user_port, start from 1  
 *\Output:
 *      state         - led state
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_get_poe_port_led_state_one_line(int user_port, POE_LED_STATE *state)
{
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX || NULL == state)
    {
        return ERR_BAD_PARAM;
    }
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
    *state = POE_PORT_INFO(user_port).led_state_cur;
    POE_CONTROL_SEM_GIVE();
    
    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_get_poe_port_led_state
 *\Description:
 *      Get Po_e port LED state.
 *\Input:
 *      user_port      - user_port, start from 1  
 *\Output:
 *      state         - led state
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_poe_port_led_state(int user_port, POE_LED_STATE *state)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }
    
    ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
    ad_poe_get_poe_port_led_state_one_line(ad_port_num, state);
    
    return ERR_NO_ERROR;
}


/*!
 *\Function:
 *      ad_poe_port_led_force_state_clear
 *\Description:
 *      clear the forced led state
 *\Input:
 *      user_port      - user_port, start from 1  
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      only used in ad_poe_flag_manage() , no lock!
 */
int ad_poe_port_led_force_state_clear(int user_port)
{

	POE_PORT_CHK(user_port);

	if(POE_MANAGE_LED_FORCE_FLAG(user_port) == 0)
	{
		POE_PORT_INFO(user_port).led_state_cur = POE_PORT_INFO(user_port).led_state_backup;
	}
	
	return (ERR_NO_ERROR);
}


#if 0
#endif

/*!
 *\fn 	  		int ad_poe_reg_write(int pse_index, unsigned char reg_addr, unsigned char data)
 *\brief   		往I2C设备写数据
 *
 *\param[in]	pse_index - PSE 设备偏移，0~5
 *				reg_addr  - PSE 寄存器地址 
 *				data     - 写到寄存器上的数据
 *\param[out]  	N/A
 *
 *\return  		ERR_NO_ERROR	- on success
 *             (Others)			- on error
 *\note		  
 */
int ad_poe_reg_write(int pse_index, unsigned char reg_addr, unsigned char data)
{
	int rv;
	unsigned char buffer[2]={0};
	unsigned int length = 2;
	int   retry = 0;
	
	PSE_CHIP_CHK(pse_index);

	buffer[0] = reg_addr;
	buffer[1] = data; 
	
	while(retry++ <= POE_RESEND_MAX_CNT)
	{
		rv = ad_poe_bus_write(pse_addr[pse_index], buffer, length);
		if( ERR_NO_ERROR == (rv) )
		{
			//DBG(" %s : [%02X] = 0x%02x\r\n", __FUNCTION__,reg_addr,data);
			return rv;
		}
		pal_usleep(10*1000);
	}

	/* 写失败，复位i2c总线，并添加i2c操作失败Log*/
	if(0)
	{
		int flag_value;
		char buff[32] = {0};	
		ad_poe_bus_reset();
		pal_usleep(10*1000);
		ad_poe_port_flag_get(pse_index+POE_UPORT_MIN, POE_I2C_RW_FAILED, &flag_value);
		if (0 == flag_value)
		{
			/* syslog */
			sprintf(buff, "on PSE %d", pse_index);
			//syslog_add_for_po_e(LOG_POE_I2C_RW_FAILED, (int)buff, 2, 3, 4, 5, 6);
			/* 3min 内不再traps */
			ad_poe_port_flag_set(pse_index+POE_UPORT_MIN,POE_I2C_RW_FAILED, POE_TIMER_I2C_RW_FAILED);
		}
	}
	return rv;
}


/*!
 *\fn 	  	int ad_poe_reg_read(int pse_index, unsigned char reg_addr, unsigned char *data)
 *\brief   	从I2C设备读数据
 *
 *\param[in]   pse_index - PSE 设备偏移，0~5
 *			reg_addr  - PSE 寄存器地址 
 *\param[out]  data     - 保存读取的值
 *
 *\return  	ERR_NO_ERROR  - on success
 *             (Others)      - on error
 *\note		  
 */
int ad_poe_reg_read(int pse_index, unsigned char reg_addr, unsigned char *data)
{
	int rv;

	unsigned char buffer[2]={0};
	unsigned int length = 2;
	int   retry = 0;

	//POINTER_IS_VALID(data);
	PSE_CHIP_CHK(pse_index);

	buffer[0] = reg_addr;
	
	while( retry++ <= POE_RESEND_MAX_CNT )
	{
		rv = ad_poe_bus_read(pse_addr[pse_index], buffer, &length);
		
		if( ERR_NO_ERROR == (rv) )
		{
			*data = buffer[1];
			//DBG(" %s : [%02X] = 0x%02x\r\n", __FUNCTION__,reg_addr,buffer[1]);
			return rv;
		}
		//pal_usleep(10*1000);
	}

	/* 读失败，复位i2c总线，并添加i2c操作失败Log*/
	if(1)
	{
		int flag_value;
		char buff[32] = {0};
		
		ad_poe_bus_reset();
		pal_usleep(10*1000);
		ad_poe_port_flag_get(pse_index+POE_UPORT_MIN, POE_I2C_RW_FAILED, &flag_value);
		if (0 == flag_value)
		{
			/* syslog */
			sprintf(buff, "on PSE %d", pse_index);
			//syslog_add_for_po_e(LOG_POE_I2C_RW_FAILED, (int)buff, 2, 3, 4, 5, 6);
			/* 3min 内不再traps */
			ad_poe_port_flag_set(pse_index+POE_UPORT_MIN,POE_I2C_RW_FAILED, POE_TIMER_I2C_RW_FAILED);
		}			
	}
	return rv;
}

/* For check_addr/dump */
int ad_poe_reg_read_no_log(int pse_index, unsigned char reg_addr, unsigned char *data)
{
	int rv;
	unsigned char buffer[2]={0};
	unsigned int length = 2;
	int   retry = 0;
	
	//POINTER_IS_VALID(data);
	PSE_CHIP_CHK(pse_index);

	buffer[0] = reg_addr;
	
	while( retry++ <= POE_RESEND_MAX_CNT )
	{
		rv = ad_poe_bus_read(pse_addr[pse_index], buffer, &length);
		
		if( ERR_NO_ERROR == (rv) )
		{
			*data = buffer[1];
			//DBG(" %s : [%02X] = 0x%02x\r\n", __FUNCTION__,reg_addr,buffer[1]);
			return rv;
		}
		//pal_usleep(10*1000);
	}

	/* 读失败，复位i2c总线，不添加i2c操作失败Log*/	
	ad_poe_bus_reset();
	pal_usleep(10*1000);
	
	return rv;
}

#if 0
/*!
 *\fn 	  	static int ad_poe_config_i2c_addr(void)
 *\brief   	配置I2C地址
 *
 *\param[in]   N/A
 *\param[out]  N/A
 *
 *\Return		配置结果
 *			0 - 成功
 *		    -1 - 打开广播地址失败，可判定为未插Po_e子板
 *\Note		调用前，已调用 soc_i2c_reset 与 soc_i2c_probe
 */
static int ad_poe_config_i2c_addr(void)
{
	int rv = ERR_NO_ERROR;
	int index = 0;
	int retry = 0;
	unsigned char buffer[2] = {0};
	
	buffer[0] = PSE_REG_UNLOCK;
	buffer[1] = PSE_TEMP_ADDR;

	while (retry++ < I2C_BUS_RETRY_TIMES)
	{
		/* 分配临时地址 */
		ad_misc_i2c_data_set(I2C_TYPE_GPIO, I2C_TPS23861_SADDR30, buffer[0], 1, &(buffer[1]));

		/* 分配真正地址 */
		PSE_CHIP_ITER(index)
		{
			rv = ad_misc_i2c_data_set(I2C_TYPE_GPIO, pse_temp_addr[index], PSE_REG_SADDR, 1, &(pse_addr[index]));
			if (ERR_NO_ERROR != rv)
			{
				DBG("%s: ad_misc_i2c_data_set failed on pse#%d.\r\n", __FUNCTION__, index);
				break;
			}
		}
		
		if (ERR_NO_ERROR == rv)
		{
			break;
		}
	}

	/* 编址完成后等待1s再Reset */
	pal_usleep(1000*1000);
	ad_poe_chip_reset();

	return rv;
}
#endif


#if 0
/*!
 *\fn 	  	static int check_i2c_addr_config(int *cfg_times)
 *\brief   	读取文件，检查是否配置了I2C地址
 *
 *\param[in]   N/A
 *\param[out]  cfg_times - 保存配置的次数
 *
 *\Return		检查结果
 *			FALSE - 没有配置地址
 *			TRUE  - 已经配置了地址
 *  	  
 */
static int check_i2c_addr_config_auto_fix(int *all_cfg_addr_times, int *con_cfg_addr_times)
{
	FILE * fp = NULL;
	int addr_cfg_times_con = 0;
	int addr_cfg_times_all = 0;

	if( NULL == all_cfg_addr_times || NULL == con_cfg_addr_times )
	{
		DBG("[%s] invalid input!\r\n", __FUNCTION__);
		return FALSE;
	}
		
	fp =  fopen(PSE_I2C_ADDR_CFG_INFO_FILE, "r");
	if( NULL == fp )
	{
		*all_cfg_addr_times = 0;
		*con_cfg_addr_times = 0;
		DBG("[%s] %s file open failed!\r\n", __FUNCTION__, PSE_I2C_ADDR_CFG_INFO_FILE);
		return FALSE;
	}
	fscanf(fp,"%d %d\n",&addr_cfg_times_all, &addr_cfg_times_con);
	DBG("[%s] addr_cfg_times_all=%d, addr_cfg_times_con=%d\n", __FUNCTION__, addr_cfg_times_all, addr_cfg_times_con);
	fclose(fp);

	// 配置成功后con_cfg_addr_times和all_cfg_addr_times都需要+1
	if( addr_cfg_times_all <= 0)
	{
		*all_cfg_addr_times = 0;
		*con_cfg_addr_times = 0;
		return FALSE;
	}
	else
	{
		*all_cfg_addr_times = addr_cfg_times_all;
		*con_cfg_addr_times = addr_cfg_times_con;
		DBG("[%s] read %s file SUCCESS! addr_cfg_times_con = %d, addr_cfg_times_all = %d.\r\n", __FUNCTION__, PSE_I2C_ADDR_CFG_INFO_FILE, addr_cfg_times_con, addr_cfg_times_all);
		return TRUE;
	}
}
#endif

#if 0
static int check_i2c_addr_config_default(int *cfg_times)
{
	FILE * fp = NULL;
	int addr_cfg_times = 0;

	if( NULL == cfg_times)
		return FALSE;
	
	fp =  fopen(PSE_I2C_ADDR_CFG_INFO_FILE, "r");

	if( NULL == fp )
	{
		*cfg_times = 0;
		return FALSE;
	}

	fscanf(fp,"%d\n",&addr_cfg_times);

	DBG(" addr_cfg_times=%d\n",addr_cfg_times);

	fclose(fp);
	
	if( addr_cfg_times <= 0 )
	{
		return FALSE;
	}
	else
	{
		*cfg_times =  addr_cfg_times;
		return TRUE;
	}
	
}
#endif

/*!
 *\fn 	  	void update_i2c_addr_config_info(void)
 *\brief   	保存最新的I2C地址配置次数到文件，为防止开关机测试失败，
 *			需要在系统启动之后才调用。
 *			
 *\param[in]   N/A
 *\param[out]  N/A
 *
 *\Return		N/A  
 */
static void update_i2c_addr_config_info_auto_fix(void)
{
	FILE * fp = NULL;

	if(FALSE == has_config_addr || all_cfg_addr_times == 0 || con_cfg_addr_times == 0)
	{
		return;
	}

	fp =  fopen(PSE_I2C_ADDR_CFG_INFO_FILE, "w+");

	if( NULL == fp )
		return ;
	
	has_config_addr = FALSE; /* 让开机后，仅能调用一次 */
	
	fprintf(fp,"%d %d\n", all_cfg_addr_times, con_cfg_addr_times);

	fclose(fp);
}

static void update_i2c_addr_config_info_default(void)
{
	FILE * fp = NULL;

	if(FALSE == has_config_addr || config_addr_times == 0 )
	{
		return;
	}

	fp =  fopen(PSE_I2C_ADDR_CFG_INFO_FILE, "w+");

	if( NULL == fp )
		return ;
	
	has_config_addr = FALSE; /* 让开机后，仅能调用一次 */
	
	fprintf(fp,"%d\n", config_addr_times);

	fclose(fp);
}

static void update_i2c_addr_config_info(void)
{
	if (i2c_addr_autofix)
	{
		return update_i2c_addr_config_info_auto_fix();
	}
	else
	{
		return update_i2c_addr_config_info_default();
	}
}

/*!
 *\Function:
 *      ad_poe_enable
 *\Description:
 *      enabe the pse power supply
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_enable(void)
{
	update_i2c_addr_config_info();
	return (ERR_NO_ERROR);
}

#if 0
int ad_poe_check_addr()
{
	int i = 0;
	unsigned char data = 0;
	
	PSE_CHIP_ITER(i)
	{
		ad_poe_reg_read_no_log(i, REG_I2C_SLAVE_ADDRESS, &data);
		if ((data&0x7F) != pse_addr[i])
		{
			DBG("[%s] PSE #%d address check ERROR!\r\n", __FUNCTION__, i);
			return ERROR;
		}
		DBG("[%s] PSE #%d address check OK!\r\n", __FUNCTION__, i);
	}
	return ERR_NO_ERROR;
}
#endif

int ad_poe_get_addr_cfg_times(int *addr_cfg_times_all, int *addr_cfg_times_con)
{
	if (addr_cfg_times_all == NULL || addr_cfg_times_con == NULL)
	{
		return ERROR;
	}

	if (i2c_addr_autofix)
	{
		*addr_cfg_times_all = all_cfg_addr_times;
		*addr_cfg_times_con = con_cfg_addr_times;
	}
	else
	{
		*addr_cfg_times_all = config_addr_times;
		*addr_cfg_times_con = -1;
	}

	return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_chip_init
 *\Description:
 *      Init the pse chip
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */

 #if 0
/* 引入地址配置出错的纠错机制:
 * (1) 先读取i2c_addr.cfg文件，得到all_cfg_addr_times和con_cfg_addr_times(连续配置地址次数、总配置地址次数)
 * (2) 如果all_cfg_addr_times和con_cfg_addr_times为0，或者无i2c_addr.cfg文件，说明从未配置地址，则正常执行地址配置；否则不配置地址
 * (3) 接着尝试进行tps23861寄存器的读写，如果出错，则查看是否满足可继续配置地址的标准来尝试纠错
 * (4) 可继续配置地址的标准为: con_cfg_addr_times未达到CON_ADDR_CFG_MAX_CNT，且all_cfg_addr_times未达到ALL_ADDR_CFG_MAX_CNT
 * (5) 满足标准的情况下进行地址配置和寄存器读写的操作；否则不再进行地址配置；若寄存器读写出错，则重复(5)
 * (6) 关键变量: all_cfg_addr_times/con_cfg_addr_times保存在flash，每次编址都要+1，成功编址后con_cfg_addr_times刷新为1
 * (7) 关键变量: ALL_ADDR_CFG_MAX_CNT/CON_ADDR_CFG_MAX_CNT为常量，初步定为2/5
 */
static int ad_poe_chip_init_auto_fix(void)
{
	int offset = 0;
	int count = 0;
	int fd_broad_cast = 0;
	int rv;
	usp_uint8_t data = 0;

	/* 检查是否有配置I2C地址，没则配置 */
	if( FALSE == check_i2c_addr_config_auto_fix(&all_cfg_addr_times, &con_cfg_addr_times) )
	{
		DBG("no i2c address config history, try to config addr!\n");
		if( 0 !=  ad_poe_config_i2c_addr() )
		{
			DBG("#1 Config i2c address for PSE failed.\n");
			//return (ERR_HW_OP_FAIL);
		}
		con_cfg_addr_times++;
		all_cfg_addr_times++;
		has_config_addr = TRUE;
		DBG("i2c address config end! con_cfg_addr_times = %d, all_cfg_addr_times = %d.\n", con_cfg_addr_times, all_cfg_addr_times);
	}
	/* 检查上一次开机的地址配置是否OK，否则尝试重配 */
	else if (ERR_NO_ERROR != ad_poe_check_addr())
	{
		if (!ADDR_CFG_CNT_EXCEED)
		{
			DBG("previous i2c address config failed, try to config addr again!\n");
			if( 0 !=  ad_poe_config_i2c_addr() )
			{
				DBG("#1 Config i2c address for PSE failed.\n");
				//return (ERR_HW_OP_FAIL);
			}
			con_cfg_addr_times++;
			all_cfg_addr_times++;
			has_config_addr = TRUE;
			DBG("i2c address config end! con_cfg_addr_times = %d, all_cfg_addr_times = %d.\n", con_cfg_addr_times, all_cfg_addr_times);
		}
		else
		{
			DBG("[%s] addr check failed, addr cfg cnt exceeded.\r\n", __FUNCTION__);
		}
	}

	if (ERR_NO_ERROR == ad_poe_check_addr())
	{
		/* 寄存器读写成功，全部地址配置OK，con_cfg_addr_times刷新为1 */
		con_cfg_addr_times = 1;
		has_config_addr = TRUE;
		DBG("i2c address config SUCCESS! con_cfg_addr_times reset to 1.\r\n");
	}

	/* 打开所有PSE，更新fd_pse数组 */
	PSE_CHIP_ITER(offset)
	{
		if(g_poe_control.standard==IEEE802_3_AF)
		{
			for(count = 0; count < sizeof(pse_init_data_af)/sizeof(pse_init_data_af[0]); count++)
			{
				rv = ad_poe_reg_write(offset, pse_init_data_af[count][0], pse_init_data_af[count][1]);
				if(rv != ERR_NO_ERROR)
				{
					DBG(" %s : [%02X] = 0x%02x write error\r\n", __FUNCTION__,pse_init_data_af[count][0],pse_init_data_af[count][1]);	
				}
				pal_usleep(100*1000);
			}
		}
		else
		{
			for(count = 0; count < sizeof(pse_init_data_at)/sizeof(pse_init_data_at[0]); count++)
			{
				rv = ad_poe_reg_write(offset, pse_init_data_at[count][0], pse_init_data_at[count][1]);
				if(rv != ERR_NO_ERROR)
				{
					DBG(" %s : [%02X] = 0x%02x write error\r\n", __FUNCTION__,pse_init_data_at[count][0],pse_init_data_at[count][1]);	
				}
				pal_usleep(100*1000);
			}
		}
	}
	
   	return (ERR_NO_ERROR);
}
#endif
#if 0
static int ad_poe_chip_init_default(void)
{
	int offset = 0;
	int count = 0;
	int fd_broad_cast = 0;
	int rv;

	usp_uint8_t data = 0;

	/* 检查是否有配置I2C地址，没则配置 */
	if( FALSE == check_i2c_addr_config_default(&config_addr_times))
	{
		DBG("no i2c address config history!\n");
		if( 0 !=  ad_poe_config_i2c_addr() )
		{
			DBG("#1 Config i2c address for PSE failed.\n");
			return (ERR_HW_OP_FAIL);
		}
		has_config_addr = TRUE;
		config_addr_times++;
	}
	//DBG(" i2c address config end!\n");
	/* 打开所有PSE，更新fd_pse数组 */
	//DBG("\r\n");	
	PSE_CHIP_ITER(offset)
	{
		if(g_poe_control.standard==IEEE802_3_AF)
		{
			for(count = 0; count < sizeof(pse_init_data_af)/sizeof(pse_init_data_af[0]); count++)
			{
				rv = ad_poe_reg_write(offset, pse_init_data_af[count][0], pse_init_data_af[count][1]);
				if(rv < 0)
				{
					printf(" %s : [%02X] = 0x%02x write error\r\n", __FUNCTION__,pse_init_data_af[count][0],pse_init_data_af[count][1]);	
				}
				pal_usleep(100*1000);
			}
		}
		else
		{
			for(count = 0; count < sizeof(pse_init_data_at)/sizeof(pse_init_data_at[0]); count++)
			{
				rv = ad_poe_reg_write(offset, pse_init_data_at[count][0], pse_init_data_at[count][1]);
				if(rv < 0)
				{
					printf(" %s : [%02X] = 0x%02x write error\r\n", __FUNCTION__,pse_init_data_at[count][0],pse_init_data_at[count][1]);	
				}
				pal_usleep(100*1000);
			}
		}
	}	
	//DBG("\r\n");
	/*PSE_CHIP_ITER(offset)
	{
		for(count = 0; count < sizeof(pse_init_data)/sizeof(pse_init_data[0]); count++)
		{
			rv = ad_poe_reg_read(offset, pse_init_data[count][0], &data);
			if(rv <0)
			{
				DBG(" %s : [%02X] = 0x%02x read error\r\n", __FUNCTION__,pse_init_data[count][0],data);
			}
			else
			{
				DBG(" %s : [%02X] = 0x%02x\r\n", __FUNCTION__,pse_init_data[count][0],data);
			}
			pal_usleep(10*1000);
		}
		rv = ad_poe_reg_read(offset,0x11, &data);
		if(rv <0)
		{
			DBG(" %s : [0X11] = 0x%02x read error\r\n", __FUNCTION__,data);
		}
		else
		{
			DBG(" %s : [0X11] = 0x%02x\r\n", __FUNCTION__,data);
		}
		pal_usleep(10*1000);
	}*/
	//DBG("\r\n");
	/*PSE_CHIP_ITER(offset)
	{
		for(count = 0; count < sizeof(pse_init_data)/sizeof(pse_init_data[0]); count++)
		{
			rv = ad_poe_reg_write(offset, pse_init_data[count][0], pse_init_data[count][1]);
			if(rv < 0)
			{
				DBG(" %s : [%02X] = 0x%02x\r\n", __FUNCTION__,pse_init_data[count][0],pse_init_data[count][1]);	
			}
			pal_usleep(10*1000);
		}
	}*/	
	//DBG("\r\n");
	/*PSE_CHIP_ITER(offset)
	{
		for(count = 0; count < sizeof(pse_init_data)/sizeof(pse_init_data[0]); count++)
		{
			rv = ad_poe_reg_read(offset, pse_init_data[count][0], &data);
			DBG(" %s : [%02X] = 0x%02x\r\n", __FUNCTION__,pse_init_data[count][0],data);
			pal_usleep(10*1000);
		}
	}*/
	//DBG("\r\n");
   	return (ERR_NO_ERROR);
}
#endif

int ad_poe_reg_write_by_addr(unsigned int addr, unsigned char reg_addr, unsigned char data)
{
    int rv = 0;
    unsigned char buffer[2]={0};
    unsigned int length = 2;
    int retry = 0;

    buffer[0] = reg_addr;
    buffer[1] = data; 
    
    while(retry++ <= POE_RESEND_MAX_CNT)
    {
        rv = ad_poe_bus_write(addr, buffer, length);
        if (ERR_NO_ERROR == rv)
        {
            return rv;
        }
        pal_usleep(10*1000);
    }
    
    return rv;
}


int ad_poe_reg_read_by_addr(unsigned int addr, unsigned char reg_addr, unsigned char *data)
{
    int rv = 0;
    unsigned char buffer[2]={0};
    unsigned int length = 2;
    int  retry = 0;
    
    if (NULL == data)
    {
        return ERR_BAD_PARAM;
    }

    buffer[0] = reg_addr;
    
    while (retry++ <= POE_RESEND_MAX_CNT)
    {
        rv = ad_poe_bus_read(addr, buffer, &length);
        
        if (ERR_NO_ERROR == rv)
        {
            *data = buffer[1];
            return rv;
        }
        pal_usleep(10*1000);
    }

    
    return rv;
}

#if 0
unsigned char check_Sram_state(void)
{
    unsigned char ICcount = 0;
    unsigned char temp=0;
    int rv = 0;
   
    for(ICcount = 0; ICcount < PSE_CHIP_NUMS; ICcount ++ )                /*所有端口*/
    {
        //SUPPLY and FAULT EVENT Register 
        rv = ad_poe_reg_read(ICcount, TPS238X_SUPPLY_EVENT_CLEAR_COMMAND, &temp);
        if(ERR_NO_ERROR != rv || temp&0x1) //bit0:Indicates that a SRAM fault has occurred
        {
            return FALSE; //出错
        }
        //Check the firmware version to make sure the SRAM code has been loaded properly
        rv = ad_poe_reg_read(ICcount, TPS238X_FIRMWARE_REVISION_COMMAND, &temp);
        if(ERR_NO_ERROR != rv || SRAM_VERSION != temp) //Indicates that a SRAM fault has occurred
        {
            return FALSE; //出错
        }
    }
    return TRUE;
}

int ad_poe_device_probe(void)
{
	int dev_num = 0; /* start from 0 */
	char barr[1] = {0};
	int dev_found = 0;
    
	for (dev_num = 0; dev_num < PSE_CHIP_NUMS; dev_num++)
	{
		if (ad_poe_reg_read(dev_num, (unsigned char)TPS238X_DEVICE_ID_COMMAND, &barr[0]) != ERR_NO_ERROR)
		{
			 DBG("	Po_e device %d# : Communication ERROR.\r\n", dev_num);
			 continue;
		}
		else
		{
			 /* invalid part ID */
			 if ((barr[0] & 0x20) != 0x20)
			 {
				 DBG("	Po_e device %d# : Part_ID=0x%x ERROR.\r\n", dev_num, barr[0]);
				 continue;
			 }

			 DBG("	Po_e device %d# : Detected OK. val: %#x\r\n", dev_num, TPS238X_DEVICE_ID_COMMAND);
			 dev_found++;
		}
	}

	return dev_found;
}
#endif

int ad_poe_set_icut_ilim_mp3924(int user_port)
{
	int rv = 0;
	usp_uint8_t pse_num = 0;
	usp_uint8_t pse_port = 0;
	
	if(user_port <POE_UPORT_MIN|| user_port > POE_UPORT_MAX)
	{
		return (ERR_BAD_PARAM);
	}

	ad_poe_user_port2Pse_port(user_port, &pse_num, &pse_port);
	
	FUNC_CALL_DONE(ad_poe_reg_write(pse_num, (MP3924_POE_ICUT1_THRESHOLD + pse_port-1), 0x04),rv);
	pal_usleep(10 * 1000);
	FUNC_CALL_DONE(ad_poe_reg_write(pse_num, (MP3924_POE_ILIM1_THRESHOLD + pse_port-1), 0x01),rv);
	
_done:
	return (ERR_NO_ERROR);

}


int  ad_poe_chip_init_mp3924()
{
    int offset = 0;
	int count = 0;
	int rv;
	usp_uint8_t data = 0;
	
	PSE_CHIP_ITER(offset)
    {
		for(count = 0; count < sizeof(pse_init_data_mp3924)/sizeof(pse_init_data_mp3924[0]); count++)
		{
			rv = ad_poe_reg_write(offset, pse_init_data_mp3924[count][0], pse_init_data_mp3924[count][1]);
			if(rv != 0)
			{
				printf(" [%02X] : [%02X] = 0x%02x write error\r\n", pse_addr[offset], pse_init_data_mp3924[count][0], pse_init_data_mp3924[count][1]);
				return (ERR_CONFIG_WRITE_PROFILE);
			}
			pal_usleep(100*1000);
		}
		ad_poe_reg_read(offset, MP3924_POE_POWER_SOURCE_STATUS1_CLEAR,  &data);

  	}	
	/*
	DBG("\r\n");
	PSE_CHIP_ITER(offset)
    {
		for(count = 0; count < sizeof(pse_init_data_mp3924)/sizeof(pse_init_data_mp3924[0]); count++)
		{
			rv = ad_poe_reg_read(offset, pse_init_data_mp3924[count][0], &data);
			DBG(" %s %d :[%02X] : [%02X] = 0x%02x\r\n", __FUNCTION__, rv, pse_addr[offset],pse_init_data_mp3924[count][0], data);
			pal_usleep(100*1000);
		}
  	}
  	*/
	return (ERR_NO_ERROR); 
}


static int ad_poe_chip_init(void)
{
	int rv=ERR_NO_ERROR;

	if(NULL != ti_lib.ad_poe_chip_init)
	{
		rv = ti_lib.ad_poe_chip_init();
	}

	return rv;

}


/*!
 *\Function:
 *      ad_poe_init_poe_port_info
 *\Description:
 *      Init data struct of poe_port_info_t.
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      n/a
 *\Note:
 *      n/a
 */
static void ad_poe_attach(void)
{
	int port_no;
	poe_port_info_t *poe_port;
	power_info_t *power_info;

 if(PSE_CHIP_TYPE_MP3924 == get_pse_type_info())
	{
		ti_lib.ad_poe_chip_init = ad_poe_chip_init_mp3924;
		ti_lib.ad_poe_get_port_current_from_reg_val = ad_poe_get_port_current_from_reg_val_mp3924;
		ti_lib.ad_poe_get_port_voltage_from_reg_val = ad_poe_get_port_voltage_from_reg_val_mp3924;
		ti_lib.ad_poe_get_port_class_from_reg_val = ad_poe_get_port_class_from_reg_val_mp3924;
		ti_lib.ad_poe_set_port_state = ad_poe_set_port_state_mp3924;
		ti_lib.ad_poe_port_power_on_control = ad_poe_port_power_on_control_mp3924;
		ti_lib.ad_poe_updt_event_status = ad_poe_updt_event_status_mp3924;
		ti_lib.ad_poe_check_class = ad_poe_check_class_mp3924;
		ti_lib.ad_poe_power_info_updt = ad_poe_power_info_updt_mp3924;
	}
	
	g_poe_control.sys_poe_enable=POE_ENABLE;
	g_poe_control.sys_power_limit=get_poe_power_limit();
	printf("g_poe_control.sys_power_limit:%d\n", g_poe_control.sys_power_limit);
	g_poe_control.sys_actual_cons=0;
	
	g_poe_control.poe_port_per_pse = PORTS_PER_PSE_CHIP;
	g_poe_control.poe_line_per_port = POE_PORT_SINGLE_LINE;
	
	if(get_poe_power_limit_per_port() == 154)
	{
		g_poe_control.standard=IEEE802_3_AF;
	}
	else if (get_poe_power_limit_per_port() == 300)
	{
		g_poe_control.standard=IEEE802_3_AT;
	}
	else
	{
		g_poe_control.standard=IEEE802_3_BT;
		g_poe_control.poe_line_per_port = POE_PORT_DUAL_LINE;
	}

	g_poe_control.poe_port_num = get_poe_port_num()*g_poe_control.poe_line_per_port;
	g_poe_control.poe_uport_min = get_poe_first_uport();
	g_poe_control.poe_uport_max = get_poe_last_uport()*g_poe_control.poe_line_per_port;

	for (port_no = 1; port_no <= MAX_POE_PORT_NUM; port_no++)
	{
		poe_port = &(g_poe_control.poe_port_info[port_no-1]);
		power_info = &(g_poe_control.poe_port_info[port_no-1].power_info);
		
		/******************************************************************/
		poe_port->port_no       =   port_no;
		poe_port->poe_enable  =   POE_ENABLE;
		poe_port->priority        =   POE_PRI_LOW_AD;
		poe_port->power_limit  =   get_poe_power_limit_per_port(); /* 300 * 0.1W or 154 * 0.1W*/
		poe_port->stm_state    =   STM_BEGIN; /* 初始状态 */
		/******************************************************************/
		
		power_info->actual_state = POE_DISABLE;
		power_info->actual_cur   = 0;
		power_info->actual_vol   = 0;
		power_info->actual_cons  = 0;
		power_info->pd_class     = POE_CLASS_CLASSUN_AD; /* --- */
		power_info->power_state = POE_OFF;	
	}
}

#if 0 /*操作硬件的函数*/
#endif


/*!
 *\Function:
 *      ad_poe_get_port_current_from_reg_val
 *\Description:
 *      Get the Po_e port current according to the register.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      current       - port current (unit: m_a)
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */




int ad_poe_get_port_current_from_reg_val_mp3924(int user_port, float *current)
{
	int rv;
	unsigned char data = 0;	
	unsigned short current_temp = 0;
	usp_uint8_t pse_num = 0;
	usp_uint8_t pse_port = 0;
	usp_uint32_t i = 0;
	float current_total = 0;
	
	
	if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX || NULL == current )
	{
	    return (ERR_BAD_PARAM);
	}

	FUNC_CALL(ad_poe_user_port2Pse_port(user_port, &pse_num, &pse_port), rv);

	for(i = 0 ; i < MP3924_AVERRAGE_Current_and_Voltage; i++)
	{
		switch(pse_port)
		{
			case 1:
				FUNC_CALL(ad_poe_reg_read(pse_num,MP3924_POE_PORT_1_CURRENT_LSB, &data), rv);
				current_temp = data;
				FUNC_CALL(ad_poe_reg_read(pse_num,  MP3924_POE_PORT_1_CURRENT_MSB, &data), rv);
				current_temp += ( unsigned short )data<< 1;
				break;
			case 2:
				FUNC_CALL(ad_poe_reg_read(pse_num,  MP3924_POE_PORT_2_CURRENT_LSB, &data), rv);
				current_temp = data;
				FUNC_CALL(ad_poe_reg_read(pse_num,  MP3924_POE_PORT_2_CURRENT_MSB, &data), rv);
				current_temp += ( unsigned short )data<< 1;
				break;
			case 3:
				FUNC_CALL(ad_poe_reg_read(pse_num,  MP3924_POE_PORT_3_CURRENT_LSB, &data), rv);
				current_temp = data;
				FUNC_CALL(ad_poe_reg_read(pse_num,MP3924_POE_PORT_3_CURRENT_MSB, &data), rv);
				current_temp += ( unsigned short )data<< 1;
				break;
			case 4:
				FUNC_CALL(ad_poe_reg_read(pse_num,  MP3924_POE_PORT_4_CURRENT_LSB, &data), rv);
				current_temp = data;
				FUNC_CALL(ad_poe_reg_read(pse_num,  MP3924_POE_PORT_4_CURRENT_MSB, &data), rv);
				current_temp += ( unsigned short )data<< 1;
				break;
			default:
				return (ERR_BAD_PARAM);
		}
		current_total += (((float)current_temp )*   MP3924_PSE_I_UNIT);
		pal_usleep(2100);
	}

	if((current_total/MP3924_AVERRAGE_Current_and_Voltage) > MP3924_PSE_I_OFFSET)
	{
		*current =current_total/MP3924_AVERRAGE_Current_and_Voltage     - MP3924_PSE_I_OFFSET;
	}
	else
	{
		*current = 0;
	}
	
	return rv;

}


int ad_poe_get_port_current_from_reg_val(int user_port, float *current)
{
	int rv=ERR_NO_ERROR;

	if(NULL != ti_lib.ad_poe_get_port_current_from_reg_val)
	{
		rv = ti_lib.ad_poe_get_port_current_from_reg_val(user_port,current);
	}

	return rv;
}

/*!
 *\Function:
 *      ad_poe_get_port_voltage_from_reg_val
 *\Description:
 *      Get the Po_e port voltage according to the register.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      voltage       - port voltage (unit: 0.1V)
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */

int ad_poe_get_port_voltage_from_reg_val_mp3924(int user_port, float *voltage)
{
  	int rv;
	unsigned char data = 0;	
	unsigned short vol_temp = 0;
	unsigned short vol_input = 0;
	usp_uint8_t pse_num = 0;
	usp_uint8_t pse_port = 0;

	usp_uint32_t i = 0;
	float vol_total = 0;
	
	if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX || NULL == voltage )
	{
	    return (ERR_BAD_PARAM);
	}

	FUNC_CALL(ad_poe_user_port2Pse_port(user_port, &pse_num, &pse_port), rv);

	for(i = 0 ; i < MP3924_AVERRAGE_Current_and_Voltage; i++)
	{
		FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_INPUT_VOLTAGE_LSB, &data), rv);
		vol_input = data;
		FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_INPUT_VOLTAGE_MSB, &data), rv);
		vol_input += ( unsigned short )data<< 1;	
		
		switch(pse_port)
		{
			case 1:
				FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_OUT_1_VOLTAGE_LSB, &data), rv);
				vol_temp = data;
				FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_OUT_1_VOLTAGE_MSB, &data), rv);
				vol_temp += ( unsigned short )data<< 1;
				break;
			case 2:
				FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_OUT_2_VOLTAGE_LSB, &data), rv);
				vol_temp = data;
				FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_OUT_2_VOLTAGE_MSB, &data), rv);
				vol_temp += ( unsigned short )data<< 1;
				break;
			case 3:
				FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_OUT_3_VOLTAGE_LSB, &data), rv);
				vol_temp = data;
				FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_OUT_3_VOLTAGE_MSB, &data), rv);
				vol_temp += ( unsigned short )data<< 1;
				break;
			case 4:
				FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_OUT_4_VOLTAGE_LSB, &data), rv);
				vol_temp = data;
				FUNC_CALL(ad_poe_reg_read(pse_num, MP3924_POE_OUT_4_VOLTAGE_MSB, &data), rv);
				vol_temp += ( unsigned short )data<< 1;
				break;
			default:
				return (ERR_BAD_PARAM);
		}

		if(vol_input > vol_temp)
		{
			vol_total +=  ((((float)(vol_input - vol_temp))* MP3924_PSE_V_UNIT)/100);
		}
		pal_usleep(2100);
	}

	*voltage =vol_total/MP3924_AVERRAGE_Current_and_Voltage;
	
	return rv;

}


int ad_poe_get_port_voltage_from_reg_val(int user_port, float *voltage)
{
	int rv=ERR_NO_ERROR;

	if(NULL != ti_lib.ad_poe_get_port_voltage_from_reg_val)
	{
		rv = ti_lib.ad_poe_get_port_voltage_from_reg_val(user_port,voltage);
	}

	return rv;
}



/*!
 *\Function:
 *      ad_poe_get_port_class_from_reg_val
 *\Description:
 *      Get the Po_e port pd class according to the register.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      pd_class       - port Class ( 0 ~ 4 )
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */



int ad_poe_get_port_class_from_reg_val_mp3924(int user_port, int *pd_class)
{
    poe_port_info_t *poe_port;
    usp_uint8_t class_value;

    if( NULL == pd_class)
    {
	    return (ERR_BAD_PARAM); 
    }
    poe_port = &POE_PORT_INFO(user_port);
    class_value = poe_port->st_cla;
    
    switch (class_value)
    {
    case CLASS_0:
        *pd_class = POE_CLASS_CLASS0_AD;
        break;
    case CLASS_1:
        *pd_class = POE_CLASS_CLASS1_AD;
        break;
    case CLASS_2:
        *pd_class = POE_CLASS_CLASS2_AD;
        break;
    case CLASS_3:
        *pd_class = POE_CLASS_CLASS3_AD;
        break;
    case CLASS_4:
        *pd_class = POE_CLASS_CLASS4_AD;
        break;
    case CLASS_UNKNOWN:
    case CLASS_OVER_CURR:
	case CLASS_5:
        *pd_class = POE_CLASS_CLASSUN_AD; /* --- */
        break;
    default:
        *pd_class = POE_CLASS_CLASSUN_AD; /* --- */
        break;
    }

   return (ERR_NO_ERROR);
}


int ad_poe_get_port_class_from_reg_val(int user_port, int *pd_class)
{
	int rv=ERR_NO_ERROR;

	if(NULL != ti_lib.ad_poe_get_port_class_from_reg_val)
	{
		rv = ti_lib.ad_poe_get_port_class_from_reg_val(user_port,pd_class);
	}

	return rv;
}


/*!
 *\Function:
 *      ad_poe_set_port_state
 *\Description:
 *      setting the pse register to enable/disable power supply.
 *\Input:
 *      user_port      - user_port, start from 1
 *      status        - POE ability enabled or not
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      if set the status to enable, the power will not power on in semi-auto mode.
 *      you need send a power-on command to power on the port if the port can be.
 */

/*!
 *\Function:
 *      ad_poe_port_check_power_allowed
 *\Description:
 *      Check whether user_port is allowed to power on.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      n/a
 *\Return:
 *      TRUE  - allowed
 *      FALSE - not allowed
 *\Note:
 *      n/a
 */


/*!
 *\Function:
 *      ad_poe_set_port_state
 *\Description:
 *      setting the pse register to enalbe/disable power supply.
 *\Input:
 *      user_port      - user_port, start from 1
 *      status        - POE ability enabled or not
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      if set the status to enable, the power will not power on in semi-auto mode.
 *      you need send a power-on command to power on the port if the port can be.
 */


void ad_poe_turn_off_mp3924(int user_port)
{
    int rv = ERR_NO_ERROR;
    usp_uint8_t dev_num = 0;
    usp_uint8_t port_num = 0;
    usp_uint8_t reg_val_old = 0;
    usp_uint8_t reg_val_new = 0;

    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)  
    {
        return;
    }
    ad_poe_user_port2Pse_port(user_port, &dev_num, &port_num);

   
    FUNC_CALL_DONE(ad_poe_reg_read(dev_num, (usp_uint8_t)MP3924_POE_OPERATION_MODE_SETING, &reg_val_old), rv);
   
    reg_val_new = (usp_uint8_t)(~(0x03 << (2*(port_num-1)))) & reg_val_old;
    FUNC_CALL_DONE(ad_poe_reg_write(dev_num, (usp_uint8_t)MP3924_POE_OPERATION_MODE_SETING, reg_val_new), rv);
_done:	
    if ((rv) != ERR_NO_ERROR)
    {
        DBG("Turn off poe failed\r\n");
        return;
    }

    return;
}


int ad_poe_set_port_state_mp3924
(
    IN  int            user_port,
    IN  int  status
)
{
	int rv;
	usp_uint8_t pse_num = 0;
	usp_uint8_t pse_port = 0;
	unsigned char data = 0;
	
	if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX )
	{
	    return (ERR_BAD_PARAM);
	}
	
	FUNC_CALL(ad_poe_user_port2Pse_port(user_port, &pse_num, &pse_port), rv);

	pal_sem_take(p_sem_poe_off[user_port-1], pal_sem_FOREVER);
		
	FUNC_CALL_DONE(ad_poe_reg_read(pse_num, MP3924_POE_OPERATION_MODE_SETING, &data), rv);
	
	data &=~ ( 0x03<<(2*(pse_port - 1)) ); 
	
	if( POE_ENABLE == POE_PORT_INFO(user_port).poe_enable && POE_ENABLE == status )
	{
		data |= (0x02) << (2*(pse_port - 1)); /* SEMI-AUTO MODE */
		FUNC_CALL_DONE(ad_poe_reg_write(pse_num, MP3924_POE_OPERATION_MODE_SETING, data), rv);

		pal_usleep(100*1000);

		FUNC_CALL_DONE(ad_poe_reg_read(pse_num, MP3924_POE_DET_CLS_ENABLE, &data), rv);
		data |=( 0x01<<(pse_port - 1)) |( 0x10 << (pse_port - 1));
		FUNC_CALL_DONE(ad_poe_reg_write(pse_num, MP3924_POE_DET_CLS_ENABLE, data), rv);	
		
	}
	else if ( status == POE_DISABLE )
	{
		usp_uint8_t reg_val_old = 0;
    	usp_uint8_t reg_val_new = 0;

		pal_usleep(100*1000);

		FUNC_CALL_DONE(ad_poe_reg_read(pse_num, (usp_uint8_t)MP3924_POE_DET_CLS_ENABLE, &reg_val_old), rv);
		reg_val_new =  (usp_uint8_t)(~(0x11 << (pse_port-1))) & reg_val_old;
		FUNC_CALL_DONE(ad_poe_reg_write(pse_num, (usp_uint8_t)MP3924_POE_DET_CLS_ENABLE,reg_val_new), rv);

		ad_poe_turn_off_mp3924(user_port);
	}
	
_done:
	pal_sem_give(p_sem_poe_off[user_port-1]);
	
	return (rv);
    
}




int ad_poe_set_port_state(IN int user_port, IN int status)
{
    int rv=ERR_NO_ERROR;

	if(NULL != ti_lib.ad_poe_set_port_state)
	{
		rv = ti_lib.ad_poe_set_port_state(user_port,status);
	}

	return rv;
}
#if 0
/*
 *	判断PSE的芯片版本，以确定是否需要通过USE_AUTO_TO_POWER_CLASS4的方式上电
 *	返回值如下：
 *	0x00: 读错误，芯片版本未知
 *	0x01: 老芯片，需要通过USE_AUTO_TO_POWER_CLASS4的方式上电
 *	0xFF: 新芯片，无需通过USE_AUTO_TO_POWER_CLASS4的方式上电
 */
int ad_poe_get_pse_version_from_reg_val(usp_uint8_t pse_num)
{
	unsigned char silicon = 0;
	unsigned char firmware = 0;
	int rv = 0;
	
	FUNC_CALL(ad_poe_reg_read(pse_num, REG_SILICON_REVISION, &silicon), rv);
	FUNC_CALL(ad_poe_reg_read(pse_num, REG_FIRMWARE_REVISION, &firmware), rv);
	if (silicon == 0x52 || firmware == 0x01)
	{
		if (silicon == 0x52 && firmware == 0x01)
		{
			DBG("[%s] PSE %d: Old silicon detected!\r\n", __FUNCTION__, pse_num);
			return 0x01;
		}
		else
		{
			FUNC_CALL(ad_poe_reg_read(pse_num, REG_SILICON_REVISION, &silicon), rv);
			FUNC_CALL(ad_poe_reg_read(pse_num, REG_FIRMWARE_REVISION, &firmware), rv);
			DBG("[%s][Reconfirm] PSE %d: %s\r\n", __FUNCTION__, pse_num, (silicon == 0x52 && firmware == 0x01) ? "Old silicon detected!" : "Read error!");
			return (silicon == 0x52 && firmware == 0x01) ? 0x01 : 0x00;
		}
	}
	else
	{
		DBG("[%s] PSE %d: New silicon detected!\r\n", __FUNCTION__, pse_num);
		return 0xFF;
	}
		
}
#endif
/*!
 *\Function:
 *      ad_poe_port_power_on_control
 *\Description:
 *      send port power-on or power-off command to pse
 *\Input:
 *      user_port      - user_port, start from 1
 *      status        - POE_ENALBE to power on 
 *                    - POE_DISABLE to power off
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */

int ad_poe_port_power_on_control_mp3924
(
    IN  int  user_port,
    IN  POE_POWER_ON_CTRL  status
)
{
	int rv ;
	usp_uint8_t pse_num = 0;
	usp_uint8_t pse_port = 0;
	unsigned char data = 0;
	
	if(user_port <POE_UPORT_MIN|| user_port > POE_UPORT_MAX)
	{
		return (ERR_BAD_PARAM);
	}
	
	if(status != POE_ENABLE && status != POE_DISABLE)
	{
		return (ERR_BAD_PARAM);
	}

	ad_poe_user_port2Pse_port(user_port, &pse_num, &pse_port);

	pal_sem_take(p_sem_poe_off[user_port-1], pal_sem_FOREVER);

	if( POE_POWER_ON == status)
	{
		ad_poe_reg_read(pse_num, (usp_uint8_t)MP3924_POE_POWER_STATUS, &data);
		data |= data >> 4;
		/* disable 未上电端口 */
		ad_poe_reg_write(pse_num, (usp_uint8_t)MP3924_POE_PORT_ENABLE, 0xF0 | data);									
		pal_usleep(10 * 1000);
		/* 打开准备上电端口 */
		ad_poe_reg_write(pse_num, (usp_uint8_t)MP3924_POE_PORT_ENABLE, 0xFF);
		
		data = 0x11 << ( pse_port-1 );
		ad_poe_reg_write(pse_num, (usp_uint8_t)MP3924_POE_DET_CLS_ENABLE, data);
		pal_usleep(100*1000);
		data = 0x01 << ( pse_port-1 );
		ad_poe_set_icut_ilim_mp3924(user_port);
		FUNC_CALL_DONE(ad_poe_reg_write(pse_num, MP3924_POWER_ON_OFF_TRIGGER_COMMAND, data), rv);		
	}
	else
	{
		data = 0x11 << ( pse_port-1 );
		ad_poe_reg_write(pse_num, (usp_uint8_t)MP3924_POE_DET_CLS_ENABLE, data);
		pal_usleep(100*1000);
		data = 0x10 << ( pse_port-1 );
		FUNC_CALL_DONE(ad_poe_reg_write(pse_num, MP3924_POWER_ON_OFF_TRIGGER_COMMAND, data), rv);
	}

_done:
	pal_sem_give(p_sem_poe_off[user_port-1]);

   	return (ERR_NO_ERROR);
    
}


int ad_poe_port_power_on_control(int user_port, POE_POWER_ON_CTRL status)
{
    int rv=ERR_NO_ERROR;

	if(NULL != ti_lib.ad_poe_port_power_on_control)
	{
		rv = ti_lib.ad_poe_port_power_on_control(user_port,status);
	}

	return rv;
}


/*!
 *\Function:
 *      ad_poe_updt_event_status
 *\Description:
 *      Update event and state from the register.
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */

int ad_poe_updt_event_status_mp3924(void)
{
    usp_uint8_t ev_overload = 0;
    usp_uint8_t ev_pwr = 0;
    usp_uint8_t ev_supply = 0;
    usp_uint8_t ev_currentlimit = 0;
    usp_uint8_t ev_disconnect = 0;
    usp_uint8_t st_port[4] = {0};
    usp_uint8_t st_pwr = 0;
    usp_uint8_t pse_index = 0;  /* start from 0... */
    int     user_port = 1;   /* start from 1... */
    int     i = 0;

    poe_port_info_t *poe_port;
    int err_cnt = 0 ;
	usp_uint8_t ev_det = 0;

    for (pse_index = 0; pse_index < PSE_CHIP_NUMS; err_cnt = 0,pse_index++)
    {
		if( ad_poe_reg_write(pse_index, (usp_uint8_t)MP3924_POE_DET_CLS_ENABLE, 0xFF) != ERR_NO_ERROR)
		{
			err_cnt++;
		}
		
		if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_POE_OVER_LOAD_STATUS_CLEAR, &ev_overload) != ERR_NO_ERROR)
		{
			err_cnt++;
		}

		if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_POE_POWER_EVENT_CLEAR, &ev_pwr) != ERR_NO_ERROR)
		{
			err_cnt++;
		}

		if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_POE_POWER_SOURCE_STATUS1_CLEAR, &ev_supply) != ERR_NO_ERROR)
		{
			err_cnt++;
		}

		if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_POE_CURRENT_LIMIT_STATUS_CLEAR, &ev_currentlimit) != ERR_NO_ERROR)
		{
			err_cnt++;
		}

		if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_POE_DISCONNECT_STATUS_CLEAR, &ev_disconnect) != ERR_NO_ERROR)
		{
			err_cnt++;
		}	
		if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_POE_POWER_STATUS, &st_pwr) != ERR_NO_ERROR)
		{
			err_cnt++;
		}
        if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_DETECTION_EVENT_CLEAR_COMMAND, &ev_det) != ERR_NO_ERROR)
        {
            err_cnt++;
        }
        if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_CHANNEL_1_STATUS_COMMAND, &st_port[0]) != ERR_NO_ERROR)
        {
            err_cnt++;
        }
        if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_CHANNEL_2_STATUS_COMMAND, &st_port[1]) != ERR_NO_ERROR)
        {
            err_cnt++;
        }
        if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_CHANNEL_3_STATUS_COMMAND, &st_port[2]) != ERR_NO_ERROR)
        {
            err_cnt++;
        }
        if( ad_poe_reg_read(pse_index, (usp_uint8_t)MP3924_CHANNEL_4_STATUS_COMMAND, &st_port[3]) != ERR_NO_ERROR)
        {
            err_cnt++;
        }

		/* 读取PSE寄存器失败，则跳过该PSE芯片的处理 */
		if( err_cnt > 0  ) 
		{
			DBG("%s(): pse_index=%d err_cnt=%d\n ", __FUNCTION__, pse_index, err_cnt);
			continue;
		}

        for (i = 0; i < PORTS_PER_PSE_CHIP; i++)
        {

            ad_poe_pse_port2User_port(pse_index, (i + 1), &user_port);

	      	poe_port = &POE_PORT_INFO(user_port);
			
            poe_port->ev_pe_chg  = (ev_pwr & (0x01<<i)) ? TRUE : FALSE;
            poe_port->ev_pg_chg  = (ev_pwr & (0x10<<i)) ? TRUE : FALSE;

            poe_port->ev_det_end = (ev_det & (0x01<<i)) ? TRUE : FALSE;
            poe_port->ev_cla_end = (ev_det & (0x10<<i)) ? TRUE : FALSE;

            poe_port->ev_i_cut   = (ev_overload & (0x10<<i)) ? TRUE : FALSE;
            poe_port->ev_ld_disc = (ev_disconnect & (0x01<<i)) ? TRUE : FALSE;

        //    poe_port->ev_start_flt = (ev_overload & (0x01<<i)) ? TRUE : FALSE;
            poe_port->ev_i_lim       = (ev_currentlimit & (0x01<<i)) ? TRUE : FALSE;

            poe_port->ev_over_temp = (ev_supply & 0x04) ? TRUE : FALSE;
            poe_port->ev_vdd_uvlo  = (ev_supply & 0x01) ? TRUE : FALSE;
        //  poe_port->ev_vpwr_uvlo  = (ev_supply & 0x10) ? TRUE : FALSE;
            
            poe_port->st_pe = (st_pwr & (0x01<<i)) ? TRUE : FALSE; 
            poe_port->st_pg = (st_pwr & (0x10<<i)) ? TRUE : FALSE;
	
            if (poe_port->ev_det_end)
            {
                poe_port->st_det = (st_port[i] & 0x07);
            }
            if (poe_port->ev_cla_end)
            {
                poe_port->st_cla = (st_port[i] & 0xF0) >> 4;

            }

        }
		/* for debug */
		if(0)
		{
			for (i = 0; i < PORTS_PER_PSE_CHIP; i++)
			{
				//user_port = (pse_index * PORTS_PER_PSE_CHIP) + (i + 1);
				ad_poe_pse_port2User_port(pse_index, (i + 1), &user_port);
				poe_port = &POE_PORT_INFO(user_port);
				printf("\r\n");
				printf("P%d.pe_chg:%d\r\n",user_port, poe_port->ev_pe_chg);
				printf("P%d.pg_chg:%d\r\n",user_port, poe_port->ev_pg_chg);
				printf("P%d.det_end:%d\r\n",user_port, poe_port->ev_det_end);
				printf("P%d.cla_end:%d\r\n",user_port, poe_port->ev_cla_end);
			
				printf("P%d.ev_i_cut:%d\r\n",user_port, poe_port->ev_i_cut);
				printf("P%d.ev_ld_disc:%d\r\n",user_port, poe_port->ev_ld_disc);
				printf("P%d.ev_start_flt:%d\r\n",user_port, poe_port->ev_start_flt);
				printf("P%d.ev_i_lim:%d\r\n",user_port, poe_port->ev_i_lim);
				printf("P%d.ev_over_temp:%d\r\n",user_port, poe_port->ev_over_temp);
				printf("P%d.ev_vdd_uvlo:%d\r\n",user_port, poe_port->ev_vdd_uvlo);
				printf("P%d.ev_vpwr_uvlo:%d\r\n",user_port, poe_port->ev_vpwr_uvlo);
			
				printf("P%d.st_pe:%d %x\r\n",user_port, poe_port->st_pe,st_pwr);
				printf("P%d.st_pg:%d %x\r\n",user_port, poe_port->st_pg,st_pwr);
				printf("P%d.st_det:%d\r\n",user_port, poe_port->st_det);
				printf("P%d.st_cla:%d\r\n",user_port, poe_port->st_cla);
			}
		}	
    }

    return (ERR_NO_ERROR);
}


int ad_poe_updt_event_status(void)
{
    int rv=ERR_NO_ERROR;

	if(NULL != ti_lib.ad_poe_updt_event_status)
	{
		rv = ti_lib.ad_poe_updt_event_status();
	}

	return rv;
}

/*!
 *\Function:
 *      ad_poe_updt_poe_port_power_info
 *\Description:
 *      Update current, voltage, power_consumption, pd_class, power_status.
 *\Input:
 *      port_no       - panel port no, start from 1..
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_updt_poe_port_power_info(int port_no)
{
    int rv = ERR_NO_ERROR;
    float cur = 0;
    float vol = 0;
    int pd_class = 0;

    poe_port_info_t *poe_port = NULL;

    if (port_no < POE_UPORT_MIN || port_no > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }
    
    POE_CONTROL_SEM_TAKE();
    poe_port = &POE_PORT_INFO(port_no);

    /* power good */
    if (poe_port->st_pg)
    {
        FUNC_CALL_DONE(ad_poe_get_port_current_from_reg_val(port_no, &cur), rv);
        FUNC_CALL_DONE(ad_poe_get_port_voltage_from_reg_val(port_no, &vol), rv);
        FUNC_CALL_DONE(ad_poe_get_port_class_from_reg_val(port_no, &pd_class), rv);
   
        poe_port->power_info.actual_cur  = (int)cur; /* unit: m_a */
        poe_port->power_info.actual_vol  = (int)vol; /* unit: 0.1V */
        poe_port->power_info.actual_cons = (int)((cur * vol) / 1000); /* unit: 0.1W */
        poe_port->power_info.pd_class    = pd_class;
/*      if (poe_port->actual_cons <= poe_port->power_limit)
        {
            poe_port->power_status = POE_ON;
        }
        else
        {
            poe_port->power_status = POE_OVERLOAD;
        }
*/
    }
    else
    {
        poe_port->power_info.actual_cur  = 0; /* unit: m_a */
        poe_port->power_info.actual_vol  = 0; /* unit: 0.1V */
        poe_port->power_info.actual_cons = 0; /* unit: 0.1W */
        poe_port->power_info.pd_class    = POE_CLASS_CLASSUN_AD; /* --- */
    
     /* if (poe_port->stm_state == STM_PORT_BAD)
        {
            poe_port->power_status = STM_POE_SHORT;
        }
        else
        {
            poe_port->power_status = STM_POE_OFF;
        }
    */
    }
_done:
    POE_CONTROL_SEM_GIVE();
    return ERR_NO_ERROR;
}

#if 0
/*!
 *\Function:
 *      ad_poe_set_op_mode_to_semi_auto
 *\Description:
 *\Input:
 *      port_no       - panel port no, start from 1..
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_set_op_mode_to_semi_auto(int user_port)
{
   	int rv;
	usp_uint8_t pse_num = 0;
	usp_uint8_t pse_port = 0;
	unsigned char data = 0;
	
	POE_PORT_CHK(user_port);
	
	FUNC_CALL(ad_poe_user_port2Pse_port(user_port, &pse_num, &pse_port), rv);
	FUNC_CALL(ad_poe_reg_read(pse_num, REG_OPERATING_MODE, &data), rv);
	
	data &=~ ( 0x03<<(2*(pse_port - 1)) ); 
	data |= (0x02) << (2*(pse_port - 1)); /* SEMI-AUTO MODE */
	
	FUNC_CALL(ad_poe_reg_write(pse_num, REG_OPERATING_MODE, data), rv);

	pal_usleep(100*1000);

	return (ERR_NO_ERROR);
}
#endif

/**************************************************************************************************/
/*                                      PUBLIC_FUNCTIONS                                          */
/**************************************************************************************************/
#if 0
#endif

void clear_start_flt_status()
{
	start_flt_flag = FALSE;
	start_flt_cnt = 0;
}

/*!
 *\Function:
 *      ad_poe_set_sys_power_limt
 *\Description:
 *      Set the system power limit.
 *\Input:
 *      power_limit    - power limit be set, 单位为0.1W
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_sys_power_limt
(
    IN  int  power_limit
)
{
	/* 获取信号量 */
    pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);
    g_poe_control.sys_power_limit = power_limit;
	printf("ad_poe_set_sys_power_limt***ad_poe_set_sys_power_limt:%d\n", power_limit);
	pal_mutex_give(sem_poe_control);
    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_get_sys_power_limt
 *\Description:
 *      Get the system power limit.
 *\Input:
 *      n/a
 *\Output:
 *      power_limit    - power limit, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_sys_power_limt
(
    OUT int  *power_limit
)
{
	if(power_limit == NULL)
	{
		return (ERR_BAD_PARAM);
	}
	/* 获取信号量 */
    pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);
	*power_limit=g_poe_control.sys_power_limit;
	
	pal_mutex_give(sem_poe_control);
	return (ERR_NO_ERROR);
}

/*!
 *\Function:
 *      ad_poe_get_sys_power_cons
 *\Description:
 *      Get the system actual power consumption.
 *\Input:
 *      n/a
 *\Output:
 *      power_cons     - actual power consumption, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_sys_power_cons
(
    OUT int  *power_cons
)
{
	int port_no;
	int temp = 0;

	if(power_cons == NULL)
	{
		return (ERR_BAD_PARAM);
	}
	/* 获取信号量 */
	pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);

	POE_PORT_ITER(port_no)
	{
		temp +=POE_PORT_POWER_INFO(port_no).actual_cons;
	}
	g_poe_control.sys_actual_cons = temp; 
	*power_cons=g_poe_control.sys_actual_cons;
	
	pal_mutex_give(sem_poe_control);

	return (ERR_NO_ERROR);
}


/*!
 *\Function:
 *      ad_poe_get_sys_power_remain_percent
 *\Description:
 *      Get the system remain power and percent.
 *\Input:
 *      n/a
 *\Output:
 *      power_cons     - actual power consumption, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_sys_power_remain_percent
(
    OUT float  *power_remain,
    OUT float  *power_percent
)
{
	int power_lim;
	int power_cons;

	if(power_remain == NULL || power_percent == NULL)
	{
		return (ERR_BAD_PARAM);
	}
	/* 获取信号量 */
	pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);

	ad_poe_get_sys_power_limt(&power_lim);
	ad_poe_get_sys_power_cons(&power_cons);

	*power_remain  = (float)(power_lim - power_cons) / 10;  // power remain, unit: w
	*power_percent = ((float)((*power_remain * 10) / power_lim)) * 100;  // power remain percent   
	
	pal_mutex_give(sem_poe_control);

	return (ERR_NO_ERROR);
}

/*!
 *\Function:
 *      ad_poe_get_sys_power_cons_cali
 *\Description:
 *      Get the system actual power consumption calibration.
 *\Input:
 *      n/a
 *\Output:
 *      power_cons     - actual power consumption, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_sys_power_cons_cali
(
    OUT int  *power_cons_cali
)
{
	int port_no;
	int temp = 0;

	if(power_cons_cali == NULL)
	{
		return (ERR_BAD_PARAM);
	}
	/* 获取信号量 */
	pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);

	POE_PORT_ITER(port_no)
	{
		temp += POE_PORT_POWER_INFO(port_no).actual_cur * POE_PORT_POWER_INFO(port_no).actual_cur ;
	}

	*power_cons_cali = temp / 200000;
	
	pal_mutex_give(sem_poe_control);

	return (ERR_NO_ERROR);
}

/*!
 *\Function:
 *      ad_poe_set_port_power_limt
 *\Description:
 *      Set the Po_e port power limit.
 *\Input:
 *      user_port      - user_port, start from 1
 *      power_limit   - port power limit
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_set_port_power_limt_one_line(int user_port, int power_limit)
{
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }

    if (power_limit < 0)
    {
        return ERR_BAD_PARAM;
    }
    
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
    POE_PORT_INFO(user_port).power_limit = power_limit;
    POE_CONTROL_SEM_GIVE();

    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_set_port_power_limt
 *\Description:
 *      Set the Po_e port power limit.
 *\Input:
 *      user_port      - user_port, start from 1
 *      power_limit   - port power limit
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_port_power_limt(int user_port, int power_limit)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    
    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport())
    {
        return ERR_BAD_PARAM;
    }

    if (power_limit < 0)
    {
        return ERR_BAD_PARAM;
    }

    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_set_port_power_limt_one_line(ad_port_num, power_limit);
    }

    return ERR_NO_ERROR;
}


/*!
 *\Function:
 *      ad_poe_get_port_power_limt
 *\Description:
 *      Get the Po_e port power limit.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *     power_limit   - port power limit
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_get_port_power_limt_one_line(int user_port, int *power_limit)
{
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX || NULL == power_limit)
    {
        return ERR_BAD_PARAM;
    }

    if (power_limit < 0)
    {
        return ERR_BAD_PARAM;
    }
    
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
    *power_limit = POE_PORT_INFO(user_port).power_limit;
    POE_CONTROL_SEM_GIVE();

    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_get_port_power_limt
 *\Description:
 *      Get the Po_e port power limit.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *     power_limit   - port power limit
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_port_power_limt(int user_port, int *power_limit)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    int power_limit_one_line = 0;
    
    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport() || NULL == power_limit)
    {
        return ERR_BAD_PARAM;
    }

    if (power_limit < 0)
    {
        return ERR_BAD_PARAM;
    }

    *power_limit = 0;
    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_get_port_power_limt_one_line(ad_port_num, &power_limit_one_line);
        if (0 == *power_limit || *power_limit > power_limit_one_line)
        {
            *power_limit = power_limit_one_line;
        }
    }

    return ERR_NO_ERROR;
}


/*!
 *\Function:
 *      ad_poe_set_port_user_status
 *\Description:
 *      Set the Po_e port user status.
 *\Input:
 *      user_port      - user_port, start from 1
 *      status        - POE func enabled or not
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_set_port_user_status_one_line(int user_port, int status)
{
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }

    if (status != POE_DISABLE && status != POE_ENABLE)
    {
        return ERR_BAD_PARAM;
    }

    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
    POE_PORT_INFO(user_port).poe_enable = status;
    POE_CONTROL_SEM_GIVE();

    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_set_port_user_status
 *\Description:
 *      Set the Po_e port user status.
 *\Input:
 *      user_port      - user_port, start from 1
 *      status        - POE func enabled or not
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_port_user_status(int user_port, int status)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    
    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport())
    {
        return ERR_BAD_PARAM;
    }

    if (status != POE_DISABLE && status != POE_ENABLE)
    {
        return ERR_BAD_PARAM;
    }

    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_set_port_user_status_one_line(ad_port_num, status);
    }

    return ERR_NO_ERROR;
}


/*!
 *\Function:
 *      ad_poe_set_port_priority
 *\Description:
 *      Set the Po_e port priority.
 *\Input:
 *      user_port      - user_port, start from 1
 *      priority      - port priority
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_set_port_priority_one_line(int user_port, POE_PRIORITY_STATE_AD priority)
{
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }

    if (priority <= POE_PRI_HIGH_AD && priority >= POE_PRI_LOW_AD)
    {
        return ERR_BAD_PARAM;
    }

    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
    POE_PORT_INFO(user_port).priority = priority;
    POE_CONTROL_SEM_GIVE();

    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_set_port_priority
 *\Description:
 *      Set the Po_e port priority.
 *\Input:
 *      user_port      - user_port, start from 1
 *      priority      - port priority
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_port_priority(int user_port, POE_PRIORITY_STATE_AD priority)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    
    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport())
    {
        return ERR_BAD_PARAM;
    }

    if (priority <= POE_PRI_HIGH_AD && priority >= POE_PRI_LOW_AD)
    {
        return ERR_BAD_PARAM;
    }

    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_set_port_priority_one_line(ad_port_num, priority);
    }

    return ERR_NO_ERROR;
}


/*!
 *\Function:
 *      ad_poe_get_port_power_cons
 *\Description:
 *      Get the system actual power consumption.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      power_cons     - actual power consumption, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_get_port_power_cons_one_line(IN  int user_port, OUT int  *power_cons)
{
    if (power_cons == NULL)
    {
        return ERR_BAD_PARAM;
    }
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();

    *power_cons = POE_PORT_POWER_INFO(user_port).actual_cons;

    POE_CONTROL_SEM_GIVE();
    
    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_get_port_power_cons
 *\Description:
 *      Get the system actual power consumption.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      power_cons     - actual power consumption, 单位为0.1W
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_port_power_cons(IN  int user_port, OUT int  *power_cons)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    int power_cons_one_line = 0;
    if (power_cons == NULL)
    {
        return ERR_BAD_PARAM;
    }
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }

    *power_cons = 0;
    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        power_cons_one_line = 0;
        ad_poe_get_port_power_cons_one_line(ad_port_num, &power_cons_one_line);
        *power_cons += power_cons_one_line;
    }
    
    return ERR_NO_ERROR;
}



/*!
 *\Function:
 *      ad_poe_get_port_power_info
 *\Description:
 *      get the poe port power supply information
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      power_info     - power info you get
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */

int ad_poe_get_port_power_info(IN  int user_port, OUT power_info_t  *power_info)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    power_info_t *p_ad_power_info = NULL;
    
    if (power_info == NULL)
    {
        return ERR_BAD_PARAM;
    }
    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport())
    {
        return ERR_BAD_PARAM;
    }
    memset(power_info, 0, sizeof(power_info_t));
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();

    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        p_ad_power_info = &POE_PORT_POWER_INFO(ad_port_num);
        if (1 == p_ad_power_info->actual_state)
        {
            power_info->actual_state = 1;
            power_info->actual_cur += p_ad_power_info->actual_cur;
            if (p_ad_power_info->actual_vol > power_info->actual_vol)
            {
                power_info->actual_vol = p_ad_power_info->actual_vol;
            }
            power_info->actual_cons += p_ad_power_info->actual_cons;
			//printf("[%d-%s] : userport = %d adport = %d : cons = %d %d \r\n", __LINE__ , __func__,user_port,ad_port_num,power_info->actual_cons,p_ad_power_info->actual_cons);
            if (0 == power_info->pd_class
                || (power_info->pd_class < p_ad_power_info->pd_class && POE_CLASS_CLASSUN_AD != p_ad_power_info->pd_class))
            {
                power_info->pd_class = p_ad_power_info->pd_class;
                power_info->power_state = p_ad_power_info->power_state;
                power_info->power_state_backup = p_ad_power_info->power_state_backup;
            }
        }
    }

    POE_CONTROL_SEM_GIVE();
    
    return ERR_NO_ERROR;
}


/*!
 *\Function:
 *      ad_poe_get_port_power_state
 *\Description:
 *      get the port power supply state
 *\Input:
 *      user_port            - user_port, start from 1
 *\Output:
 *      power_state         - current power state
 *      power_state_backup  - last power state
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_get_port_power_state_one_line(int user_port, POE_POWER_STATE *power_state, POE_POWER_STATE *power_state_backup)
{
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }
    if (power_state== NULL || power_state_backup == NULL)
    {
        return ERR_BAD_PARAM;
    }
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
    
    *power_state = POE_PORT_POWER_INFO(user_port).power_state;
    *power_state_backup = POE_PORT_POWER_INFO(user_port).power_state_backup;

    POE_CONTROL_SEM_GIVE();
    
    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_get_port_power_state
 *\Description:
 *      get the port power supply state
 *\Input:
 *      user_port            - user_port, start from 1
 *\Output:
 *      power_state         - current power state
 *      power_state_backup  - last power state
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_port_power_state(int user_port, POE_POWER_STATE *power_state, POE_POWER_STATE *power_state_backup)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    POE_POWER_STATE power_state_one_line = 0;
    POE_POWER_STATE power_state_backup_one_line = 0;
    
    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport())
    {
        return ERR_BAD_PARAM;
    }
    if (power_state== NULL || power_state_backup == NULL)
    {
        return ERR_BAD_PARAM;
    }

    *power_state = 0;
    *power_state_backup = 0;
    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_get_port_power_state_one_line(ad_port_num, &power_state_one_line, &power_state_backup_one_line);
        if (POE_ON == power_state_one_line)
        {
            *power_state = power_state_one_line;
            *power_state_backup = power_state_backup_one_line;
            break;
        }
        else if (power_state_one_line != power_state_backup_one_line)
        {
            *power_state = power_state_one_line;
            *power_state_backup = power_state_backup_one_line;
        }
    }
    
    return ERR_NO_ERROR;
}


/*!
 *\Function:
 *      ad_poe_set_port_power_state
 *\Description:
 *      Get the pd standard.
 *\Input:
 *      user_port      - user_port, start from 1
 *      power_state    - power supply state to set
 *					POE_OFF
 *					POE_ON 
 *					POE_OVERLOAD
 *					POE_SHORT
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_set_port_power_state_one_line(int user_port, POE_POWER_STATE power_state)
{
    if (user_port <POE_UPORT_MIN|| user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
    
    POE_PORT_POWER_INFO(user_port).power_state_backup = POE_PORT_POWER_INFO(user_port).power_state;
    POE_PORT_POWER_INFO(user_port).power_state = power_state;
    
    switch(power_state)
    {
        case POE_OFF:
		case POE_SHORT:
        case POE_START_FAULT:
            POE_PORT_POWER_INFO(user_port).actual_state = POE_DISABLE;
            break;
        case POE_ON:
        case POE_OVERLOAD:
            POE_PORT_POWER_INFO(user_port).actual_state = POE_ENABLE;
            break;
        default:
            POE_PORT_POWER_INFO(user_port).actual_state = POE_DISABLE;
            break;
    }
    POE_CONTROL_SEM_GIVE();
    
    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_set_port_power_state
 *\Description:
 *      Get the pd standard.
 *\Input:
 *      user_port      - user_port, start from 1
 *      power_state    - power supply state to set
 *                  POE_OFF
 *                  POE_ON 
 *                  POE_OVERLOAD
 *                  POE_SHORT
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_set_port_power_state(int user_port, POE_POWER_STATE power_state)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    POE_POWER_STATE power_state_one_line = 0;
    POE_POWER_STATE power_state_backup_one_line = 0;
    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport())
    {
        return ERR_BAD_PARAM;
    }
    
    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_get_port_power_state_one_line(ad_port_num, &power_state_one_line, &power_state_backup_one_line);
        //if (POE_OFF != power_state_one_line)
        {
            ad_poe_set_port_power_state_one_line(ad_port_num, power_state);
        }
    }
    
    return ERR_NO_ERROR;
}


/*!
 *\Function:
 *      ad_poe_get_port_status
 *\Description:
 *      Get the Po_e port POE_DISABLE_AD or POE_ENABLE_AD
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      status        - POE func enabled or not
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_port_status(int user_port, int *status)
{

  	POE_PORT_CHK(user_port);

	if(status == NULL)
	{
		return (ERR_BAD_PARAM);
	}
	/* 获取信号量 */
	pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);
	*status = POE_PORT_INFO(user_port).power_info.actual_state;
	pal_mutex_give(sem_poe_control);
	
	return (ERR_NO_ERROR);
}

/*!
 *\Function:
 *      ad_poe_get_standard
 *\Description:
 *      Get the pd standard.
 *\Input:
 *      user_port      - user_port, start from 1
 *\Output:
 *      standard      - pd standard
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_get_standard
(
    IN  int            user_port,
    OUT  int  *standard
)
{


	POE_PORT_CHK(user_port);

	if(standard == NULL)
	{
		return (ERR_BAD_PARAM);
	}
	/* 获取信号量 */
	pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);
	*standard = POE_STANDARD;
	pal_mutex_give(sem_poe_control);
	
    return (ERR_NO_ERROR);
}





/*!
 *\Function:
 *      ad_poe_clear_poe_port_power_info
 *\Description:
 *      Clear power info.
 *\Input:
 *      user_port    - user Port , start from 1
 *\Output:
 *      n/a
 *\Return:
 *      n/a
 *\Note:
 *      Only ad_poe_stm_enter() can invoke this function!!
 */
static void ad_poe_clear_poe_port_power_info_one_line(int user_port)
{
/* 获取信号量 */
    POE_CONTROL_SEM_TAKE();

    POE_PORT_INFO(user_port).power_info.actual_state = POE_DISABLE;
    POE_PORT_INFO(user_port).power_info.actual_cur   = 0;
    POE_PORT_INFO(user_port).power_info.actual_vol   = 0;
    POE_PORT_INFO(user_port).power_info.actual_cons  = 0;
    POE_PORT_INFO(user_port).power_info.pd_class     = POE_CLASS_CLASSUN_AD; /* --- */
    //POE_PORT_INFO(user_port).power_info.power_state = POE_OFF; /*由调用者改变状态*/
    
    POE_PORT_INFO(user_port).ev_pe_chg    = FALSE;
    POE_PORT_INFO(user_port).ev_pg_chg    = FALSE;
    POE_PORT_INFO(user_port).ev_det_end   = FALSE;
    POE_PORT_INFO(user_port).ev_cla_end   = FALSE;
    POE_PORT_INFO(user_port).ev_i_cut     = FALSE;
    POE_PORT_INFO(user_port).ev_ld_disc   = FALSE;
    POE_PORT_INFO(user_port).ev_start_flt = FALSE;
    POE_PORT_INFO(user_port).ev_i_lim     = FALSE;
    POE_PORT_INFO(user_port).ev_over_temp = FALSE;
    POE_PORT_INFO(user_port).ev_vdd_uvlo  = FALSE;
    POE_PORT_INFO(user_port).ev_vpwr_uvlo  = FALSE;
    POE_PORT_INFO(user_port).st_pe        = FALSE;
    POE_PORT_INFO(user_port).st_pg        = FALSE;
    POE_PORT_INFO(user_port).st_det       = DET_UNKNOWN;
    POE_PORT_INFO(user_port).st_cla       = CLASS_UNKNOWN;

    /* 获取信号量 */
    POE_CONTROL_SEM_GIVE();
    
}

/*!
 *\Function:
 *      ad_poe_clear_poe_port_power_info
 *\Description:
 *      Clear power info.
 *\Input:
 *      user_port    - user Port , start from 1
 *\Output:
 *      n/a
 *\Return:
 *      n/a
 *\Note:
 *      Only ad_poe_stm_enter() can invoke this function!!
 */
void ad_poe_clear_poe_port_power_info(int user_port)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    
    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ;
    }
    
    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_clear_poe_port_power_info_one_line(ad_port_num);
    }
}

/*!
 *\Function:
 *      ad_poe_check_class
 *\Description:
 *      Check classification succeed or not.
 *\Input:
 *      class_value - class_value in register
 *\Output:
 *      n/a
 *\Return:
 *      TRUE        - on success
 *      FALSE       - on error
 *\Note:
 *      n/a
 */
int ad_poe_check_class_mp3924(usp_uint8_t class_value)
{
    int class_status = FALSE;
    
    switch (class_value)
    {
    case CLASS_1:
    case CLASS_2:
    case CLASS_3:
    case CLASS_4:
    case CLASS_0:
        class_status = TRUE;
        break;
    case CLASS_UNKNOWN:
    case CLASS_OVER_CURR:
        class_status = FALSE;
        break;
    default:
        break;
    }

    return class_status;
}


int ad_poe_check_class(usp_uint8_t class_value)
{
	int rv=ERR_NO_ERROR;

	if(NULL != ti_lib.ad_poe_check_class)
	{
		rv = ti_lib.ad_poe_check_class(class_value);
	}

	return rv;
}

/*!
 *\Function:
 *      ad_poe_stm_enter
 *\Description:
 *      Actions in specified state.
 *\Input:
 *      poe_port      - pointer of poe_port_info_t
 *      target        - into this state and do actions
 *\Output:
 *      state         - state to store the target state
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */


/*!
 *\Function:
 *      ad_poe_stm_step
 *\Description:
 *      Step into one state.
 *\Input:
 *      poe_port      - pointer of poe_port_info_t
 *      state         - current state
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */


/*!
 *\Function:
 *      ad_poe_port_info_updt
 *\Description:
 *      read the newest power information per port, and calculate the total power
 *	   of all ports to the g_poe_control.sys_actual_cons
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */



/*!
 *\Function:
 *      ad_poe_stm_enter
 *\Description:
 *      Actions in specified state.
 *\Input:
 *      poe_port      - pointer of poe_port_info_t
 *      target        - into this state and do actions
 *\Output:
 *      state         - state to store the target state
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */


/*!
 *\Function:
 *      ad_poe_stm_step
 *\Description:
 *      Step into one state.
 *\Input:
 *      poe_port      - pointer of poe_port_info_t
 *      state         - current state
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */

/*!
 *\Function:
 *      ad_poe_port_info_updt
 *\Description:
 *      read the newest power information per port, and calculate the total power
 *     of all ports to the g_poe_control.sys_actual_cons
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */

/*!
 *\Function:
 *      ad_poe_stm_step
 *\Description:
 *      Step into one state.
 *\Input:
 *      poe_port      - pointer of poe_port_info_t
 *      state         - current state
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
void ad_poe_stm_enter_mp3924(poe_port_info_t *poe_port, poe_stm_state_t *state, poe_stm_state_t target)
{

    if( NULL == poe_port || NULL == state )
    {
		DBG("%s(): arg is null",__FUNCTION__);
		return;
    }

    switch (target)
    {
    case STM_PORT_OFF:

		ad_poe_clear_poe_port_power_info(poe_port->port_no);
		ad_poe_set_port_power_state(poe_port->port_no, POE_OFF);
		
		break;

    case STM_PORT_TURNING_ON:
		ad_poe_updt_poe_port_power_info(poe_port->port_no);
		ad_poe_set_port_power_state(poe_port->port_no, POE_OFF);
		
		break;

    case STM_PORT_ON:
        ad_poe_updt_poe_port_power_info(poe_port->port_no);
		if(1)
		{
			POE_POWER_STATE port_poe_state_backup = POE_OFF;
			POE_POWER_STATE port_poe_state = POE_OFF;
			int port_poe_cons = 0;
			int port_power_limit = 0;

			ad_poe_get_port_power_state(poe_port->port_no, &port_poe_state,&port_poe_state_backup);
			ad_poe_get_port_power_cons(poe_port->port_no,&port_poe_cons);
			ad_poe_get_port_power_limt(poe_port->port_no, &port_power_limit);

			/* 处理被功率管理模块标志为POE_OVERLOAD的端口状态 */
			if( POE_OVERLOAD == port_poe_state ) 
			{
				if( port_poe_cons <= port_power_limit )
				{
					ad_poe_set_port_power_state(poe_port->port_no, POE_ON);
				}
				else
				{
					ad_poe_set_port_power_state(poe_port->port_no, POE_OVERLOAD);
				}
			}
			else
			{
				ad_poe_set_port_power_state(poe_port->port_no, POE_ON);
			}
		}
        break;

    case STM_PORT_BAD:
		ad_poe_updt_poe_port_power_info(poe_port->port_no);
		ad_poe_set_port_power_state(poe_port->port_no, POE_SHORT);
		/* 让端口在POWER_RECONNECT_TIME后才能重新上电 */
		//printf("disconnect %d\n",poe_port->port_no);
		ad_poe_set_port_state(poe_port->port_no, POE_DISABLE);
		ad_poe_port_flag_set(poe_port->port_no,POE_MANAGE_POWER_DIS,POWER_RECONNECT_TIME);
        break;

    default:
        break;
    }

    /* debugging */
    if (*state != target)
    {
        DBG("\n Po_e STM on Port %.2d enter %s.\n", poe_port->port_no, poe_stm_state_str[target]);
    }
	
   /* 仅有此处会改变端口的stm_state,因此无需加锁 */
    *state = target; 
}



void ad_poe_stm_step_mp3924(poe_port_info_t *poe_port, poe_stm_state_t *state)
{
    poe_stm_state_t cur_state  ;

   if( NULL == poe_port || NULL == state )
   {
   	DBG("%s() null pointer\n",__FUNCTION__);
   	return;
   }
   cur_state = *state;

//   DBG("STM:P%d:%s\r\n",poe_port->port_no, poe_stm_state_str[cur_state]);

    switch (cur_state)
    {
        
    case STM_BEGIN:
        ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_OFF);
        break;
        
    case STM_PORT_OFF:
        if (poe_port->ev_cla_end
            && ad_poe_check_class(poe_port->st_cla)
            && (poe_port->poe_enable == POE_ENABLE)
            && !poe_port->st_pg)
         {
            ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_TURNING_ON);
         }
		 else
		 {
			ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_OFF);
		 }
        break;
        
    case STM_PORT_TURNING_ON:
		if (poe_port->ev_start_flt && !start_flt_flag)
		{
			start_flt_flag = TRUE;
			start_flt_cnt = 0;
			DBG("[%s] case STM_PORT_TURNING_ON: start_flt = %d!\r\n", __FUNCTION__, poe_port->ev_start_flt);
		}

		if ((poe_port->ev_i_lim)
            && !poe_port->st_pg)
        {
            clear_start_flt_status();
			ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_OFF);
        }
        else if (poe_port->ev_i_cut
            && !poe_port->st_pg)
        {
            clear_start_flt_status();
			ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_BAD);
        }
        else if (!poe_port->st_pg)
        {
        	if (start_flt_flag)
        	{
				if (start_flt_cnt++ == 3)
				{
					clear_start_flt_status();
					ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_OFF);
				}
				else
				{
					ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_TURNING_ON);
				}
        	}
			else
			{
				ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_TURNING_ON);
			}
        }
        else if (poe_port->st_pg)
        {
            clear_start_flt_status();
			ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_ON);
        }
        break;
        
    case STM_PORT_ON:
        if (poe_port->ev_i_cut
            && !poe_port->st_pg)
        {
            ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_BAD);
        }
        else if (!poe_port->st_pg)
        {
            ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_OFF);
        }
        else if (poe_port->st_pg)
        {
            ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_ON);
        }
        break;

    case STM_PORT_BAD:
        if (!poe_port->st_pg)
        {
            ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_OFF);
        }
	 else
	 {
		ad_poe_stm_enter_mp3924(poe_port, state, STM_PORT_BAD);
	 }
        break;
        
    default:
        break;
    }
}

int ad_poe_power_info_updt_mp3924(void)
 {
 	int rv;
	poe_port_info_t *poe_port;
	int user_port = 0;
	int actual_cons = 0;

	/*reset check*/
	ad_poe_re_config_mp3924();
	/* Update event and state from the register */
	
    FUNC_CALL(ad_poe_updt_event_status(), rv);

	for (user_port = POE_UPORT_MIN; user_port <= POE_UPORT_MAX; user_port++)
	{
	    poe_port = &POE_PORT_INFO(user_port);
		ad_poe_stm_step_mp3924(poe_port, (poe_stm_state_t *)&poe_port->stm_state);
	}

	pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);
	
	for (user_port = POE_UPORT_MIN; user_port <= POE_UPORT_MAX; user_port++)
	{
		poe_port = &POE_PORT_INFO(user_port);
		actual_cons += poe_port->power_info.actual_cons;
	}
	
	POE_CONTROL.sys_actual_cons = actual_cons;
	
	pal_mutex_give(sem_poe_control);
	
	return (ERR_NO_ERROR);
	
 }


int ad_poe_power_info_updt(void)
{
    int rv=ERR_NO_ERROR;

	if(NULL != ti_lib.ad_poe_power_info_updt)
	{
		rv = ti_lib.ad_poe_power_info_updt();
	}

	return rv;
}


#if 0
#endif

/*!
 *\Function:
 *      ad_poe_manage_check
 *\Description:
 *      check whether the port can be disconnected due to the power manage algorithm
 *\Input:
 *      user_port      - user_port, start from 1
 *      lowest        - port' priority must be lower than this argument 
 *\Output:
 *      n/a
 *\Return:
 *      POE_MANAGE_EN  - the port can be disconnected
 *      (Others)       - the port cannot be disconnected
 *\Note:
 *      n/a
 */
int ad_poe_manage_check(int user_port,int lowest)
{
	power_info_t *power_info;
	poe_port_info_t *port_info;
	int timeseg_dis = 0;
	int recovery_dis = 0;
	
	if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return (ERR_BAD_PARAM);
    }

	/* 获取信号量 */
    pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);
	
	power_info=&(POE_PORT_POWER_INFO(user_port));
	port_info=&(POE_PORT_INFO(user_port));
	timeseg_dis = POE_MANAGE_TIMESEG_DIS_FLAG(user_port);
	recovery_dis = POE_MANAGE_RECOVERY_DIS_FLAG(user_port);
	DBG("userport:%d,power_info->actual_state:%d, power_info->actual_cons:%d, port_info->priority:%d, timeseg_dis:%d, recovery_dis:%d\n",
		user_port,power_info->actual_state, power_info->actual_cons, port_info->priority, timeseg_dis, recovery_dis);
	if (power_info->actual_state == POE_ENABLE\
                    && power_info->actual_cons != 0 \
                    && port_info->priority >= lowest \
                    && timeseg_dis == POE_RELEASE
                    && recovery_dis == POE_RELEASE)
    {
		
		pal_mutex_give(sem_poe_control);
		return POE_MANAGE_EN;
	}

	pal_mutex_give(sem_poe_control);
	return POE_MANAGE_DIS;
}

#if 0
#endif

/*!
 *\Function:
 *      ad_poe_flag_manage
 *\Description:
 *      deal with the flag for power manage
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      n/a
 *\Note:
 *      n/a
 */
void ad_poe_flag_manage()
{
	int port_no;
	int type;
	
    /* 获取信号量 */
	//DBG("ad_poe_flag_manage BEGIN\n");
    pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);
	/* 以下代码不能有再获取信号量，以防死锁 */ 
	for(type=0;type<POE_MANAGE_PORT_FLAG_COUNT;type++)
    {
    	//DBG("ad_poe_flag_manage 1\n");
		for (port_no = POE_UPORT_MIN; port_no <= POE_UPORT_MAX; port_no++)  
		{
			//DBG("ad_poe_flag_manage 2\n");
			if(POE_MANAGE_PORT_FLAG(type,port_no)>=POE_FLAG_TIMER)
			{
				//DBG("ad_poe_flag_manage 3\n");
				POE_MANAGE_PORT_FLAG(type,port_no) -=POE_FLAG_TIMER;

				/*特殊flag清零时需要回调处理*/
				/*功率管理结束后会重新connect*/
				//DBG("ad_poe_flag_manage 4\n");
				if((type == POE_MANAGE_POWER_DIS)&&  (POE_MANAGE_PORT_FLAG(type,port_no) == 0))
				{
					//DBG("ad_poe_flag_manage 5\n");
					ad_poe_reconnect_port(port_no);
				}
				/*led 强制模式恢复*/
				if((type == POE_MANAGE_LED_FORCE)&&  (POE_MANAGE_PORT_FLAG(type,port_no) == 0))
				{
					//DBG("ad_poe_flag_manage 6\n");
					ad_poe_port_led_force_state_clear(port_no);
				}
			}
		}
		
    }
	//DBG("ad_poe_flag_manage 7\n");
	for(type=0;type<POE_MANAGE_SYS_FLAG_COUNT;type++)
    {
		//DBG("ad_poe_flag_manage 8\n");
		if(POE_MANAGE_SYS_FLAG(type)>=POE_FLAG_TIMER)
		{
			//DBG("ad_poe_flag_manage 9\n");
			POE_MANAGE_SYS_FLAG(type) -=POE_FLAG_TIMER;
		}
    }

	pal_mutex_give(sem_poe_control);
}


/*!
 *\Function:
 *      ad_poe_sys_flag_get
 *\Description:
 *      get the system level flag value
 *\Input:
 *      sys_flag - flag type
 *\Output:
 *      value    - flag value
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_sys_flag_get(POE_MANAGE_SYS_FLAG_TYPE sys_flag,int *value)
{

	if(sys_flag >= POE_MANAGE_SYS_FLAG_COUNT)
	{
		return (ERR_BAD_PARAM);
	}

	if(value == NULL)
	{
		return (ERR_BAD_PARAM);
	}

	
    /* 获取信号量 */
    pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);
	  
  	*value = POE_MANAGE_SYS_FLAG(sys_flag);
	
	pal_mutex_give(sem_poe_control);
	return (ERR_NO_ERROR);
}

/*!
 *\Function:
 *      ad_poe_sys_flag_set
 *\Description:
 *      set the system level flag value
 *\Input:
 *      sys_flag - flag type
 *      value    - flag value to set
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_sys_flag_set(POE_MANAGE_SYS_FLAG_TYPE sys_flag,int value)
{

	if(sys_flag >= POE_MANAGE_SYS_FLAG_COUNT)
	{
		return (ERR_BAD_PARAM);
	}
	
    /* 获取信号量 */
    pal_mutex_take(sem_poe_control, pal_mutex_FOREVER);
	  
  	POE_MANAGE_SYS_FLAG(sys_flag) = value;
	
	pal_mutex_give(sem_poe_control);
	return (ERR_NO_ERROR);
}

/*!
 *\Function:
 *      ad_poe_port_flag_get
 *\Description:
 *      get the port level flag value
 *\Input:
 *      user_port   - user_port, start from 1
 *      port_flag  - flag type
 *\Output:
 *      value      - flag value		
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_port_flag_get_one_line(int user_port, POE_MANAGE_PORT_FLAG_TYPE port_flag, int *value)
{
    if (port_flag >= POE_MANAGE_PORT_FLAG_COUNT)
    {
        return ERR_BAD_PARAM;
    }

    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }

    if (value == NULL)
    {
        return ERR_BAD_PARAM;
    }
    
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
      
    *value = POE_MANAGE_PORT_FLAG(port_flag,user_port);
    
    POE_CONTROL_SEM_GIVE();
    
    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_port_flag_get
 *\Description:
 *      get the port level flag value
 *\Input:
 *      user_port   - user_port, start from 1
 *      port_flag  - flag type
 *\Output:
 *      value      - flag value     
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_port_flag_get(int user_port, POE_MANAGE_PORT_FLAG_TYPE port_flag, int *value)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    int value_one_line = 0;
    if (port_flag >= POE_MANAGE_PORT_FLAG_COUNT)
    {
        return ERR_BAD_PARAM;
    }

    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport())
    {
        return ERR_BAD_PARAM;
    }

    if (value == NULL)
    {
        return ERR_BAD_PARAM;
    }

    *value = 0;
    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_port_flag_get_one_line(ad_port_num, port_flag, &value_one_line);
        *value += value_one_line;
    }
    
    return ERR_NO_ERROR;
}



/*!
 *\Function:
 *      ad_poe_port_flag_set
 *\Description:
 *      set the port level flag value
 *\Input:
 *      user_port   - user_port, start from 1
 *      port_flag  - flag type
 *      value      - flag value to set
 *\Output:
 *      n/a		
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
static int ad_poe_port_flag_set_one_line(int user_port, POE_MANAGE_PORT_FLAG_TYPE port_flag, int value)
{
    if (port_flag >= POE_MANAGE_PORT_FLAG_COUNT)
    {
        return ERR_BAD_PARAM;
    }

    if (user_port < POE_UPORT_MIN || user_port > POE_UPORT_MAX)
    {
        return ERR_BAD_PARAM;
    }
    
    /* 获取信号量 */
    POE_CONTROL_SEM_TAKE();
      
    POE_MANAGE_PORT_FLAG(port_flag, user_port) = value;
    
    POE_CONTROL_SEM_GIVE();

    return ERR_NO_ERROR;
}

/*!
 *\Function:
 *      ad_poe_port_flag_set
 *\Description:
 *      set the port level flag value
 *\Input:
 *      user_port   - user_port, start from 1
 *      port_flag  - flag type
 *      value      - flag value to set
 *\Output:
 *      n/a     
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_port_flag_set(int user_port, POE_MANAGE_PORT_FLAG_TYPE port_flag, int value)
{
    int ad_port_index = 0;
    int ad_port_num = 0;
    
    if (port_flag >= POE_MANAGE_PORT_FLAG_COUNT)
    {
        return ERR_BAD_PARAM;
    }

    if (user_port < get_poe_first_uport() || user_port > get_poe_last_uport())
    {
        return ERR_BAD_PARAM;
    }

    for (ad_port_index = 0; ad_port_index < PSE_PORT_LINE_NUM; ad_port_index++)
    {
        ad_port_num = PSE_USERPORT_TO_ADPORT(user_port, ad_port_index);
        ad_poe_port_flag_set_one_line(ad_port_num, port_flag, value);
    }

    return ERR_NO_ERROR;
}






#if 0
/*!
 *\fn		int  ad_poe_get_pse_temperature( void )
 *\brief		get PSE average temperature
 *\param[in]	N/A
 *\param[out]	N/A
 *\return		PSE average temperature, unit: 0.1 degree
 *\note 		added by zengjunjun
 */
static int  ad_poe_get_pse_temperature( void )
{
	int i = 0;
	int rv;
	unsigned char data = 0;
	unsigned int temp_sum = 0;
	unsigned int pse_cnt = 0;
	float average_temp = 0.0;
	
	PSE_CHIP_ITER(i)
	{
		rv = ad_poe_reg_read(i, REG_TEMPERATURE, &data);
		if( ERR_NO_ERROR == (rv) )
		{
			pse_cnt++;
			temp_sum += data;
		}
	}

	if( 0 !=  pse_cnt )
	{
		average_temp =  ((float)temp_sum / (float)pse_cnt) * 0.652 - 20; 
	}
	
	return (int)(average_temp*10);
		
}
#endif





/* poe max led */
void ad_poe_max_led_flag_set(void)
{
	if(PWR_MAX_LED_STATE_ON == poe_max_led_blink_flag)
	{
		poe_max_led_blink_flag = PWR_MAX_LED_STATE_BLINK;
	}
}

/*!
 *\fn		void ad_poe_update_poe_max_led(void)
 *\brief		udpate the poe max led state 
 *\param[in]	N/A
 *\param[out]	N/A
 *\return		void
 *\note 		add by zengjunjun
 */
void ad_poe_update_poe_max_led()
{

	int sys_actual_cons = 0;
	int sys_power_limit = 0;

	int burst = 6; /* 0.6w 滞回 */

	struct timespec interval = {0};

	ad_poe_get_sys_power_cons(&sys_actual_cons);
	ad_poe_get_sys_power_limt(&sys_power_limit);

	if (PWR_MAX_LED_STATE_OFF == poe_max_led_blink_flag)
	{
		if( ad_poe_max_led_timer_id != ERROR )
		{
			del_timer(ad_poe_max_led_timer_id);
			ad_poe_max_led_timer_id = ERROR;
		}	
		/* 剩余功率小于POWER_REMAIN_MAX_LED_ON */
		if (sys_actual_cons + POWER_REMAIN_MAX_LED_ON - burst >= sys_power_limit )
		{
			poe_max_led_blink_flag = PWR_MAX_LED_STATE_ON; /* red led on */
			
			ad_led_poe_max_led_green_on();
			interval.tv_sec = 120;
			interval.tv_nsec = 0;
			ad_poe_max_led_timer_id = add_timer(&interval,ad_poe_max_led_flag_set,0,EXECUTE_SINGLE) ;
		}
		else
		{
			ad_led_poe_max_led_off();
		}
	}
	else if(PWR_MAX_LED_STATE_ON == poe_max_led_blink_flag) 
	{
		if(sys_actual_cons + POWER_REMAIN_MAX_LED_ON + burst <= sys_power_limit)
		{
			poe_max_led_blink_flag = PWR_MAX_LED_STATE_OFF;
			ad_led_poe_max_led_off();
		}
		else
		{
			ad_led_poe_max_led_green_on();
		}
	}
	else if(PWR_MAX_LED_STATE_BLINK == poe_max_led_blink_flag)
	{
		if(sys_actual_cons + POWER_REMAIN_MAX_LED_ON + burst <= sys_power_limit)
		{
			poe_max_led_blink_flag = PWR_MAX_LED_STATE_OFF;
			ad_led_poe_max_led_off();
		}
		else
		{
			ad_led_poe_max_led_green_blink();
		}
	}

}

/*	生产优化：由于硬件设计的原因，0x20/0x28这对地址在编址失败的情况下仍然
 *	会响应I2C读写请求，可能造成PSE子板撞件等故障需要在到老化阶段才能拦截。
 *	为了提高工厂生产效率，后续Po_e新机型统一切换到新地址方案(pse_addr_new)。
 *	[注] maple/marvel需要切换；cypress方案已经是这套地址，无需切换。*/
static void ad_poe_addr_ctrl(void)
{
	/* pse_addr方案初始化 */
	switch (get_model_info())
	{
		case AD_MARVELL_ER7212PC:
			pse_addr = pse_addr_mp3924_new;
			break;
		default:
			pse_addr = pse_addr_mp3924_new;
			break;
	}

	/* Marvell方案I2C地址直接从AD层调用SDK写入，不用同步 */
}

/**************************************************************************************************/
/*                                      GLOBAL_FUNCTIONS                                          */
/**************************************************************************************************/
/*!
 *\Function:
 *      ad_poe_init
 *\Description:
 *      ad layer init entry of the poe module 
 *\Input:
 *      n/a
 *\Output:
 *      n/a
 *\Return:
 *      ERR_NO_ERROR  - on success
 *      (Others)      - on error
 *\Note:
 *      n/a
 */
int ad_poe_init(void)
{
	int rv;
	int i = 0;
	DBG("ad_poe_init Begin\n");
	sem_poe_control = pal_mutex_create(sem_poe_control_des);/*使用pal层接口创建信号量消息队列等*/
	if(sem_poe_control == NULL)
	{
		return (ERR_HW_OP_FAIL);/*return需统一调用此宏*/
	}

	sem_poe_time = pal_sem_create("sem_poe_time",0,0);
	if(sem_poe_time == NULL)
	{
		return (ERR_HW_OP_FAIL);/*return需统一调用此宏*/
	}
	
	/* pse_addr初始化 */
	ad_poe_addr_ctrl();
	
	/*数据结构初始化*/
   	ad_poe_attach();
	   
	/*!<创建信号量  */
	memset(sem_poe_off_desc, 0, sizeof(sem_poe_off_desc));
	for (i = POE_UPORT_MIN; i <=POE_UPORT_MAX; i++) 
	{ 
	    sprintf(&sem_poe_off_desc[i-POE_UPORT_MIN][0], "sem_poe_off_port%d", i);
	    p_sem_poe_off[i-POE_UPORT_MIN] = pal_sem_create((char*)&(sem_poe_off_desc[i-POE_UPORT_MIN][0]),0,1);
	    if (p_sem_poe_off[i-POE_UPORT_MIN] == NULL)
	    {
	        printf("Create semaphore for Port %d failed\r\n", i);
	        return (ERR_HW_OP_FAIL);
	    }
	}
	
	
	FUNC_CALL(ad_poe_misc_init(), rv);
	
	/* GPIO初始化 */
	DBG("ad_poe_board_init Begin\n");
    FUNC_CALL(ad_poe_board_init(), rv);
	
	/* 芯片复位 */
	FUNC_CALL(ad_poe_chip_reset(), rv);
	
	/*I2C初始化*/
	//FUNC_CALL(ad_poe_bus_init(), rv);
	DBG("ad_poe_board_init Begin\n");
	/* 芯片配置 */
    FUNC_CALL(ad_poe_chip_init(), rv);
	
	
	DBG(" %s : init ok\r\n", __FUNCTION__);
    return (ERR_NO_ERROR);
}


/*#endif *//* TPLINK_T1600G_28PS */

#if 0
/**************************************************************************************************/
/*                            TEST          FUNCTIONS                                             */
/**************************************************************************************************/
#endif



/*!
*\Function:
*      ad_poe_disable_reset_detect
*\Description:
*      关闭复位检测，reboot时使用
*/
void ad_poe_disable_reset_detect()
{
	rst_det_enable = FALSE;
}

/*!
*\Function:
*      ad_poe_in_reset_state
*\Description:
*      检查PSE芯片是否处于复位状态
*\Input:
*      dev      - PSE芯片编号，从0开始
*\Output:
*     
 *\Return:
*      TRUE  - 复位
*      FALSE - 未复位
*\Note:
*      n/a
*/
int ad_poe_in_reset_state_mp3924(int dev)
{
     int rv = 0;
	 usp_uint8_t poe_status;
	 usp_uint8_t mode;	 

	 rv = ad_poe_reg_read(dev, (usp_uint8_t)MP3924_POE_POWER_SOURCE_STATUS1_CLEAR, &poe_status);
     if(ERR_NO_ERROR != (rv))
     {
        return FALSE;
     }

	 rv = ad_poe_reg_read(dev, (usp_uint8_t)MP3924_POE_INTERRUPT_MASK, &mode);
     if(ERR_NO_ERROR != (rv))
     {
        return FALSE;
     }	 

	 if((poe_status != 0) || (mode != MP3924_RESET_DETECT))
	 {
		return TRUE;
	 }

     return FALSE;
}


/*!
*\Function:
*      ad_poe_re_config
*\Description:
*      复位检测，以及重新配置被复位的PSE芯片
*/
void ad_poe_re_config_mp3924()
{
	int dev = 0;
	int count = 0;
	int rv = 0;
	int index = 0;
	poe_port_info_t *port_info = NULL;
	//usp_uint8_t 		op_mode = 0;
	int     	user_port = 1;   /* start from 1... */
	usp_uint8_t data;

	if(rst_det_enable == FALSE)
	{
		return;
	}

	/*遍历每一个PSE*/
	for(dev = 0; dev < PSE_CHIP_NUMS; dev++)
	{
		if(FALSE == ad_poe_in_reset_state_mp3924(dev))
		{
			continue;
		}
		pal_usleep(20*1000);
		if(FALSE == ad_poe_in_reset_state_mp3924(dev))
		{
			continue;
		}

		DBG("Start Re_config PSE %d\r\n", dev);
		//syslog_add_for_po_e(LOG_POE_PSE_RESET_DETECTED, (int)dev+1, 2, 3, 4, 5, 6);

		for(count = 0; count < sizeof(pse_init_data_mp3924)/sizeof(pse_init_data_mp3924[0]); count++)
		{
			rv = ad_poe_reg_write(dev, pse_init_data_mp3924[count][0], pse_init_data_mp3924[count][1]);
			if(rv != 0)
			{
				printf(" %s : [%02X] = 0x%02x write error\r\n", __FUNCTION__, pse_init_data_mp3924[count][0], pse_init_data_mp3924[count][1]);	
			}
			pal_usleep(100*1000);
		}
		ad_poe_reg_read(dev, MP3924_POE_POWER_SOURCE_STATUS1_CLEAR,  &data);

		for (index = 0; index < PORTS_PER_PSE_CHIP; index++)
		{
			ad_poe_pse_port2User_port(dev, (index + 1), &user_port);
			port_info = &POE_PORT_INFO(user_port);
			if(POE_DISABLE == port_info->poe_enable)
			{
				DBG("disable %d !\r\n", user_port);
				ad_poe_set_port_state(user_port, POE_DISABLE);
			}
		}
		pal_usleep(20*1000);
	}
}