/*******************************************************************************
 *
 * Copyright (c) 2008 TP-Link Technologies CO.,LTD.
 * All rights reserved.
  
 * Filename:		sw_timer.c
 * Version:			1.0
 * Abstract:		
 *
 * Author:			郑星固 <zhengxinggu@tp-link.net>
 * Created Date:	02/16/2008
 *
 * Modified History:	
 *
 *
 *******************************************************************************/
#include "timer.h"
#include "usp_pal_sync.h"
#include "usp_pal_thread.h"




extern usp_uint32_t get_timer_task_stack(void);
#define TIMER_TASK_STACK_SIZE	(16*1024)
#define NO_WAIT		0
#define WAIT_FOREVER	(-1)
#define BILLION  (1000000000)
#define SEC_PER_TEN_YEAR (315360000)

static struct timer_list task_list_head;						// 定时任务链表指针
static struct timer_list *p_idle_list_head;						// 空闲链表
static struct timer_list timer_list_node[MAX_TIMER_NUMBER];	// 使用数组模拟链表

static pal_sem_t sem_list_is_changed = NULL;
static pal_mutex_t mutex_list_is_safe = NULL;

static char *sem_list_is_change_des = "sem_list_is_change_des";
static char *mutex_list_is_safe_des = "mutex_list_is_safe_des";

STATUS timer_struct_timespec_check(struct timespec *p_time_spec)
{
    if(NULL == p_time_spec)
    {
        return ERROR;
    }

    if(p_time_spec->tv_sec > SEC_PER_TEN_YEAR)
    {
        return ERROR;
    }

    if(p_time_spec->tv_nsec > BILLION)
    {
        return ERROR;
    }
    
    return OK;
}

void timer_struct_timespec_plus(struct timespec *p_sum, struct timespec *p_src)
{
    if( (OK != timer_struct_timespec_check(p_sum)) || (OK != timer_struct_timespec_check(p_src)) )
    {
        return ;
    }
    
    p_sum->tv_sec += p_src->tv_sec;
    p_sum->tv_nsec += p_src->tv_nsec;

    if(p_sum->tv_nsec >= BILLION)
    {
        p_sum->tv_nsec -= BILLION;
        p_sum->tv_sec += 1;
    }
}

STATUS timer_struct_timespec_subtracton(struct timespec *p_sum, struct timespec *p_src)
{
    if( (OK != timer_struct_timespec_check(p_sum)) || (OK != timer_struct_timespec_check(p_src)) )
    {
        return ERROR;
    }

    if(p_sum->tv_sec < p_src->tv_sec)
    {
        return ERROR;
    }
    p_sum->tv_sec -= p_src->tv_sec;



    if(p_sum->tv_nsec < p_src->tv_nsec )
    {
        if(p_sum->tv_sec < 1)
        {
            return ERROR;
        }

        p_sum->tv_sec -= 1;
        p_sum->tv_nsec += BILLION;
    }

    p_sum->tv_nsec -= p_src->tv_nsec;

    return OK;
}

BOOL timer_struct_timespec_is_bigger_equal(struct timespec *p_a, struct timespec *p_b)
{
    if( (OK != timer_struct_timespec_check(p_a)) || (OK != timer_struct_timespec_check(p_b)) )
    {
        return FALSE;
    }

    if(p_a->tv_sec > p_b->tv_sec)
    {
        return TRUE;
    }

    if(p_a->tv_sec == p_b->tv_sec && p_a->tv_nsec >= p_b->tv_nsec)
    {
        return TRUE;
    }

    return FALSE;
}



/* 函数名称	: timer_add2Task_list
 * 函数介绍	: 按到期顺序将所给结点插入到链表中, 调用者需确保访问数据安全
 * 输入参数	: p_new_task_node - 所需插入结点的指针
 * 输出参数 : 无
 * 返 回 值 : 无
 */
void timer_add2Task_list(struct timer_list *p_new_task_node)
{
	struct timer_list *p_next_new_node;
	p_next_new_node = task_list_head.next;

	// 查找到新增结点所应在位置的后一结点指针
	while ((&task_list_head != task_list_head.next) &&
		   (&task_list_head != p_next_new_node) &&
		   (TRUE == timer_struct_timespec_is_bigger_equal(&(p_new_task_node->task.expires), &(p_next_new_node->task.expires)))
		   )
	{
		p_next_new_node = p_next_new_node->next;
	}
	// 将结点p_new_task_node插入到结点p_next_new_node之前
	p_next_new_node->prior->next = p_new_task_node;
	p_new_task_node->prior = p_next_new_node->prior;
	p_new_task_node->next = p_next_new_node;
	p_next_new_node->prior = p_new_task_node;

	return;
}



/* 函数名称	: add_timer
 * 函数介绍	: 注册定时任务,以回调函数方式实现
 * 输入参数	: interval - 任务的执行间隔(单位:tick);
 *			  p_function - 所需执行的回调函数指针地址;
 *			  param	   - 回调函数的参数;
 *			  style	   - 任务执行类型 : EXECUTE_FOREVER - 周期循环执行 
 *										EXECUTE_SINGLE	- 只执行一次
 * 输出参数 : 无
 * 返 回 值	: 新增定时任务状态 - 定时任务ID(成功); ERROR(失败)
 */
TIMER_ID add_timer(struct timespec *p_time_interval, void *p_function, usp_uint param, TIMER_STYLE style)
{
	int n_timer_ID = ERROR;
	struct timespec current_time;
	struct timer_list *p_new_task_node;
	
    //printf("add_timer \n");
	if (((EXECUTE_FOREVER != style) && (EXECUTE_SINGLE != style)) || (NULL == p_function))	// 输入参数合法性检查
	{
		return ERROR;
	}
    //printf("add_timer 1\n");
    if(ERROR == timer_struct_timespec_check(p_time_interval))
    {
        return ERROR;
    }
    //printf("add_timer 2\n");
    clock_gettime(CLOCK_MONOTONIC, &current_time);
    
    //printf("add_timer 2.1\n");
	if (ERROR == pal_mutex_take(mutex_list_is_safe, WAIT_FOREVER))			// 如定时器模块未被初始化，此处返回ERROR
	{
		
		//printf("add_timer 2.2\n");
		return ERROR;
	}
    //printf("add_timer 3\n");
	if (NULL != p_idle_list_head)									// 空闲链表非空
	{
		// 从空闲链表中取得结点
		p_new_task_node = p_idle_list_head;
		p_idle_list_head = p_idle_list_head->next;

		// 对所取得结点上变量赋值
		memcpy(&(p_new_task_node->task.expires), &current_time, sizeof(struct timespec));
        timer_struct_timespec_plus(&(p_new_task_node->task.expires), p_time_interval);
        
		p_new_task_node->task.p_function = p_function;
		memcpy(&(p_new_task_node->task.interval),p_time_interval, sizeof(struct timespec));
		p_new_task_node->task.param = param;
		p_new_task_node->task.style = style;
		n_timer_ID = p_new_task_node - &timer_list_node[0];			// n_timer_ID=结点所在数组下标

		// 将该结点按到期顺序插入到定时任务链表
		timer_add2Task_list(p_new_task_node);
	}
    //printf("add_timer 4\n");
	pal_mutex_give(mutex_list_is_safe);
    //printf("add_timer 5\n");
	// 通知管理模块当前定时任务链表结构已改变
	pal_sem_give(sem_list_is_changed);

	return n_timer_ID;
}


/* 函数名称	: del_timer
 * 函数介绍	: 删除指定定时任务
 * 输入参数	: n_timer_ID - 定时任务ID
 * 输出参数 : 无
 * 返 回 值	: 删除定时任务状态  -OK(成功); ERROR(失败)
 */
STATUS del_timer(usp_uint n_timer_ID)
{
	if (MAX_TIMER_NUMBER <= n_timer_ID)							// 输入参数合法性检查
	{
		return ERROR;
	}

	if (ERROR == pal_mutex_take(mutex_list_is_safe, WAIT_FOREVER))			// 如定时器模块未被初始化，此处返回ERROR
	{
		return ERROR;
	}

	if (NULL != timer_list_node[n_timer_ID].prior)					// 结点不在空闲队列中
	{
		timer_list_node[n_timer_ID].prior->next = timer_list_node[n_timer_ID].next;
		timer_list_node[n_timer_ID].next->prior = timer_list_node[n_timer_ID].prior;
		timer_list_node[n_timer_ID].next = p_idle_list_head;
		timer_list_node[n_timer_ID].prior = NULL;
		p_idle_list_head = &timer_list_node[n_timer_ID];
	}
	pal_mutex_give(mutex_list_is_safe);

	// 通知管理模块当前定时任务链表结构被改变
	// 此处如果不通知将有可能管理模块是因不能获取到互斥信号量
	// 而处于等待同步信号量状态下不能再次进入
	pal_sem_give(sem_list_is_changed);

	return OK;
}



/* 函数名称	: timer_clear
 * 函数介绍	: 清空定时器列表
 * 输入参数	: 无
 * 输出参数	: 无
 * 返 回 值	: 无
 */
void timer_clear()
{
	struct timer_list *p_first_task_node;

	// 获取异步信号量,保证访问数据安全
	if(OK == pal_mutex_take(mutex_list_is_safe, NO_WAIT))
	{
		// 只判断除头结点外第一个结点是否已到期
		while ((&task_list_head != task_list_head.next))
		{
			p_first_task_node = task_list_head.next;			

			// 将该结点从定时任务链表中删除,但不改变p_first_task_node指向地址
			task_list_head.next = task_list_head.next->next;
			task_list_head.next->prior = &task_list_head;

			// 将该结点加入到空闲链表中
			p_first_task_node->next = p_idle_list_head;
			p_first_task_node->prior = NULL;
			p_idle_list_head = p_first_task_node;

		}// End of while
	}// End of if(OK == sem_take(mutex_list_is_safe, NO_WAIT))		
	pal_mutex_give(mutex_list_is_safe);
	return;
}

void timer_thread(void *arg)
{
    //int n_time_out_usec;
    //ULONG current_tick;
    struct timespec current_time = {0, 0};
    struct timer_list *p_first_task_node;

    int n_time_out = WAIT_FOREVER;            // 初始化后无定时任务，永远等待
    while (1)
    {
        // 获取同步信号量或等待超时
        pal_sem_take(sem_list_is_changed, n_time_out);

        // 获取异步信号量,保证访问数据安全
        if(OK == pal_mutex_take(mutex_list_is_safe, NO_WAIT))
        {
            clock_gettime(CLOCK_MONOTONIC, &current_time);

            // 只判断除头结点外第一个结点是否已到期
            while ( (&task_list_head != task_list_head.next) && 
                    (TRUE == timer_struct_timespec_is_bigger_equal(&(current_time), &(task_list_head.next->task.expires)))
                  )
            {
                p_first_task_node = task_list_head.next;         

                // 将该结点从定时任务链表中删除,但不改变p_first_task_node指向地址
                task_list_head.next = task_list_head.next->next;
                task_list_head.next->prior = &task_list_head;
            
                if (EXECUTE_SINGLE == p_first_task_node->task.style)           // 删除只执行一次的任务             
                {
                    // 将该结点加入到空闲链表中
                    p_first_task_node->next = p_idle_list_head;
                    p_first_task_node->prior = NULL;
                    p_idle_list_head = p_first_task_node;
                }   
                else                                                        // 重新插入到链表中                             
                {
                    /*32位可以标识26亿，超过2秒，只要保证tv_nsec不超过10亿就不会出现越界 */
                    p_first_task_node->task.expires.tv_sec += p_first_task_node->task.interval.tv_sec;
                    p_first_task_node->task.expires.tv_nsec += p_first_task_node->task.interval.tv_nsec;

                    if(p_first_task_node->task.expires.tv_nsec >= BILLION)
                    {
                        p_first_task_node->task.expires.tv_nsec -= BILLION;
                        p_first_task_node->task.expires.tv_sec += 1;
                    }
                    
                    timer_add2Task_list(p_first_task_node);                     // 按序插入到定时任务链表中
                }

                p_first_task_node->task.p_function(p_first_task_node->task.param); // 执行回调函数

            }// End of while 
            
            // 定时任务链表中无到期定时任务时, 重新计算下次等待的超时时间
            if (&task_list_head != task_list_head.next)                         // 定时任务链表不为空
            {
                n_time_out = (task_list_head.next->task.expires.tv_sec - current_time.tv_sec) * 1000000;
                n_time_out += (task_list_head.next->task.expires.tv_nsec - current_time.tv_nsec) / 1000;

                if(((task_list_head.next->task.expires.tv_nsec - current_time.tv_nsec) % 1000) > 0)
                {
                    n_time_out ++;
                }
                
                if (n_time_out < 0 || n_time_out > 60000000)                  //1分钟时间内，总会执行一次
                {
                    n_time_out = 60000000;                                  // n_time_out为int型，防止数值越界
                }
            }
            else                                                            // 定时任务链表为空
            {
                n_time_out = WAIT_FOREVER;
            }

            pal_mutex_give(mutex_list_is_safe);
        }// End of if(OK == sem_take(mutex_list_is_safe, NO_WAIT))
    }//End of while (1)

    return;
}


/* 函数名称	: timer_init
 * 函数介绍	: 初始化定时器模块,只能被调用过一次; 将创建一个高优先级任务来管理定时任务
 * 输入参数	: 无
 * 输出参数 : 无
 * 返 回 值	: 初始化成功状态  -OK(成功); ERROR(失败)
 */
STATUS timer_init(void)
{
	int i;
	pal_thread_t timer_id = NULL;
	int n_stacksize;

	if (sem_list_is_changed != NULL && mutex_list_is_safe != NULL)	// 只能被调用成功过一次
	{
		return ERROR;
	}

	// 初始化定时任务链表
	task_list_head.next = &task_list_head;
	task_list_head.prior = &task_list_head;

	// 初始化空闲链表
	p_idle_list_head = &timer_list_node[0];	
	for (i = 0; i < MAX_TIMER_NUMBER - 1; i++)
	{
		timer_list_node[i].next = &timer_list_node[i+1];
		timer_list_node[i].prior = NULL;
	}
	timer_list_node[i].next = NULL;
	timer_list_node[i].prior = NULL;

	// 创建信号量
	mutex_list_is_safe = pal_mutex_create(mutex_list_is_safe_des);	
	sem_list_is_changed = pal_sem_create(sem_list_is_change_des, 0, 0);	
    
	if (NULL == mutex_list_is_safe || NULL == sem_list_is_changed)
	{
		pal_mutex_destroy(mutex_list_is_safe);
		pal_sem_destroy(sem_list_is_changed);
		return ERROR;
	}

	// 创建管理定时器的高优先级任务
	n_stacksize = TIMER_TASK_STACK_SIZE;

	timer_id = pal_thread_create("tp_systimer", n_stacksize, 100, timer_thread, (void *)NULL);
    
	if (PAL_THREAD_ERROR == timer_id)   
	{
		pal_mutex_destroy(mutex_list_is_safe);
		pal_sem_destroy(sem_list_is_changed);
		mutex_list_is_safe = NULL;
		sem_list_is_changed = NULL;
		return ERROR;
	}
    
	return OK;
}

