/*! Copyright(c) 2008-2016 Shenzhen TP-LINK Technologies Co.Ltd.
 *
 *\file     ipstat_sfe.c
 *\brief
 *\details
 *
 *\author   Yuan Fengjia
 *\version  0.0.1
 *\date     18May16
 *
 *\warning
 *
 *\history  \arg    0.0.1, 22May15, Yuan Fengjia, Create the file.
 *\history  \arg    0.0.2, 13Oct17, Zhang Zhongwei, Modify the file.
 */

/**************************************************************************************************/
/*                                      CONFIGURATIONS                                            */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      INCLUDE_FILES                                             */
/**************************************************************************************************/
#include <linux/types.h>
#include <linux/if_ether.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/module.h>
#include <linux/netfilter_ipv4.h>
#include <linux/netfilter_bridge.h>
#include <linux/spinlock.h>
#include <linux/timer.h>
#include <linux/version.h>
#include <linux/proc_fs.h>
#include <net/neighbour.h>
#include <net/arp.h>
#include <linux/list.h>
#include <linux/hash.h>
#include <linux/if_bridge.h>
#include <linux/uaccess.h>
#include <linux/kthread.h>
#include "sfe_cm.h"
#include "sfe_ipstat.h"

#ifdef IPSTAT_SUPPORT_IPV6
#include <linux/inet.h>
#include <linux/ipv6.h>
#endif
/**************************************************************************************************/
/*                                      DEFINES                                                   */
/**************************************************************************************************/
#define ETHER_PAYLOAD_LEN           14
#define IPV6_NET_LEN           40
#define LAN_IFNAME                  "br-lan"
#define LAN_IFNAME_LEN              6
#define SFE_IPSTAT_PROC_NAME        "sfe_ipstat"
#define SFE_IPMAC_PROC_NAME         "sfe_ip_mac"
#define SFE_IPSTAT_STEP_PROC_NAME   "sfe_ipstat_step"
#define SFE_CLIENTSTAT_PROC_NAME    "sfe_clientstat"
#ifdef SFE_CONNMARK_STAT
#define SFE_CONNMARK_PROC_NAME    "sfe_connmark"
#endif
//todo
//bandcontrol switch,legacy code with no macro
#define SFE_IPSTAT_SWITCH_NAME "sfe_ipstat_switch"
#define FRAGMENT_SWITCH_NAME "sfe_fragment_switch"
#define PPP_TUNNEL_SWITCH_NAME "sfe_ppp_tunnel_switch"
#define SFE_FASTPATH_NAT_SWITCH_NAME "sfe_fastpath_nat_switch"
#define SFE_QOS_FASTPATH_NAT_SWITCH_NAME "sfe_qos_fastpath_nat_switch"
#define IPSTAT_MAX_NAME "ipstat_max"

#define MAX_IP_STR_LEN              20
#define MAX_IPV6_STR_LEN              40

#define IPSTAT_HASH_BITNUM     (10)
#define IPSTAT_HASHTBL_MAX     (1UL << IPSTAT_HASH_BITNUM)

#define ARP_GC_THRESH_TIME     (60)


#define UPTIME_BITS	2
#define ipstat_debug(fmt, args...)\
        /*printk(KERN_INFO"[LOG]%s,%d:"fmt"\r\n", __FUNCTION__,__LINE__,##args)*/

/**************************************************************************************************/
/*                                      TYPES                                                     */
/**************************************************************************************************/
typedef struct _IP_SUBNET
{
    u_int32_t ip;
    u_int32_t mask;
#ifdef IPSTAT_SUPPORT_IPV6
    struct sfe_ipv6_addr ipv6;
    __be32 v6_mask[4];
#endif
}IP_SUBNET;


typedef struct _IPSTAT_STATISTIC
{
    u_int64_t src_packets;
    u_int64_t src_bytes;

    u_int64_t dst_packets;
    u_int64_t dst_bytes;
}IPSTAT_STATISTIC;

typedef struct _IPSTAT_IP_ENTRY
{
    bool is_usb;
    u_int32_t ip;
    u_int8_t  mac[ETH_ALEN];
    IPSTAT_STATISTIC cur_stat;
    IPSTAT_STATISTIC cur_stat_update;
    IPSTAT_STATISTIC pre_stat_update;

    unsigned long arp_timer;
    u_int32_t offline;

    struct hlist_node hostHashNode;
    struct _IPSTAT_IP_ENTRY *pNextLruHost;
    struct _IPSTAT_IP_ENTRY *pPrevLruHost;
    bool clientFlag;
    char devname[IFNAMSIZ];
}IPSTAT_IP_ENTRY;
#ifdef SFE_CONNMARK_STAT
typedef struct _CONNMARK_ENTRY
{
    u_int32_t connmark;
    IPSTAT_STATISTIC cur_stat;
    IPSTAT_STATISTIC cur_stat_update;
    IPSTAT_STATISTIC pre_stat_update;

    unsigned long arp_timer;
    u_int32_t offline;

    struct hlist_node hostHashNode;
    struct _CONNMARK_ENTRY *pNextLruHost;
    struct _CONNMARK_ENTRY *pPrevLruHost;
    bool clientFlag;
}CONNMARK_ENTRY;

typedef struct _CONNMARK_MNGR
{
    CONNMARK_ENTRY *mem_pool;
    int used;
    CONNMARK_ENTRY hostLstHead;
    CONNMARK_ENTRY *pLastLruHost;
    struct hlist_head hostHashTbl[IPSTAT_HASHTBL_MAX];
}CONNMARK_MNGR;
#endif
typedef struct _IPSTAT_IP_MNGR
{
    IPSTAT_IP_ENTRY *mem_pool;
    int used;
    IPSTAT_IP_ENTRY hostLstHead;
    IPSTAT_IP_ENTRY *pLastLruHost;
    struct hlist_head hostHashTbl[IPSTAT_HASHTBL_MAX];
}IPSTAT_IP_MNGR;

/**************************************************************************************************/
/*                                      EXTERN_PROTOTYPES                                         */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      LOCAL_PROTOTYPES                                          */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      VARIABLES                                                 */
/**************************************************************************************************/
IP_SUBNET g_ip_subnet;
static struct proc_dir_entry *sfe_ipstat_proc = NULL;
static struct proc_dir_entry *sfe_ipmac_proc = NULL;
static struct proc_dir_entry *sfe_ipstat_step_proc = NULL;
static struct proc_dir_entry *sfe_clientstat_proc = NULL;
#ifdef SFE_CONNMARK_STAT
static struct proc_dir_entry *sfe_connmark_proc = NULL;
#endif
//todo
static struct proc_dir_entry *sfe_ipstat_switch_proc = NULL;
static struct proc_dir_entry *fragment_switch_proc = NULL;
static struct proc_dir_entry *ppp_tunnel_switch_proc = NULL;
static struct proc_dir_entry *sfe_fastpath_nat_switch_proc = NULL;
static struct proc_dir_entry *sfe_qos_fastpath_nat_switch_proc = NULL;
static struct proc_dir_entry *ipstat_max_proc = NULL;
static DEFINE_SPINLOCK(g_stat_lock);
unsigned long stat_lock_flag;
static IPSTAT_IP_MNGR g_ip_entry_table;
#ifdef SFE_CONNMARK_STAT
static CONNMARK_MNGR g_connmark_entry_table;
#endif
static int g_interval_time = 1;
static int g_ipstat_interval = 10;
static int g_interval_update_time = (1 << UPTIME_BITS);
static struct timer_list g_arp_timer;
static struct timer_list g_update_timer;
volatile unsigned long g_cur_timer_cycle = 0;
volatile unsigned long g_cur_update_timer_cycle = 0;
static int g_ipstat_max = 4096;
//todo
//legacy code,no macro
int sfe_ipstat_switch = 0;
static struct task_struct *ipstat_kthread = NULL;
int fragment_switch = 1;
int ppp_tunnel_switch = 0;
int fastpath_nat_enabled = 0;
int qos_fastpath_nat_enabled = 0;


//func declare
int sfe_ipstat_flush(void);
/**************************************************************************************************/
/*                                      LOCAL_FUNCTIONS                                           */
/**************************************************************************************************/
static void ipstat_lock(void)
{
    spin_lock_bh(&g_stat_lock);
}

static void ipstat_unlock(void)
{
    spin_unlock_bh(&g_stat_lock);
}

static u_int32_t do_hash_key(u_int32_t data)
{
    u_int32_t hashIn = 0;
    hashIn = data;
    return hash_32(hashIn, IPSTAT_HASH_BITNUM);
}

/**************************/
/*       connmark         */
/**************************/

#ifdef IPSTAT_SUPPORT_IPV6
int sfe_ipv6_addr_mask_equal(struct sfe_ipv6_addr ip1, __be32 mask1[4], struct sfe_ipv6_addr ip2, __be32 mask2[4])
{
    int index = 0;
    for (index = 0; index < 4; index++)
    {
        if ((ip1.addr[index] & mask1[index]) != (ip2.addr[index] & mask2[index]))
        {
            return 0;
        }
    }
    return 1;
}
#endif
void remove_connmark_entry(CONNMARK_ENTRY *pHost);

#ifdef SFE_CONNMARK_STAT
void connmark_init(int is_init)
{
    int idx = 0;
	struct _CONNMARK_ENTRY *pHost = NULL;

    if (is_init)
    {
        memset(&g_connmark_entry_table, 0, sizeof(g_connmark_entry_table));
    } else{
		pHost = g_connmark_entry_table.hostLstHead.pNextLruHost;
		while (pHost != NULL) {
			remove_connmark_entry(pHost);
			pHost = pHost->pNextLruHost;
		}
		memset(g_connmark_entry_table.mem_pool,0, g_ipstat_max);
		g_connmark_entry_table.used = 0;    
    }

    g_connmark_entry_table.pLastLruHost = &g_connmark_entry_table.hostLstHead;
    for ( idx = 0; idx < IPSTAT_HASHTBL_MAX; idx++ ) {
        INIT_HLIST_HEAD(&g_connmark_entry_table.hostHashTbl[idx]);
    }

    return;
}

CONNMARK_ENTRY *connmark_get_entry(u_int32_t connmark, bool clientFlag)
{
    u_int32_t hashKey = do_hash_key(connmark);
    struct hlist_head *head = &g_connmark_entry_table.hostHashTbl[hashKey];
    struct hlist_node *_p = NULL;
    CONNMARK_ENTRY *pHost = NULL;
    if ( head == NULL) {
        return NULL;
    }

    hlist_for_each(_p, head) {
        pHost = hlist_entry(_p, CONNMARK_ENTRY, hostHashNode);
        if ( pHost == NULL ) {
            continue ;
        }
        if ( (pHost->connmark == connmark) && (clientFlag == pHost->clientFlag)) {
            break;
        }
    }
    return ((_p == NULL) ? NULL : pHost);
}

static void init_connmark_host(CONNMARK_ENTRY *pHost, u_int32_t connmark)
{
    memset(pHost, 0, sizeof(CONNMARK_ENTRY));
    pHost->connmark = connmark;
    pHost->offline = 1;
}

static void add_connmark_entry_toLruLst(CONNMARK_ENTRY *pHost)
{
    if (g_connmark_entry_table.pLastLruHost == &g_connmark_entry_table.hostLstHead) {
        g_connmark_entry_table.pLastLruHost = pHost;
    }

    pHost->pPrevLruHost = &g_connmark_entry_table.hostLstHead;
    pHost->pNextLruHost = g_connmark_entry_table.hostLstHead.pNextLruHost;

    if (pHost->pNextLruHost != NULL) {
        pHost->pNextLruHost->pPrevLruHost = pHost;
    }
    g_connmark_entry_table.hostLstHead.pNextLruHost = pHost;

    return;
}

static void add_connmark_entry_toHashTbl(CONNMARK_ENTRY *pHost)
{
    u_int32_t hashKey = do_hash_key(pHost->connmark);
    struct hlist_head *head = &g_connmark_entry_table.hostHashTbl[hashKey];
    hlist_add_head(&pHost->hostHashNode, head);

    return;
}

static void add_connmark_entry(CONNMARK_ENTRY *pHost)
{
    add_connmark_entry_toLruLst(pHost);
    add_connmark_entry_toHashTbl(pHost);
}

static void remove_connmark_entry_fromHashTbl(CONNMARK_ENTRY *pHost)
{
    hlist_del_init(&pHost->hostHashNode);
}

static void remove_connmark_entry_fromLruLst(CONNMARK_ENTRY *pHost)
{
    pHost->pPrevLruHost->pNextLruHost = pHost->pNextLruHost;

    if (g_connmark_entry_table.pLastLruHost == pHost) {
        g_connmark_entry_table.pLastLruHost = pHost->pPrevLruHost;
    } else {
        pHost->pNextLruHost->pPrevLruHost = pHost->pPrevLruHost;
    }

    return;
}

void remove_connmark_entry(CONNMARK_ENTRY *pHost)
{
    remove_connmark_entry_fromLruLst(pHost);
    remove_connmark_entry_fromHashTbl(pHost);
}



static void replace_connmark_entry(CONNMARK_ENTRY *pHost, const u_int32_t connmark, bool clientFlag)
{
    remove_connmark_entry(pHost);
    init_connmark_host(pHost, connmark);
    pHost->clientFlag = clientFlag;
    add_connmark_entry(pHost);
}

static CONNMARK_ENTRY *replace_exist_connmark_entry(const u_int32_t connmark, bool clientFlag)
{
    CONNMARK_ENTRY *pHost = NULL;

    pHost = g_connmark_entry_table.pLastLruHost;
    if (pHost == NULL ) {
        return NULL;
    }

    replace_connmark_entry(pHost, connmark, clientFlag);
    return pHost;
}


static CONNMARK_ENTRY *add_new_connmark_entry(const u_int32_t connmark, bool clientFlag)
{
    CONNMARK_ENTRY *pNewHost = NULL;

    if(g_connmark_entry_table.mem_pool)
    {
        if (g_connmark_entry_table.used >= g_ipstat_max) {
            return replace_exist_connmark_entry(connmark, clientFlag);
        }
        pNewHost = &g_connmark_entry_table.mem_pool[g_connmark_entry_table.used];
        g_connmark_entry_table.used ++;
        init_connmark_host(pNewHost, connmark);
        pNewHost->clientFlag = clientFlag;

        add_connmark_entry(pNewHost);
    }
    return pNewHost;
}

CONNMARK_ENTRY *find_conn_entry(const u_int32_t connmark, bool clientFlag)
{
    CONNMARK_ENTRY *pHost = NULL;
    pHost = connmark_get_entry(connmark,clientFlag);
    if (pHost) {
            return pHost;
    }

    pHost = add_new_connmark_entry(connmark, clientFlag);
    return pHost;	
}

CONNMARK_ENTRY *get_connmark_firstLruHost(void)
{
    return g_connmark_entry_table.hostLstHead.pNextLruHost;
}

static int connmark_proc_show(struct seq_file *seq, void *v)
{
    CONNMARK_ENTRY *pHost = NULL;
    const int MAX_LINE_LEN = 512;
    char line[MAX_LINE_LEN];
    char *cur = &line[0];
    int len;

    spin_lock_bh(&g_stat_lock);
    u_int32_t denominator = (UPTIME_BITS + 10); // for KBps unit
    pHost = get_connmark_firstLruHost();
     while (pHost != NULL) {
        cur = &line[0];
        len = sprintf(cur, "0x%08x",pHost->connmark);
        cur += len;
        // calc KB/s and Pkt/s
        len = sprintf(cur, ";%llu;%llu;%llu;%llu",
                      ((pHost->cur_stat_update.src_packets - pHost->pre_stat_update.src_packets) >> UPTIME_BITS),
                      ((pHost->cur_stat_update.src_bytes - pHost->pre_stat_update.src_bytes) >> denominator),
                      ((pHost->cur_stat_update.dst_packets - pHost->pre_stat_update.dst_packets) >> UPTIME_BITS),
                      ((pHost->cur_stat_update.dst_bytes - pHost->pre_stat_update.dst_bytes) >> denominator)
                     );
        cur += len;
        len = sprintf(cur, ";%llu;%llu;%llu;%llu;",
                      pHost->cur_stat.src_packets,
                      pHost->cur_stat.src_bytes,
                      pHost->cur_stat.dst_packets,
                      pHost->cur_stat.dst_bytes);
        cur += len;
	len = sprintf(cur,"%s\n",(pHost->clientFlag)?"client":"server");
	cur += len;
        seq_puts(seq, line);
        pHost = pHost->pNextLruHost;
    }
    spin_unlock_bh(&g_stat_lock);

    return 0;
}

//todo
static int ipstat_switch_proc_show(struct seq_file *seq, void *v)
{
    char switch_ptr[2] = {0};
    switch_ptr[0] = sfe_ipstat_switch ? '1':'0';
    switch_ptr[1] = '\0';

    spin_lock_bh(&g_stat_lock);
    seq_puts(seq, &switch_ptr);
    spin_unlock_bh(&g_stat_lock);

    return 0;
}

static int fragment_switch_proc_show(struct seq_file *seq, void *v)
{
    char switch_ptr[2] = {0};
    switch_ptr[0] = fragment_switch ? '1':'0';
    switch_ptr[1] = '\0';

    spin_lock_bh(&g_stat_lock);
    seq_puts(seq, &switch_ptr);
    spin_unlock_bh(&g_stat_lock);

    return 0;
}

static int ppp_tunnel_switch_proc_show(struct seq_file *seq, void *v)
{
    char switch_ptr[2] = {0};
    switch_ptr[0] = ppp_tunnel_switch ? '1':'0';
    switch_ptr[1] = '\0';

    spin_lock_bh(&g_stat_lock);
    seq_puts(seq, &switch_ptr);
    spin_unlock_bh(&g_stat_lock);

    return 0;
}

static int fastpath_nat_switch_proc_show(struct seq_file *seq, void *v)
{
    char switch_ptr[2] = {0};
    switch_ptr[0] = fastpath_nat_enabled ? '1':'0';
    switch_ptr[1] = '\0';

    spin_lock_bh(&g_stat_lock);
    seq_puts(seq, &switch_ptr);
    spin_unlock_bh(&g_stat_lock);

    return 0;
}

static int fastpath_qos_nat_switch_proc_show(struct seq_file *seq, void *v)
{
    char switch_ptr[2] = {0};
    switch_ptr[0] = qos_fastpath_nat_enabled ? '1':'0';
    switch_ptr[1] = '\0';

    spin_lock_bh(&g_stat_lock);
    seq_puts(seq, &switch_ptr);
    spin_unlock_bh(&g_stat_lock);

    return 0;
}

static int ipstat_max_proc_show(struct seq_file *seq, void *v)
{
    char count_ptr[10] = {0};
    snprintf(count_ptr, sizeof(count_ptr), "%d", g_ipstat_max);

    spin_lock_bh(&g_stat_lock);
    seq_puts(seq, &count_ptr);
    spin_unlock_bh(&g_stat_lock);

    return 0;
}

void gc_thresh_connmark_entry(CONNMARK_ENTRY *pHost)
{
    pHost->arp_timer = g_cur_timer_cycle;
    pHost->offline = 0;
}


void connmark_tx(CONNMARK_ENTRY *pHost, u_int32_t connmark, u_int32_t len)
{

    pHost->cur_stat.src_bytes += len;
    pHost->cur_stat.src_packets += 1;
    pHost->connmark = connmark;
    gc_thresh_connmark_entry(pHost);
}

void connmark_rx(CONNMARK_ENTRY *pHost, u_int32_t connmark, u_int32_t len)
{
    pHost->cur_stat.dst_bytes += len;
    pHost->cur_stat.dst_packets += 1;
    pHost->connmark = connmark;
    gc_thresh_connmark_entry(pHost);
}
#endif
/***************************/
IPSTAT_IP_ENTRY *get_firstLruHost(void);
void remove_entry(IPSTAT_IP_ENTRY *pHost);

void ip_entry_init(int is_init)
{
    int idx = 0;
	IPSTAT_IP_ENTRY *pHost = NULL;

    if (is_init)
    {
	    memset(&g_ip_entry_table, 0, sizeof(g_ip_entry_table));
    }
    else{
		pHost = get_firstLruHost();
		while (pHost  != NULL) {
			remove_entry(pHost);
			pHost = pHost->pNextLruHost;
		}
		memset(g_ip_entry_table.mem_pool,0, g_ipstat_max);
	    g_ip_entry_table.used = 0;    
    }

    g_ip_entry_table.pLastLruHost = &g_ip_entry_table.hostLstHead;
    for ( idx = 0; idx < IPSTAT_HASHTBL_MAX; idx++ ) {
        INIT_HLIST_HEAD(&g_ip_entry_table.hostHashTbl[idx]);
    }

    return;
}

IPSTAT_IP_ENTRY *ipstat_get_entry(u_int32_t ipaddr, bool is_usb)
{
    ipstat_debug("ipaddr:%x",ipaddr);
    u_int32_t hashKey = do_hash_key(ipaddr);
    struct hlist_head *head = &g_ip_entry_table.hostHashTbl[hashKey];
    ipstat_debug();
    struct hlist_node *_p = NULL;
    IPSTAT_IP_ENTRY *pHost = NULL;

    ipstat_debug();
    if ( ipaddr == 0 || head == NULL) {
        return NULL;
    }

    ipstat_debug();
    hlist_for_each(_p, head) {
        ipstat_debug();
        pHost = hlist_entry(_p, IPSTAT_IP_ENTRY, hostHashNode);
        if ( pHost == NULL ) {
            continue ;
        }
        ipstat_debug("pHost:%x",pHost);
        if ( pHost->ip == ipaddr && pHost->is_usb == is_usb) {
            ipstat_debug();
            break;
        }
        ipstat_debug();
    }
    ipstat_debug();
    return ((_p == NULL) ? NULL : pHost);
}

static void init_host(IPSTAT_IP_ENTRY *pHost, u_int32_t ipaddr, const u_int8_t mac[ETH_ALEN], bool is_usb)
{
    memset(pHost, 0, sizeof(IPSTAT_IP_ENTRY));
    pHost->ip = ipaddr;
    pHost->is_usb = is_usb;
    if (mac[0] != 0 || mac[1] != 0 || mac[2] != 0 || mac[3] != 0 || mac[4] != 0 || mac[5] != 0) {
        memcpy(&pHost->mac, mac, ETH_ALEN);
    }
    pHost->offline = 1;
}

static void add_entry_toLruLst(IPSTAT_IP_ENTRY *pHost)
{
    if (g_ip_entry_table.pLastLruHost == &g_ip_entry_table.hostLstHead) {
        g_ip_entry_table.pLastLruHost = pHost;
    }

    pHost->pPrevLruHost = &g_ip_entry_table.hostLstHead;
    pHost->pNextLruHost = g_ip_entry_table.hostLstHead.pNextLruHost;

    if (pHost->pNextLruHost != NULL) {
        pHost->pNextLruHost->pPrevLruHost = pHost;
    }
    g_ip_entry_table.hostLstHead.pNextLruHost = pHost;

    return;
}

static void add_entry_toHashTbl(IPSTAT_IP_ENTRY *pHost)
{
    u_int32_t hashKey = do_hash_key(pHost->ip);
    struct hlist_head *head = &g_ip_entry_table.hostHashTbl[hashKey];
    hlist_add_head(&pHost->hostHashNode, head);

    return;
}

static void add_entry(IPSTAT_IP_ENTRY *pHost)
{
    add_entry_toLruLst(pHost);
    add_entry_toHashTbl(pHost);
}

static void remove_entry_fromHashTbl(IPSTAT_IP_ENTRY *pHost)
{
    hlist_del_init(&pHost->hostHashNode);
}

static void remove_entry_fromLruLst(IPSTAT_IP_ENTRY *pHost)
{
    pHost->pPrevLruHost->pNextLruHost = pHost->pNextLruHost;

    if (g_ip_entry_table.pLastLruHost == pHost) {
        g_ip_entry_table.pLastLruHost = pHost->pPrevLruHost;
    } else {
        pHost->pNextLruHost->pPrevLruHost = pHost->pPrevLruHost;
    }

    return;
}

void remove_entry(IPSTAT_IP_ENTRY *pHost)
{
    remove_entry_fromLruLst(pHost);
    remove_entry_fromHashTbl(pHost);
}



static void replace_entry(IPSTAT_IP_ENTRY *pHost, u_int32_t ipaddr, const u_int8_t mac[ETH_ALEN], bool clientFlag, bool is_usb)
{
    remove_entry(pHost);
    init_host(pHost, ipaddr, mac, is_usb);
    pHost->clientFlag = clientFlag;
    add_entry(pHost);
}


static IPSTAT_IP_ENTRY *replace_exist_entry(u_int32_t ipaddr, const u_int8_t mac[ETH_ALEN], bool clientFlag, bool is_usb)
{
    IPSTAT_IP_ENTRY *pHost = NULL;
    int client_count = 1;
    pHost = g_ip_entry_table.pLastLruHost;
    if (pHost == NULL ) {
        return NULL;
    }
    while(pHost && pHost->clientFlag && (client_count < IPSTAT_HASHTBL_MAX))
    {
    	pHost = pHost->pPrevLruHost;
	client_count++;
    }
    if ((pHost && pHost->clientFlag && (client_count == IPSTAT_HASHTBL_MAX)) || (!pHost))
    {
    	pHost = g_ip_entry_table.pLastLruHost;
    }

    replace_entry(pHost, ipaddr, mac, clientFlag, is_usb);
    return pHost;
}


static IPSTAT_IP_ENTRY *add_newEntry(u_int32_t ipaddr, const u_int8_t mac[ETH_ALEN], bool clientFlag, bool is_usb)
{
    IPSTAT_IP_ENTRY *pNewHost = NULL;
    if(g_ip_entry_table.mem_pool)
    {
        if (g_ip_entry_table.used >= g_ipstat_max) {
            return replace_exist_entry(ipaddr, mac, clientFlag, is_usb);
        }
    
        pNewHost = &g_ip_entry_table.mem_pool[g_ip_entry_table.used];
        g_ip_entry_table.used ++;
        init_host(pNewHost, ipaddr, mac, is_usb);
        pNewHost->clientFlag = clientFlag;

        add_entry(pNewHost);
    }

    return pNewHost;
}


IPSTAT_IP_ENTRY *find_ipentry(u_int32_t ipaddr, const u_int8_t mac[ETH_ALEN], bool clientFlag, bool is_usb)
{
    IPSTAT_IP_ENTRY *pHost = NULL;
    ipstat_debug();
    pHost = ipstat_get_entry(ipaddr, is_usb);
    if (pHost) {
            return pHost;
    }

    ipstat_debug();
    pHost = add_newEntry(ipaddr, mac, clientFlag, is_usb);
    return pHost;
}

IPSTAT_IP_ENTRY *get_firstLruHost(void)
{
    return g_ip_entry_table.hostLstHead.pNextLruHost;
}

int format_ip(char *buff, u_int32_t ip)
{
    int len;
	ip = ntohl(ip);
    len = sprintf(buff, "%d.%d.%d.%d", (u_int8_t)(ip >> 24), (u_int8_t)(ip >> 16), (u_int8_t)(ip >> 8), (u_int8_t)(ip));
   // len = sprintf(buff, "%d.%d.%d.%d", (u_int8_t)(ip), (u_int8_t)(ip >> 8), (u_int8_t)(ip >> 16), (u_int8_t)(ip >> 24));
    return len;
}

int format_mac(char *buff, u_int8_t  mac[ETH_ALEN])
{
    int len;
    len = sprintf(buff, ";%02X:%02X:%02X:%02X:%02X:%02X\n", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
    return len;
}


static struct file *open_ipstat_file(char* string, mm_segment_t* fs)
{
	struct file *filp = NULL;
	filp = filp_open(string, O_WRONLY | O_CREAT | O_TRUNC, 0644);
	if (IS_ERR(filp)) {
		return NULL;
	}
#if LINUX_VERSION_CODE < KERNEL_VERSION(5,10,0)
	*fs = get_fs();
	set_fs(KERNEL_DS);
#else
	*fs = force_uaccess_begin();
#endif
	return filp;
}

static int close_ipstat_file(struct file * filp, mm_segment_t* fs)
{
	if (filp == NULL || fs == NULL)
		return -EFAULT;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5,10,0)
	set_fs(*fs);
#else
	force_uaccess_end(*fs);
#endif
	filp_close(filp, NULL);
	return 0;
}

static int write_ipstat_file(struct file * filp, char* buf, int len)
{
	int ret = 0;
	if (filp == NULL || buf == NULL)
		return -EFAULT;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 14, 0)
	ret = kernel_write(filp, buf, len, &filp->f_pos);
#else
	ret = vfs_write(filp, (const char __user *)buf, len, &filp->f_pos);
#endif
	memset(buf, 0, len);
	return ret;
}

void ipstat_update_file(void)
{
	struct file *filp = NULL;
	IPSTAT_IP_ENTRY *pHost = NULL;
	const int MAX_LINE_LEN = 512;
	char cur[MAX_LINE_LEN];
	int len = 0, msg_len = 0;
	mm_segment_t fs;
	int ret = 0;
	int used = 0, i = 0;
	while ((!kthread_should_stop())){
		msleep(g_ipstat_interval * 1000);
		if ((1 != sfe_ipstat_switch) || (!g_ip_entry_table.mem_pool)) {
     			continue;
		}
		filp = open_ipstat_file(SFE_FILE_PATH, &fs);
		if (filp == NULL) {
	        	printk("%s %d: open file %s failed.\n", __func__, __LINE__, SFE_FILE_PATH);
			continue;
	        }
		ipstat_lock();
		used = g_ip_entry_table.used;
		ipstat_unlock();
		for(i = 0; i < used; i++)
		{
			ipstat_lock();
			pHost = &g_ip_entry_table.mem_pool[i];
			if (!pHost || ((pHost->ip & g_ip_subnet.mask) != (g_ip_subnet.ip & g_ip_subnet.mask))) 
			{
				ipstat_unlock();
				continue;
			}
			memset(cur, 0, sizeof(cur));
			msg_len = format_ip(cur, pHost->ip);
			len = snprintf(cur + msg_len, MAX_LINE_LEN, ";%llu;%llu;%llu;%llu;%s\n",
				pHost->cur_stat.src_packets,
				pHost->cur_stat.src_bytes,
				pHost->cur_stat.dst_packets,
				pHost->cur_stat.dst_bytes,
				pHost->devname);
			ipstat_unlock();
			ret = write_ipstat_file(filp, cur, strlen(cur));
			if (ret < 0) {
				printk("%s %d: write file %s failed: %d.\n", __func__, __LINE__, SFE_FILE_PATH, ret);
			}
		}
		ret = close_ipstat_file(filp, &fs);
		if (ret) {
			printk("%s %d: close file %s failed. ret = %d\n", __func__, __LINE__, SFE_FILE_PATH, ret);
		}
	}
}



static int ipstat_proc_show(struct seq_file *seq, void *v)
{
    const int MAX_LINE_LEN = 128;
    char line[MAX_LINE_LEN];
    char *cur = &line[0];
    int len;

    memset(line, 0, sizeof(line));
    cur = &line[0];
    ipstat_lock();
    len = format_ip(cur, g_ip_subnet.ip);
    cur += len;
    *cur = ' ';
    cur++;
    len = format_ip(cur, g_ip_subnet.mask);
    ipstat_unlock();
    cur += len;
    seq_puts(seq, line);
    return 0;
}

static int ipstat_step_proc_show(struct seq_file *seq, void *v)
{
    IPSTAT_IP_ENTRY *pHost = NULL;
    const int MAX_LINE_LEN = 512;
    char line[MAX_LINE_LEN];
    char *cur = &line[0];
    int len;
    u_int32_t denominator = (UPTIME_BITS + 10); // for KBps unit

    spin_lock_bh(&g_stat_lock);
    pHost = get_firstLruHost();
    while (pHost != NULL) {
        cur = &line[0];
        len = format_ip(cur, pHost->ip);
        cur += len;
        // calc KB/s and Pkt/s
        len = sprintf(cur, ";%llu;%llu;%llu;%llu",
                      ((pHost->cur_stat_update.src_packets - pHost->pre_stat_update.src_packets) >> UPTIME_BITS),
                      ((pHost->cur_stat_update.src_bytes - pHost->pre_stat_update.src_bytes) >> denominator),
                      ((pHost->cur_stat_update.dst_packets - pHost->pre_stat_update.dst_packets) >> UPTIME_BITS),
                      ((pHost->cur_stat_update.dst_bytes - pHost->pre_stat_update.dst_bytes) >> denominator)
                     );
        cur += len;
        len = sprintf(cur, ";%llu;%llu;%llu;%llu;\n",
                      pHost->cur_stat.src_packets,
                      pHost->cur_stat.src_bytes,
                      pHost->cur_stat.dst_packets,
                      pHost->cur_stat.dst_bytes);
	cur += len;
        seq_puts(seq, line);
        pHost = pHost->pNextLruHost;
    }
    spin_unlock_bh(&g_stat_lock);

    return 0;
}


static int ipmac_proc_show(struct seq_file *seq, void *v)
{
    IPSTAT_IP_ENTRY *pHost = NULL;
    const int MAX_LINE_LEN = 512;
    char line[MAX_LINE_LEN];
    char *cur = &line[0];
    int len;

    ipstat_debug();
    spin_lock_bh(&g_stat_lock);
    pHost = get_firstLruHost();
    while (pHost != NULL) {
        cur = &line[0];
        if (pHost->offline) {
            ipstat_debug("%x is offline",pHost->ip);
            pHost = pHost->pNextLruHost;
            continue ;
        }
        len = format_ip(cur, pHost->ip);
        cur += len;
        len = format_mac(cur, pHost->mac);
        cur += len;
        seq_puts(seq, line);
        pHost = pHost->pNextLruHost;
    }
    spin_unlock_bh(&g_stat_lock);
    ipstat_debug();

    return 0;
}

static int clientstat_proc_show(struct seq_file *seq, void *v)
{
    IPSTAT_IP_ENTRY *pHost = NULL;
    const int MAX_LINE_LEN = 512;
    char line[MAX_LINE_LEN];
    char *cur = &line[0];
    int len;

    spin_lock_bh(&g_stat_lock);
    pHost = get_firstLruHost();
    while(pHost != NULL)
    {
        if(pHost->clientFlag)
        {
            cur = &line[0];
            len = format_ip(cur, pHost->ip);
            cur += len;
            len = sprintf(cur, ";%llu;%llu;%llu;%llu\n",
                          pHost->cur_stat.src_packets,
                          pHost->cur_stat.src_bytes,
                          pHost->cur_stat.dst_packets,
                          pHost->cur_stat.dst_bytes);
            cur += len;
            seq_puts(seq, line);
         }
        pHost = pHost->pNextLruHost;
    }
    spin_unlock_bh(&g_stat_lock);

    return 0;
}


static int ipstat_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, ipstat_proc_show, inode->i_private);
}

static int ipmac_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, ipmac_proc_show, inode->i_private);
}

static int ipstat_step_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, ipstat_step_proc_show, inode->i_private);
}

static int clientstat_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, clientstat_proc_show, inode->i_private);
}
#ifdef SFE_CONNMARK_STAT
static int connmark_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, connmark_proc_show, inode->i_private);
}
#endif
//todo
static int ipstat_switch_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, ipstat_switch_proc_show, inode->i_private);
}

static int fragment_switch_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, fragment_switch_proc_show, inode->i_private);
}

static int ppp_tunnel_switch_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, ppp_tunnel_switch_proc_show, inode->i_private);
}

static int fastpath_nat_switch_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, fastpath_nat_switch_proc_show, inode->i_private);
}

static int qos_fastpath_nat_switch_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, fastpath_qos_nat_switch_proc_show, inode->i_private);
}

static int ipstat_max_proc_open(struct inode *inode, struct file *fp)
{
    return single_open(fp, ipstat_max_proc_show, inode->i_private);
}

#ifdef IPSTAT_SUPPORT_IPV6
static int parse_para(const char *input, char lanIp[MAX_IP_STR_LEN + 1], char lanMask[MAX_IP_STR_LEN + 1], char lan6Ip[MAX_IPV6_STR_LEN + 1], char lan6Mask[MAX_IPV6_STR_LEN + 1])
#else
static int parse_para(const char *input, char lanIp[MAX_IP_STR_LEN + 1], char lanMask[MAX_IP_STR_LEN + 1])
#endif
{
    char *space = NULL;
    int lanIpLen,lanMaskLen;
#ifdef IPSTAT_SUPPORT_IPV6
    int lan6IpLen,lan6MaskLen;
    struct sfe_ipv6_addr ipv6_addr;
    int prefixLen, prefixTol = 128;
    int i = 0, j = 0;
#endif
    char *p = input;
    space = strchr(p, ' ');
    if (NULL == space) {
        ipstat_debug("input para format error!");
        return -1;
    }

    lanIpLen = space - p;
    if (lanIpLen > MAX_IP_STR_LEN) {
        ipstat_debug("input para too long!");
        return -1;
    }
    strncpy(lanIp, input, lanIpLen);
    lanIp[lanIpLen] = '\0';

#ifdef IPSTAT_SUPPORT_IPV6    
    p = input + lanIpLen + 1;
    space = strchr(p, ' ');
    if (NULL == space) {
        ipstat_debug("input para format error!");
	lan6Ip = "0";
        lan6Mask = "0";
        lanMaskLen = strlen(p);
        strncpy(lanMask, p, lanMaskLen);
        lanMask[lanMaskLen] = '\0';
        return 0;
    }
    lanMaskLen = space - p;
    if (lanMaskLen > MAX_IP_STR_LEN) {
            ipstat_debug("input para too long!");
            return -1;
    }
    strncpy(lanMask, p, lanMaskLen);
    lanMask[lanMaskLen] = '\0';

    p = p + lanMaskLen + 1;
    space = strchr(p, ' ');
    if (NULL == space) {
        ipstat_debug("input para invalid!");
        return -1;
    }
    lan6IpLen = space - p;
    if (lan6IpLen > MAX_IPV6_STR_LEN) {
            ipstat_debug("input para too long!");
            return -1;
    }
    strncpy(lan6Ip, p, lan6IpLen);
    lan6Ip[lan6IpLen] = '\0';

    lan6MaskLen = strlen(space + 1);
    if (lan6MaskLen > MAX_IPV6_STR_LEN) {
            ipstat_debug("input para too long!");
            return -1;
    }
    strncpy(lan6Mask, space+1, lan6MaskLen);
    lan6Mask[lan6MaskLen] = '\0';
    prefixLen = simple_strtol(lan6Mask,NULL,0);
    if (prefixLen > 128 || prefixLen < 0)
    {
         ipstat_debug("Prefix len invalid:%s",lan6Mask);
            return -1;
    }
    for (i=0; i < MAX_IPV6_STR_LEN - 1;i++) //128 mask+ 7 ':'
    {  
        if(prefixLen >= 4){
            lan6Mask[i] = 'f';
            prefixLen -= 4;
        }else if(prefixLen == 0)
        {
            lan6Mask[i] = '0';
        } else if(prefixLen == 1)
        {
            lan6Mask[i] = '1';
            prefixLen = 0;
        }else if(prefixLen == 2)
        {
            lan6Mask[i] = '3';
            prefixLen = 0;
        }else{
            lan6Mask[i] = '7';
            prefixLen = 0;
        }
        if (((++j%4) == 0) && (++i < MAX_IPV6_STR_LEN - 1))
        {
            lan6Mask[i] = ':';
            j = 0;
        }
        
    }
    lan6Mask[MAX_IPV6_STR_LEN] = '\0';
#else
    lanMaskLen = strlen(space + 1);
    if (lanMaskLen > MAX_IP_STR_LEN) {
            ipstat_debug("input para too long!");
            return -1;
    }
    strncpy(lanMask, space+1, lanMaskLen);
    lanMask[lanMaskLen] = '\0';
#endif   
    return 0;
}

static int parse_ip(char *ipStr, u_int32_t ipData[4])
{
    int scanResult;
    int idx;

    scanResult = sscanf(ipStr, "%u.%u.%u.%u", &ipData[0], &ipData[1], &ipData[2], &ipData[3]);
    if (scanResult != 4) {
        return -1;
    }

    for (idx = 0; idx < 4; idx++) {
        if (ipData[idx] > 255) {
                return -1;
        }
    }

    return 0;
}

#ifdef IPSTAT_SUPPORT_IPV6
static int parse_ipv6(char *ipv6Str, __be32 *ipv6Data)
{
    int ret;
    ret = in6_pton(ipv6Str, strlen(ipv6Str), (void *)ipv6Data, -1, NULL);
    return (ret == 1)?0:-1;
}


static int update_config(u_int32_t addr[4], u_int32_t mask[4], struct sfe_ipv6_addr ipv6Data, __be32 v6_mask[4])
#else
static int update_config(u_int32_t addr[4], u_int32_t mask[4])
#endif
{
    int i ;
    ipstat_lock();
    g_ip_subnet.ip = htonl( (addr[0] << 24) | (addr[1] << 16) | (addr[2] << 8) | (addr[3]));
    g_ip_subnet.mask = htonl( (mask[0] << 24) | (mask[1] << 16) | (mask[2] << 8) | (mask[3]));
#ifdef IPSTAT_SUPPORT_IPV6
    for(i=0; i<4; i++)
    {
        g_ip_subnet.ipv6.addr[i] = ipv6Data.addr[i];
        g_ip_subnet.v6_mask[i] = v6_mask[i];
    }
#endif
    ipstat_unlock();
    printk(KERN_INFO"Update Subnet-> ip: %08X, mask: %08X\n", g_ip_subnet.ip, g_ip_subnet.mask);
    return 0;
}

static ssize_t ipstat_proc_write(struct file *fp, const char __user *buff, size_t len, loff_t *ppos)
{
    char lanIp[MAX_IP_STR_LEN + 1];
    char lanMask[MAX_IP_STR_LEN + 1];
    u_int32_t addr[4];
    u_int32_t mask[4];
    int ret, ret2;
#ifdef IPSTAT_SUPPORT_IPV6
    char lan6Ip[MAX_IPV6_STR_LEN + 1];
    char lan6Mask[MAX_IPV6_STR_LEN + 1];
    struct sfe_ipv6_addr ipv6Data;
    __be32 v6_mark[4];
#endif

    ipstat_debug();
#ifdef IPSTAT_SUPPORT_IPV6
    ret = parse_para(buff, lanIp, lanMask, lan6Ip, lan6Mask);
#else
    ret = parse_para(buff, lanIp, lanMask);
#endif
    if (ret != 0) {
        return -1;
    }

    ret = parse_ip(lanIp, addr);
    if (ret != 0) {
        printk("[stat]Input ip invalid!\n");
        return -1;
    }

    ret = parse_ip(lanMask, mask);
    if (ret != 0) {
        printk("[stat]Input mask invalid!\n");
        return -1;
    }
#ifdef IPSTAT_SUPPORT_IPV6
    ipstat_debug("lan6=%s",lan6Ip);
    ret = parse_ipv6(lan6Ip, ipv6Data.addr);
    ret2 = parse_ipv6(lan6Mask, v6_mark);
    if (ret != 0 || ret2 != 0) 
    {
        printk("[stat]Input ipv6 or ipv6 mask invalid!\n");
        update_config(addr, mask, g_ip_subnet.ipv6, g_ip_subnet.v6_mask);
    } else {
        update_config(addr, mask, ipv6Data, v6_mark);
    }
#else
    update_config(addr, mask);
#endif
    ipstat_debug();

    return len;
}

//todo
static ssize_t ipstat_switch_proc_write(struct file *fp, const char __user *buff, size_t len, loff_t *ppos)
{
    unsigned int new_value;
    if(len <= 0)
    {
        return len;
    }

    if(buff[0] == '0' || buff[0] == '1')
    {
        new_value = buff[0] - '0';
        ipstat_debug("new_value=%d",new_value);
        if(new_value != sfe_ipstat_switch)
        {
            sfe_ipstat_switch = new_value;
            sfe_ipstat_flush();
            printk("####ipstat_switch_proc_write####update sfe_ipstat_switch value to:%d\n",sfe_ipstat_switch);
        }
    }

    return len;
}

static ssize_t fragment_switch_proc_write(struct file *fp, const char __user *buff, size_t len, loff_t *ppos)
{
    unsigned int new_value;
    if(len <= 0)
    {
        return len;
    }

    if(buff[0] == '0' || buff[0] == '1')
    {
        new_value = buff[0] - '0';
        ipstat_debug("new_value=%d",new_value);
        if(new_value != fragment_switch)
        {
            fragment_switch = new_value;
            printk("####fragment_switch_proc_write####update fragment_switch value to:%d\n",fragment_switch);
        }
    }

    return len;
}

static ssize_t ppp_tunnel_switch_proc_write(struct file *fp, const char __user *buff, size_t len, loff_t *ppos)
{
    unsigned int new_value;
    if(len <= 0)
    {
        return len;
    }

    if(buff[0] == '0' || buff[0] == '1')
    {
        new_value = buff[0] - '0';
        ipstat_debug("new_value=%d",new_value);
        if(new_value != ppp_tunnel_switch)
        {
            ppp_tunnel_switch = new_value;
            printk("####ppp_tunnel_switch_proc_write####update ppp_tunnel_switch value to:%d\n",ppp_tunnel_switch);
        }
    }

    return len;
}

static ssize_t fastpath_nat_switch_proc_write(struct file *fp, const char __user *buff, size_t len, loff_t *ppos)
{
    unsigned int new_value;
    if(len <= 0)
    {
        return len;
    }

    if(buff[0] == '0' || buff[0] == '1')
    {
        new_value = buff[0] - '0';
        ipstat_debug("new_value=%d",new_value);
        if(new_value != fastpath_nat_enabled)
        {
            fastpath_nat_enabled = new_value;
            printk("####fastpath_nat_switch_proc_write####update fastpath_nat_enabled value to:%d\n",fastpath_nat_enabled);
        }
    }

    return len;
}

static ssize_t qos_fastpath_nat_switch_proc_write(struct file *fp, const char __user *buff, size_t len, loff_t *ppos)
{
    unsigned int new_value;
    if(len <= 0)
    {
        return len;
    }

    if(buff[0] == '0' || buff[0] == '1')
    {
        new_value = buff[0] - '0';
        ipstat_debug("new_value=%d",new_value);
        if(new_value != qos_fastpath_nat_enabled)
        {
            qos_fastpath_nat_enabled = new_value;
            printk("####fastpath_qos_nat_switch_proc_write####update qos_fastpath_nat_enabled value to:%d\n",qos_fastpath_nat_enabled);
        }
    }

    return len;
}

static ssize_t ipstat_max_proc_write(struct file *fp, const char __user *buff, size_t len, loff_t *ppos)
{
    unsigned long new_value = 0;
    if(len <= 0)
    {
    	    return len;
    }
    new_value = simple_strtoul(buff, NULL, 10);
    if (new_value > 0)
    {
        ipstat_lock();
        g_ipstat_max = new_value;
	printk("#### ipstat_max_proc_write ####:set max ipstat count to %d\n",g_ipstat_max);
	if(g_connmark_entry_table.mem_pool)
	{
		kfree(g_connmark_entry_table.mem_pool);
	}
	if(g_ip_entry_table.mem_pool)
	{
		kfree(g_ip_entry_table.mem_pool);
	}
	g_connmark_entry_table.mem_pool = (CONNMARK_ENTRY *)kzalloc(sizeof(CONNMARK_ENTRY) * g_ipstat_max, GFP_ATOMIC);
	g_ip_entry_table.mem_pool = (IPSTAT_IP_ENTRY *)kzalloc(sizeof(IPSTAT_IP_ENTRY) * g_ipstat_max, GFP_ATOMIC);
	if((!g_connmark_entry_table.mem_pool) || (!g_ip_entry_table.mem_pool))
	{
		printk("#### ipstat_max_proc_write ####: kmalloc failed.\n");
		if(g_connmark_entry_table.mem_pool)
		{
			kfree(g_connmark_entry_table.mem_pool);
		}
		if(g_ip_entry_table.mem_pool)
		{
			kfree(g_ip_entry_table.mem_pool);
		}
	}else{
		printk("#### ipstat_max_proc_write ####: kmalloc success.\n");
	}
	ipstat_unlock();
    }
    return len;
}

static const struct file_operations stat_file_op =
{
    .open    = ipstat_proc_open,
    .read    = seq_read,
    .write   = ipstat_proc_write,
    .llseek  = seq_lseek,
    .release = single_release,
};

static const struct file_operations ipmac_file_op =
{
    .open    = ipmac_proc_open,
    .read    = seq_read,
    .llseek  = seq_lseek,
    .release = single_release,
};

static const struct file_operations ipstat_step_file_op =
{
    .open    = ipstat_step_proc_open,
    .read    = seq_read,
    .llseek  = seq_lseek,
    .release = single_release,
};

static const struct file_operations clientstat_file_op =
{
    .open    = clientstat_proc_open,
    .read    = seq_read,
    .llseek  = seq_lseek,
    .release = single_release,
};
#ifdef SFE_CONNMARK_STAT
static const struct file_operations connmark_file_op =
{
    .open    = connmark_proc_open,
    .read    = seq_read,
    .llseek  = seq_lseek,
    .release = single_release,
};
#endif

//todo
static const struct file_operations ipstat_switch_file_op =
{
    .open    = ipstat_switch_proc_open,
    .read    = seq_read,
    .write   = ipstat_switch_proc_write,
    .llseek  = seq_lseek,
    .release = single_release,
};

static const struct file_operations fragment_switch_file_op =
{
    .open    = fragment_switch_proc_open,
    .read    = seq_read,
    .write   = fragment_switch_proc_write,
    .llseek  = seq_lseek,
    .release = single_release,
};

static const struct file_operations ppp_tunnel_switch_file_op =
{
    .open    = ppp_tunnel_switch_proc_open,
    .read    = seq_read,
    .write   = ppp_tunnel_switch_proc_write,
    .llseek  = seq_lseek,
    .release = single_release,
};

static const struct file_operations fastpath_nat_switch_file_op =
{
    .open    = fastpath_nat_switch_proc_open,
    .read    = seq_read,
    .write   = fastpath_nat_switch_proc_write,
    .llseek  = seq_lseek,
    .release = single_release,
};

static const struct file_operations qos_fastpath_nat_switch_file_op =
{
    .open    = qos_fastpath_nat_switch_proc_open,
    .read    = seq_read,
    .write   = qos_fastpath_nat_switch_proc_write,
    .llseek  = seq_lseek,
    .release = single_release,
};

static const struct file_operations ipstat_max_file_op =
{
    .open    = ipstat_max_proc_open,
    .read    = seq_read,
    .write   = ipstat_max_proc_write,
    .llseek  = seq_lseek,
    .release = single_release,
};

static int creat_proc_file (void)
{
    if ( sfe_ipstat_proc != NULL ) {
            return -ENOENT;
    }

    sfe_ipstat_proc = proc_create(SFE_IPSTAT_PROC_NAME, 0644, init_net.proc_net, &stat_file_op);
    if (!sfe_ipstat_proc) {
        printk("Create proc file %s failed!\n", SFE_IPSTAT_PROC_NAME);
        return -ENOENT;
    }

    if ( sfe_ipmac_proc != NULL ) {
        return -ENOENT;
    }
    sfe_ipmac_proc = proc_create(SFE_IPMAC_PROC_NAME, 0644, init_net.proc_net, &ipmac_file_op);
    if (!sfe_ipmac_proc) {
        printk("Create proc file %s failed!\n", SFE_IPMAC_PROC_NAME);
        return -ENOENT;
    }

    if ( sfe_ipstat_step_proc != NULL ) {
        return -ENOENT;
    }
    sfe_ipstat_step_proc = proc_create(SFE_IPSTAT_STEP_PROC_NAME, 0644, init_net.proc_net, &ipstat_step_file_op);
    if (!sfe_ipstat_step_proc) {
        printk("Create proc file %s failed!\n", SFE_IPSTAT_STEP_PROC_NAME);
        return -ENOENT;
    }

    if ( sfe_clientstat_proc != NULL ) {
            return -ENOENT;
    }
    sfe_clientstat_proc = proc_create(SFE_CLIENTSTAT_PROC_NAME, 0644, init_net.proc_net, &clientstat_file_op);
    if (!sfe_clientstat_proc) {
        printk("Create proc file %s failed!\n", SFE_CLIENTSTAT_PROC_NAME);
        return -ENOENT;
    }
#ifdef SFE_CONNMARK_STAT
    if ( sfe_connmark_proc != NULL ) {
            return -ENOENT;
    }
    sfe_connmark_proc = proc_create(SFE_CONNMARK_PROC_NAME, 0644, init_net.proc_net, &connmark_file_op);
    if (!sfe_connmark_proc) {
        printk("Create proc file %s failed!\n", SFE_CONNMARK_PROC_NAME);
        return -ENOENT;
    }
#endif
    //todo
    if (sfe_ipstat_switch_proc != NULL ) {
            return -ENOENT;
    }
    sfe_ipstat_switch_proc = proc_create(SFE_IPSTAT_SWITCH_NAME, 0666, init_net.proc_net, &ipstat_switch_file_op);
    if (!sfe_ipstat_switch_proc) {
        printk("Create proc file %s failed!\n", SFE_IPSTAT_SWITCH_NAME);
        return -ENOENT;
    }

    if (fragment_switch_proc != NULL ) {
            return -ENOENT;
    }
    fragment_switch_proc = proc_create(FRAGMENT_SWITCH_NAME, 0666, init_net.proc_net, &fragment_switch_file_op);
    if (!fragment_switch_proc) {
        printk("Create proc file %s failed!\n", FRAGMENT_SWITCH_NAME);
        return -ENOENT;
    }

    if (ppp_tunnel_switch_proc != NULL ) {
            return -ENOENT;
    }
    ppp_tunnel_switch_proc = proc_create(PPP_TUNNEL_SWITCH_NAME, 0666, init_net.proc_net, &ppp_tunnel_switch_file_op);
    if (!ppp_tunnel_switch_proc) {
        printk("Create proc file %s failed!\n", PPP_TUNNEL_SWITCH_NAME);
        return -ENOENT;
    }

    if (sfe_fastpath_nat_switch_proc != NULL ) {
            return -ENOENT;
    }
    sfe_fastpath_nat_switch_proc = proc_create(SFE_FASTPATH_NAT_SWITCH_NAME, 0666, init_net.proc_net, &fastpath_nat_switch_file_op);
    if (!sfe_fastpath_nat_switch_proc) {
        printk("Create proc file %s failed!\n", SFE_FASTPATH_NAT_SWITCH_NAME);
        return -ENOENT;
    }

    if (sfe_qos_fastpath_nat_switch_proc != NULL ) {
            return -ENOENT;
    }
    sfe_qos_fastpath_nat_switch_proc = proc_create(SFE_QOS_FASTPATH_NAT_SWITCH_NAME, 0666, init_net.proc_net, &qos_fastpath_nat_switch_file_op);
    if (!sfe_qos_fastpath_nat_switch_proc) {
        printk("Create proc file %s failed!\n", SFE_QOS_FASTPATH_NAT_SWITCH_NAME);
        return -ENOENT;
    }

    if (ipstat_max_proc != NULL ) {
            return -ENOENT;
    }
    ipstat_max_proc = proc_create(IPSTAT_MAX_NAME, 0666, init_net.proc_net, &ipstat_max_file_op);
    if (!ipstat_max_proc) {
        printk("Create proc file %s failed!\n", IPSTAT_MAX_NAME);
        return -ENOENT;
    }

    return 0;
}

static void remove_proc_file(void)
{
    if ( sfe_ipstat_proc != NULL ) {
        remove_proc_entry(SFE_IPSTAT_PROC_NAME, init_net.proc_net);
    }

    if ( sfe_ipmac_proc != NULL ) {
        remove_proc_entry(SFE_IPMAC_PROC_NAME, init_net.proc_net);
    }

    if ( sfe_ipstat_step_proc != NULL ) {
        remove_proc_entry(SFE_IPSTAT_STEP_PROC_NAME, init_net.proc_net);
    }

    if ( sfe_clientstat_proc != NULL ) {
        remove_proc_entry(SFE_CLIENTSTAT_PROC_NAME, init_net.proc_net);
    }
#ifdef SFE_CONNMARK_STAT
    if ( sfe_connmark_proc != NULL ) {
        remove_proc_entry(SFE_CONNMARK_PROC_NAME, init_net.proc_net);
    }
#endif
    //todo
    if ( sfe_ipstat_switch_proc != NULL ) {
        remove_proc_entry(SFE_IPSTAT_SWITCH_NAME, init_net.proc_net);
    }

    if ( fragment_switch_proc != NULL ) {
        remove_proc_entry(FRAGMENT_SWITCH_NAME, init_net.proc_net);
    }

    if ( ppp_tunnel_switch_proc != NULL ) {
        remove_proc_entry(PPP_TUNNEL_SWITCH_NAME, init_net.proc_net);
    }

    if ( sfe_fastpath_nat_switch_proc != NULL ) {
        remove_proc_entry(SFE_FASTPATH_NAT_SWITCH_NAME, init_net.proc_net);
    }

    if ( sfe_qos_fastpath_nat_switch_proc != NULL ) {
        remove_proc_entry(SFE_QOS_FASTPATH_NAT_SWITCH_NAME, init_net.proc_net);
    }

    if ( ipstat_max_proc != NULL ) {
        remove_proc_entry(IPSTAT_MAX_NAME, init_net.proc_net);
    }
}

void gc_thresh_entry(IPSTAT_IP_ENTRY *pHost)
{
    pHost->arp_timer = g_cur_timer_cycle;
    pHost->offline = 0;
}

void refresh_entry_mac(IPSTAT_IP_ENTRY *pHost, const u_int8_t mac[ETH_ALEN])
{
    if (mac[0] != 0 || mac[1] != 0 || mac[2] != 0 || mac[3] != 0 || mac[4] != 0 || mac[5] != 0) {
        memcpy(&pHost->mac, mac, ETH_ALEN);
    }
}

void ipstat_tx(IPSTAT_IP_ENTRY *pHost, u_int32_t protocol, u_int32_t len)
{

    pHost->cur_stat.src_bytes += len;
    pHost->cur_stat.src_packets += 1;
    gc_thresh_entry(pHost);
}

void ipstat_rx(IPSTAT_IP_ENTRY *pHost, u_int32_t protocol, u_int32_t len)
{
    pHost->cur_stat.dst_bytes += len;
    pHost->cur_stat.dst_packets += 1;
    gc_thresh_entry(pHost);
}

void check_offline_entry(unsigned long curTime, IPSTAT_IP_ENTRY *pHost)
{
    if (pHost->offline == 1) {
        return;
    }

    if (time_after(curTime, pHost->arp_timer + HZ*ARP_GC_THRESH_TIME)) {
        ipstat_debug("%x is offline",pHost->ip);
        pHost->offline = 1;
    }
}

void entry_timer_proc(unsigned long curTime)
{
    IPSTAT_IP_ENTRY *pHost = NULL;

    ipstat_lock();
    pHost = get_firstLruHost();
    while (pHost != NULL) {
        check_offline_entry(curTime, pHost);
        pHost = pHost->pNextLruHost;
    }
    ipstat_unlock();
}
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0))
void arp_timer_proc(unsigned long nul)
#else
void arp_timer_proc(struct timer_list *t)
#endif
{
    entry_timer_proc(g_cur_timer_cycle);
    g_cur_timer_cycle = jiffies + g_interval_time * HZ;
    mod_timer(&g_arp_timer, g_cur_timer_cycle);
}

void arp_timer_init(void)
{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0))
    init_timer(&g_arp_timer);
    g_arp_timer.data     = (unsigned long)0;
#else
    timer_setup(&g_arp_timer, NULL, 0);
#endif
    g_arp_timer.function = arp_timer_proc;
    arp_timer_proc(0);
}

void arp_timer_exit(void)
{
    del_timer(&g_arp_timer);
}
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0))
void update_timer_proc(unsigned long nul)
#else
void update_timer_proc(struct timer_list *t)
#endif
{
    IPSTAT_IP_ENTRY *pHost = NULL;
#ifdef SFE_CONNMARK_STAT 
    CONNMARK_ENTRY *cHost = NULL;
#endif
    ipstat_lock();
    pHost = get_firstLruHost();
#ifdef SFE_CONNMARK_STAT 
    cHost = get_connmark_firstLruHost();
#endif

    while(pHost != NULL)
    {
        memcpy(&(pHost->pre_stat_update), &(pHost->cur_stat_update), sizeof(pHost->cur_stat_update));
        memcpy(&(pHost->cur_stat_update), &(pHost->cur_stat), sizeof(pHost->cur_stat));
        pHost = pHost->pNextLruHost;
    }
#ifdef SFE_CONNMARK_STAT
    while(cHost != NULL)
    {
        memcpy(&(cHost->pre_stat_update), &(cHost->cur_stat_update), sizeof(cHost->cur_stat_update));
        memcpy(&(cHost->cur_stat_update), &(cHost->cur_stat), sizeof(cHost->cur_stat));
        cHost = cHost->pNextLruHost;
    }
#endif
    ipstat_unlock();

    g_cur_update_timer_cycle = jiffies + g_interval_update_time * HZ;
    mod_timer(&g_update_timer, g_cur_update_timer_cycle);
}


void update_timer_init(void)
{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0))
    init_timer(&g_update_timer);
    g_update_timer.data     = (unsigned long)0;
#else
    timer_setup(&g_update_timer, NULL, 0);
#endif
    g_update_timer.function = update_timer_proc;
    update_timer_proc(0);
}

void update_timer_exit(void)
{
    del_timer(&g_update_timer);
}


/**************************************************************************************************/
/*                                      PUBLIC_FUNCTIONS                                          */
/**************************************************************************************************/
int sfe_ipstat_init(void)
{
    g_ip_subnet.ip = 0xC0A80001;
    g_ip_subnet.mask = 0xFFFFFF00;
#ifdef IPSTAT_SUPPORT_IPV6
    g_ip_subnet.ipv6.addr[0] = 0x00;
    g_ip_subnet.ipv6.addr[1] = 0x0;
    g_ip_subnet.ipv6.addr[2] = 0x0;
    g_ip_subnet.ipv6.addr[3] = 0x0;
    g_ip_subnet.v6_mask[0] = 0x0;
    g_ip_subnet.v6_mask[1] = 0x0;
    g_ip_subnet.v6_mask[2] = 0x0;
    g_ip_subnet.v6_mask[3] = 0x0;
#endif
    creat_proc_file();
    ip_entry_init(1);
#ifdef SFE_CONNMARK_STAT
        connmark_init(1);
#endif
    arp_timer_init();
    update_timer_init();
    ipstat_kthread = kthread_run(ipstat_update_file, NULL, "sfe_ipstat_update"); 
    if (!ipstat_kthread) {
	printk("%s %d:kthread_run fail", __func__, __LINE__);
    }
    printk("sizeof(CONNMARK_ENTRY)=%d,sizeof(IPSTAT_IP_ENTRY)=%d\n",sizeof(CONNMARK_ENTRY),sizeof(IPSTAT_IP_ENTRY));
    return 0;
}

int sfe_ipstat_exit(void)
{
    remove_proc_file();
    arp_timer_exit();
    update_timer_exit();
    if (ipstat_kthread) {
        printk("%s %d:kthread_stop",__func__,__LINE__);
        kthread_stop(ipstat_kthread); //停止内核线程
        ipstat_kthread = NULL;
    }
    if(g_connmark_entry_table.mem_pool)
    {
        kfree(g_connmark_entry_table.mem_pool);
    }
    if(g_ip_entry_table.mem_pool)
    {
        kfree(g_ip_entry_table.mem_pool);
    }
    return 0;
}

int sfe_ipstat_flush(void)
{
    ipstat_lock();
    ip_entry_init(0);
#ifdef SFE_CONNMARK_STAT
    connmark_init(0);
#endif
    ipstat_unlock();
    return 0;
}
/**************************************************************************************************/
/*                                      GLOBAL_FUNCTIONS                                          */
/**************************************************************************************************/
void sfe_ipstat_update(u_int32_t ipaddr, u_int64_t src_bytes, u_int64_t src_pkts, u_int64_t dst_bytes, u_int64_t dst_pkts, bool is_usb)
{
    IPSTAT_IP_ENTRY *pHost = NULL;
    ipstat_debug();
    ipstat_lock();
    pHost = ipstat_get_entry(ipaddr, is_usb);
    if (pHost) {
        pHost->cur_stat.src_bytes += src_bytes;
        pHost->cur_stat.src_packets += src_pkts;
        pHost->cur_stat.dst_bytes += dst_bytes;
        pHost->cur_stat.dst_packets += dst_pkts;

        gc_thresh_entry(pHost);
    }
    ipstat_unlock();
}
#ifdef SFE_CONNMARK_STAT
void sfe_connmark_update(u_int32_t connmark, bool clientFlag, u_int64_t src_bytes, u_int64_t src_pkts, u_int64_t dst_bytes, u_int64_t dst_pkts)
{
    CONNMARK_ENTRY *pHost = NULL;

    ipstat_lock();
    pHost = connmark_get_entry(connmark, clientFlag);
    if (pHost) {
        pHost->cur_stat.src_bytes += src_bytes;
        pHost->cur_stat.src_packets += src_pkts;
        pHost->cur_stat.dst_bytes += dst_bytes;
        pHost->cur_stat.dst_packets += dst_pkts;

        gc_thresh_connmark_entry(pHost);
    }
    ipstat_unlock();
}
#endif

#ifdef TPLINK_SWITCH_DRIVER
extern int tplink_get_netdev_type(char *name);
#endif

void sfe_ipstat_in(const struct sfe_ipv4_stat_info *sisi, const struct sk_buff *skb)
{
    const struct iphdr *iph = ip_hdr(skb);
    u_int32_t len = ntohs(iph->tot_len) + ETHER_PAYLOAD_LEN;
    IPSTAT_IP_ENTRY *host = NULL;
    bool is_usb = false;
    bool no_mac = false;
    u_int8_t fake_mac[ETH_ALEN] = {0};
#ifdef SFE_CONNMARK_STAT
    void *host_conn = NULL;   //connmark entry
#endif
    struct net *this_net;
    if (1 != sfe_ipstat_switch) {
        return;
    }

    this_net = sisi->in ? dev_net(sisi->in) : (sisi->out ? dev_net(sisi->out) : dev_net(skb->dev));
    if (&init_net != this_net) {
        return;
    }

//    ipstat_debug();
    //if (unlikely(0 == g_ip_subnet.ip && 0 == g_ip_subnet.mask)) {
    //    ipstat_debug("No ip subnet!!!\n");
    //    goto next;
    //}

    ipstat_debug();
    if ((sisi->src_ip & g_ip_subnet.mask) == (g_ip_subnet.ip & g_ip_subnet.mask)) {
        ipstat_debug();

        if (unlikely(skb_mac_header(skb) < skb->head))
        {
            ipstat_debug("tx mac is not valid\n");
            goto next;
        }
        if((skb_mac_header(skb) + ETH_HLEN) > skb->data)
        {
            if(skb->dev && strncmp(skb->dev,"inf",3) && (skb_mac_header(skb) == skb->data))
            {
                ipstat_debug("This is vpn/ppp dev=%s, which has no mac header, continue\n",skb->dev->name);
                no_mac = true;
            }else{
                ipstat_debug("tx mac is not valid\n");
                goto next;
            }
        }
        ipstat_lock();
#ifndef TPLINK_SWITCH_DRIVER
	if(sisi->in && strstr(sisi->in->name,"br-lan"))
#else
	if(sisi->in && 0 == tplink_get_netdev_type(sisi->in->name))
#endif
        {
	    is_usb = (strcmp(sisi->in->name, "usb0") == 0 || strcmp(sisi->in->name, "usb1") == 0)?true:false;
	    if(!no_mac)
                host = find_ipentry(sisi->src_ip,eth_hdr(skb)->h_source, true, is_usb);
	    else
                host = find_ipentry(sisi->src_ip,fake_mac, true, is_usb);
#ifdef SFE_CONNMARK_STAT
	    host_conn = find_conn_entry(sisi->connmark,true);
#endif
        }
        else
        {
	    if(sisi->in)
	    {
	    	is_usb = (strcmp(sisi->in->name, "usb0") == 0 || strcmp(sisi->in->name, "usb1") == 0)?true:false;
	    }
	    if(!no_mac)
                host = find_ipentry(sisi->src_ip,eth_hdr(skb)->h_source, false, is_usb);
	    else
                host = find_ipentry(sisi->src_ip,fake_mac, false, is_usb);
#ifdef SFE_CONNMARK_STAT
	    host_conn = find_conn_entry(sisi->connmark,false);
#endif
        }
        if (unlikely((NULL == host) || (NULL == host_conn)))
		{
            ipstat_unlock();
            ipstat_debug("host can't match and add.\n");
            goto next;
        }
        if(!no_mac)
            refresh_entry_mac(host, eth_hdr(skb)->h_source);
		
        if(sisi->in)
        {
            memset(host->devname, 0, sizeof(host->devname));
    	    strncpy(host->devname, sisi->in->name, strlen(sisi->in->name));
        }
        ipstat_tx(host, sisi->protocol, len);
#ifdef SFE_CONNMARK_STAT
        connmark_tx(host_conn, sisi->connmark, len);
#endif
        ipstat_unlock();
    } 
    if ((sisi->dst_ip & g_ip_subnet.mask) == (g_ip_subnet.ip & g_ip_subnet.mask)) 
    {
        ipstat_debug();
        ipstat_lock();

        ipstat_debug();
#ifndef TPLINK_SWITCH_DRIVER
	if(sisi->out && strstr(sisi->out->name,"br-lan"))
#else	
	if(sisi->out &&  0 == tplink_get_netdev_type(sisi->out->name))
#endif
        {
            is_usb = (strcmp(sisi->out->name, "usb0") == 0 || strcmp(sisi->out->name, "usb1") == 0)?true:false;
	    host = find_ipentry(sisi->dst_ip, sisi->dst_mac, true, is_usb);
#ifdef SFE_CONNMARK_STAT
            host_conn = find_conn_entry(sisi->connmark, true);
#endif
        }
        else
        {
	    if(sisi->out)
	    {
	    	is_usb = (strcmp(sisi->out->name, "usb0") == 0 || strcmp(sisi->out->name, "usb1") == 0)?true:false;
	    }
	    
            host = find_ipentry(sisi->dst_ip, sisi->dst_mac, false, is_usb);
#ifdef SFE_CONNMARK_STAT
            host_conn = find_conn_entry(sisi->connmark, false);
#endif
        }
#ifdef SFE_CONNMARK_STAT
        if (unlikely((NULL == host)||(host_conn == NULL)))
#else
	if (unlikely(NULL == host))
#endif
        {
            ipstat_unlock();
            ipstat_debug("host can't match and add.\n");
            goto next;
        }

        ipstat_debug();
        refresh_entry_mac(host, sisi->dst_mac);
	if(sisi->out && strlen(host->devname) == 0)
	{
		strncpy(host->devname, sisi->out->name, strlen(sisi->out->name));
	}

	ipstat_rx(host, sisi->protocol, len);
#ifdef SFE_CONNMARK_STAT
	connmark_rx(host_conn, sisi->connmark, len);
#endif

        ipstat_unlock();
    }
    if (!host)
    {
        ipstat_debug("package doesn't have lan_dev or src/dst_ip != lan_ip\n");
    }

next:
    return;
}

#ifdef IPSTAT_SUPPORT_IPV6
void sfe_ipstat_in6(const struct sfe_ipv6_stat_info *sisi, const struct sk_buff *skb)
{
     struct ipv6hdr *ipv6h = ipv6_hdr(skb);
     u_int32_t len = ntohs(ipv6h->payload_len) + ETHER_PAYLOAD_LEN + IPV6_NET_LEN;
#ifdef SFE_CONNMARK_STAT
    void *host_conn = NULL;   //connmark entry
#endif
    struct net *this_net;

    if (1 != sfe_ipstat_switch) {
        return;
    }
    this_net = sisi->in ? dev_net(sisi->in) : (sisi->out ? dev_net(sisi->out) : dev_net(skb->dev));
    if (&init_net != this_net) {
        return;
    }
    if (sfe_ipv6_addr_mask_equal(sisi->src_ip, g_ip_subnet.v6_mask,g_ip_subnet.ipv6, g_ip_subnet.v6_mask) > 0) 
    {
        ipstat_lock();
        if (unlikely((skb_mac_header(skb) < skb->head) || ((skb_mac_header(skb) + ETH_HLEN) > skb->data)))
        {
            ipstat_unlock();
                        ipstat_debug("tx mac is not valid\n");
            goto next;
        }
#ifndef TPLINK_SWITCH_DRIVER
	if(sisi->in && strstr(sisi->in->name,"br-lan")) 
#else
	if(sisi->in && 0 == tplink_get_netdev_type(sisi->in->name))
#endif
        {
#ifdef SFE_CONNMARK_STAT
            host_conn = find_conn_entry(sisi->connmark,true);
#endif
        }
        else
        {
#ifdef SFE_CONNMARK_STAT
            host_conn = find_conn_entry(sisi->connmark,false);
#endif
        }
        if (unlikely(NULL == host_conn))
        {
            ipstat_unlock();
            ipstat_debug("host can't match and add.\n");
            goto next;
        }

#ifdef SFE_CONNMARK_STAT
                connmark_tx(host_conn, sisi->connmark, len);
#endif
        ipstat_unlock();
    }
    if (sfe_ipv6_addr_mask_equal(sisi->dst_ip , g_ip_subnet.v6_mask, g_ip_subnet.ipv6, g_ip_subnet.v6_mask) > 0)
    {
        ipstat_debug();
        ipstat_lock();
#ifndef TPLINK_SWITCH_DRIVER
	if(sisi->out && strstr(sisi->out->name,"br-lan"))	 
#else
	if(sisi->out && 0 == tplink_get_netdev_type(sisi->out->name))
#endif
	{
#ifdef SFE_CONNMARK_STAT
            host_conn = find_conn_entry(sisi->connmark, true);
#endif
        }
        else
        {
#ifdef SFE_CONNMARK_STAT
            host_conn = find_conn_entry(sisi->connmark, false);
#endif
        }
#ifdef SFE_CONNMARK_STAT
        if (unlikely(host_conn == NULL))
#endif
        {
            ipstat_unlock();
            ipstat_debug("host can't match and add.\n");
            goto next;
        }

#ifdef SFE_CONNMARK_STAT
        connmark_rx(host_conn, sisi->connmark, len);

#endif

        ipstat_unlock();
    }
next:
    return;
}
                                                                                              
#endif

EXPORT_SYMBOL(sfe_ipstat_in);
#ifdef IPSTAT_SUPPORT_IPV6
EXPORT_SYMBOL(sfe_ipstat_in6);
#endif
EXPORT_SYMBOL(sfe_ipstat_update);
#ifdef SFE_CONNMARK_STAT
EXPORT_SYMBOL(sfe_connmark_update);
#endif
EXPORT_SYMBOL(sfe_ipstat_flush);

EXPORT_SYMBOL(sfe_ipstat_switch);

EXPORT_SYMBOL(fragment_switch);
EXPORT_SYMBOL(ppp_tunnel_switch);
EXPORT_SYMBOL(fastpath_nat_enabled);
EXPORT_SYMBOL(qos_fastpath_nat_enabled);
