/**************************************************************************************************/
/*                                      CONFIGURATIONS                                            */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      INCLUDE_FILES                                             */
/**************************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <getopt.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include "tpcrypt.h"

/**************************************************************************************************/
/*                                      DEFINES                                                   */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      TYPES                                                     */
/**************************************************************************************************/
struct tpcrypt_cmd_opts {
    bool  encrypt;
    bool  decrypt;
    char* in;
    char* out;
};

/**************************************************************************************************/
/*                                      EXTERN_PROTOTYPES                                         */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      LOCAL_PROTOTYPES                                          */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      VARIABLES                                                 */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      LOCAL_FUNCTIONS                                           */
/**************************************************************************************************/
static int _tpcrypt_execCmd(char* cmd, ...)
{
	char buf[1024+128];
	int pid = 0, status = 0;
	char *argv[4] = {0};
    va_list vaList;

    va_start (vaList, cmd);
    vsprintf (buf, cmd, vaList);
    va_end (vaList);
	
    if ((pid = fork()) < 0)
    {
    	return -1;
    }
    else if (pid == 0)
	{               
		argv[0] = "sh";
		argv[1] = "-c";
		argv[2] = buf;
		argv[3] = 0;

		execve("/bin/sh", argv, NULL);
		exit(127);       
    }
		
   	/* wait for child process return */
	do 
	{
		if ( waitpid(pid, &status, 0) == -1 )
		{
			if ( errno != EINTR )
			{
	         	return -1;
			}
	   	}
		else
		{
	     	return 0;
		}
	} while(1);

	return 0;
}

/**************************************************************************************************/
/*                                      PUBLIC_FUNCTIONS                                          */
/**************************************************************************************************/

/**************************************************************************************************/
/*                                      GLOBAL_FUNCTIONS                                          */
/**************************************************************************************************/
int main(int argc,char *argv[])
{
	const char aes_key[] = "dec_for_userconfig";
	struct tpcrypt_cmd_opts opts = {0};
    struct option long_options[] = {
        {"encrypt", 0, NULL, 'e'},
        {"decrypt", 0, NULL, 'd'},
        {"in", 1, NULL, 'i'},
        {"out", 1, NULL, 'o'},
        {0, 0, 0, 0}
    };
    char *short_options = "i:o:ed";

	char *in = NULL;
	char *out = NULL;
	int ret = 0;
    int opt = 0;

    while (-1 != (opt = getopt_long(argc, argv, short_options, long_options, NULL))) {
        switch (opt) {
			case 'e':
				opts.encrypt = true;
				break;
				
			case 'd':
				opts.decrypt = true;
				break;

            case 'i':
				opts.in = optarg;
                break;
				
            case 'o':
				opts.out = optarg;
                break;

            default:
               break;
        }
    }

	if ((opts.in == NULL) || (opts.out == NULL))
	{
		tpcrypt_err("Missing arg(in or out missing)!\r\n");
		ret = -1;
		goto out;
	}

	if ((opts.decrypt == false) && (opts.encrypt == false))
	{
		tpcrypt_err("Missing arg(decrypt or encrypt missing)!\r\n");
		ret = -1;
		goto out;
	}

	if ((opts.decrypt == true) && (opts.encrypt == true))
	{
		tpcrypt_err("Duplicate arg(decrypt or encrypt duplicate)!\r\n");
		ret = -1;
		goto out;
	}

	_tpcrypt_execCmd("openssl enc -aes-128-cbc -%s -in %s -out %s -pass pass:%s",
		opts.decrypt ? "d":"e", opts.in, opts.out, aes_key);

out:
	return ret;
}

