/*
 *  TP-LINK TL-WVR1300G board support
 *
 *  Copyright (C) 2015-2016 chenqiang 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License version 2 as published
 *  by the Free Software Foundation.
 */

#include <linux/pci.h>
#include <linux/phy.h>
#include <linux/platform_device.h>
#include <linux/ath9k_platform.h>
#include <linux/ar8216_platform.h>

#include <asm/mach-ath79/ar71xx_regs.h>

#include "common.h"
#include "pci.h"
#include "dev-usb.h"
#include "dev-eth.h"
#include "dev-gpio-buttons.h"
#include "dev-leds-gpio.h"
#include "dev-m25p80.h"
#include "dev-spi.h"
#include "dev-wmac.h"
#include "machtypes.h"

#define TL_WVR1300G_GPIO_LED_SYSTEM     1
#define TL_WVR1300G_GPIO_BTN_RESET      2
#define TL_WVR1300G_GPIO_LED_ENABLE     14
#define WVR1300G_GPIO_MDC               8
#define WVR1300G_GPIO_MDIO              10


#define TL_WVR1300G_KEYS_POLL_INTERVAL          20      /* msecs */
#define TL_WVR1300G_KEYS_DEBOUNCE_INTERVAL      (3 * TL_WVR1300G_KEYS_POLL_INTERVAL)


static const char *tl_r4299gv2_part_probes[] = {
        "tp-link",
        NULL,
};

static struct flash_platform_data tl_r4299gv2_flash_data = {
        .part_probes    = tl_r4299gv2_part_probes,
};

static struct gpio_led tl_r4299gv2_leds_gpio[] __initdata = {
        {
                .name           = "SYS_LED",
                .gpio           = TL_WVR1300G_GPIO_LED_SYSTEM,
                .active_low     = 0,
        },
        {
                .name           = "LED_ENALBE",
                .gpio           = TL_WVR1300G_GPIO_LED_ENABLE,
                .active_low     = 0,
        },
};

static struct gpio_keys_button tl_r4299gv2_gpio_keys[] __initdata = {
        {
                .desc           = "reset",
                .type           = EV_KEY,
                .code           = KEY_RESTART,
                .debounce_interval = TL_WVR1300G_KEYS_DEBOUNCE_INTERVAL,
                .gpio           = TL_WVR1300G_GPIO_BTN_RESET,
                .active_low     = 1,
        }
};

/* GMAC0 of the AR8327 switch is connected to the QCA9558 SoC via SGMII */
static struct ar8327_pad_cfg r4299gv2_ar8327_pad0_cfg = {
        .mode = AR8327_PAD_MAC_SGMII,
        .sgmii_txclk_phase_sel = AR8327_SGMII_CLK_PHASE_RISE,
        .sgmii_rxclk_phase_sel = AR8327_SGMII_CLK_PHASE_FALL,
};


static struct ar8327_led_cfg r4299gv2_ar8327_led_cfg = {
        .led_ctrl0 = 0xCC35CC35,        
        .led_ctrl1 = 0xC335C335,        
        .led_ctrl2 = 0x08010801,
        .led_ctrl3 = 0x00F3CF00,
        .open_drain = false,            //driver mode, PWS_REG(Address:0x10, bit:24)
};

static struct ar8327_platform_data r4299gv2_ar8327_data = {
        .pad0_cfg = &r4299gv2_ar8327_pad0_cfg,
        .cpuport_cfg = {
                .force_link = 1,
                .speed = AR8327_PORT_SPEED_1000,
                .duplex = 1,
                .txpause = 1,
                .rxpause = 1,
        },

        .led_cfg = &r4299gv2_ar8327_led_cfg,
};

static struct mdio_board_info r4299gv2_mdio0_info[] = {
        {
                .bus_id = "ag71xx-mdio.0",
                .phy_addr = 0,
                .platform_data = &r4299gv2_ar8327_data,
        },
};

static void __init tl_r4299gv2_setup(void)
{
        printk(KERN_ERR"tl_r4299gv2_setup()\n");

        ath79_register_m25p80(&tl_r4299gv2_flash_data);
        printk(KERN_ERR"ath79_register_m25p80()\n");

        ath79_register_leds_gpio(-1, ARRAY_SIZE(tl_r4299gv2_leds_gpio),
                                 tl_r4299gv2_leds_gpio);
        ath79_register_gpio_keys_polled(-1, TL_WVR1300G_KEYS_POLL_INTERVAL,
                                         ARRAY_SIZE(tl_r4299gv2_gpio_keys),
                                         tl_r4299gv2_gpio_keys);

        ath79_gpio_output_select(WVR1300G_GPIO_MDC, QCA956X_GPIO_OUT_MUX_GE0_MDC);
        ath79_gpio_output_select(WVR1300G_GPIO_MDIO, QCA956X_GPIO_OUT_MUX_GE0_MDO);

        mdiobus_register_board_info(r4299gv2_mdio0_info,
                                    ARRAY_SIZE(r4299gv2_mdio0_info));
}

MIPS_MACHINE(ATH79_MACH_TL_R4299GV2, "TL-R4299GV2",
             "TP-LINK TL-R4299G 2.0", tl_r4299gv2_setup);
