/*
 * Atheros AP147 reference board support
 *
 * Copyright (c) 2013 The Linux Foundation. All rights reserved.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
#include <linux/platform_device.h>
#include <linux/ath9k_platform.h>
#include <linux/ar8216_platform.h>

#include <asm/mach-ath79/ar71xx_regs.h>
#include <asm/delay.h>

#include "common.h"
#include "dev-eth.h"
#include "dev-gpio-buttons.h"
#include "dev-leds-gpio.h"
#include "dev-m25p80.h"
#include "dev-spi.h"
#include "dev-usb.h"
#include "dev-wmac.h"
#include "machtypes.h"
#include "pci.h"

#define AP147_GPIO_LED_WPS		13
#define AP147_GPIO_LED_STATUS		13

#if 0
#define AP147_GPIO_LED_WLAN		12
#define AP147_GPIO_LED_WAN		4
#define AP147_GPIO_LED_LAN1		16
#define AP147_GPIO_LED_LAN2		15
#define AP147_GPIO_LED_LAN3		14
#define AP147_GPIO_LED_LAN4		11
#else
#define AP147_GPIO_LED_WLAN		14
#define AP147_GPIO_LED_WAN	    16	
#define AP147_GPIO_LED_LAN1		1
#define AP147_GPIO_LED_LAN2		2
#define AP147_GPIO_LED_LAN3		3
#define AP147_GPIO_LED_LAN4		4
#endif /* 0 */

#define AP147_GPIO_BTN_WPS		17

#define AP147_KEYS_POLL_INTERVAL	20	/* msecs */
#define AP147_KEYS_DEBOUNCE_INTERVAL	(3 * AP147_KEYS_POLL_INTERVAL)

#define AP147_MAC0_OFFSET		0
#define AP147_MAC1_OFFSET		6
#define AP147_WMAC_CALDATA_OFFSET	0x1000

#define TL_R473P_AC_GPIO_BTN_RESET	       13
#define TL_R473P_AC_GPIO_LED_WLAN		    14
#define TL_R473P_AC_GPIO_LED_SYSTEM	       12
#define TL_R473P_AC_KEYS_POLL_INTERVAL		 20	/* msecs */
#define TL_R473P_AC_KEYS_DEBOUNCE_INTERVAL (3 * TL_R473P_AC_KEYS_POLL_INTERVAL)
#define TL_R473P_AC_GPIO_LED_FIXED_WAN     15
#define TL_R473P_AC_GPIO_LED_FIXED_LAN     16

static const char *tl_r473p_ac_part_probes[] = {
	"tp-link",
	NULL,
};

static struct flash_platform_data tl_r473p_ac_flash_data = {
	.part_probes	= tl_r473p_ac_part_probes,
};

static struct gpio_led tl_r473p_ac_leds_gpio[] __initdata = {
	{
		.name		= "SYS_LED",
		.gpio		= TL_R473P_AC_GPIO_LED_SYSTEM,
		.active_low	= 0,
	}
};

static struct gpio_keys_button tl_r473p_ac_gpio_keys[] __initdata = {
	{
		.desc		= "reset",
		.type		= EV_KEY,
		.code		= KEY_RESTART,
		.debounce_interval = TL_R473P_AC_KEYS_DEBOUNCE_INTERVAL,
		.gpio		= TL_R473P_AC_GPIO_BTN_RESET,
		.active_low	= 1,
	}
};

void self_delay(void)
{
    int i = 0x7fffffff;
    while (i > 0)
        i --;
}

void gpio_set_value(unsigned gpio, int value);
static void __init tl_r473p_ac_setup(void)
{

	ath79_register_m25p80(&tl_r473p_ac_flash_data);
   
   ath79_register_leds_gpio(-1, ARRAY_SIZE(tl_r473p_ac_leds_gpio),
				 tl_r473p_ac_leds_gpio);
	ath79_register_gpio_keys_polled(-1, TL_R473P_AC_KEYS_POLL_INTERVAL,
					 ARRAY_SIZE(tl_r473p_ac_gpio_keys),
					 tl_r473p_ac_gpio_keys);
}

MIPS_MACHINE(ATH79_MACH_TL_R473P_ACV1, "TL-R473P-ACV1", "TP-LINK TL-R473P-AC V1",
	     tl_r473p_ac_setup);
