/*
 * Atheros AP147 reference board support
 *
 * Copyright (c) 2013 The Linux Foundation. All rights reserved.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
#include <linux/platform_device.h>
#include <linux/ath9k_platform.h>
#include <linux/ar8216_platform.h>
#include <linux/mdio-gpio.h>

#include <asm/mach-ath79/ar71xx_regs.h>
#include <asm/delay.h>

#include "common.h"
#include "dev-eth.h"
#include "dev-gpio-buttons.h"
#include "dev-leds-gpio.h"
#include "dev-m25p80.h"
#include "dev-spi.h"
#include "dev-usb.h"
#include "dev-wmac.h"
#include "machtypes.h"
#include "pci.h"

#define TL_R479P_AC_GPIO_MDC	       3
#define TL_R479P_AC_GPIO_MDIO	       2

#define TL_R479P_AC_GPIO_BTN_RESET	       13
#define TL_R479P_AC_GPIO_LED_SYSTEM	       12
#define TL_R479P_AC_KEYS_POLL_INTERVAL	   20	/* msecs */
#define TL_R479P_AC_KEYS_DEBOUNCE_INTERVAL (3 * TL_R479P_AC_KEYS_POLL_INTERVAL)

static const char *tl_r479p_ac_part_probes[] = {
	"tp-link",
	NULL,
};

static struct flash_platform_data tl_r479p_ac_flash_data = {
	.part_probes	= tl_r479p_ac_part_probes,
};

static struct gpio_led tl_r479p_ac_leds_gpio[] __initdata = {
	{
		.name		= "SYS_LED",
		.gpio		= TL_R479P_AC_GPIO_LED_SYSTEM,
		.active_low	= 0,
	}
};

static struct gpio_keys_button tl_r479p_ac_gpio_keys[] __initdata = {
	{
		.desc		= "reset",
		.type		= EV_KEY,
		.code		= KEY_RESTART,
		.debounce_interval = TL_R479P_AC_KEYS_DEBOUNCE_INTERVAL,
		.gpio		= TL_R479P_AC_GPIO_BTN_RESET,
		.active_low	= 1,
	}
};

struct mdio_gpio_platform_data mdio_platform_data = {
    .mdc = TL_R479P_AC_GPIO_MDC,
    .mdio = TL_R479P_AC_GPIO_MDIO,
    .phy_mask = ~0,
    .irqs = {},
    .reset = NULL,
};

struct platform_device rtl8309n_mdio_device = {
        .name           = "rtl8309n-mdio",
        .id             = 0,
        .resource       = NULL,
        .num_resources  = 0,
        .dev = {
                .platform_data = &mdio_platform_data,
        },
};


static void __init tl_r479p_ac_setup(void)
{
    int ret = -1;
    ath79_register_m25p80(&tl_r479p_ac_flash_data);
    ath79_register_leds_gpio(-1, ARRAY_SIZE(tl_r479p_ac_leds_gpio), tl_r479p_ac_leds_gpio);
    ath79_register_gpio_keys_polled(-1, TL_R479P_AC_KEYS_POLL_INTERVAL, ARRAY_SIZE(tl_r479p_ac_gpio_keys), tl_r479p_ac_gpio_keys);

    ret = platform_device_register(&rtl8309n_mdio_device);
    printk("@@@@@@@@@@@@@@@@ platform_device_register(rtl8309n_mdio_device) ext_mdio, ret = %d\n", ret);
    return;
}

MIPS_MACHINE(ATH79_MACH_TL_R479P_ACV1, "TL-R479P-ACV1", "TP-LINK TL-R479P-AC V1", tl_r479p_ac_setup);
