/*
 * Atheros AP147 reference board support
 *
 * Copyright (c) 2013 The Linux Foundation. All rights reserved.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <linux/gpio.h>
#include <linux/spinlock.h>
#include <linux/platform_device.h>
#include <linux/ath9k_platform.h>
#include <linux/ar8216_platform.h>

#include <asm/mach-ath79/ar71xx_regs.h>
#include <asm/delay.h>

#include "common.h"
#include "dev-eth.h"
#include "dev-gpio-buttons.h"
#include "dev-leds-gpio.h"
#include "dev-m25p80.h"
#include "dev-spi.h"
#include "dev-usb.h"
#include "dev-wmac.h"
#include "machtypes.h"
#include "pci.h"

#define AP147_GPIO_LED_STATUS           13

#define AP147_GPIO_LED_PORT1            18
#define AP147_GPIO_LED_PORT2            15
#define AP147_GPIO_LED_PORT3            14
#define AP147_GPIO_LED_PORT4            22
#define AP147_GPIO_LED_PORT5            21

#define QCA953X_GPIO_OUT_MUX_LED_PHY0   16
#define QCA953X_GPIO_OUT_MUX_LED_PHY1   17
#define QCA953X_GPIO_OUT_MUX_LED_PHY2   34
#define QCA953X_GPIO_OUT_MUX_LED_PHY3   35
#define QCA953X_GPIO_OUT_MUX_LED_PHY4   36

#define AP147_KEYS_POLL_INTERVAL        20      /* msecs */
#define AP147_KEYS_DEBOUNCE_INTERVAL    (3 * AP147_KEYS_POLL_INTERVAL)

#define AP147_MAC0_OFFSET               0
#define AP147_MAC1_OFFSET               6
#define AP147_WMAC_CALDATA_OFFSET       0x1000

#define TL_WVR450V1_GPIO_BTN_RESET      2
#define TL_WVR450V1_GPIO_LED_SYSTEM     1
#define TL_WVR450V1_KEYS_POLL_INTERVAL  20      /* msecs */
#define TL_WVR450V1_KEYS_DEBOUNCE_INTERVAL  (3 * TL_WVR450V1_KEYS_POLL_INTERVAL)

static const char *tl_wvr450v1_part_probes[] = {
        "tp-link",
        NULL,
};

static struct flash_platform_data tl_wvr450v1_flash_data = {
        .part_probes    = tl_wvr450v1_part_probes,
};

static struct gpio_led tl_wvr450v1_leds_gpio[] __initdata = {
        {
                .name           = "SYS_LED",
                .gpio           = TL_WVR450V1_GPIO_LED_SYSTEM,
                .active_low     = 0,
        },
};

static struct gpio_keys_button tl_wvr450v1_gpio_keys[] __initdata = {
        {
                .desc           = "reset",
                .type           = EV_KEY,
                .code           = KEY_RESTART,
                .debounce_interval = TL_WVR450V1_KEYS_DEBOUNCE_INTERVAL,
                .gpio           = TL_WVR450V1_GPIO_BTN_RESET,
                .active_low     = 1,
        }
};

static void __init tl_wvr450v1_setup(void)
{
#ifdef CONFIG_AG71XX_LAN_WAN_PHY_SWAP
    void __iomem *base = NULL;    
#endif
    u8 *art = (u8 *) KSEG1ADDR(0x1fff0000);

    ath79_register_m25p80(&tl_wvr450v1_flash_data);

    /* LED blink & setup is done at u-boot  */

#if 0
    ath79_gpio_direction_select(AP147_GPIO_LED_PORT1, true);
    ath79_gpio_direction_select(AP147_GPIO_LED_PORT2, true);
    ath79_gpio_direction_select(AP147_GPIO_LED_PORT3, true);
    ath79_gpio_direction_select(AP147_GPIO_LED_PORT4, true);
    ath79_gpio_direction_select(AP147_GPIO_LED_PORT5, true);

    ath79_gpio_output_select(AP147_GPIO_LED_PORT1,
            QCA953X_GPIO_OUT_MUX_LED_PHY0);
    ath79_gpio_output_select(AP147_GPIO_LED_PORT2,
            QCA953X_GPIO_OUT_MUX_LED_PHY1);
    ath79_gpio_output_select(AP147_GPIO_LED_PORT3,
            QCA953X_GPIO_OUT_MUX_LED_PHY2);
    ath79_gpio_output_select(AP147_GPIO_LED_PORT4,
            QCA953X_GPIO_OUT_MUX_LED_PHY3);
    ath79_gpio_output_select(AP147_GPIO_LED_PORT5,
            QCA953X_GPIO_OUT_MUX_LED_PHY4);
#endif

    ath79_register_leds_gpio(-1, ARRAY_SIZE(tl_wvr450v1_leds_gpio),
                                 tl_wvr450v1_leds_gpio);
    ath79_register_gpio_keys_polled(-1, TL_WVR450V1_KEYS_DEBOUNCE_INTERVAL,
                                         ARRAY_SIZE(tl_wvr450v1_gpio_keys),
                                         tl_wvr450v1_gpio_keys);

    ath79_register_wmac(art + AP147_WMAC_CALDATA_OFFSET, NULL);

#ifdef CONFIG_AG71XX_LAN_WAN_PHY_SWAP
    base = ioremap(AR71XX_PLL_BASE, AR71XX_PLL_SIZE);       
    if (base) {             
            __raw_writel(0x3100,base + 0x20000);
    }       
    iounmap(base);
#endif
}

MIPS_MACHINE(ATH79_MACH_TL_WVR450V1, "TL-WVR450V1", "TP-LINK TL-WVR450 V1",
             tl_wvr450v1_setup);
