#include <linux/bitops.h>
#include <linux/phy.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/gpio.h>


#define PMA_CTRL_REG_MMD          1
#define PMA_CTRL_REG_ADDR         0x0
#define PWR_DOWN_BIT                  11
#define SPD_SELECT_LOW_BIT          13
#define SPD_SELECT_HIGH_BIT 6
#define AN_CTRL_REG_MMD            7
#define AN_CTRL_REG_ADDR           0x0
#define AN_EN_BIT                           12
#define RESTART_AN_BIT                  9
#define AN_AD_REG_MMD               7
#define AN_AD_REG_ADDR              0x10
#define PAUSE_EN_BIT              10
#define DUPLEX_CTRL_REG_MMD           31
#define DUPLEX_CTRL_REG_ADDR    0x0a40
#define DUPLEX_SELECT_BIT             8
#define setbit(x,y)  x|=(1<<y)
#define clrbit(x,y)  x&=~(1<<y)

int debug_level = 5;

#define SW_DBG_NONE      0
#define SW_DBG_ERROR     1
#define SW_DBG_NOTICE    2
#define SW_DBG_WARNING   3
#define SW_DBG_INFO      4
#define SW_DBG_VERBOSE   5
							   
#define SW_DBG(debug, fmt, ...)  \
							   do{	\
								   if (debug <= debug_level) {printk("SW_DBG %s(%d): " fmt "\n", __func__, __LINE__, ##__VA_ARGS__);}  \
							   }while(0)
								 

typedef struct _RTL8221B_PHY
{
	/* data */
	int rtl8221b_led[2];
	int reset_gpio;
}RTL8221B_PHY;

RTL8221B_PHY rtl8221b[]=
{
	{
		.rtl8221b_led={1,2},//1000M~LED1,2.5G~LED2
		.reset_gpio=43,
	},
};

int  MAX_RTL8221B = 1;

static int  phy_index = 0;
static struct phy_device *phydevs[10];


int rtl8221b_set_auto(struct phy_device *phydev)
{
	//支持10,100M
	int reg_val = 0;
			reg_val = phy_read_mmd(phydev, 7, 0x10);
			setbit(reg_val, 5);
			setbit(reg_val, 6);
			setbit(reg_val, 7);
			setbit(reg_val, 8);
			phy_write_mmd(phydev, 7, 0x10,reg_val);
			//开启1000M
			phy_write(phydev, 31, 0xa41);
			reg_val = phy_read(phydev, 17);
			printk("0x%x\n", reg_val);
			setbit(reg_val, 9);
			printk("0x%x\n", reg_val);
			phy_write(phydev, 17, reg_val);
			//支持2.5G
			reg_val = phy_read_mmd(phydev, 7, 0x20);
			setbit(reg_val,7);
			phy_write_mmd(phydev, 7, 0x20,reg_val);
			//重启AN
			phy_write_mmd(phydev, 7, 0x0, 0x3200);
}

int rtl8221b_get_link(void)
{
	int reg_val; 
	int link_state;
	struct phy_device *phydev;
	
	phydev = phydevs[0];
	phy_write(phydev, 31, 0xa43);
	reg_val = phy_read(phydev, 18);
	reg_val &= 0x4;
	link_state = reg_val >> 2;

	return link_state;
}
EXPORT_SYMBOL(rtl8221b_get_link);

int rtl8221b_dbg(void)
{
	int reg_val = 0;
	struct phy_device *phydev;
	phydev = phydevs[0];
	reg_val = phy_read_mmd(phydev, 7, 0x0);
	printk("8221b mmd7 0x0 0x%x\n", reg_val);
	reg_val = phy_read_mmd(phydev, 7, 0x1);
	printk("8221b mmd7 0x0 0x%x\n", reg_val);
	reg_val = phy_read_mmd(phydev, 7, 0x10);
	printk("8221b mmd7 0x0 0x%x\n", reg_val);
	reg_val = phy_read_mmd(phydev, 7, 0x13);
	printk("8221b mmd7 0x0 0x%x\n", reg_val);
	phy_write(phydev, 31, 0xa41);
	reg_val = phy_read(phydev, 17);
	printk("8221b mmd31 0xa412 0x%x\n", reg_val);
}

EXPORT_SYMBOL(rtl8221b_dbg);

int rtl8221b_init(struct phy_device *phydev)
{
	int reg_val;
	
	reg_val= phy_read(phydev, 0x2);
	printk("8221b oui is 0x%x\n", reg_val);
	reg_val= phy_read(phydev, 0x3);
	printk("8221b oui is 0x%x\n", reg_val);
	reg_val = phy_read_mmd(phydev, 30, 0x75f3);
	reg_val &= 0xfffe;
	phy_write_mmd(phydev, 30, 0x75f3, reg_val);
	reg_val = phy_read_mmd(phydev, 30, 0x75f3);
	SW_DBG(SW_DBG_INFO, "rtl8221b 0x75f3:%x\n", reg_val);
	
	reg_val = phy_read_mmd(phydev, 30, 0x697a);
	SW_DBG(SW_DBG_INFO, "rtl8221b 0x697a:%x\n", reg_val);
	phy_write_mmd(phydev, 30, 0x697a, 0x2);
	reg_val = phy_read_mmd(phydev, 30, 0x697a);
	SW_DBG(SW_DBG_INFO, "rtl8221b 0x697a:%x\n", reg_val);
	
	reg_val = phy_read_mmd(phydev, 30, 0x6a04);
	SW_DBG(SW_DBG_INFO, "rtl8221b 0x6a04:%x\n", reg_val);
	phy_write_mmd(phydev, 30, 0x6a04, 0x0503);
	reg_val = phy_read_mmd(phydev, 30, 0x6a04);
	SW_DBG(SW_DBG_INFO, "rtl8221b 0x6a04:%x\n", reg_val);
	
	reg_val = phy_read_mmd(phydev, 30, 0x6f10);
	SW_DBG(SW_DBG_INFO, "rtl8221b 0x6f10:%x\n", reg_val);
	phy_write_mmd(phydev, 30, 0x6f10, 0xd455);
	reg_val = phy_read_mmd(phydev, 30, 0x6f10);
	SW_DBG(SW_DBG_INFO, "rtl8221b 0x6f10:%x\n", reg_val);
	
	reg_val = phy_read_mmd(phydev, 30, 0x6f11);
	SW_DBG(SW_DBG_INFO, "rtl8221b 0x6f11:%x\n", reg_val);
	phy_write_mmd(phydev, 30, 0x6f11, 0x8020);
	reg_val = phy_read_mmd(phydev, 30, 0x6f11);
	SW_DBG(SW_DBG_INFO, "rtl8221b 0x6f11:%x\n", reg_val);

	//10~1000M LED配置
	phy_write(phydev, 31, 0x0d03);
	phy_write(phydev, 17+rtl8221b[phy_index].rtl8221b_led[0], 0x87);
	//2.5G LED配置
	phy_write(phydev, 31, 0x0d03);
	phy_write(phydev, 17+rtl8221b[phy_index].rtl8221b_led[1], 0x20);
	
	rtl8221b_set_auto(phydev);
	
	
	return 0;

}

int rtl8221b_probe(struct phy_device *phydev)
{
	int i=0;
	int flag=0;
	phydevs[phy_index] = phydev;
	char *name = "reset_gpio";
	
	SW_DBG(SW_DBG_INFO, "rtl8221b%d_rst\n",phy_index);
	for ( i = 0; i < phy_index; i++)
	{
		if(rtl8221b[phy_index].reset_gpio == rtl8221b[i].reset_gpio)
		{
			flag = 1;//reset_gpio is shared with other rtl8221b
		}
	}
	if(flag == 0 && rtl8221b[phy_index].reset_gpio != -1)
	{
		gpio_request(rtl8221b[phy_index].reset_gpio, name);
		gpio_direction_output(rtl8221b[phy_index].reset_gpio, 0);
		gpio_set_value(rtl8221b[phy_index].reset_gpio, 1);
		gpio_free(rtl8221b[phy_index].reset_gpio);
		mdelay(500);
	}

	rtl8221b_init(phydev);

	phy_index++;
	
	return 0;
}

struct phy_driver tplink_phy = {
		.phy_id		= 0x001cc849,
		.name		= "RTL8221B",
		.phy_id_mask	= 0x001fffff,
		.probe	= &rtl8221b_probe,
};

static int __init rtl8221b_module_init(void)
{
	return phy_driver_register(&tplink_phy,THIS_MODULE);
}
module_init(rtl8221b_module_init);


static void __exit rtl8221b_module_exit(void)
{
	phy_driver_unregister(&tplink_phy);
}
module_exit(rtl8221b_module_exit);
