#include "rtl8367sc_api.h"
#include <linux/switch.h>
#include <string.h>
#include <linux/module.h>
#include <linux/spinlock.h>

static rtk_switch_halCtrl_t halCtrl =
{
    /* Switch Chip */
    CHIP_RTL8367D,

    /* Logical to Physical */
    {0, 1, 2, 3, 4, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
     6, 7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF },

    /* Physical to Logical */
    {UTP_PORT0, UTP_PORT1, UTP_PORT2, UTP_PORT3, UTP_PORT4, UNDEFINE_PORT, EXT_PORT0, EXT_PORT1,
     UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT,
     UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT,
     UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT, UNDEFINE_PORT},

    /* Port Type */
    {UTP_PORT, UTP_PORT, UTP_PORT, UTP_PORT, UTP_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT,
     UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT,
     EXT_PORT, EXT_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT,
     UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT, UNKNOWN_PORT},

    /* PTP port */
    {0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0 },

    /* Valid port mask */
    ( (0x1 << UTP_PORT0) | (0x1 << UTP_PORT1) | (0x1 << UTP_PORT2) | (0x1 << UTP_PORT3) | (0x1 << UTP_PORT4) | (0x1 << EXT_PORT0) | (0x1 << EXT_PORT1) ),

    /* Valid UTP port mask */
    ( (0x1 << UTP_PORT0) | (0x1 << UTP_PORT1) | (0x1 << UTP_PORT2) | (0x1 << UTP_PORT3) | (0x1 << UTP_PORT4) ),

    /* Valid EXT port mask */
    ( (0x1 << EXT_PORT0) | (0x1 << EXT_PORT1) ),

    /* Valid CPU port mask */
    0x00,

    /* Minimum physical port number */
    0,

    /* Maxmum physical port number */
    7,

    /* Physical port mask */
    0xFF,

    /* Combo Logical port ID */
    4,

    /* HSG Logical portmask */
    ((0x1 << EXT_PORT0) |  (0x1 << EXT_PORT1)),

    /* SGMII Logical portmask */
    ((0x1 << EXT_PORT0) |  (0x1 << EXT_PORT1)),

    /* Max Meter ID */
    39,

    /* MAX LUT Address Number */
    2056,

    /* Trunk Group Mask */
    0x03,

    /* Packet buffer page number */
    2048
};
ret_t rtk_switch_setPHYReg(rtk_uint32 phyNo, rtk_uint32 phyAddr, rtk_uint32 phyData)
{
    return dal_rtl8367d_port_phyReg_set(phyNo, phyAddr, phyData);
}

ret_t rtk_switch_setReg(rtk_uint32 reg, rtk_uint32 value)
{
    return rtl8367d_setAsicReg(reg, value);
}

ret_t rtk_switch_getReg(rtk_uint32 reg, rtk_uint32 *pValue)
{
    return rtl8367d_getAsicReg(reg, pValue);
}

ret_t rtk_switch_getPHYReg(rtk_uint32 phyNo, rtk_uint32 phyAddr, rtk_uint32 *pRegData)
{
    return dal_rtl8367d_port_phyReg_get(phyNo, phyAddr, pRegData);
}

int rtk_acl_init(void)
{
	rtk_filter_igrAcl_init();
	rtk_filter_igrAcl_state_set(EXT_PORT1, DISABLED);
	return 1;
}

int rtk_acl_cfg_start(void **handl)
{
	rtk_filter_cfg_t    *cfg;

	cfg = kmalloc(sizeof(*cfg), GFP_KERNEL);
	if (!cfg)
		return -1;

	memset(cfg, 0x00, sizeof(rtk_filter_cfg_t));
	*handl = cfg;
	return 0;
}

int rtk_acl_cfg_add_snet(void *handl, unsigned int ipaddr, unsigned int ipmask)
{
	rtk_filter_cfg_t    *cfg = handl;
	rtk_filter_field_t  *field;
	rtk_api_ret_t ret_val = 0;

	field = kmalloc(sizeof(*field), GFP_KERNEL);
	if (!field)
		return -1;
	memset(field, 0x00, sizeof(rtk_filter_field_t));
	field->fieldType = FILTER_FIELD_IPV4_SIP;
	field->filter_pattern_union.inData.dataType = FILTER_FIELD_DATA_MASK;
	field->filter_pattern_union.inData.value = ipaddr;
	field->filter_pattern_union.inData.mask = ipmask;

	ret_val = rtk_filter_igrAcl_field_add(cfg, field);
	if (ret_val != RT_ERR_OK) {
		return -1;
	}
	return 0;
}

int rtk_acl_cfg_add_dnet(void *handl, unsigned int ipaddr, unsigned int ipmask)
{
	rtk_filter_cfg_t    *cfg = handl;
	rtk_filter_field_t  *field;
	rtk_api_ret_t ret_val = 0;

	field = kmalloc(sizeof(*field), GFP_KERNEL);
	if (!field)
		return -1;
	memset(field, 0x00, sizeof(rtk_filter_field_t));
	field->fieldType = FILTER_FIELD_IPV4_DIP;
	field->filter_pattern_union.inData.dataType = FILTER_FIELD_DATA_MASK;
	field->filter_pattern_union.inData.value = ipaddr;
	field->filter_pattern_union.inData.mask = ipmask;

	ret_val = rtk_filter_igrAcl_field_add(cfg, field);
	if (ret_val != RT_ERR_OK) {
		return -1;
	}
	return 0;
}

int rtk_acl_cfg_add_vid(void *handl, unsigned short vid)
{
	rtk_filter_cfg_t    *cfg = handl;
	rtk_filter_field_t  *field;
	rtk_api_ret_t ret_val = 0;

	field = kmalloc(sizeof(*field), GFP_KERNEL);
	if (!field)
		return -1;
	memset(field, 0x00, sizeof(rtk_filter_field_t));
	field->fieldType = FILTER_FIELD_CTAG;
	field->filter_pattern_union.ctag.pri.value = 0;
	field->filter_pattern_union.ctag.pri.mask = 0;
	field->filter_pattern_union.ctag.cfi.value = 0;
	field->filter_pattern_union.ctag.cfi.mask = 0;
	field->filter_pattern_union.ctag.vid.value = vid;
	field->filter_pattern_union.ctag.vid.mask = 0x0FFF;

	ret_val = rtk_filter_igrAcl_field_add(cfg, field);
	if (ret_val != RT_ERR_OK) {
		return -1;
	}
	return 0;
}

int rtk_acl_cfg_add_padding(void *handl)
{
	rtk_filter_cfg_t    *cfg = handl;
	rtk_filter_field_t  *field;
	rtk_api_ret_t ret_val = 0;

	field = kmalloc(sizeof(*field), GFP_KERNEL);
	if (!field)
		return -1;
	memset(field, 0x00, sizeof(rtk_filter_field_t));
	field->fieldType = FILTER_FIELD_PATTERN_MATCH;

	ret_val = rtk_filter_igrAcl_field_add(cfg, field);
	if (ret_val != RT_ERR_OK) {
		return -1;
	}
	return 0;
}
/****
action
****/
int rtk_acl_cfg_commit(void *handl, int action, int index, int port, int *outdex)
{
	rtk_filter_cfg_t    *cfg = handl;
	rtk_filter_action_t act;
	unsigned int     ruleNum;
	rtk_portmask_t   portmask;
	rtk_api_ret_t ret_val = RT_ERR_INPUT;

	if (!cfg->fieldHead) {
		ret_val = rtk_acl_cfg_add_padding(handl);
		if (ret_val != 0) {
			return ret_val;
		}
	}

	portmask.bits[0] = halCtrl.valid_utp_portmask;
	if (port >= 0) {
		portmask.bits[0] = 0;
		port = rtk_switch_port_P2L_get(port);
		RTK_PORTMASK_PORT_SET(portmask, port);
	}

	ret_val = RT_ERR_INPUT;
	switch (action) {
		case CMT_ACTION_FLUSH:
			if (index >= 0) {
				ret_val = rtk_filter_igrAcl_cfg_del(index);
				if (ret_val == RT_ERR_OK) {
					*outdex = index - 1;
				}
			}
			else {
				ret_val = rtk_filter_igrAcl_cfg_delAll();
				if (ret_val == RT_ERR_OK) {
					*outdex = index - 1;
				}
			}
			break;
		case CMT_ACTION_ADD_DENY:
			memset(&act, 0x00, sizeof(rtk_filter_action_t));
			cfg->activeport.value.bits[0] = portmask.bits[0];
			rtk_switch_logPortMask_get(&cfg->activeport.mask);
			cfg->invert = 0;

			act.actEnable[FILTER_ENACT_DROP] = 1;
			ret_val = rtk_filter_igrAcl_cfg_add(index, cfg, &act, &ruleNum);
			if (ret_val == RT_ERR_OK) {
				*outdex = index + ruleNum - 1;
			}
			break;
		case CMT_ACTION_ADD_ALLOW:
			memset(&act, 0x00, sizeof(rtk_filter_action_t));
			cfg->activeport.value.bits[0] = portmask.bits[0];
			rtk_switch_logPortMask_get(&cfg->activeport.mask);
			cfg->invert = 0;

			act.actEnable[FILTER_ENACT_ADD_DSTPORT] = 1;
			//rtk_switch_logPortMask_get(&act.filterPortmask);
			ret_val = rtk_filter_igrAcl_cfg_add(index, cfg, &act, &ruleNum);
			if (ret_val == RT_ERR_OK) {
				*outdex = index + ruleNum - 1;
			}
			break;
		default:
			break;
	}
	if (ret_val != RT_ERR_OK) {
		return -1;
	}
	return 0;
}

int rtk_acl_cfg_finish(void *handl)
{
	rtk_filter_cfg_t    *cfg = handl;
	rtk_filter_field_t  *field;

	if (cfg) {
		while (cfg->fieldHead) {
			field = cfg->fieldHead;
			cfg->fieldHead = field->next;
			kfree(field);
		}
		kfree(cfg);
	}
	return 0;
}
