#!/bin/sh
[ -n "$VNET_BOOTING" ] && exit 0
[ -z "$INTERFACE" -o -z "$DEVICE" ] && exit 0
[ "$DEVICE" == "lo" ] && exit 0

WPORTAL_LIBDIR=/lib/wportal
. $WPORTAL_LIBDIR/config.sh
. /lib/zone/zone_api.sh
. /lib/freeStrategy/core_rule.sh

update_wportalEntry_interface_by_zone()
{
sqlite3 /etc/config/portal.db <<EOF
update wportal set interface='$1',gw_id='$1' where zone='$2';
EOF
}

del_wportalEntry_interface_by_zone()
{
sqlite3 /etc/config/portal.db <<EOF
delete from wportal where zone='$1';
EOF
}

local ssid_zones=""

vnethotplug_get_zone_from_vlanid(){
	local vlanid=$2
	if [ "$vlanid" == "0" ];then
		config_get s_system $1 system
		if [ "$s_system" == "1" ];then
			config_get ssid_zone $1 zone
			ssid_zones="${ssid_zones} ${ssid_zone}"
		fi
	else
		config_get _vlanid $1 vlanid
		config_get vlanType_t $1 vlanType
		config_get vlans $1 vlans
		config_get ssid_zone $1 zone
                if [ "$vlanType_t" == "" ];then
			vlanType_t=0
		fi
		if [ "$vlanType_t" == "0" ] && [ "$_vlanid" == "$vlanid" ];then
			ssid_zones="${ssid_zones} ${ssid_zone}"
		else if [ "$vlanType_t" == "1" ] && [ -n "$vlans" ];then
			tmp1=$(echo $vlans | awk -F "," '{for(i=1;i<=NF;i++) print $i}')
			for i in $tmp1;do
				tmp2=$(echo $i | awk -F "-" '{for(i=1;i<=NF;i++) print $i}')
				n=$(echo $tmp2 | awk '{print NF}')
				if [ $n == 1 ] && [ "$vlanid" == "$tmp2" ];then
					ssid_zones="${ssid_zones} ${ssid_zone}"
				else if [ $n == 2 ];then
					n1=$(echo $tmp2 | awk '{print $1}')
					n2=$(echo $tmp2 | awk '{print $2}')
					if [ $vlanid -ge $n1 ] && [ $vlanid -le $n2 ];then
					ssid_zones="${ssid_zones} ${ssid_zone}"
					fi
				fi
				fi
			done
			fi
		fi
	fi
}

case "$ACTION" in
        add)
                config_load "wportal"
                config_get zones dft_wportal zones
                config_get global_enable dft_wportal enable
                zone=`zone_get_zone_byif $INTERFACE`      
                for i in ${zones}
                do
                        if [ "$i" == "$zone" ];then
                                echo "wportal add interface:"$DEVICE > /dev/console
                                uci add_list wportal.dft_wportal.interface=$DEVICE
                                uci add_list wportal.dft_wportal.gw_id=$DEVICE
                                uci commit

                                [ "$global_enable" == "on" ] && update_wportalEntry_interface_by_zone "$DEVICE" "$zone"  ##update database
                        fi
                done
        ;;
        delete)      
                ##delete阶段及时删除network对应的interface，但network对应的zone保留到commit阶段再决定是否删除
                config_load "wportal"
                config_get interfaces dft_wportal interface
                for i in ${interfaces}
                do
                        if [ "$i" == "$DEVICE" ];then
                                echo "wportal delete interface:"$DEVICE > /dev/console
                                uci del_list wportal.dft_wportal.interface=$DEVICE
                                uci del_list wportal.dft_wportal.gw_id=$DEVICE
                                uci commit
                        fi
                done
        ;;
        commit)
                config_load "wportal"
                config_get zones dft_wportal zones
                config_get global_enable dft_wportal enable
                config_get ssids dft_wportal ssid_interface_for_freeStrategy_init
                include=0

                if [ $ssids ];then
                        config_load vnetwork
                        for i in $ssids; do
                                local vlanid=$(uci get ssid_list.$i.vlan)
                                config_foreach vnethotplug_get_zone_from_vlanid vinterface $vlanid
                        done
                        ssid_zones=$(echo $ssid_zones | awk '{for(i=1;i<=NF;i++) print $i}' | sort -u )
                        config_load "wportal"
                fi

                for zact in $ZACTION #ZACTION=LAN1=delete,add
                do
                        action=""
                        zn=${zact%%=*}
                        echo "$zones" | grep "$zn"
                        if [ $? -eq 0 ];then
                                include=1
                                op=${zact##*=}
                                for act in ${op//,/ }
                                do
                                        if [ "$act" == "delete" ]; then
                                                action="del"
                                        elif [ "$act" == "add" -a "$action" == "del" ];then
                                                action="set"
                                        fi
                                done
                                [ "$action" == "del" ] && {
                                        uci del_list wportal.dft_wportal.zones=$zn
                                        uci del_list wportal.dft_wportal.interface_for_freeStrategy_init=$zn
                                        uci commit
                                        [ "$global_enable" == "on" ] && del_wportalEntry_interface_by_zone "$zn"
                                }
                        fi

                        echo "$ssid_zones" | grep "$zn"
                        if [ $? -eq 0 ];then
                                include=1
                        fi
                done

                if [ "$global_enable" == "on" -a "$include" == "1" ]
                then
                        echo "[hotplug vnet] wifidog restarting..." > /dev/console
                        set_wifidog_conf
                        vnete wifidog-init restart
			rule_restart
                        vnete curl $ngx_portal_url    ##update share memory
                fi        
        ;;
        *)
                # do nothing
        ;;
esac
