#!/bin/sh
# This script is executed as part of the hotplug event with
# HOTPLUG_TYPE=iface, triggered by various scripts when an interface
# is configured (ACTION=ifup) or deconfigured (ACTION=ifdown).  The
# interface is available as INTERFACE, the real device as DEVICE.

. /lib/zone/zone_api.sh
. /lib/functions.sh
. /lib/nat/nat_core.sh

if [[ -z "$INTERFACE" ]]; then
	[ -z "$DEVICE" ] && exit 0
	INTERFACE=$(zone_get_iface_bydev $DEVICE)
fi

[ -z "$INTERFACE" ] && exit 0
[ "$DEVICE" == "lo" ] && exit 0

mkdir -p /tmp/.nat
local HOTP_LOG="/tmp/.nat/hotplug.log"
if [[ "`ls -l $HOTP_LOG | awk '{print $5}'`" -gt 50000 ]]; then
	echo -n "" > $HOTP_LOG
fi
HP_LOG()
{
	echo -e "`date +%Y%m%d-%H:%M:%S` $@" >> $HOTP_LOG
}
HP_LOG "$INTERFACE $ACTION"

{
	flock -x 50
	if [[ ! -f "$NAT_READY" ]]; then
		HP_LOG "NAT is not ready, ingore this event"
		exit 0
	fi
	flock -u 50
} 50<>/tmp/nat.lock

case "$ACTION" in
	ifup|ifdown) nat_hotplug_check_rule "$INTERFACE" "$ACTION" 1;;
esac
