#!/bin/sh
# add/delete route rule, add/delete nat rule
#
#${ACTION}
#${IFNAME}
#${PPPD_USERNAME}
#${PPPD_PID}
#${PPPD_TYPE}
#${PPPD_CONFIGPATH}
#${IPREMOTE}
#${IPLOCAL}
#

. /lib/functions.sh
. /lib/firewall/fw.sh
. /lib/firewall/core_tpfirewall.sh
. /lib/pppox/pppox-default-variables.sh
. /lib/zone/zone_api.sh
. /lib/pptp/pptp-log.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
VPN_RULE_NUM=25
VPN_RULE_NUM_FULLMODE=221
ARP_RULE_NUM=24
VPN_FWMARK=0x20000
VPN_FORWARD_TABLE=VPN_FORWARD_MARK
VPN_ROUTING_TABLE=VPN_ROUTING_MARK
##FOR SINGLE NET STACK
M_VPN_INPUT_RULE=VPN_INPUT_RULE
M_VPN_OUTPUT_RULE=VPN_OUTPUT_RULE
VPN_INPUT_RULE=ppp_input_rule
VPN_INPUT_RULE_PRE=ppp_input_rule_pre
VPN_PRESERVICE_RULE=ppp_preservice_rule
VPN_SERVICE_RULE=ppp_service_rule

##FOR L2TP ENCRYPT BANDWIDTH CONTROL
. /lib/qos-tplink/qos_public.sh
PROFILE="/etc/profile.d"
QOS_SFE_CHAIN="sfe_mark"
QOS_HWNAT_CHAIN="hwnat_mark"
NFMARK_MUTEX_MASK="$QOS_NFMARK_MUTEX_MASK"
IPT_PREFIX="iptables -w -t mangle"
L2TP_QOS_CHAIN="qoschain_l2tp"
RATE_LIMIT=`uci -c ${PROFILE} get profile.@l2tp[0].enc_rate_limit_enable`
UPDOWNRATE=`uci -c ${PROFILE} get profile.@l2tp[0].enc_rate_limit`



qos_nf_add_rule_l2tp()
{
	local chain=${L2TP_QOS_CHAIN}
	local rules="${IFNAME} ${IFNAME}_mate"

	#echo " ${vvtype} ${ipsecenc} ${RATE_LIMIT} ${UPDOWNRATE}" > /dev/console
	[ "${vvtype}" != "l2tp" -o "${ipsecenc}" != "yes" ] && return
	[ ! -n "${RATE_LIMIT}" -o "${RATE_LIMIT}" != "enable" -o ! -n "${UPDOWNRATE}" ] && return

	$IPT_PREFIX  -L FORWARD --line | grep -q -w "$chain" || {
		echo " info, create qos default chain $chain" > /dev/console
		echo "$IPT_PREFIX -N $chain" > /dev/console
		$IPT_PREFIX -N $chain 2>/dev/null

		# anti-reentry mechanism
		$IPT_PREFIX -A $chain -m mark ! --mark 0/${NFMARK_MUTEX_MASK} -j RETURN
		echo "$IPT_PREFIX -A $chain -m mark ! --mark 0/${NFMARK_MUTEX_MASK} -j RETURN" > /dev/console

		# global fastpath mark.
		$IPT_PREFIX -A $chain -m tplimit --rate 0x7FFFFFFF --maskbit $QOS_NFMARK_MUTEX_START --info global -j DROP
		echo "$IPT_PREFIX -A $chain -m tplimit --rate 0x7FFFFFFF --maskbit $QOS_NFMARK_MUTEX_START --info global -j DROP" > /dev/console

		ln=`$IPT_PREFIX  -L FORWARD --line | awk '/zone.*MSSFIX/ {print $1}' | awk 'NR==1'`
		[ ${#ln} -ne 0 ] && plus="-I FORWARD $ln" || plus="-A FORWARD"
		echo "$IPT_PREFIX $plus -j $chain" > /dev/console
		$IPT_PREFIX $plus -j $chain

	}
	for rule in $rules;
	do
		[ "${rule#*_}" == "mate" ] && dir="-o" || dir="-i"
		mt_limit="-m tplimit --rate ${UPDOWNRATE/kbps} --type share --maskbit $QOS_NFMARK_MUTEX_START --info $rule"

		##dont do sfe/hw accelerate
		$IPT_PREFIX -A $chain ${dir} ${IFNAME} -m comment --comment $rule -j $QOS_SFE_CHAIN
		if fastpath_hw_on; then
			$IPT_PREFIX -A $chain ${dir} ${IFNAME} -m comment --comment $rule -j $QOS_HWNAT_CHAIN
		fi
		#add iptables 
		$IPT_PREFIX -A $chain ${dir} ${IFNAME} ${mt_limit} -j DROP

	done
}

qos_nf_del_rule_l2tp()
{
	local rules="${IFNAME} ${IFNAME}_mate"
	local chain="$L2TP_QOS_CHAIN"

	##ipsecenc may not be got while deleting config, just delete it while it exists
	[ "${vvtype}" != "l2tp" ] && return
	[ ! -n "${RATE_LIMIT}" -o "${RATE_LIMIT}" != "enable" -o ! -n "${UPDOWNRATE}" ] && return

	for rule in $rules; 
	do
		local idx=`$IPT_PREFIX -nvL $chain --line | sed -e '1,4d' | grep -w "rate.*type.*maskbit.*, info[ ]$rule" | awk '{print $1}'`
		[ ${#idx} -ne 0 ] && {
			$IPT_PREFIX -D $chain $idx

			# delete sfe xfwdm mark rule
			idx=`$IPT_PREFIX -nvL $chain --line | sed -e '1,4d' | grep -w $QOS_SFE_CHAIN | grep -w "/\* $rule \*/" | awk '{print $1}'`
			[ ${#idx} -ne 0 ] && $IPT_PREFIX -D $chain $idx
			( fastpath_hw_on ) && {	
				idx=`$IPT_PREFIX -nvL $chain --line | sed -e '1,4d' | grep -w $QOS_HWNAT_CHAIN | grep -w "/\* $rule \*/" | awk '{print $1}'`
				[ ${#idx} -ne 0 ] && $IPT_PREFIX -D $chain $idx
			}
		}
	done
		
	# IF $QOS_CHAIN is empty, smash it and delete target from FORWARD
	[ `$IPT_PREFIX -S $chain | awk 'END{print NR}'` = 3 ] && {
		echo " info, $chain is empty, should destroy! " > /dev/console
		local i=`$IPT_PREFIX -L FORWARD --line | grep -w "$chain" | awk '{print $1}'`
		[ ${#i} -ne 0 ] && $IPT_PREFIX -D FORWARD $i
		$IPT_PREFIX -F $chain
		$IPT_PREFIX -X $chain
	}
}

check_is_lan()
{
	ipset test VMe $1
	[ x$? == x0 ] && echo "yes" && return
	echo "no"
}

add_vpn_service()
{
	#echo "IPLOCAL $IPLOCAL IPREMOTE $IPREMOTE $PPPD_CONFIGPATH" > /dev/console
	iptables -w -t raw -I $VPN_PRESERVICE_RULE -i $IFNAME -p udp --dport 53 -d $1 -j CT --notrack -m comment --comment "if=${IFNAME}"
	iptables -w -t raw -I $VPN_PRESERVICE_RULE -i $IFNAME -p tcp --dport 53 -d $1 -j CT --notrack -m comment --comment "if=${IFNAME}"
	iptables -w -t nat -A $VPN_SERVICE_RULE -o $IFNAME -p udp --sport 53 -s $IPLOCAL -j SNAT --to-source $1 -m comment --comment "if=${IFNAME}"
	iptables -w -t nat -A $VPN_SERVICE_RULE -o $IFNAME -p tcp --sport 53 -s $IPLOCAL -j SNAT --to-source $1 -m comment --comment "if=${IFNAME}"
	
	#echo "iptables -w -t raw -I $VPN_PRESERVICE_RULE -i $IFNAME -p udp --dport 53 -d $1 -j CT --notrack -m comment --comment "if=${IFNAME}"" >/dev/console
	#echo "iptables -w -t nat -A $VPN_SERVICE_RULE -o $IFNAME -p udp --sport 53 -s $IPLOCAL -j SNAT --to-source $1 -m comment --comment "if=${IFNAME}"" > /dev/console
}

do_service_vpn_up()
{
	#echo "dns1: $dns1 dns2 $dns2" > /dev/console
	[ -z "$dns1" ] && [ -z "$dns2" ] && return
	[ -n "$dns1" ] && [ `check_is_lan "$dns1"` == "yes" ] && add_vpn_service "$dns1" 
	[ -n "$dns2" ] && [ `check_is_lan "$dns2"` == "yes" ] && add_vpn_service "$dns2"
}

do_service_vpn_down()
{
	nums=$(iptables -w -t nat -nvL $VPN_SERVICE_RULE --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t nat -D $VPN_SERVICE_RULE ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL $VPN_SERVICE_RULE --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done

	nums=$(iptables -w -t raw -nvL $VPN_PRESERVICE_RULE --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
		while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t raw -D $VPN_PRESERVICE_RULE ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t raw -nvL $VPN_PRESERVICE_RULE --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done
}

do_internet_vpn_up()
{
	local CHAIN=zone_vpn_${IFNAME}
	if ! fw list 4 f | grep "\-N ${CHAIN}"
	then
		fw add 4 f ${CHAIN}
		fw add 4 f forward ${CHAIN}
	else
		fw flush 4 f ${CHAIN}
		fw del 4 f forward ${CHAIN}
		fw del 4 f ${CHAIN}
	fi

	local wanmode=`uci get network.global.wanmode`
	local wan_numbers="`uci get network.global.wan_numbers`"
    local WAN_LIST=""

	[ -z "${wan_numbers}" ] && wan_numbers="$(seq 1 $wanmode)"

        for i in ${wan_numbers}; do
            WAN_LIST="WAN$i $WAN_LIST"
        done

	for wan in $WAN_LIST; do
		device_list=`zone_get_wan_devices $wan`
		for wan_device in $device_list; do
			fw add 4 f ${CHAIN} DROP $ { "-s ${IPREMOTE} -i ${IFNAME} -o $wan_device" }
		done
	done

	fw_event_manage_dev "${IFNAME}:up"		# Fix Bug 195146, let the packets from host to vpn go out.
}

do_internet_vpn_down()
{
	local CHAIN=zone_vpn_${IFNAME}
	if fw list 4 f | grep "\-N ${CHAIN}"
	then
		fw flush 4 f ${CHAIN}
		fw del 4 f forward ${CHAIN}
		fw del 4 f ${CHAIN}
	fi

	fw_event_manage_dev "${IFNAME}:down"	# Fix Bug 195146, let the packets from host to vpn go out.
}

fix_mss()
{
	[ $mss_offset == "0" ] && {
		return
	}

	local chain=${IFNAME}_MSSFIX

	[ $1 == "up" ] && {
		iface=$(zone_get_effect_ifaces "${zone}")
		[ -n "$iface" ] && {
			local cur_mtu=$(uci get network.$iface.mtu)
			local cur_mru=$(uci get network.$iface.mru)
            local vpn_mtu=`cat /sys/class/net/${IFNAME}/mtu`
			local cur_mss=1460

            cur_mtu=${cur_mtu:=$vpn_mtu}
			cur_mtu=${cur_mtu:=$cur_mru}
			cur_mtu=${cur_mtu:=1500}
			cur_mss=$(($cur_mtu-$mss_offset))
			#cur_mss=$(($cur_mss-$mss_offset))		

			local rule="-p tcp -m tcp --tcp-flags SYN,RST SYN"
			local target_size="TCPMSS --set-mss ${cur_mss}"
			local target_mtu="TCPMSS --clamp-mss-to-pmtu"

			iptables -w -t mangle -C zone_MSSFIX -i $IFNAME -j ${chain} >/dev/null 2>&1
			if [[ "$?" -ne 0 ]]; then
				fw add 4 m ${chain}
				fw add 4 m zone_MSSFIX ${chain} ^ { -o ${IFNAME} }
				fw add 4 m zone_MSSFIX ${chain} ^ { -i ${IFNAME} }
				fw add 4 m ${chain} "$target_size" $ { "$rule" }
				fw add 4 m ${chain} "$target_mtu" $ { "$rule" }
			fi
		}
	}

	[ $1 == "down" ] && {
		fw flush 4 m ${chain}
		fw del 4 m zone_MSSFIX ${chain} ^ { -o ${IFNAME} }
		fw del 4 m zone_MSSFIX ${chain} ^ { -i ${IFNAME} }
		fw del 4 m ${chain}
	}
}

get_param()
{
	config_get netmode $1 "netmode"
	config_get remotesubnet $1 "remotesubnet"
	config_get local_network $1 "local_network"
	config_get workmode $1 "workmode"
	config_get zone $1 "outif"
	config_get en $1 "enable"
}

route_table_init()
{
	ret=`ip rule list | grep ^$VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		iptables -w -t mangle -F ${VPN_FORWARD_TABLE}
		iptables -w -t mangle -F ${VPN_ROUTING_TABLE}
		ip rule add pref $VPN_RULE_NUM fwmark ${VPN_FWMARK}/${VPN_FWMARK} table $VPN_RULE_NUM
	fi
    ret=`ip rule list | grep ^$VPN_RULE_NUM_FULLMODE:`
    if [ -z "$ret" ]; then
        ip rule add pref $VPN_RULE_NUM_FULLMODE fwmark ${VPN_FWMARK}/${VPN_FWMARK} table $VPN_RULE_NUM_FULLMODE
    fi
}

vnete_route_table_init()
{
	ret=`vnete ip rule list | grep ^$VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		vnete ip rule add pref $VPN_RULE_NUM table $VPN_RULE_NUM
	fi
}
#1: local_network
#2: remotesubnet
route_table_mangle()
{
	local lnets="${2}"
	local rnets="${3}"
	local label="${4}"

	if [ "${1}" == "add" ]; then
		if [ -z "$lnets" ]; then
			lnet="0.0.0.0/0"
			for rnet in ${rnets}; do
				iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -m set ! --match-set VMe dst -s ${lnet} -d ${rnet} -m comment --comment "if=${label},vnet=${lnet}" -j MARK --or-mark ${VPN_FWMARK}
				iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -m set ! --match-set VMe dst -s ${lnet} -d ${rnet} -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
				iptables -w -t mangle -A ${VPN_ROUTING_TABLE} -i "${IFNAME}" -s ${rnet} -m set --match-set $(zone_allwan_group) dst -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
				iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${VPN_FWMARK}/${VPN_FWMARK} -m comment --comment "if=${label},vnet=${lnet}" -j MARK --or-mark ${VPN_FWMARK}
				iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -s ${lnet} -d ${IPREMOTE} -m comment --comment "if=${label},vnet=${lnet}" -j MARK --or-mark ${VPN_FWMARK}
				iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -s ${lnet} -d ${IPREMOTE} -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
				[ "$rnet" == "0.0.0.0/0" -a "$PPPD_TYPE" == "client" ] && {
					iptables -w -t mangle -A ${M_VPN_INPUT_RULE} -i "${IFNAME}" -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
					iptables -w -t mangle -A ${M_VPN_OUTPUT_RULE} -m connmark --mark ${VPN_FWMARK}/${VPN_FWMARK} -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --restore-mark --nfmask ${VPN_FWMARK}
				}
			done
			iptables -w -t filter -I ${VPN_INPUT_RULE} -i "${IFNAME}" -m set --match-set VMe dst -m comment --comment "if=${label},vnetwork=${lnet}" -j ACCEPT
			iptables -w -t filter -I ${VPN_INPUT_RULE_PRE} -i "${IFNAME}" -m set --match-set VMe dst -m comment --comment "if=${label},vnetwork=${lnet}" -j RETURN
		else
			local net_type=`name_or_network ${lnets}`
			case "${net_type}" in
				"name")
					for lnet in ${lnets}; do
						local netmask=$(uci get vnetwork."${lnet}".netmask)
						local ipaddr=$(uci get vnetwork."${lnet}".ipaddr)
						local network=$(ipcalc -n ${ipaddr} ${netmask} | cut -d '=' -f2)
						local prefix=$(ipcalc -p ${ipaddr} ${netmask} | cut -d '=' -f2)
						local cidr="${network}/${prefix}" 
						local r_default=0

						for rnet in ${rnets}; do
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -m set ! --match-set VMe dst -s ${cidr} -d ${rnet} -m comment --comment "if=${label},vnet=${lnet}" -j MARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -m set ! --match-set VMe dst -s ${cidr} -d ${rnet} -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_ROUTING_TABLE} -i "${IFNAME}" -s ${rnet} -m set --match-set $(zone_allwan_group) dst -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${VPN_FWMARK}/${VPN_FWMARK} -m comment --comment "if=${label},vnet=${lnet}" -j MARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -s ${cidr} -d ${IPREMOTE} -m comment --comment "if=${label},vnet=${lnet}" -j MARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -s ${cidr} -d ${IPREMOTE} -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
							[ "$rnet" == "0.0.0.0/0" -a "$PPPD_TYPE" == "client" ] && r_default=1
						done
						[ $r_default -eq 1 ] && {
								iptables -w -t mangle -A ${M_VPN_INPUT_RULE} -i "${IFNAME}" -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
								iptables -w -t mangle -A ${M_VPN_OUTPUT_RULE}  -m connmark --mark ${VPN_FWMARK}/${VPN_FWMARK} -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --restore-mark --nfmask ${VPN_FWMARK}
						}
						iptables -w -t filter -I ${VPN_INPUT_RULE} -i "${IFNAME}"  -d ${cidr} -m comment --comment "if=${label},vnetwork=${lnet}" -j ACCEPT
						iptables -w -t filter -I ${VPN_INPUT_RULE_PRE} -i ${IFNAME} -d ${cidr} -m comment --comment "if=${label},vnetwork=${lnet}" -j RETURN
					done
					;;
				"network")
					for lnet in ${lnets}; do
						local cidr=${lnet}
						local r_default=0
						for rnet in ${rnets}; do
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -m set ! --match-set VMe dst -s ${cidr} -d ${rnet} -m comment --comment "if=${label},vnet=${lnet}" -j MARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -m set ! --match-set VMe dst -s ${cidr} -d ${rnet} -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_ROUTING_TABLE} -i "${IFNAME}" -s ${rnet} -m set --match-set $(zone_allwan_group) dst -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${VPN_FWMARK}/${VPN_FWMARK} -m comment --comment "if=${label},vnet=${lnet}" -j MARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -s ${cidr} -d ${IPREMOTE} -m comment --comment "if=${label},vnet=${lnet}" -j MARK --or-mark ${VPN_FWMARK}
							iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set lan_iface src -s ${cidr} -d ${IPREMOTE} -m comment --comment "if=${label},vnet=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
							[ "$rnet" == "0.0.0.0/0" -a "$PPPD_TYPE" == "client" ] && r_default=1
						done
						[ $r_default -eq 1 ] && {
								iptables -w -t mangle -A ${M_VPN_INPUT_RULE} -i "${IFNAME}" -m comment --comment "if=${label},vnetwork=${lnet}" -j CONNMARK --or-mark ${VPN_FWMARK}
								iptables -w -t mangle -A ${M_VPN_OUTPUT_RULE}  -m connmark --mark ${VPN_FWMARK}/${VPN_FWMARK} -m comment --comment "if=${label},vnetwork=${lnet}" -j CONNMARK --restore-mark --nfmask ${VPN_FWMARK}
						}
						iptables -w -t filter -I ${VPN_INPUT_RULE} -i "${IFNAME}"  -d ${cidr} -m comment --comment "if=${label},vnetwork=${lnet}" -j ACCEPT
						iptables -w -t filter -I ${VPN_INPUT_RULE_PRE} -i ${IFNAME} -d ${cidr} -m comment --comment "if=${label},vnetwork=${lnet}" -j RETURN
					done
					;;
			esac
		fi

		iptables -w -t filter -A ${VPN_INPUT_RULE} -i "${IFNAME}" -m set --match-set VMe dst -m comment --comment "if=${label},vnet=${lnet}" -j DROP
		iptables -w -t filter -A ${VPN_INPUT_RULE_PRE} -i "${IFNAME}" -m set --match-set VMe dst -m comment --comment "if=${label},vnet=${lnet}" -j DROP
	else
		tables="${VPN_FORWARD_TABLE} ${VPN_ROUTING_TABLE} ${M_VPN_INPUT_RULE} ${M_VPN_OUTPUT_RULE}" 
		for tbl in $tables
		do
			nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${label}\b" | cut -d ' ' -f 1)
			while [ -n "${nums}" ]; do
				for num in $nums;
				do
					iptables -w -t mangle -D ${tbl} ${num}
					break; # just delete the first once.
				done
				nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${label}\b" | cut -d ' ' -f 1)
			done
		done
		##del filter rules
		tables="${VPN_INPUT_RULE} ${VPN_INPUT_RULE_PRE}"
		for tbl in $tables
		do
			nums=$(iptables -w -t filter -nL ${tbl} --line-numbers | grep "\b${label}\b" | cut -d ' ' -f 1)
			while [ -n "${nums}" ]; do
				for num in $nums;
				do
					iptables -w -t filter -D ${tbl} ${num}
					break; # just delete the first once.
				done
				nums=$(iptables -w -t filter -nL ${tbl} --line-numbers | grep "\b${label}\b" | cut -d ' ' -f 1)
			done
		done
	fi
}

route_table_add()
{
	ip route add "$@" table $VPN_RULE_NUM
	ip route add "$@" table main
}

route_table_del()
{
	ip route del "$@" table $VPN_RULE_NUM
	ip route del "$@" table main
}

route_table_add_fullmode()
{
	ip route add "$@" table $VPN_RULE_NUM_FULLMODE
	ip route add "$@" table main
}

route_table_del_fullmode()
{
	ip route del "$@" table $VPN_RULE_NUM_FULLMODE
	ip route del "$@" table main
}

# sdnc5.6 add by local network
name_or_network() 
{
    network=$1
    # eg. network="NETWORK1 NETWORK2" or "192.168.0.1/24 192.168.1.1/22"
    [ `expr "$network" : 'NETWORK'` -ne 0 ] && echo "name" && return 
    echo "network"
}

do_ifup()
{
	local hostnet=

	route_table_init
	vnete_route_table_init

	ip route del ${IPREMOTE} dev ${IFNAME} table $VPN_RULE_NUM
	case "${PPPD_TYPE}" in
		"server")
			ip route add ${IPREMOTE} metric ${metric} dev ${IFNAME} table $VPN_RULE_NUM # Fix Bug 195146, prior to Static Route

#			vnete sysctl -w /proc/sys/net/ipv4/conf/all/proxy_arp=1
			#vnete ip route add ${IPREMOTE} via 169.254.11.21 table $VPN_RULE_NUM
			#vnete ip route add ${IPREMOTE} via 169.254.11.21 table main

			if [ "${netmode}" = "client2lan" -o "${netmode}" = "pc2lan" ]; then
#				pppd will do this
#				route add -host ${IPREMOTE} dev ${IFNAME}
				echo ""
			else
				#route add ${hostnet} ${remotesubnet} gw ${IPREMOTE} metric ${metric} dev ${IFNAME}
				for subnet in ${remotesubnet}; do
					hostnet="-net"
					[ "${subnet#*/}" = "32" ] && { hostnet="-host"; 
						subnet=${subnet%/*}
					}
					#echo "route add ${hostnet} ${subnet} gw ${IPREMOTE} metric ${metric} dev ${IFNAME}" > /dev/console
					route_table_add ${subnet} via ${IPREMOTE} metric ${metric} dev ${IFNAME}
				done
			fi
			ip route add ${IPREMOTE} dev ${IFNAME} table ${ARP_RULE_NUM}
			ret=`ip rule list | grep ^$ARP_RULE_NUM:`
			if [ -z "$ret" ]; then
				ip rule add from all lookup ${ARP_RULE_NUM}
			fi
			;;

		"client")
			[ "${lns}" != "${IPREMOTE}" ] && {
				ip route add ${IPREMOTE} metric ${metric} dev ${IFNAME} table $VPN_RULE_NUM
			}
			#route add ${hostnet} ${remotesubnet} gw ${IPREMOTE} metric ${metric} dev ${IFNAME}
			#[ "$?" != "0" ] && route add ${hostnet} ${remotesubnet} metric ${metric} dev ${IFNAME}
			for subnet in ${remotesubnet}; do
				hostnet="-net"
				[ "${subnet#*/}" = "32" ] && { hostnet="-host"; 
					subnet=${subnet%/*}
				}
				#echo "route add ${hostnet} ${subnet} gw ${IPREMOTE} metric ${metric} dev ${IFNAME}" > /dev/console
                if [ "${subnet}" == "0.0.0.0/0" ]; then
                    route_table_add_fullmode ${subnet} via ${IPREMOTE} metric ${metric} dev ${IFNAME}
                    [ "$?" != "0" ] && route_table_add_fullmode ${subnet} metric ${metric} dev ${IFNAME}
                else
                    route_table_add ${subnet} via ${IPREMOTE} metric ${metric} dev ${IFNAME}
                    [ "$?" != "0" ] && route_table_add ${subnet} metric ${metric} dev ${IFNAME}
                fi
			done

			;;
	esac
#	add interface into ZV_WANx ipset item
	[ -n "$zone" -a -n "$IFNAME" ] && {
		zone_add_vgroup "${IFNAME}" "${zone}"
	}
#	add interface into dnsproxy
	[ -n "$IFNAME" -a "${PPPD_TYPE}" == "server" ] && {
		/lib/dnsproxy/api.sh add "$IFNAME"
	}
	if [ "${PPPD_TYPE}" == "server" -a -e "/etc/init.d/hwnat_ipstat" ];then
		echo "### update hwnat_ipstat" >/dev/console
		hwnat_id=`ps -w | grep hwnat_ipstat | awk '/\/usr\/sbin\/hwnat_ipstat/ {print $1}'`
		[ "$hwnat_id" -gt 0 ] && kill -10 ${hwnat_id} 2 >/dev/null
	fi

	[ ${workmode} = "nat" ] && {
		iptables -w -t nat -C postrouting_rule_multinat -o ${IFNAME} -j MASQUERADE 2>/dev/null
		[ "$?" != "0" ] && fw add 4 nat postrouting_rule_multinat MASQUERADE  $ { -o ${IFNAME} }
	}

	unset UCI_CONFIG_DIR
	do_internet_vpn_up
	do_service_vpn_up
	qos_nf_add_rule_l2tp

	#echo "local $local_network" > /dev/console
	if [ -z "$local_network" ]; then
		#fw add 4 f ppp_forward_rule ACCEPT { -i br-lan -o ${IFNAME} -m comment --comment "vnet="}
		iptables -w -A ppp_forward_rule -m set --match-set lan_iface src -o ${IFNAME} -m comment --comment "vnet=" -j ACCEPT 
	else
		local net_type=`name_or_network ${local_network}`
		case "${net_type}" in
			"name")
				for lnet in ${local_network}; do
					local netmask=$(uci get vnetwork."${lnet}".netmask)
					local ipaddr=$(uci get vnetwork."${lnet}".ipaddr)
					local network=$(ipcalc -n ${ipaddr} ${netmask} | cut -d '=' -f2)
					local prefix=$(ipcalc -p ${ipaddr} ${netmask} | cut -d '=' -f2)
					local cidr="${network}/${prefix}" 
					#fw add 4 f ppp_forward_rule ACCEPT { -i br-lan -o ${IFNAME} -s ${cidr} -m comment --comment "vnet=${lnet}"} > /dev/console
					iptables -w -A ppp_forward_rule -m set --match-set lan_iface src -o ${IFNAME} -s ${cidr} -m comment --comment "vnet=${lnet}" -j ACCEPT
					iptables -w -I ppp_forward_rule -i ${IFNAME} -m set --match-set lan_iface dst -d ${cidr} -m comment --comment "receive_local_subnet_${lnet}_and_drop_other_vlan" -j ACCEPT
					iptables -w -A ppp_forward_rule -i ${IFNAME} -m set --match-set lan_iface dst -d 0.0.0.0/0 -m comment --comment "drop_other_vlan_except_${lnet}" -j DROP
				done
				;;
			"network")
				for lnet in ${local_network}; do
					iptables -w -A ppp_forward_rule -m set --match-set lan_iface src -o ${IFNAME} -s ${lnet} -m comment --comment "custom_net=${lnet}" -j ACCEPT
					iptables -w -I ppp_forward_rule -i ${IFNAME} -m set --match-set lan_iface dst -d ${lnet} -m comment --comment "receive_custom_net_${lnet}_and_drop_other_vlan" -j ACCEPT
					iptables -w -A ppp_forward_rule -i ${IFNAME} -m set --match-set lan_iface dst -d 0.0.0.0/0 -m comment --comment "drop_other_vlan_except_${lnet}" -j DROP
				done
				;;
		esac
	fi

	route_table_mangle add "${local_network}" "${remotesubnet}" "${IFNAME}"

	#fix mss for tcp pkt in vpn tunnel
	fix_mss "up"

	case "${PPPD_TYPE}" in
		"server")
			echo "";;
		"client")
			sleep 1
			for rnet in ${remotesubnet}; do
				local r_netmask=$(ipcalc -m ${rnet} | sed 's/NETMASK=//g')
				local r_ipaddr=$(ipcalc -n ${rnet} | sed 's/NETWORK=//g')
				if [ -z "$local_network" ]; then
					#echo "conntrack -D --dst ${r_ipaddr} --mask-dst ${r_netmask}" > /dev/console
					if [ $rnet = "0.0.0.0/0" ]; then
						[ -x /usr/sbin/conntrack ] && conntrack -D -m 0x10000 2>/dev/console
					else
						[ -x /usr/sbin/conntrack ] && conntrack -D --dst ${r_ipaddr} --mask-dst ${r_netmask} 2>/dev/console
						[ -x /usr/sbin/conntrack ] && conntrack -D --src ${r_ipaddr} --mask-src ${r_netmask} 2>/dev/console
					fi
				else
					local net_type=`name_or_network ${local_network}`
					case "${net_type}" in
						"name")
							for lnet in ${local_network}; do
								local l_netmask=$(uci get vnetwork."${lnet}".netmask)
								local l_ipaddr=$(uci get vnetwork."${lnet}".ipaddr)
								#echo "conntrack -D --src ${l_ipaddr} --mask-src ${l_netmask} --dst ${r_ipaddr} --mask-dst ${r_netmask}" > /dev/console
								#echo "conntrack -D --src ${r_ipaddr} --mask-src ${r_netmask} --dst ${l_ipaddr} --mask-dst ${l_netmask}" > /dev/console
								if [ $rnet = "0.0.0.0/0" ]; then
									[ -x /usr/sbin/conntrack ] && conntrack -D -m 0x10000 2>/dev/console
								else
									[ -x /usr/sbin/conntrack ] && conntrack -D --src ${l_ipaddr} --mask-src ${l_netmask} --dst ${r_ipaddr} --mask-dst ${r_netmask} 2>/dev/console
									[ -x /usr/sbin/conntrack ] && conntrack -D --src ${r_ipaddr} --mask-src ${r_netmask} --dst ${l_ipaddr} --mask-dst ${l_netmask} 2>/dev/console
								fi
							done
							;;
						"network")
							for lnet in ${local_network}; do
								local l_netmask=$(ipcalc -m "${lnet}" | cut -d "=" -f2)
								local l_ipaddr=$(echo "${lnet}" | cut -d "/" -f1)
								if [ $rnet = "0.0.0.0/0" ]; then
									[ -x /usr/sbin/conntrack ] && conntrack -D -m 0x10000 2>/dev/console
								else
									[ -x /usr/sbin/conntrack ] && conntrack -D --src ${l_ipaddr} --mask-src ${l_netmask} --dst ${r_ipaddr} --mask-dst ${r_netmask} 2>/dev/console
									[ -x /usr/sbin/conntrack ] && conntrack -D --src ${r_ipaddr} --mask-src ${r_netmask} --dst ${l_ipaddr} --mask-dst ${l_netmask} 2>/dev/console
								fi
							done
							;;
					esac
				fi
			done
			[ -x /usr/sbin/xfwdm ] && xfwdm -F
			[ -e "/etc/init.d/hwnat" ] && hwnat -y
			[ -e /sys/kernel/debug/ecm/ecm_db/defunct_all ] && echo 1 > /sys/kernel/debug/ecm/ecm_db/defunct_all
			;;
	esac

}

do_ifdown()
{
	local hostnet=

	route_table_init
	vnete_route_table_init

	case "${PPPD_TYPE}" in
		"server")
			route_table_del ${IPREMOTE} dev ${IFNAME}

			#vnete ip route del ${IPREMOTE} via 169.254.11.21 table $VPN_RULE_NUM
			#vnete ip route del ${IPREMOTE} via 169.254.11.21 table main

			if [ "${netmode}" = "client2lan" -o "${netmode}" = "pc2lan" ]; then
#				pppd will do this
#				route del -host ${IPREMOTE} dev ${IFNAME}
				echo ""
			else
				#route del ${hostnet} ${remotesubnet} dev ${IFNAME}
				#echo "remotesubnet $remotesubnet" > /dev/console
				for subnet in ${remotesubnet}; do
					hostnet="-net"
					[ "${subnet#*/}" = "32" ] && { hostnet="-host"; 
						subnet=${subnet%/*}
					}
					#echo "route del ${hostnet} ${subnet} dev ${IFNAME}" > /dev/console
					route_table_del ${subnet} dev ${IFNAME}
				done
			fi
			;;

		"client")
			fw del 4 nat postrouting_rule_multinat MASQUERADE  $ { -o ${IFNAME} }
			#route del ${hostnet} ${remotesubnet} dev ${IFNAME}
			for subnet in ${remotesubnet}; do
				hostnet="-net"
				[ "${subnet#*/}" = "32" ] && { hostnet="-host"; 
					subnet=${subnet%/*}
				}
				#echo "route del ${hostnet} ${subnet} dev ${IFNAME}" > /dev/console
                if [ "${subnet}" == "0.0.0.0/0" ]; then
                    route_table_del_fullmode ${subnet} dev ${IFNAME}
                else
                    route_table_del ${subnet} dev ${IFNAME}
                fi
			done
			route_table_del ${IPREMOTE} dev ${IFNAME}

			[ ${vvtype} == "pptp" ] && [ ${en} == "on" -o ${en} == "1" ] && [ -f ${path}/ppp ] && [ -f ${path}/mgr ] && [ -f ${path}/options.pptp ] && {
				shell_log_pptp "[pptp vpn][client]: client ${IFNAME} do_ifdown."
				ppp_id=`cat ${path}/ppp`
				mgr_id=`cat ${path}/mgr`
				sleep 2
				cnt=`ps | grep "pppd" | grep -E "$ppp_id | $mgr_id" | wc -l`
				if [ $cnt -lt 2 ] ; then
				# restart new pppd
					shell_log_pptp "[pptp vpn][client]: client ${IFNAME} pppd process less then 2, delete and restart new pppd."
					lusername=`uci -c ${path} get myconfig.@pnc[0].username`
					lpassword=`uci -c ${path} get myconfig.@pnc[0].password`
					lpns=`uci -c ${path} get myconfig.@pnc[0].pns`
					lworkmode=`uci -c ${path} get myconfig.@pnc[0].workmode`
					lremotesubnet=`uci -c ${path} get myconfig.@pnc[0].remotesubnet`
					loutif=`uci -c ${path} get myconfig.@pnc[0].outif`
					local_network=`uci -c ${path} get myconfig.@pnc[0].local_network`

					local lmppe=1
					if cat ${path}/options.pptp | grep "#mppe" ; then
						lmppe=0
					fi

					/lib/pptp/pptp-client-update.sh ${PPPD_CONFIGPATH} on on "$lusername" "$lpassword" $lpns $lworkmode "$lremotesubnet" $loutif $lmppe "${local_network}"
					pptp_client_update="y"
				else
					shell_log_pptp "[pptp vpn][client]: client ${IFNAME} ppp_id=$ppp_id is exist."
					echo "ppp_id=$ppp_id is exist" > /dev/console
				fi
			}
;;

	esac
#               delete interface into ZV_WANx ipset item
	[ -n "$zone" -a -n "$IFNAME" ] && {
		zone_delete_vgroup "${IFNAME}" "${zone}"
	}
#	delete interface in dnsproxy
	[ -n "$IFNAME" -a "${PPPD_TYPE}" == "server" ] && {
		/lib/dnsproxy/api.sh delete "${IFNAME}"
	}
	if [ "${PPPD_TYPE}" == "server" -a -e "/etc/init.d/hwnat_ipstat" ];then
                echo "### update hwnat_ipstat" >/dev/console
                hwnat_id=`ps -w | grep hwnat_ipstat | awk '/\/usr\/sbin\/hwnat_ipstat/ {print $1}'`
                [ "$hwnat_id" -gt 0 ] && kill -10 ${hwnat_id} 2 >/dev/null
        fi	
	#[ ${workmode} = "nat" ] && fw del 4 nat POSTROUTING MASQUERADE  $ { -o ${IFNAME} }	

	do_internet_vpn_down
	do_service_vpn_down
	qos_nf_del_rule_l2tp
	#echo "local_network down ${local_network}" > /dev/console
	#if [ -z "$local_network" ]; then
	#	fw del 4 f ppp_forward_rule ACCEPT { -i br-lan -o ${IFNAME} }
	#else
	#	for local in ${local_network}; do
	#		fw del 4 f ppp_forward_rule ACCEPT { -i br-lan -o ${IFNAME} -s ${local} }
	#	done
	#fi
	local localnet=$(iptables -w -nvL ppp_forward_rule | grep ${IFNAME} | awk '{print $8}')
	#for local in ${localnet}; do
		#echo "fw del $-i br-lan -o ${IFNAME} -s ${local}" > /dev/console
	#	fw del 4 f ppp_forward_rule ACCEPT { -i br-lan -o ${IFNAME} -s ${local} }
	#done
	
	nums=$(iptables -w -nvL ppp_forward_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -D ppp_forward_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -nvL ppp_forward_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done

	route_table_mangle del "${localnet}" "${remotesubnet}" "${IFNAME}"

	fix_mss "down"

	if [ "$pptp_client_update" == "y" ]; then
		new_ppp_id=`cat ${path}/ppp`
		new_mgr_id=`cat ${path}/mgr`
		shell_log_pptp "[pptp vpn][client]: client ${IFNAME} has restarted new pppd, new ppp_id:${new_ppp_id}, new_mgr_id:${new_mgr_id}."
	fi

}
echo "pppox if ${ACTION}" > /dev/console
#PPPD_CONFIGPATH like "/tmp/l2tp/server/server0", note: no "config"
path=${PPPD_CONFIGPATH}/config
UCI_CONFIG_DIR=${path}
pidpath=/tmp/ppp/pid

metric=0
mss_offset=40
vvtype=l2tp
opt_name=lns
if echo ${PPPD_CONFIGPATH}|grep -q "^/tmp/l2tp"; then
	metric=`uci get system_params.metric.l2tp 2>/dev/null`
	vvtype=l2tp
	mss_offset=40
	vpt_name=lns
elif echo ${PPPD_CONFIGPATH}|grep -q "^/tmp/pptp"; then
	metric=`uci get system_params.metric.pptp 2>/dev/null`
	vvtype=pptp
	mss_offset=56
	opt_name=pns
elif echo ${PPPD_CONFIGPATH}|grep -q "^/tmp/pppoe"; then
	metric=`uci get system_params.metric.pppoe 2>/dev/null`
	vvtype=pppoe
	mss_offset=0
fi
metric=${metric:-0}

if [ "${PPPD_TYPE}" = "server" ]; then
#workmode for server must be route
	pppuser=`asciimaker -a "${PPPD_USERNAME}"`
	netmode=`uci -c ${path} get ${pppox_if_config}.${pppuser}.netmode 2>/dev/null`
	remotesubnet=`uci -c ${path} get ${pppox_if_config}.${pppuser}.remotesubnet 2>/dev/null`
	local_network=`uci -c ${path} get ${pppox_if_config}.${pppuser}.local_network 2>/dev/null`
	if [ -z $local_network ]; then
		server_name=`echo "${PPPD_CONFIGPATH}" | cut -d '/' -f 5 2>/dev/null`
		server_type=`echo "${PPPD_CONFIGPATH}" | cut -d '/' -f 3 2>/dev/null`
		loop=0
		section_name="pns"
		if [ ${server_type} = "l2tp" ]; then
			section_name="lns"
		fi
		result=$(uci get ${server_type}-server.@${section_name}[$loop].server 2>/dev/null)
		while [ $result != &{server_name} ];do
			loop=$(($loop+1))
		done
		local_network=`uci get ${server_type}-server.@${section_name}[$loop].local_network`
	fi

	zone=`uci -c ${path} get ${pppox_configname}.@${opt_name}[0].bindif 2>/dev/null`
	ipsecenc=`uci -c ${path} get ${pppox_configname}.@${opt_name}[0].ipsecenc 2>/dev/null`
	workmode=route
	if [ "${netmode}" = "client2lan" -o "${netmode}" = "pc2lan" ]; then
		remotesubnet="0.0.0.0/0"
	fi
	if [ -f /tmp/pppox.controller.mode ]; then
		#controller mode
		local vpn_type=${PPPD_CONFIGPATH:5:4}
		if [ "$vpn_type" = "pptp" ]; then
			server=$(echo "$PPPD_CONFIGPATH" | cut -d '/' -f 5)
			if [ -n "$server" ]; then
				local loop=0
				local server_max=`uci -c /etc/profile.d get profile.@pptp[0].server_list_max`
				local read_server=$(uci get pptp-server.@pns[$loop].server 2>/dev/null)
				while [ -n "$read_server" -a "$read_server" != "$server" -a "$loop" -le "$server_max" ]; do
					loop=$(($loop+1))
					read_server=$(uci get pptp-server.@pns[$loop].server 2>/dev/null)
				done
				dns1=`uci get pptp-server.@pns[$loop].ms_dns1 2>/dev/null`
				dns2=`uci get pptp-server.@pns[$loop].ms_dns2 2>/dev/null`
			fi
		else
			server=$(echo "$PPPD_CONFIGPATH" | cut -d '/' -f 5)
			[ -n "$server" ] && dns1=`uci get l2tp-server.$server.ms_dns1 2>/dev/null`
			[ -n "$server" ] && dns2=`uci get l2tp-server.$server.ms_dns2 2>/dev/null`
		fi
	else
		#standalone mode
		dns1=`uci -c ${path} get ${pppox_if_config}.${pppuser}.ms_dns1 2>/dev/null`
		dns2=`uci -c ${path} get ${pppox_if_config}.${pppuser}.ms_dns2 2>/dev/null`
	fi
	#echo "uci -c ${path} get ${pppox_if_config}.${pppuser} out ${remotesubnet} ${local_network}" > /dev/console
elif [ "${PPPD_TYPE}" = "client" ]; then
	configname=${pppox_configname}
	
	config_load ${configname}
	config_foreach get_param
	config_foreach config_clear

	ipsecenc=`uci -c ${path} get ${pppox_configname}.@lac[0].ipsecenc 2>/dev/null`
#	netmode=`uci -c ${path} get ${configname}.@${vvtype}[-1].netmode 2>/dev/null`
#	remotesubnet=`uci -c ${path} get ${configname}.@${vvtype}[-1].remotesubnet 2>/dev/null`
#	workmode=`uci -c ${path} get ${configname}.@${vvtype}[-1].workmode 2>/dev/null`

	lns=`cat ${path}/serverip`
	netmode=lan2lan	
else
	return 1
fi

if [ "${ACTION}" = "ifup" ]; then
    upTime=$(awk '{print int($1)}' /proc/uptime)
	uci -c ${pidpath} set ${PPPD_PID}.@info[0].uptime=${upTime}
	uci -c ${pidpath} commit ${PPPD_PID}
	[ -n "$PPPD_PID" ] && echo -1000 > /proc/${PPPD_PID}/oom_score_adj
	
	#echo "uci -c ${pidpath} set ${PPPD_PID}.@info[0].uptime = $(date +%s)" > /dev/console
fi

#hostnet="-net"
#[ "${remotesubnet#*/}" = "32" ] && { hostnet="-host"; 
#		remotesubnet=${remotesubnet%/*}
#	}
case "${ACTION}" in
	"ifup")
		do_ifup;;
	"ifdown")
		do_ifdown;;
	*)
		return 1;;
esac
if [ "${PPPD_TYPE}" = "client" ]; then
	[ -x /etc/init.d/dnsproxy ] && /etc/init.d/dnsproxy restart
fi