-- local sys    = require "luci.sys"
-- local socket = require "socket"
local uci    = require "luci.model.uci"
local dbg    = require "luci.tools.debug"
local utl    = require "luci.util"
local sys    = require "luci.sys"
local uci_r = uci.cursor()

local zone_file = "/tmp/.acl_zone"
local zone_file_set = "/tmp/.acl_zone_set"
CONFIG_NAME="access_ctl"
SECTION_NAME="rule_acl_inner"

local function isVNET(interface)
	return type(interface) ~= "table" and interface == "VNET" or interface[1] == "VNET"
end

local function getNetworkName(name)
	local i,j
	i,j = string.find(name,"_REV")
	if i and j then
		return string.sub(name,1,i-1)
	end
	return name
end

local function getIfaceName(vnetZone)
	local ifname = nil
	local rev = false
	
	local i,j = string.find(vnetZone, "_REV")
	if i and j then
		vnetZone = string.sub(vnetZone, 1, i-1)
		rev = true
	end

	uci_r:foreach("vnetwork", "vinterface",
		function(section)
			if (vnetZone == uci_r:get("vnetwork", section[".name"], "zone")) then
				ifname = uci_r:get("vnetwork", section[".name"], "ifname")
			end
		end
	)

	if rev then
		return ifname .. "_REV"
	else
		return ifname
	end
end

function acl_config_delete_rule()
    local success = {}
    local zones=nil
	local fp_zone = io.open(zone_file, "r")
    zones=fp_zone:read("*a")
    fp_zone:close()
    if not zones or zones == "" then
        return false
    end
    local index=utl.split(zones,"\n")
    index=utl.split(index[1],",")
    zones=(type(index) == "table") and index or {index}

    uci_r:foreach(CONFIG_NAME, SECTION_NAME,
        function(section)
            local interface = section["zone"]
			if isVNET(interface) then
				local srcNetwork = getNetworkName(section["srcIf"]) or ""
				local destNetwork = getNetworkName(section["destIf"]) or ""
				for _,j in pairs(zones) do
					if j == srcNetwork or j == destNetwork then
						success[#success+1] = uci_r:delete(CONFIG_NAME,section[".name"]) and true or false
                        if success[#success] == true then
                            if section["v6flag"] and section["v6flag"] == "1" then
                                sys.fork_call("/lib/access_ctl/access_func_v6.sh "..section["name"].." 1")
                            else
                                sys.fork_call("/lib/access_ctl/access_func.sh "..section["name"].." 1")
                            end
                            if section["time"] ~= "Any" then
                                sys.fork_exec(". /lib/access_ctl/time.sh && del_timeobj "..section["time"].." "..section["name"])
                            end
                        end
					end
				end
			end			
        end        
    )

    for _,j in ipairs(success) do
        if j == false then
            return false
        end
    end

    if not uci_r:commit(CONFIG_NAME) then
        return false
    end
	
    return true
end

function acl_config_update_rule()
    local success = {}
    local zones=nil
	local fp_zone = io.open(zone_file_set, "r")
    zones=fp_zone:read("*a")
    fp_zone:close()
    if not zones or zones == "" then
        return false
    end
    local index=utl.split(zones,"\n")
    index=utl.split(index[1],",")
    zones=(type(index) == "table") and index or {index}

    uci_r:foreach(CONFIG_NAME, SECTION_NAME,
        function(section)
            local interface = section["zone"]
			if isVNET(interface) then
				local srcNetwork = getNetworkName(section["srcIf"]) or ""
				local destNetwork = getNetworkName(section["destIf"]) or ""
				for _,j in pairs(zones) do
                    if j == srcNetwork then
						local ret = uci_r:set(CONFIG_NAME,section[".name"],"srcIfName",getIfaceName(section["srcIf"])) and true or false
                        if ret == true then
                            success[#success+1] = {}
                            success[#success]["name"] = section[".name"]
                            success[#success]["v6flag"] = section["v6flag"]
                            success[#success]["time"] = section["time"]
                        end
                    elseif j == destNetwork then
						local ret = uci_r:set(CONFIG_NAME,section[".name"],"destIfName",getIfaceName(section["destIf"])) and true or false
                        if ret == true then
                            success[#success+1] = {}
                            success[#success]["name"] = section[".name"]
                            success[#success]["v6flag"] = section["v6flag"]
                            success[#success]["time"] = section["time"]
                        end
                    end
				end
			end			
        end        
    )

    if not uci_r:commit(CONFIG_NAME) then
        return false
    end

    for _,j in ipairs(success) do
        if j["time"] ~= "Any" then
            sys.fork_call(". /lib/access_ctl/time.sh && del_timeobj "..j["time"].." "..j["name"])
        end

        if j["v6flag"] and j["v6flag"] == "1" then
            sys.fork_call("/lib/access_ctl/access_func_v6.sh "..j["name"].." 1")
        else
            sys.fork_call("/lib/access_ctl/access_func.sh "..j["name"].." 1")
        end
		
        if j["time"] ~= "Any" then	
            sys.fork_call(". /lib/access_ctl/time.sh && add_timeobj "..j["time"].." "..j["name"])
        else
            if j["v6flag"] and j["v6flag"] == "1" then
                sys.fork_call("/lib/access_ctl/access_func_v6.sh "..j["name"])
            else
                sys.fork_call("/lib/access_ctl/access_func.sh "..j["name"])
            end
        end
    end	

    return true
end

local action = arg[1]
if action == "del" then
    acl_config_delete_rule()
elseif action == "update" then
    acl_config_update_rule()
end
