#!/bin/sh /etc/rc.common
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file     access_ctl core.sh
# brief    
# author   Chen chen
# version  1.0.0
# date     24July23
# histry   arg 1.0.0, 08June15, Chen chen, Create the file.  

. /lib/access_ctl/time.sh
. /lib/access_ctl/markdef.sh
. /usr/share/libubox/jshn.sh

lua /lib/access_ctl/acl_wireless_support.lua
acl_device_info_file="/tmp/device-info.json"
local acl_file="/tmp/acl_support_wireless"
local known_service="FTP SSH TELNET SMTP DNS HTTP POP3 SNTP H323 TCPUDP"
[ -f "$acl_file" ] && wireless_config="1"

## releated functions ##
ACL_LOG()
{
	[ ! -e /tmp/access_ctl ] && mkdir touch /tmp/access_ctl
	[ -f /tmp/access_ctl/access_ctl.log ] && {
		local size="`ls -l /tmp/access_ctl/access_ctl.log | awk '{ print $5 }'`"
		if [ $size -gt 3072000 ]; then
			rm /tmp/access_ctl/access_ctl.log
		fi
	}
	[ ! -f /tmp/access_ctl/access_ctl.log ] && touch /tmp/access_ctl/access_ctl.log
	echo "$1" >> /tmp/access_ctl/access_ctl.log
}

fw_check() {
  local vnet_prefix=$4
  local ret=$(${vnet_prefix} iptables -w -t $1 -C $2 $3 2>&1)
  [ -n "$ret" ] && return 0||return 1
}

acl_wan_zone_convert()
{
	local num=`echo $1| sed 's/[^0-9]//g'`
	echo "ZV_W$num"
}

acl_zone_in_flag()
{
	local wan=`echo "$1" | grep "^WAN[0-9]\+$"`
	[ -n "$wan" ] && {
		echo "WAN"
		return
	}
	local vpns=`zone_get_ifaces "VPN"`
	for vpn in $vpns
	do
		if [ "$1" == "$vpn" ];then
			echo "VPN"
			return
		fi
	done
	echo "0"
}

_wan_get_l3_device_by_zone_ipv6()
{
	local zone=$1
	local interface=""
	local ret=""
	local l3_device=""
	zone=`echo "$zone" | tr 'A-Z' 'a-z'`
	interface="$zone""_v6"
	ret=`ubus call network.interface.$interface status`
	[ "$?" == 0 ] && json_load "$ret" && json_get_var l3_device l3_device
	echo "$l3_device"
}

_acl_zone_get_wan_devices()
{
	local zone=$1
	local v6_flag=$2
	local ret_devices=""
	if [ "$v6_flag" == "1" ];then
		ret_devices=`_wan_get_l3_device_by_zone_ipv6 $zone`
	else
		ret_devices="$(zone_get_wan_devices $zone)"
	fi
	echo "$ret_devices"
}

get_extmark_info()
{
	local states=$1
	local extmark=0
	if [ x$states == 'x' ];then
		extmark=$CTMARK_DEF
	else
		case "$states" in
			*new*)
				extmark=$(($extmark + $CTMARK_NEW))
			;;
		esac
		case "$states" in
			*related*)
				extmark=$(($extmark + $CTMARK_RELATED))
			;;
		esac
		case "$states" in
			*established*)
				extmark=$(($extmark + $CTMARK_ESTBLISHED))
			;;
		esac
		case "$states" in
			*invalid*)
				extmark=$(($extmark + $CTMARK_INVALID))
			;;
		esac
	fi
	echo $extmark
}

get_src_rule() 
{
	local set_src_rule=""
	local rule_acl_inner_src=$1
	local rule_acl_inner_srcIfName=$2
	local rule_acl_inner_wireless_flag=$3
	
	if [ "${rule_acl_inner_src:0:7}" == "country" ] || [ "${rule_acl_inner_src:0:7}" == "ctgroup" ]; then
		local src_country=${rule_acl_inner_src#*:}
		set_src_rule="-m geoip --src-cc $src_country"
	elif [ "$rule_acl_inner_src" != "" ] && [ "$rule_acl_inner_src" != "IPGROUP_ANY" ];then
		local ipset_src=${rule_acl_inner_src%_REV}
		if [ "$ipset_src" == "$rule_acl_inner_src" ]; then
			set_src_rule="-m set --match-set $ipset_src src"
		else
			set_src_rule="-m set ! --match-set $ipset_src src"
		fi
		set_src_rule="$set_src_rule,src"
		#echo "[rule_acl_inner_src]set_src_rule=$set_src_rule"
	fi
	if [ "$rule_acl_inner_srcIfName" != "" ];then
		if [ "$rule_acl_inner_wireless_flag" == "1" -o "$rule_acl_inner_wireless_flag" == "3" ]; then
			set_src_rule="-m bridgeport --multisports $rule_acl_inner_srcIfName"
		else
			local src_lan_ifname=${rule_acl_inner_srcIfName%_REV}
			if [ "$src_lan_ifname" == "$rule_acl_inner_srcIfName" ]; then
				set_src_rule="-i $src_lan_ifname"
			else
				set_src_rule="! -i $src_lan_ifname"
			fi
		fi
		#echo "[rule_acl_inner_srcIfName]set_src_rule=$set_src_rule"
	fi
	echo $set_src_rule
}

get_dst_rule()
{
## 这里不处理FQDN的dst_rule ##
	local set_dst_rule=""
	local rule_acl_inner_dest=$1
	local rule_acl_inner_destIfName=$2
	local rule_acl_inner_wireless_flag=$3
	echo $@ > /dev/console
	if [ "${rule_acl_inner_dest:0:7}" == "country" ] || [ "${rule_acl_inner_dest:0:7}" == "ctgroup" ]; then
		local dst_country=${rule_acl_inner_dest#*:}
		set_dst_rule="-m geoip --dst-cc $dst_country"
	elif [ "$rule_acl_inner_dest" != "" ] && [ "$rule_acl_inner_dest" != "IPGROUP_ANY" ];then
		local ipset_dst=${rule_acl_inner_dest%_REV}
		if [ $ipset_dst != "Me" ];then
			if [[ "$ipset_dst" == "$rule_acl_inner_dest" ]]; then
				set_dst_rule="-m set --match-set $ipset_dst dst"
			else
				set_dst_rule="-m set ! --match-set $ipset_dst dst"
			fi
			set_dst_rule="$set_dst_rule,dst"
		fi
		#echo "[rule_acl_inner_dest]set_dst_rule=$set_dst_rule"
	fi
	if [ "$rule_acl_inner_destIfName" != "" ];then
		if [ "$rule_acl_inner_wireless_flag" == "2" -o "$rule_acl_inner_wireless_flag" == "3" ]; then
			set_dst_rule="-m bridgeport --multidports $rule_acl_inner_destIfName"
		else
			local dst_lan_ifname=${rule_acl_inner_destIfName%_REV}
			if [ $dst_lan_ifname != "Me" ];then
				if [ "$dst_lan_ifname" == "$rule_acl_inner_destIfName" ];then
					set_dst_rule="-o $dst_lan_ifname"
				else
					set_dst_rule="! -o $dst_lan_ifname"
				fi
			fi
		fi
		#echo "[rule_acl_inner_destIfName]set_dst_rule=$set_dst_rule"
	fi
	echo $set_dst_rule
}

add_fqdn_rule() 
{
    ## fqdn暂时仅支持IPv4 ##
	local accesss_ctl_chain=$1 
	local direction_rules=$2 
	local set_src_rule=$3
	local rule_acl_inner_name=$4 
	local rule_acl_inner_policy=$5 
	local rule_acl_inner_policy_rev=$6
	local service_type=$7
	local dst_domains=$(uci get access_ctl.$rule_acl_inner_name.dst_domain 2>/dev/null)
	for dst_domain in ${dst_domains};do
		local num_fqdn_group=$(uci get fqdn.$dst_domain.option_nums 2>/dev/null)
		num_fqdn_group=$((num_fqdn_group))
		while [ $num_fqdn_group -ge 1 ];do
			local fqdn_ipset_name=$(uci get fqdn.$dst_domain.name_$num_fqdn_group 2>/dev/null)
			if [ $service_type == "ALL" ]; then
				local fqdn_ports_name=$(uci get fqdn.$dst_domain.port_$num_fqdn_group 2>/dev/null)
				fqdn_ports_name=$(echo $fqdn_ports_name|tr '-' ':')
				local set_dst_rule="-m set --match-set $fqdn_ipset_name dst -m multiport --dports $fqdn_ports_name"
				local set_dst_rule_rev="-m set --match-set $fqdn_ipset_name dst -m multiport ! --dports $fqdn_ports_name"
				local set_dst_rule_all="-m set --match-set $fqdn_ipset_name dst"
				# 所有支持配置port的协议：tcp/UDP/DCCP(33)/SCTP(132)/UDP-Lite(136)
				local protos="tcp udp 33 132 136"
				for proto in $protos; do
					##port
					local rule="$direction_rules -p $proto $set_src_rule $set_dst_rule -m comment --comment \
						${rule_acl_inner_name}_1acl"
					
					fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
					[ x$? == x0 ] && {
						ACL_LOG "[FQDN rule] add rule : iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}"
						iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}
					}

					# iptables -w -A $accesss_ctl_chain $direction_rules -p $proto $set_src_rule $set_dst_rule -m comment --comment \
					# ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}
					
					# ACL_LOG "[FQDN rule] add rule :iptables -w -A $accesss_ctl_chain $direction_rules -p $proto $set_src_rule \ 
					# $set_dst_rule -m comment --comment ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}"
					
					## !port
					rule="$direction_rules -p $proto $set_src_rule $set_dst_rule_rev -m comment \
						--comment ${rule_acl_inner_name}_1acl"
					
					fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
					[ x$? == x0 ] && {
						ACL_LOG "[FQDN rule_rev] add rule : iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy_rev}"
						iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy_rev}
					}
					# iptables -w -A $accesss_ctl_chain $direction_rules -p $proto $set_src_rule $set_dst_rule_rev -m comment \
					# --comment ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy_rev}
					
					# ACL_LOG "[FQDN rule_rev] add rule :iptables -w -A $accesss_ctl_chain $direction_rules -p $proto $set_src_rule \
					# $set_dst_rule_rev -m comment --comment ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy_rev}"
				done
				## ALL
				local rule="$direction_rules $set_src_rule $set_dst_rule_all -m comment --comment \
					${rule_acl_inner_name}_1acl"
				
				fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
				[ x$? == x0 ] && {
					ACL_LOG "[FQDN rule_all] add rule : iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}"
					iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}
				}

				# iptables -w -A $accesss_ctl_chain $direction_rules $set_src_rule $set_dst_rule_all -m comment --comment \
				# ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}

				# ACL_LOG "[FQDN rule_all] add rule :iptables -w -A $accesss_ctl_chain $direction_rules $set_src_rule \
				# $set_dst_rule_all -m comment --comment ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}"

			else	#service_type=proto
				if [ "$service_type" == "6" -o "$service_type" == "17" -o "$service_type" == "33" -o "$service_type" == "132" -o "$service_type" == "136" ]; then
					local fqdn_ports_name=$(uci get fqdn.$dst_domain.port_$num_fqdn_group 2>/dev/null)
					fqdn_ports_name=$(echo $fqdn_ports_name|tr '-' ':')
					local set_dst_rule="-m set --match-set $fqdn_ipset_name dst -m multiport --dports $fqdn_ports_name"
					local proto=$service_type
					local rule="$direction_rules -p $proto $set_src_rule $set_dst_rule -m comment --comment \
						${rule_acl_inner_name}_1acl"
					fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
					[ x$? == x0 ] && {
						echo "[FQDN service_type=proto] add rule : iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}" > /dev/console
						iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}
					}

					# iptables -w -A $accesss_ctl_chain $direction_rules -p $proto $set_src_rule $set_dst_rule -m comment --comment \
					# ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}
				else
					local proto=$service_type
					local set_dst_rule="-m set --match-set $fqdn_ipset_name dst"
					local rule="$direction_rules -p $proto $set_src_rule $set_dst_rule -m comment --comment \
						${rule_acl_inner_name}_1acl"
					
					fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
					[ x$? == x0 ] && {
						echo "[FQDN NONE] add rule : iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}" > /dev/console
						iptables -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}
					}				
					# iptables -w -A $accesss_ctl_chain $direction_rules -p $proto $set_src_rule $set_dst_rule -m comment --comment \ 
					# ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}
				fi
			fi
			num_fqdn_group=$(($num_fqdn_group-1))
		done
	done
}

service_config_get() {
	fw_config_get_section "$1" service { \
		string name "" \
		string proto "" \
		string sport "" \
		string dport "" \
		string type "" \
		string code "" \
		string comment "" \
		string flag "" \
	} || return
}

fw_acl_do_acl_inner_rule()
{
	local PREFIX="rule_acl_inner_"
	local rule_item="service policy policy_rev action src srcIf srcIfName time dest destIf destIfName zone flag name wireless_flag \
	dst_domain_flag syslogEnable v6flag"

    # local IPTABLES_CMD="iptables"
    # [ "$rule_acl_inner_v6flag" == "1" ] && IPTABLES_CMD="ip6tables"
    
	local loop=0
	local destroy_flag=$2
	local vnet_prefix=0
	local LAN_INF="lan_iface"
	local result=$(uci get access_ctl.@rule_acl_inner[$loop].name 2>/dev/null)

	while [ x$result != 'x' ];do
		loop=$(($loop+1))
		result=$(uci get access_ctl.@rule_acl_inner[$loop].name 2>/dev/null)
	done

	loop=$(($loop-1))

	while [ $loop -ge 0 ];do
		for item in $rule_item; do
			eval $PREFIX$item=\"$(uci get access_ctl.@rule_acl_inner[$loop].$item 2>/dev/null)\"
			echo "$PREFIX$item = `eval echo '$'$PREFIX$item`"
		done
		local IPTABLES_CMD="iptables"
    	[ "$rule_acl_inner_v6flag" == "1" ] && IPTABLES_CMD="ip6tables"
		#以上获取了rule_acl_inner_*的变量值
		rule_acl_inner_policy=$(echo $rule_acl_inner_policy|tr '[a-z]' '[A-Z]')
		rule_acl_inner_action=$rule_acl_inner_policy
		([ "$rule_acl_inner_destIf" == "Me" ] || [ "$rule_acl_inner_destIf" == "Me_REV" ]) && rule_acl_inner_destIfName="$rule_acl_inner_destIf"
        #LAN2LAN处理与VNET一致
		[ "$rule_acl_inner_zone" == "LAN2LAN" ] && rule_acl_inner_zone="VNET"

		json_load "`cat $acl_device_info_file`"
		json_get_var mic_type mic_type
		[ "$mic_type" == "" ] && acl_log=1
        if [ x$rule_acl_inner_syslogEnable == x"1" -a x$acl_log != x"1" ];then
            rule_acl_inner_log_params="--log $rule_acl_inner_name"
        else
            rule_acl_inner_log_params=""
        fi
		##service_name get
		service_config_get $rule_acl_inner_service
		states=$(uci get access_ctl.@rule_acl_inner[$loop].states 2>/dev/null)
		extmark=$(get_extmark_info $states)

        case "$rule_acl_inner_policy" in
            ACCEPT)
                rule_acl_inner_policy=" CTSTATEMARK --ctstate-accept $extmark/$CTMARK_shift  $rule_acl_inner_log_params"
				rule_acl_inner_policy_rev=" CTSTATEMARK --ctstate-drop $extmark/$CTMARK_shift "
            ;;
            DROP)
				rule_acl_inner_policy=" CTSTATEMARK --ctstate-drop $extmark/$CTMARK_shift  $rule_acl_inner_log_params"
				rule_acl_inner_policy_rev=" CTSTATEMARK --ctstate-accept $extmark/$CTMARK_shift "
            ;;
        esac

		
		[ "$rule_acl_inner_flag" == "1" ] && {
			if [ $destroy_flag == 1 ];then
				ACL_LOG "add rule ${loop}"
				local set_src_rule=""
				local set_dst_rule=""
				[ "$rule_acl_inner_time" == "Any" ] && {
				## 非any场景在其他路径处理 ##	
					local interface=""
					local device=""
					
					for interface in ${rule_acl_inner_zone};do

						set_src_rule=$(get_src_rule "$rule_acl_inner_src" "$rule_acl_inner_srcIfName" "$rule_acl_inner_wireless_flag")
						set_dst_rule=$(get_dst_rule "$rule_acl_inner_dest" "$rule_acl_inner_destIfName" "$rule_acl_inner_wireless_flag")

						local xyy="eth0"
						local accesss_ctl_chain=""
						local me_flag="0"
						([ "$rule_acl_inner_dest" == "Me" ] || [ "$rule_acl_inner_destIfName" == "Me" ]) && me_flag="1"
						[ "$me_flag" == "1" ] && accesss_ctl_chain="access_ctl_me"
						[ "$me_flag" != "1" ] && accesss_ctl_chain="access_ctl"
						local zone_in_flag=`acl_zone_in_flag $interface`
						[ $zone_in_flag == "VPN" ] && xyy=$(zone_get_wan_devices $interface)

						for device in $xyy;do
							local direction_rules=""
							[ -z "$device" ] && continue
							#direction_rules
							if [ $interface == "VNET" ];then
								direction_rules="-m set --match-set $LAN_INF src -m set --match-set $LAN_INF dst"
								[ "$me_flag" == "1" ] && direction_rules="-m set --match-set $LAN_INF src"
							elif [ $interface != "ALL" ];then
								if [ $interface == "LAN" ];then
									direction_rules="-m set --match-set $LAN_INF src -m set ! --match-set $LAN_INF dst"
									[ "$me_flag" == "1" ] && direction_rules="-m set --match-set $LAN_INF src"
								else
									[ -z "$device" ] && return
									if [ $zone_in_flag == "VPN" ];then
										direction_rules="-i $device"
									elif [ $zone_in_flag == "WAN" ];then
										direction_rules="-m set --match-set `acl_wan_zone_convert $interface` src"
									fi
								fi
							fi
							echo "direction_rules=$direction_rules"
							if [ "$service_name" == "ALL" ]; then #all
								if [ "$rule_acl_inner_dst_domain_flag" == "1" ]; then
									add_fqdn_rule "$accesss_ctl_chain" "$direction_rules" "$set_src_rule" "$rule_acl_inner_name" \
									"$rule_acl_inner_policy" "$rule_acl_inner_policy_rev" "ALL"
								else
									local rule="$direction_rules $set_src_rule $set_dst_rule -m comment \
										--comment ${rule_acl_inner_name}_1acl"
									
									fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
									[ x$? == x0 ] && {
										ACL_LOG "[position 12] add rule : $IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}"
										$IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}
									}

									# $IPTABLES_CMD -w -A $accesss_ctl_chain $direction_rules $set_src_rule $set_dst_rule -m comment \
									# --comment ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}

									# ACL_LOG "[position 12] add rule :$IPTABLES_CMD -w -A $accesss_ctl_chain $direction_rules \
									# $set_src_rule $set_dst_rule -m comment --comment ${rule_acl_inner_name}_1acl -j \
									# ${rule_acl_inner_policy}"
								fi
							else
								echo "enter into non-all branch"
								local SERVICE_PREFIX="service_"
								local service_item="name sport dport proto type code"
								for i in $rule_acl_inner_service; do
									for j in $service_item; do
										#service_name service_sport service_dport service_proto service_type service_code
										eval $SERVICE_PREFIX$j=$(uci get service.$i.$j 2>/dev/null)
										echo "$SERVICE_PREFIX$j = `eval echo '$'$SERVICE_PREFIX$j`"
									done 

									service_name=$(echo $service_name | tr '[a-z]' '[A-Z]')
									service_proto=$(echo $service_proto | tr '[A-Z]' '[a-z]')
					
									if [ "$service_proto" != "icmp" ]; then
										# for UDP/TCP
										if [ "$service_proto" == "tcp" -o "$service_proto" == "udp" -o "$service_proto" == "tcp-udp" ];then 
											[ "$service_proto" == "tcp-udp" ] && service_proto="tcp udp"
											echo "service_proto=$service_proto"
											service_sport=$(echo $service_sport|tr '-' ':')
											service_dport=$(echo $service_dport|tr '-' ':')
											for proto in $service_proto; do
												local rule="$direction_rules -p $proto -m $proto --sport \
													$service_sport --dport $service_dport $set_src_rule $set_dst_rule -m comment \
													--comment ${rule_acl_inner_name}_1acl"
												
												fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
												[ x$? == x0 ] && {
													ACL_LOG "[position 13] add rule : $IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}"
													$IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}
												}
												# $IPTABLES_CMD -w -A $accesss_ctl_chain $direction_rules -p $proto -m $proto --sport \
												# $service_sport --dport $service_dport $set_src_rule $set_dst_rule -m comment \
												# --comment ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}

												# ACL_LOG "[position 13] add rule :$IPTABLES_CMD -w -A $accesss_ctl_chain $direction_rules \
												# -p $proto -m $proto --sport $service_sport --dport $service_dport $set_src_rule \
												# $set_dst_rule -m comment --comment ${rule_acl_inner_name}_1acl -j \
												# ${rule_acl_inner_policy}"
											done
										else  # for other
											echo "enter into proto other branch"
											if [ "$rule_acl_inner_dst_domain_flag" == "1" ]; then
												add_fqdn_rule "$accesss_ctl_chain" "$direction_rules" "$set_src_rule" \
												"$rule_acl_inner_name" "$rule_acl_inner_policy" "$rule_acl_inner_policy_rev" "$i"
											else
												for proto in $service_proto; do
												local rule="$direction_rules -p $proto $set_src_rule \
													$set_dst_rule -m comment --comment ${rule_acl_inner_name}_1acl"
												
												fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
												[ x$? == x0 ] && {
													ACL_LOG "[position 5] add rule : $IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}"
													$IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}
												}

													# $IPTABLES_CMD -w -A $accesss_ctl_chain $direction_rules -p $proto $set_src_rule \
													# $set_dst_rule -m comment --comment ${rule_acl_inner_name}_1acl -j \
													# ${rule_acl_inner_policy}

													# ACL_LOG "[position 5] add rule :$IPTABLES_CMD -w -A $accesss_ctl_chain \
													# $direction_rules -p $proto $set_src_rule $set_dst_rule -m comment --comment \
													# ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}"
												done	
											fi 
										fi
									else  # for icmp
										if [ "$service_name" == "ICMP_ALL" ]; then #icmp_all
											local rule="$direction_rules -p $service_proto --icmp-type any \
												$set_src_rule $set_dst_rule -m comment --comment ${rule_acl_inner_name}_1acl"
											
											fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
											[ x$? == x0 ] && {
												ACL_LOG "[position 6] add rule : $IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}"
												$IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}
											}

											# $IPTABLES_CMD -w -A $accesss_ctl_chain $direction_rules -p $service_proto --icmp-type any \
											# $set_src_rule $set_dst_rule -m comment --comment ${rule_acl_inner_name}_1acl \
											# -j ${rule_acl_inner_policy}

											# ACL_LOG "[position 6] add rule :$IPTABLES_CMD -w -A $accesss_ctl_chain $direction_rules -p \
											# $service_proto --icmp-type any $set_src_rule $set_dst_rule -m comment --comment \
											# ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}"
										else # icmp service_type & service_code
											local rule="$direction_rules -p $service_proto -m $service_proto \
												--icmp-type $service_type/$service_code $set_src_rule $set_dst_rule -m comment \
												--comment ${rule_acl_inner_name}_1acl"
											
											fw_check "filter" "$accesss_ctl_chain" "$rule -j ${rule_acl_inner_policy}"
											[ x$? == x0 ] && {
												ACL_LOG "[position 7] add rule : $IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}"
												$IPTABLES_CMD -w -I $accesss_ctl_chain 1 $rule -j ${rule_acl_inner_policy}
											}

											# $IPTABLES_CMD -w -A $accesss_ctl_chain $direction_rules -p $service_proto -m $service_proto \
											# --icmp-type $service_type/$service_code $set_src_rule $set_dst_rule -m comment \
											# --comment ${rule_acl_inner_name}_1acl -j ${rule_acl_inner_policy}
											
											# ACL_LOG "[position 7] add rule :$IPTABLES_CMD -w -A $accesss_ctl_chain $direction_rules -p \
											# $service_proto -m $service_proto --icmp-type $service_type/$service_code \
											# $set_src_rule $set_dst_rule -m comment --comment ${rule_acl_inner_name}_1acl \
											# -j ${rule_acl_inner_policy}"
										fi
									fi
								done
							fi
						done
					done
				}
			elif [ $destroy_flag == 2 ];then
				ACL_LOG "delelte rule ${loop}, actually do nothing"
				[ "$rule_acl_inner_time" != "Any" ] && del_timeobj $rule_acl_inner_time $rule_acl_inner_name
			else
				ACL_LOG "destroy_flag is not 1 or 2"
				return 1
			fi
			anete conntrack -F
		}
		loop=$(($loop-1))
		# result=$(uci get access_ctl.@rule_acl_inner[$loop].name 2>/dev/null)
	done
}

fw_acl_load_rules()
{
	#INPUT
	if ! iptables -w -C remote_mngt_chain -j access_ctl_me 2>/dev/null; then
		iptables -w -N access_ctl_me
		iptables -w -I remote_mngt_chain 1 -j access_ctl_me
	else
		iptables -w -F access_ctl_me
	fi
	#IPv4-FORWARD
	if ! iptables -w -C forwarding_rule -j access_ctl 2>/dev/null; then
		iptables -w -N access_ctl
		iptables -w -I forwarding_rule 1 -j access_ctl
	else
		iptables -w -F access_ctl
	fi
    #IPv6-FORWARD   
    if ! ip6tables -w -C forwarding_rule -j access_ctl 2>/dev/null; then
		ip6tables -w -N access_ctl
		ip6tables -w -I forwarding_rule 1 -j access_ctl
	else
		ip6tables -w -F access_ctl
	fi

	ACL_LOG "start load iptables"

	fw_acl_do_acl_inner_rule rule_acl_inner 1
	lua /lib/access_ctl/attach_timeobj.lua
	ACL_LOG "stop"
}

fw_acl_clear()
{
	# original stack
	if iptables -w -C remote_mngt_chain -j access_ctl_me 2>/dev/null; then
		iptables -w -F access_ctl_me
	fi
	if iptables -w -C forwarding_rule -j access_ctl 2>/dev/null; then
		iptables -w -D forwarding_rule -j access_ctl
		iptables -w -F access_ctl
		iptables -w -X access_ctl
	fi
    if ip6tables -w -C forwarding_rule -j access_ctl 2>/dev/null; then
		ip6tables -w -F access_ctl
		ip6tables -w -D forwarding_rule -j access_ctl
		ip6tables -w -X access_ctl
	fi
        
	ACL_LOG "free access_ctl rule"
	fw_acl_do_acl_inner_rule rule_acl_inner 2
	ACL_LOG "free access_ctl rule stop"

}

