#!/bin/sh
. /lib/functions.sh
. /lib/access_ctl/markdef.sh

local acl_debug=0
local acl_config="customAcl"
local acl_section_type_v4="acl_ipv4"
local acl_section_type_v6="acl_ipv6"
local acl_tmp_dir="/tmp/access_ctl/"
local acl_inner_lock="$acl_tmp_dir""acl_inner.lock"
local acl_outer_lock="$acl_tmp_dir""acl_outer.lock"
local acl_log_file="$acl_tmp_dir""access_ctl_v2.log"
local acl_hitcount_file="$acl_tmp_dir""hit_count"
local fqdn_proc_path="/proc/fqdn"

local acl_forward_chain="acl_custom_forward"
local acl_input_chain="acl_custom_input"
local acl_forward_chain_v6="acl_custom_forward_v6"
local acl_input_chain_v6="acl_custom_input_v6"

local LAN_IFACE="lan_iface"

#acl_forward_chain ---> {direction:["outbound","inbound",""], iptype:"ipv4"}
#acl_forward_chain_v6 ---> {direction:["outbound","inbound",""], iptype:"ipv6"}
#acl_input_chain ---> {direction:["local"], iptype:"ipv4"}
#acl_input_chain_v6 ---> {direction:["local"], iptype:"ipv6"}

local g_acl_rule_name=""

local g_acl_position_count=0
local g_acl_position_count_tmp=0

acl_log()
{
    [ "$acl_debug" -eq 0 ] && return;
    case $acl_debug in 
        log)
            [ ! -e "$acl_tmp_dir" ] && mkdir "$acl_tmp_dir"
            [ -f "$acl_log_file" ] && {
                local size="`ls -l "$acl_log_file" | awk '{ print $5 }'`"
                if [ $size -gt 3072000 ]; then
                    rm "$acl_log_file"
                fi
            }
            [ ! -f "$acl_log_file" ] && touch "$acl_log_file"
            echo "`date +[%m/%d-%H:%M:%S]` --- $*" >> "$acl_log_file"
        ;;
        console)
            echo "`date +[%m/%d-%H:%M:%S]` --- $*" > /dev/console
        ;;
    esac
}

_acl_target()
{
    local rule=""
    local log=`_acl_syslog`
    local extmark=""
    local action=""
    config_get states $g_acl_rule_name states
    config_get policy $g_acl_rule_name policy
    config_get description $g_acl_rule_name description
    if [ "$policy" == "0" ];then
        action="DROP"
    elif [ "$policy" == "1" ];then
        action="ACCEPT"
    fi

    if [ x$log == x"1" ];then
        description=${description//\'/\'\\\'\'}
        log_param="--log '$description'"
    fi

    if [ x$states == 'x' ];then
        extmark=$CTMARK_DEF
    else
        case "$states" in
            *new*)
                extmark=$(($extmark + $CTMARK_NEW))
            ;;
        esac
        case "$states" in
            *related*)
                extmark=$(($extmark + $CTMARK_RELATED))
            ;;
        esac
        case "$states" in
            *established*)
                extmark=$(($extmark + $CTMARK_ESTBLISHED))
            ;;
        esac
        case "$states" in
            *invalid*)
                extmark=$(($extmark + $CTMARK_INVALID))
            ;;
        esac
    fi

    case "$action" in
        ACCEPT)
            policy=" CTSTATEMARK --ctstate-accept $extmark/$CTMARK_shift $log_param"
        ;;
        DROP)
            policy=" CTSTATEMARK --ctstate-drop $extmark/$CTMARK_shift $log_param"
        ;;
    esac

    acl_log "stop get other info for rule $g_acl_rule_name extmark=$extmark $policy"

    rule=" -j $policy "
    acl_log "_acl_target=$rule"
    echo "$rule"
}

_acl_chain()
{
    local rule=""
    #namespace #iptype #chain
    config_get iptype $g_acl_rule_name iptype
    config_get directions $g_acl_rule_name directions

    if [ "$directions" == "3" ];then  #0:LAN-LAN, 1:LAN-WAN, 2:WAN-LAN, 3:LOCAL
        if [ "$iptype" == "ipv4" ];then
            rule="$acl_input_chain"
        elif [ "$iptype" == "ipv6" ];then
            rule="$acl_input_chain_v6"
        else
            acl_log "iptype invalid error."
        fi
    else
        if [ "$iptype" == "ipv4" ];then
            rule="$acl_forward_chain"
        elif [ "$iptype" == "ipv6" ];then
            rule="$acl_forward_chain_v6"
        else
            acl_log "iptype invalid error."
        fi   
    fi

    #acl_log "_acl_chain=$rule"
    echo "$rule"
}

_acl_direction()
{
    local rule=""
    config_get directions $g_acl_rule_name directions
    #0:LAN-LAN, 1:LAN-WAN, 2:WAN-LAN, 3:LOCAL
    case $directions in 
    3)
        rule=""
    ;;
    0)
        rule="-m set --match-set $LAN_IFACE src -m set --match-set $LAN_IFACE dst"
    ;;
    1)
        rule="-m set --match-set $LAN_IFACE src -m set ! --match-set $LAN_IFACE dst"
    ;;
    2)
        rule="-m set ! --match-set $LAN_IFACE src -m set --match-set $LAN_IFACE dst"
    ;;
    *)
        acl_log "[_acl_direction] directions not valid"
    ;;
    esac

    acl_log "_acl_direction=$rule"
    echo "$rule"
}

_acl_count_position()
{
    local section_name="$1"
    local directions="$2"
    local cur_num="" 
    local chain=`_acl_chain`
    config_get tmp_direction "$section_name" directions
    config_get status "$section_name" status
    if [ "$tmp_direction" == "3" -a "$directions" == "3" -a "$status" == "1" ] || 
        [ "$tmp_direction" != "3" -a "$directions" != "3" -a "$status" == "1" ];then #other condition to be added.
        #ret=`iptables -w -L $chain | grep "$section_name"`
        #[ -n "$ret" ] && let g_acl_position_count_tmp++
	let g_acl_position_count_tmp++
        if [ "$section_name" == "$g_acl_rule_name" ];then
            #let g_acl_position_count_tmp++
            g_acl_position_count=$g_acl_position_count_tmp

            cur_num=`iptables -w -L $chain| tail -n +3 | wc -l`
            if [ $g_acl_position_count -gt $cur_num ];then
                g_acl_position_count=0
            fi
        fi
    fi
}

_acl_position()
{
    local rule=""
    #namespace #iptype #chain
    config_get iptype $g_acl_rule_name iptype
    config_get directions $g_acl_rule_name directions
    g_acl_position_count=0
    g_acl_position_count_tmp=0
    if [ "$iptype" == "ipv6" ];then
        config_foreach _acl_count_position "$acl_section_type_v6" "$directions"
    else
        config_foreach _acl_count_position "$acl_section_type_v4" "$directions"
    fi

    rule="$g_acl_position_count"
    acl_log "_acl_position=$rule"
    echo "$rule"
}

_acl_protocol()
{
    local rule=""
    local src_port=""
    local dst_port=""
    config_get service $g_acl_rule_name service
    config_get sourcePort $g_acl_rule_name sourcePort
    config_get destinationPort $g_acl_rule_name destinationPort
    [ -n "$service" -a "$service" != "256" ] && rule=" -p $service " #ALL:256
    if [ "$service" == "6" -o "$service" == "17" ];then  #TCP/UDP
        for port in ${sourcePort}
        do
            if [ -n "$(echo $port | grep -E '^[1-9][0-9]{0,4}$')" ];then
                [ ! -n "$src_port" ] && src_port="$port" || src_port="$src_port,$port"
            elif [ -n "$(echo $port | grep -E '^[1-9][0-9]{0,4}-[1-9][0-9]{0,4}$')" ];then
                [ ! -n "$src_port" ] && src_port="${port/-/:}" || src_port="$src_port,${port/-/:}"
            else
                acl_log "src_port:$port not valid"
            fi    
        done
        [ -n "$src_port" ] && rule="$rule -m multiport --source-ports $src_port "

        for port in ${destinationPort}
        do
            if [ -n "$(echo $port | grep -E '^[1-9][0-9]{0,4}$')" ];then
                [ ! -n "$dst_port" ] && dst_port="$port" || dst_port="$dst_port,$port"
            elif [ -n "$(echo $port | grep -E '^[1-9][0-9]{0,4}-[1-9][0-9]{0,4}$')" ];then
                [ ! -n "$dst_port" ] && dst_port="${port/-/:}" || dst_port="$dst_port,${port/-/:}"
            else
                acl_log "dst_port:$port not valid"
            fi    
        done
        [ -n "$dst_port" ] && rule="$rule -m multiport --destination-ports $dst_port "

    fi

    acl_log "_acl_protocol=$rule"
    echo "$rule"
}

_acl_ipset_check()
{
    local name=$1
    local ns=$2
    $ns ipset list $1 &>/dev/null
    echo "$?"
}

_acl_ipset_entry_check()
{
    local name=$1
    local entry=$2
    local ns=$3
    $ns ipset test $1 $2 &>/dev/null
    echo "$?"
}

_create_ipset_net() #$1=name
{
    local name=$1
    local ret=`_acl_ipset_check $name`
    if [ "$ret" == "0" ];then
        acl_log "[ipset] $name already exist."
        return
    else
        ret=`ipset create $name hash:net 2>&1`
        if [ -n "$ret" ];then
            acl_log "[ipset]$name:$ret"
        fi
    fi
}

_create_ipset_domain() #$1=name
{
    local name="$1"
    local ret=`_acl_ipset_check $name`
    if [ "$ret" == "0" ];then
        acl_log "[ipset] $name already exist."
        return
    else
        if [ "${name:0:1}" == "*" ]; then
            ret=`ipset create $name hash:ip timeout 900 maxelem 1024 2>&1`
        else
            ret=`ipset create $name hash:ip timeout 900 maxelem 256 2>&1`
        fi

        if [ -n "$ret" ];then
            acl_log "[ipset]$name:$ret"
            return
        fi
        _create_domain_in_kernel "$name"
    fi
}

_create_ipset_list_set() #$1=name
{
    local name="$1"
    local ret=`_acl_ipset_check $name`
    if [ "$ret" == "0" ];then
        acl_log "[ipset] $name already exist."
        return
    else
        ret=`ipset create $name list:set 2>&1`
        if [ -n "$ret" ];then
            acl_log "[ipset]$name:$ret"
        fi
    fi
}

_ipset_add_entry()  #$1=name $2=entry
{
    local name="$1"
    local entry="$2"

    ret=`_acl_ipset_entry_check $name $entry`
    if [ "$ret" == "0" ];then
        acl_log "[ipset] $name:$entry already exist."
        return
    else
        ret=`ipset add $name $entry 2>&1`
        if [ -n "$ret" ];then
            acl_log "[ipset]$name:$ret"
        fi
    fi
}

_destroy_ipset()
{
    local name="$1"
    local ret=`_acl_ipset_check $name`
    if [ "$ret" != "0" ];then
        acl_log "[ipset] $name does not exist."
        return
    else
        ret=`ipset destroy $name 2>&1`
        if [ -n "$ret" ];then
            acl_log "[ipset]$name:$ret"
            return
        fi
        _destroy_domain_in_kernel "$name"
    fi
}

_nslookup_domain()
{
    local domain="$1"
    if [ "${domain:0:1}" == "*" ];then
        nslookup "${domain:2}"
    else
        nslookup "$domain"
    fi
}

_create_domain_in_kernel()
{
    #("echo \"%d %d %s\" > %s",tonumber(operation),len,cmd_str,fqdn_proc_path)
    echo 1 1 "$1" > $fqdn_proc_path
    _nslookup_domain "$1"&
}

_destroy_domain_in_kernel()
{
    #("echo \"%d %d %s\" > %s",tonumber(operation),len,cmd_str,fqdn_proc_path)
    echo 2 1 "$1" > $fqdn_proc_path
}

_acl_handle_relevant_item_by_direction() #$1=src/dst #2=create/destroy
{
    local ipset_name=""
    local ipsetlist_name=""

    case $1 in 
    src)
        config_get src_rule $g_acl_rule_name source
        [ ! -n "$src_rule" ] && return

        array=${src_rule//,/ }
        ipsetlist_name=`_acl_ipset_match_name src`
    ;;
    dst)
        config_get dst_rule $g_acl_rule_name destination
        [ ! -n "$dst_rule" ] && return

        array=${dst_rule//,/ }
        ipsetlist_name=`_acl_ipset_match_name dst`
    ;;
    *)
        return
    ;;
    esac

    case $2 in 
    create)
        _create_ipset_list_set "$ipsetlist_name"

        for value in ${array}
        do
            match=$(echo "$value" | grep -E '^(((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5]))\.){3}((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5]))(\/(([1-9])|([1-2]\d)|(3[0-2]))){0,1}$')  #ipv4
            if [ "$match" != "" ];then
                if [ "$ipset_name" == "" ];then
                    ipset_name="acl_${g_acl_rule_name}_net_$1"
                    _create_ipset_net "$ipset_name"
                    _ipset_add_entry "$ipsetlist_name" "$ipset_name"
                fi
                _ipset_add_entry "$ipset_name" "$value"
                continue
            fi

            match=$(echo "$value" | grep -E '^(\*\.){0,1}([a-zA-Z0-9]([-a-zA-Z0-9]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,6}$')  #domain
            if [ "$match" != "" ];then
                _create_ipset_domain "$value"
                _ipset_add_entry "$ipsetlist_name" "$value"
                continue
            fi

            match=$(echo "$value" | grep -E '^([0-9a-fA-F]{0,4}:){1,7}[0-9a-fA-F]{0,4}$') #ipv6
            if [ "$match" != "" ];then
                #TODO
                continue
            fi
        done
    ;;
    destroy)  
        _destroy_ipset "$ipsetlist_name"

        for value in ${array}
        do
            match=$(echo "$value" | grep -E '^(((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5]))\.){3}((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5]))(\/(([1-9])|([1-2]\d)|(3[0-2]))){0,1}$')  #ipv4
            if [ "$match" != "" ];then
                if [ "$ipset_name" == "" ];then
                    ipset_name="acl_${g_acl_rule_name}_net_$1"
                    _destroy_ipset "$ipset_name"
                fi
                continue
            fi

            match=$(echo "$value" | grep -E '^(\*\.){0,1}([a-zA-Z0-9]([-a-zA-Z0-9]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,6}$')  #domain
            if [ "$match" != "" ];then
                _destroy_ipset "$value"
                continue
            fi

            match=$(echo "$value" | grep -E '^([0-9a-fA-F]{0,4}:){1,7}[0-9a-fA-F]{0,4}$') #ipv6
            if [ "$match" != "" ];then
                #TODO
                continue
            fi
        done
    ;;
    *)
        return
    ;;
    esac
}

_acl_create_relevant_item()
{
    _acl_handle_relevant_item_by_direction src create
    _acl_handle_relevant_item_by_direction dst create
}

_acl_destroy_relevant_item()
{
    _acl_handle_relevant_item_by_direction src destroy
    _acl_handle_relevant_item_by_direction dst destroy
}

_acl_ipset_match_name()
{
    local ipsetlist_name=""
    case $1 in 
    src)
        ipsetlist_name="acl_${g_acl_rule_name}_src"
        echo "$ipsetlist_name"
    ;;
    dst)
        ipsetlist_name="acl_${g_acl_rule_name}_dst"
        echo "$ipsetlist_name"
    ;;
    *)
        return
    ;;
    esac  
}

_acl_ipset_match_rule()
{
    local rule=""
    
    case $1 in 
    src)
        config_get src_rule $g_acl_rule_name source
        [ ! -n "$src_rule" ] && return

        rule=`_acl_ipset_match_name src`
        [ -n "$rule" ] && echo " -m set --match-set $rule src,src "
    ;;
    dst)
        config_get dst_rule $g_acl_rule_name destination
        [ ! -n "$dst_rule" ] && return

        rule=`_acl_ipset_match_name dst`
        [ -n "$rule" ] && echo " -m set --match-set $rule dst,dst "
    ;;
    *)
        return
    ;;
    esac  
}

_acl_src_rule()
{
    local rule=""
    rule=`_acl_ipset_match_rule src`

    acl_log "_acl_src_rule=$rule"
    echo "$rule"
}

_acl_dst_rule()
{
    local rule=""
    rule=`_acl_ipset_match_rule dst`

    acl_log "_acl_dst_rule=$rule"
    echo "$rule"
}

_acl_syslog()
{
    local rule=""
    config_get log $g_acl_rule_name syslogEnable
    rule=$log
    acl_log "syslog=$rule"
    echo "$rule"
}

_acl_hit_count_write_each()
{   
    g_acl_rule_name="$1"
    local iptables_cmd="iptables"
    local chain=""
    config_get iptype "$g_acl_rule_name" iptype
    [ "$iptype" == "ipv6" ] && iptables_cmd="ip6tables"
    chain=`_acl_chain`

    local pkt_count=`$iptables_cmd -w -nvxL $chain | grep $g_acl_rule_name | awk '{printf $1}'`
    echo "$g_acl_rule_name,$pkt_count" >> $acl_hitcount_file
}

_acl_hit_count_write_all()
{   
    local chain_list_v4="$acl_forward_chain $acl_input_chain"
    local chain_list_v6="$acl_forward_chain_v6 $acl_input_chain_v6"

    for chain in $chain_list_v4
    do
        #iptables -w -nvxL $chain | awk 'NR>2{for(i=1;i<=NF;i++){if ($i ~/id:[0-9]+/) {gsub(/[^0-9]/, "", $i);print $i,$1;break}}}' > $acl_hitcount_file
        echo "$(iptables -w -nvxL $chain | awk 'NR>2{for(i=1;i<=NF;i++){if ($i ~/id:[0-9]+/) {gsub(/[^0-9]/, "", $i);print $i,$1;break}}}')"
        iptables -w -Z $chain
    done
    #for chain in $chain_list_v6
    #do
        #ip6tables -w -nvxL $chain | awk 'NR>2{for(i=1;i<=NF;i++){if ($i ~/id:[0-9]+/) {gsub(/[^0-9]/, "", $i);print $i,$1;break}}}' >> $acl_hitcount_file
        #echo "$(ip6tables -w -nvxL $chain | awk 'NR>2{for(i=1;i<=NF;i++){if ($i ~/id:[0-9]+/) {gsub(/[^0-9]/, "", $i);print $i,$1;break}}}')"
        #ip6tables -w -Z $chain
    #done
}

_acl_add_iptables()
{
    local rules="$*"
    local chain=""
    chain=`_acl_chain`

    acl_log "[iptables]:$rules "
    check_rule=`echo $rules | sed -n 's/ -I '"$chain"' [0-9]* / -C '"$chain"' /p'`
    [ "$check_rule" == "" ] && check_rule=`echo $rules | sed -n 's/ -A '"$chain"' / -C '"$chain"' /p'`
    acl_log check:"$check_rule"

    eval "$check_rule" 2>/dev/null
    if [ "$?" ==  "0" ];then
        acl_log "rule:$g_acl_rule_name exists, just return."
        return
    fi

    eval "$rules"
    [ "$?" !=  "0" ] && acl_log "rule:$g_acl_rule_name iptables cmd failed. "
}

_acl_del_iptables()
{
    local rules="$*"

    acl_log "[iptables]:$rules "

    local check_rule=${rules/ -D $chain/ -nvL $chain}
    local ret=`$check_rule 2>&1`

    if [ -n "$ret" ];then
        acl_log "rule:$g_acl_rule_name delete iptables:$ret"
    fi

    eval "$rules"
    [ "$?" !=  "0" ] && acl_log "rule:$g_acl_rule_name iptables cmd failed. "
}

_acl_assemble_add_rules_()
{
    local rule=""
    local target=""
    local chain=""
    local direction=""
    local position=""
    local protocol=""
    local src_rule=""
    local dst_rule=""
    local namespace=""
    local action=""
    local iptables=""
    local comment=""

    target=`_acl_target`
    chain=`_acl_chain`
    direction=`_acl_direction`
    position=`_acl_position`
    protocol=`_acl_protocol` #tcp/udp/icmp/icmpv6 [src port/dst port]
    src_rule=`_acl_src_rule`
    dst_rule=`_acl_dst_rule`
    
    if [ "$position" == "0" -o "$position" == "" ];then
        action=" -A $chain "
    else
        action=" -I $chain $position "
    fi

    config_get iptype $g_acl_rule_name iptype
    [ "$iptype" == "ipv4" ] && iptables="iptables -w "
    [ "$iptype" == "ipv6" ] && iptables="ip6tables -w "

    comment=" -m comment --comment id:${g_acl_rule_name} "

    rule=" $namespace $iptables $action $protocol $direction $src_rule $dst_rule $comment $target"
    echo "$rule"
}

_acl_assemble_del_rules_()
{
    local chain=""
    local position=""
    local action=""
    local namespace=""
    local iptables=""
    local rule=""

    chain=`_acl_chain`
    #position=`_acl_position`
    position=`iptables -w -L $chain | tail -n +3 | awk '/'$g_acl_rule_name'/{print NR}'`
    if [ -n "$position" ];then
        action=" -D $chain $position"

        config_get iptype $g_acl_rule_name iptype
        [ "$iptype" == "ipv4" ] && iptables="iptables -w "
        [ "$iptype" == "ipv6" ] && iptables="ip6tables -w "

        rule=" $namespace $iptables $action "
    fi
    echo "$rule"
}

_acl_rule_add()
{
    config_get status $1 status
    if [ "$status" != "1" ];then
        return
    fi
    g_acl_rule_name="$1"
    _acl_create_relevant_item
    local iptables_rules=`_acl_assemble_add_rules_`
    _acl_add_iptables "$iptables_rules"
}

_acl_rule_del()
{
    config_get status $1 status
    if [ "$status" != "1" ];then
        return
    fi
    g_acl_rule_name="$1"
    local mode="fast"  #accurate/multi/fast

    [ "$mode" == "fast" ] && {
        local iptables_rules=`_acl_assemble_del_rules_`
        _acl_del_iptables "$iptables_rules"
    } 

    _acl_destroy_relevant_item
}

_acl_destroy_relevant_item_byname()
{
    g_acl_rule_name="$1"
    _acl_destroy_relevant_item
}

_acl_destroy_relevant_all_items()
{
    config_foreach _acl_destroy_relevant_item_byname
}

acl_hit_count_report()
{
    {
        flock -x 5
        #config_foreach _acl_hit_count_write_each "$acl_section_type_v4"
        #config_foreach _acl_hit_count_write_each "$acl_section_type_v6"
        #_acl_hit_count_write_all
        ret=`_acl_hit_count_write_all`
        echo "$ret"
        flock -u 5
    } 5<>"$acl_outer_lock"
}

acl_hit_count_start()
{   
    {
        flock -x 5
        #TODO
        flock -u 5
    } 5<>"$acl_outer_lock"
}

acl_hit_count_clear()
{
    {
        flock -x 5
        #TODO
        flock -u 5
    } 5<>"$acl_outer_lock"
}

acl_rule_add()
{
    {
        flock -x 5
        uci_load "$acl_config"
        _acl_rule_add $1
        flock -u 5
    } 5<>"$acl_outer_lock"
}

acl_rule_del()
{
    {
        flock -x 5
        uci_load "$acl_config"
        _acl_rule_del $1
        flock -u 5
    } 5<>"$acl_outer_lock"
} 

acl_start()
{
    {
        flock -x 5
        iptables -w -C forwarding_rule -j $acl_forward_chain 2> /dev/null
        if [ "$?" != "0" ];then
            iptables -w -N $acl_forward_chain
            iptables -w -I forwarding_rule 1 -j $acl_forward_chain
        else
            iptables -w -F $acl_forward_chain
        fi
        iptables -w -C remote_mngt_chain -j $acl_input_chain 2> /dev/null
        if [ "$?" != "0" ];then
            iptables -w -N $acl_input_chain
            iptables -w -I remote_mngt_chain 1 -j $acl_input_chain
        else
            iptables -w -F $acl_input_chain
        fi

        #TODO ipv6
        uci_load "$acl_config"
        config_foreach _acl_rule_add "$acl_section_type_v4"
        #config_foreach _acl_rule_add "$acl_section_type_v6"
        flock -u 5
    } 5<>"$acl_outer_lock"
}

acl_stop()
{
    {
        flock -x 5
        uci_load "$acl_config"
        iptables -w -F $acl_forward_chain
        iptables -w -F $acl_input_chain
        #TODO ipv6
        _acl_destroy_relevant_all_items
        flock -u 5
    } 5<>"$acl_outer_lock"
}
