
local sys  = require "luci.sys"
local json = require 'luci.json'
local utl = require "luci.util"
local DBG  = require ('luci.tools.debug')
local form = require "luci.tools.form"
local uci  = require "luci.model.uci"
local switch = require "luci.controller.admin.switch"
local error_print = require "luci.tools.error_print"

local uci_r = uci.cursor()

local CFG_PATH = "/etc/config"

function show_mirror_port()
	local result = "SUCCESS"
	local data = {}
	local str_mirroredports=""
	--读取默认参数配置
	data = switch.get_mirror()
	mode = data["mode"]
	mirrorport = data["mirrorport"]
	mirroredports = data["mirroredports"]
	state = data["state"]
	DBG.print_tty("---------------------------------------------------------------------");
	DBG.print_tty("mode:"..mode)
	DBG.print_tty("Mirroring Port:"..mirrorport)
	for i,v in ipairs(mirroredports) do
		str_mirroredports = str_mirroredports.." "..v
	end
	DBG.print_tty("Mirrored Port:"..str_mirroredports)
	DBG.print_tty("state:"..state)
	DBG.print_tty("---------------------------------------------------------------------");
	return result
end

function set_mirror_config(input)
	local result = "SUCCESS"
	local data = {}
	local http_form = {}
	local mirrorport_str=""
	local mirrorredport_str=""
	
	--读取默认参数配置
	data = switch.get_mirror()
	mode = data["mode"]
	mirrorport = data["mirrorport"]
	mirroredports = data["mirroredports"]
	state = data["state"]
	--设置mirror port
	if input.mirrorport ~= nil then
		local flag = 0
		--转换为字符串
		mirrorport_str = string.format("%d",input.mirrorport)
		for i,v in ipairs(mirroredports) do
			if mirrorport_str == v then
				flag = 1 
			end
			mirrorredport_str = mirrorredport_str.." "..v
		end
		--mirrored ports包含将要设置的mirror port
		if flag == 1 then
			DBG.print_tty("Error: The port is already occupied,current mirrored ports are:"..mirrorredport_str..".")
			return "FAILED"
		end
		mirrorport = input.mirrorport
	end
	--设置mirrored port
	if input.mirroredports ~= nil and input.mirroredports ~= "" then
		--mirrorport是字符串形式
		mirrorport_str = mirrorport
		--mirror port和将要设置的mirrored port冲突
		if string.find(input.mirroredports,mirrorport) ~= nil then
			DBG.print_tty("Error: Failed to set mirrored ports.The current mirror port is:"..mirrorport_str..".")
			return "FAILED"
		end
		mirroredports = input.mirroredports
	end
	if input.no_mirroredports ~= nil and input.no_mirroredports ~= "" then
		local mirroredports_res =""
		--转换为字符串
		for i,v in ipairs(mirroredports) do
			if mirrorport_str == v then
				flag = 1 
			end
			mirrorredport_str = mirrorredport_str.." "..v
		end
		--检查是no mirroredports 是否存在
		for i=1,#input.no_mirroredports do
			if string.sub(input.no_mirroredports,i,i) ~= ' ' and string.find(mirrorredport_str,string.sub(input.no_mirroredports,i,i)) == nil then
				DBG.print_tty("Error: Failed to disable the ports. The current ports that can be disabled are:"..mirrorredport_str..".")
				return "FAILED"
			end
		end
		--保留no mirroredports后的mirroredports
		for i=1,#mirrorredport_str do
			if string.sub(mirrorredport_str,i,i) ~= ' ' and string.find(input.no_mirroredports,string.sub(mirrorredport_str,i,i)) == nil then
				mirroredports_res = mirroredports_res.." "..string.sub(mirrorredport_str,i,i)
			end
		end

		if mirroredports_res ~= "" then
			mirroredports = mirroredports_res
		else
			DBG.print_tty("Error: Mirroredports must exist at least one.The current ports that can be disabled are:"..mirrorredport_str)
			return "FAILED"
		end
	end

	if input.state ~=nil and input.state ~= "" then
		state = input.state
	end
	if input.mode ~=nil and input.mode ~= "" then
		mode = input.mode
	end
	http_form.data =json.encode(
		{
			["method"] = "set",
			["params"] = 
			{
				["mirrorport"] = mirrorport,
				["mirroredports"] = mirroredports ,
				["state"] = state,
				["mode"] = mode
			}
		}
	)
	http_form.form = "mirror"

	ret = switch.dispatch(http_form)

	if (not ret) or (ret.error_code ~= "0") then
		DBG.print_tty(error_print.check_error(ret))
		return "FAILED"
	end

	return result

end



function monitor_port(cmd,session_num,port_id)
	local result = "SUCCESS"
	local port_str =""
	local phy_port = 0
	input={}

	if cmd == "monitor session port" and port_id ~= nil then
		local ports_total = string.format("%d",#switch.get_state()+1)
		phy_port = uci_r:get("switch", "global", "phy_port_list")
		ports_total = tonumber(ports_total)
		port_id = tonumber(port_id)
		if not phy_port then
			if port_id < ports_total and port_id > 0 then
				input.mirrorport = port_id
			else
				DBG.print_tty("Error: port parameters are illegal.")
				return "FAILED"
			end
		else
			phy_port = tonumber(phy_port)
			if port_id < ports_total and port_id > 0 and phy_port ~= port_id then
				input.mirrorport = port_id
			else
				DBG.print_tty("Error: port parameters are illegal.")
				return "FAILED"
			end
		end
	end

	if cmd == "no monitor session" and session_num ~= nil then
		input.state ="off"
	end
	
	if cmd == "monitor session source port" or cmd == "no monitor session source port" then
		if port_id ~= nil then
			local ports_total = string.format("%d",#switch.get_state()+1)
			ports_total = tonumber(ports_total)
			phy_port = uci_r:get("switch", "global", "phy_port_list")
			local source_port
			for i=1,#port_id do
				source_port = string.sub(port_id,i,i)
				source_port = tonumber(source_port)
				if i%2 == 1 then 
					if not phy_port then
						if source_port > 0 and source_port < ports_total then
							port_str = port_str.." "..string.sub(port_id,i,i)
						else 
							DBG.print_tty("Error: Source port parameters are illegal.")
							return "FAILED"
						end
					else
						phy_port = tonumber(phy_port)
						if source_port > 0 and source_port < ports_total and  phy_port ~= source_port then
							port_str = port_str.." "..string.sub(port_id,i,i)
						else 
							DBG.print_tty("Error: Source port parameters are illegal.")
							return "FAILED"
						end
					end
				else 
					if string.sub(port_id,i,i) ~= ',' then
						DBG.print_tty("Error: Source port parameters are illegal.")
						return "FAILED"
					end
				end
			end
			--初始化port_table字典
			port_table={}
			for i=1,tonumber(ports_total) do
				port_table[i]= -1
			end
			--过滤port_str重复的值
			for i=1,#port_str do
				if string.sub(port_str,i,i) ~= " "then
				port_table[tonumber(string.sub(port_str,i,i))] = 0
				end
			end
			--遍历port_table的key值重新赋值为port_str
			port_str=""
			for k,v in ipairs(port_table) do
				if port_table[k]~= -1 then
				port_str = port_str.." "..k
				end
			end
			if cmd == "monitor session source port" then
				input.mirroredports =port_str
			else
				input.no_mirroredports =port_str
			end
		end
		
	end

	if cmd == "monitor session enable" then
		input.state = "on"
    end

	if cmd == "no monitor session enable" then
		input.state ="off"
	end

	if cmd == "monitor session ingress" then
		input.mode = "ingress"
	end

	if cmd == "monitor session egress" then
		input.mode = "egress"
	end

	if cmd == "monitor session both" then
		input.mode = "both"
	end

	if cmd == "show monitor session" and session_num~=nil then
		return show_mirror_port()
	end


	uci_r:set_confdir(CFG_PATH)
	result = set_mirror_config(input)
	return result
end


-- 脚本入口
local cmd = arg[1]
local session_num = arg[2]
local port_id = arg[3]

local result = "SUCCESS"

result = monitor_port(cmd, session_num, port_id)


io.stdout:write(result)