#!/bin/sh
# Copyright (C) 2006-2013 OpenWrt.org
# Copyright (C) 2010 Vertical Communications

#add directories here
RAMFS_DIRS="/etc/ /www/luci-static/resources/authserver/"
DEVICE_INFO_PATH="/tmp/device-info.json"
SPECIALCFG_PATH="/etc/config_special"
TDDP_REGION_CFG_DIR="/usr/sbin"
TDDP_REGION_CFG_FILE="tddp_region_cfg"
AGILECONFIG_PARTTTION_NAME="device-config"
AGILECONFIG_DIR="/tmp/agileconfig/"
TMP_AGILECONFIG_FILE_PATH="/tmp/agileconfigfile/"
USERCONFIG_PATH="/tmp/userconfig/"
AGILE_FILE="ag-config"

get_conffiles() {
    local file="$1"  
    ( (sed -ne '/^[[:space:]]*$/d; /^#/d; p' /etc/sysupgrade.conf /lib/upgrade/keep.d/* 2>/dev/null);opkg list-changed-conffiles ) | sort -u > "$file"    
} 

get_cfg_sync() {
    local file="$1"    
    find /etc/cfgsync.d/ -type f|sort -n > "$file"
}

cfg_dftcfg_to_usercfg() {
    find /tmp/userconfig -type f ! -name "cloud" | xargs rm -f

    get_conffiles "/tmp/conffiles.tmp"

    echo "Copy romconfig to userconfig"
    [ "$fs_overlay_support" == "n" ] && {
        for dir in $RAMFS_DIRS;
        do
            tmp_dir=/tmp/`basename ${dir}`_tmp           
            cp -a ${dir} ${tmp_dir}
            mount -t ramfs -n none ${dir}
            cp -a ${tmp_dir}/* ${dir}
            rm -rf ${tmp_dir}        
        done        
    }

    #copy conffiles to jffs2
    cfg_dir=/tmp/userconfig
    for CONFFILES in `cat /tmp/conffiles.tmp`
    do
        DIR=${CONFFILES%/*}
        #echo "mkdir ${cfg_dir}${DIR}"
        mkdir -p ${cfg_dir}${DIR}
        #DIR or FILE
        STRLEN=${#CONFFILES}
        let "STRLEN-=1"
        STRTYPE=${CONFFILES:$STRLEN}
        if [ "$STRTYPE" == "/" ];then
            PARENT_DIR=`dirname $CONFFILES`
            #echo "cp -a ${CONFFILES} ${cfg_dir}${PARENT_DIR}"
            cp -a ${CONFFILES} ${cfg_dir}${PARENT_DIR}       
        else
            #echo "cp -a ${CONFFILES} ${cfg_dir}${CONFFILES}"
            cp -a ${CONFFILES} ${cfg_dir}${CONFFILES}              
        fi
    done	

    rm /tmp/conffiles.tmp

    # cloud config is not in rom, after resetting, copy userconfig to root to remain the info
    cp /tmp/userconfig/etc/config/cloud /etc/config/
}

cfg_specialcfg_to_usercfg() {
	model_name=`cat /tmp/devinfo/model_name | sed 's/ /_/g'`
	region=`cat /tmp/devinfo/region`
	product_version=`cat /tmp/devinfo/model_version | awk -F"." '{print $1"_"substr($2,1,1)}' | sed 's/_0$//'`

	specialcfg_path=`echo /etc/config_special/${model_name}_${region}_v${product_version} | tr [A-Z] [a-z]`
	specialcfg_list=`ls $specialcfg_path| sed 's#.*/##'`

	for config_file in $specialcfg_list ; do
		section_list=`uci -c $specialcfg_path show $config_file | cut -d "=" -f1 | cut -d "." -f2 | uniq`

		for section_name in $section_list ;do
			option_list=`uci -c $specialcfg_path show $config_file.$section_name | sed '1d' | cut -d "=" -f1 | cut -d "." -f3`

			# 如果追加的section不存在，则新增
			section_is_exist=`uci get $config_file.$section_name`
			[ -z "$section_is_exist" ] && {
				uci set $config_file.$section_name=`uci -c $specialcfg_path get $config_file.$section_name`
			}

			for option_name in $option_list ;do
				line=`uci -c $specialcfg_path export $config_file | grep -rnw $option_name | head -n1 `
				type=`echo $line | awk -F $option_name '{print $1}' | cut -d ":" -f2 | sed 's/[\t]//g' | sed 's/ //g'`

				# 如果是option类型的配置，则使用uci set
				if [ "$type" == "option" ] ; then
					value=`uci -c $specialcfg_path get $config_file.$section_name.$option_name`
					uci set $config_file.$section_name.$option_name="$value"

				# 如果是list类型的配置，则使用uci add_list
				elif [ "$type" == "list" ] ; then
					list_value=`uci -c $specialcfg_path get $config_file.$section_name.$option_name`
					for list_member in $list_value ;do
						uci add_list $config_file.$section_name.$option_name="$list_member"
					done

				else
					echo "UCI: error type of $config_file.$section_name.$option_name" > /dev/console
				fi
			done
		done
		uci commit $config_file
	done
	/lib/cfg_save/config.sh save
}

cfg_userconfig_to_memory() {
	get_conffiles "/tmp/conffiles.tmp"
	get_cfg_sync "/tmp/cfgsync.tmp"

    if [ ! -f "/tmp/userconfig/userconfig_default.tmp" ];then
	echo "Warning:Detect userconfig_default file not exist!" > /dev/console
	touch "/tmp/userconfig/userconfig_default.tmp"
	echo "Default" > "/tmp/userconfig/userconfig_default.tmp"
    fi

    echo "Copy userconfig to memory"
    [ "$fs_overlay_support" == "n" ] && {
        for dir in $RAMFS_DIRS;
        do
            tmp_dir=/tmp/`basename ${dir}`_tmp           
            cp -a ${dir} ${tmp_dir}
            mount -t ramfs -n none ${dir}
            cp -a ${tmp_dir}/* ${dir}
            rm -rf ${tmp_dir}        
        done        
    }

    #Configuration compatibility
    cfg_dir=/tmp/userconfig
    for CFGSYNC in `cat /tmp/cfgsync.tmp`
    do
        chmod 0700 $CFGSYNC
        $CFGSYNC "/" "/tmp/userconfig/"
    done
    #copy jffs2 to conffiles
    for CONFFILES in `cat /tmp/conffiles.tmp`
    do
        isDiff=`diff ${cfg_dir}${CONFFILES} ${CONFFILES} 2>/dev/null|wc -l`
        if [ ! -e "${CONFFILES}" ] || [ $isDiff -gt 0 ];then
            DIR=${CONFFILES%/*}
            #echo "mkdir -p ${DIR}"
            mkdir -p ${DIR}
            #DIR or FILE
            STRLEN=${#CONFFILES}
            let "STRLEN-=1"
            STRTYPE=${CONFFILES:$STRLEN}                           
            if [ "$STRTYPE" == "/" ];then
                PARENT_DIR=`dirname $CONFFILES`
                #echo "cp -a ${cfg_dir}${CONFFILES} ${PARENT_DIR}"
                cp -a ${cfg_dir}${CONFFILES} ${PARENT_DIR}
            else
                #echo "cp -a ${cfg_dir}${CONFFILES} ${CONFFILES}"
                cp -a ${cfg_dir}${CONFFILES} ${CONFFILES}
            fi                        
        fi
    done	

    cp "/tmp/userconfig/userconfig_default.tmp" "/tmp/userconfig_default.tmp"
    cp "/tmp/userconfig/userpwd_default.tmp" "/tmp/userpwd_default.tmp"
	
    rm /tmp/conffiles.tmp
    rm /tmp/cfgsync.tmp    
}

cfg_copy()
{
fs=$(find  "$1")
for i in $fs;
do
    [ -d $i ] && {
        mkdir -p $2/$i
       }
done

for i in $fs;
do
    [ -f $i ] && {
        if [ -f $2/$i ] ;then
            difs=$(diff -q $i  $2/$i)
            [ -z "$difs" ] || {
				if [ "$i" != "/etc/config/wlancfg.bin" ]; then
					cp -f $i  $2/$i
				fi
			}
        else
            cp -f $i $2/$i
        fi
        sync
       }
done

}

cfg_save_to_flash() {
    if [ -f /var/run/restorefactory.pid -o -f /tmp/config_restore_start ];then
        echo "restorfactory or restore process is on-going! can not save config to flash!" > /dev/console
        return 0
    fi

    exec 3<>"/tmp/cfg_save.stat"
    flock -w 30 3
    [ $? -eq 1 ] && { echo "lock fail" > /dev/console; exit; }
    echo "start cfg_save_to_flash ,add file lock" > /dev/console
    #status=`cat /tmp/cfg_save.stat`    
    #if [ "$status" == "Saved" ];then
    #    echo "Saved"
    #    exit
    #fi    
    touch "/tmp/conffiles.tmp"
    get_conffiles "/tmp/conffiles.tmp"
    echo "Saving" > "/tmp/cfg_save.stat"
    
    cfg_dir=/tmp/userconfig
    for configs in `cat /tmp/conffiles.tmp`
    do
        DIR=${configs%/*}
        mkdir -p ${cfg_dir}${DIR}
        #DIR or FILE
        STRLEN=${#configs}
        let "STRLEN-=1"
        STRTYPE=${configs:$STRLEN}
        
        isDiff=`diff ${configs} ${cfg_dir}${configs} 2>/dev/null|wc -l`
        if [ ! -e "${cfg_dir}${configs}" ] || [ $isDiff -gt 0 ];then
            if [ "$STRTYPE" == "/" ];then
                cfg_copy ${configs} ${cfg_dir} 
            else
                #echo "cp -a ${configs} ${cfg_dir}${configs}"
                cp -a ${configs} ${cfg_dir}${configs}
                sync
            fi
        fi
    done    
    cat /dev/null > /tmp/conffiles.tmp

    cp "/tmp/userconfig_default.tmp" "/tmp/userconfig/userconfig_default.tmp"
    cp "/tmp/userpwd_default.tmp" "/tmp/userconfig/userpwd_default.tmp"    
    cd /tmp/userconfig/
    sync
    echo "Saved" > "/tmp/cfg_save.stat"
    echo "Saved"
}

cfg_reset_to_dftcfg() {

    exec 3<>"/tmp/cfg_save.stat"
    flock -w 30 3
    [ $? -eq 1 ] && { echo "lock fail"; return 1; } 

    rm -f /tmp/userconfig/userconfig_default.tmp
    #rm -rf /tmp/userconfig/*
    find /tmp/userconfig -type f ! -name "cloud" | xargs rm -f

    echo "Saved" > "/tmp/cfg_save.stat"
    return 0
}

cfg_modify() {
    exec 3<>"/tmp/cfg_save.stat"
    flock -w 60 3 
    [ $? -eq 1 ] && { echo "lock fail"; exit; } 
    
    echo "Modify" > "/tmp/userconfig_default.tmp" 
    echo "Not Saved" > "/tmp/cfg_save.stat"
    echo "Modify OK"

}

dft_pwd_modify() {
	
    echo "Modify" > "/tmp/userconfig_default.tmp" 
    echo "Modify" > "/tmp/userpwd_default.tmp" 
    echo "Modify OK"
	
    #cfg_save_to_flash
    cp "/etc/config/accountmgnt" "/tmp/userconfig/etc/config/accountmgnt"
    
    cp "/tmp/userconfig_default.tmp" "/tmp/userconfig/userconfig_default.tmp"
    cp "/tmp/userpwd_default.tmp" "/tmp/userconfig/userpwd_default.tmp"
}
cfg_status_get() {
    exec 3<>"/tmp/cfg_save.stat"
    flock -w 30 3 
    [ $? -eq 1 ] && { echo "Saved"; exit; }
    
    cat "/tmp/cfg_save.stat"
}

cfg_check_isdftcfg() {
    if [ -f "/tmp/userconfig_default.tmp" ];then
        cat "/tmp/userconfig_default.tmp"
    else
        echo "Default"
    fi
}

pwd_check_isdftcfg() {
    if [ -f "/tmp/userpwd_default.tmp" ]; then
        cat "/tmp/userpwd_default.tmp"
    else
        echo "Default"
    fi
}

cfg_file_check_isdftcfg() {
    if [ -n "$1" ]; then
        isDiff=`diff /etc/config/$1 /tmp/.default_config/$1 2>/dev/null|wc -l`
        if [ $isDiff -gt 0 ]; then
            echo -n "Modified"
        else
            echo -n "Default"
        fi
    else
        echo -n "Error"
    fi
}

cfg_memory_reset_to_dftcfg() {
    local config="$1"
    cp /tmp/.default_config/$config /etc/config/$config
}

set_cfg_copy_success_flag() {
    touch "/tmp/userconfig/userpwd_default.tmp"
    echo "Default" > "/tmp/userconfig/userpwd_default.tmp"
    cp "/tmp/userconfig/userpwd_default.tmp" "/tmp/userpwd_default.tmp"
    touch "/tmp/userconfig/userconfig_default.tmp"
    echo "Default" > "/tmp/userconfig/userconfig_default.tmp"
    cp "/tmp/userconfig/userconfig_default.tmp" "/tmp/userconfig_default.tmp"
}

set_agile_cfg_copy_success_flag() {
    touch "/tmp/userconfig/userpwd_default.tmp"
    cp ${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig/userpwd_default.tmp "/tmp/userconfig/userpwd_default.tmp"
    cp "/tmp/userconfig/userpwd_default.tmp" "/tmp/userpwd_default.tmp"
    touch "/tmp/userconfig/userconfig_default.tmp"
    cp ${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig/userconfig_default.tmp "/tmp/userconfig/userconfig_default.tmp"
    cp "/tmp/userconfig/userconfig_default.tmp" "/tmp/userconfig_default.tmp"
    rm -rf $TMP_AGILECONFIG_FILE_PATH
    echo "set_agile_cfg_copy_success_flag ok" > /dev/console
}

mount_agileconfig() {
	agileconfig_mtd=`cat /proc/mtd | grep ${AGILECONFIG_PARTTTION_NAME} | head -n 1 | awk -F : '{print $1}'`
	agileconfig_mtd_num=${agileconfig_mtd##mtd}
	mkdir -p ${AGILECONFIG_DIR}
	/usr/sbin/ubiattach -m ${agileconfig_mtd_num} -d 6 -b 5 /dev/ubi_ctrl
	if [ $? == 0 ]; then
		echo "ubifs mount already" > /dev/console
		/bin/mount -t ubifs -o ro /dev/ubi6_0 ${AGILECONFIG_DIR} -o bulk_read
	else
		echo "ubifs format already" > /dev/console
		/usr/sbin/ubiformat -y -q /dev/mtd${agileconfig_mtd_num}
		/usr/sbin/ubiattach -m ${agileconfig_mtd_num} -d 6 -b 5 /dev/ubi_ctrl
		/usr/sbin/ubimkvol /dev/ubi6 -m -N agile-config
		/bin/mount -t ubifs -o ro /dev/ubi6_0 ${AGILECONFIG_DIR} -o bulk_read
	fi
}

dec_and_untar_agilecfg() {
    # 参考firmware.lua config_restore
    local dft_key="2EB38F7EC41D4B8E1422805BCD5F740BC3B95BE163E39D67579EB344427F7836"
    local dft_iv="360028C9064242F81074F4C127D299F6"
    local agile_back_tmp="/tmp/agile_backup.tar.gz"
	local model_name=`cat /tmp/devinfo/model_name | sed 's/ /_/g'`
    agile_config_tmp="${AGILECONFIG_DIR}/${AGILE_FILE}"
    if [ -f "$agile_config_tmp" ]; then
        #解密
        openssl aes-256-cbc -md md5 -d -in $agile_config_tmp -k $model_name -K $dft_key -iv $dft_iv | openssl zlib -d > $agile_back_tmp
        # 解压
        mkdir -p $TMP_AGILECONFIG_FILE_PATH
        tar -zxvf $agile_back_tmp -C $TMP_AGILECONFIG_FILE_PATH > /dev/null 2>&1
        # 删除黑名单文件
        file_list=`uci -c /etc/profile.d get profile.@backup_block_file[0].file`
        for file in $file_list; do
            file_path="${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig/${file}"
            if [ -f "$file_path" ]; then
                rm -rf "$file_path"
                echo "rm -rf $file_path" > /dev/console
            fi
        done
        #判断机型名是否匹配
        result_info=`cat ${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig/product_name`
        # 获取当前系统的配置信息
        cur_model_name=`cat /tmp/devinfo/model_name | sed 's/ /_/g'`
        cur_region=`cat /tmp/devinfo/region`
        cur_model_version=`cat /tmp/devinfo/model_version`
        cur_model_info="${cur_model_name}_${cur_region}_v${cur_model_version}"
        # 比较当前产品信息与文件中的产品信息
        if [[ "${cur_model_info}" != "${result_info}" ]]; then
            echo "cur_model_info doesnot match file_info" > /dev/console
            echo "cur_model_info is ${cur_model_info} and file_info is ${result_info}" > /dev/console
            rm -rf $TMP_AGILECONFIG_FILE_PATH
            rm -rf $agile_back_tmp
            exit 1
        fi
        # 解密解压成功 解压目录：$TMP_AGILECONFIG_FILE_PATH/tmp/userconfig/
        echo "Default" > /tmp/agileconfig_default.tmp
        echo "dec_and_untar_agilecfg ok" > /dev/console
        rm -rf $agile_back_tmp
    fi
}

# 完全参考 cfg_userconfig_to_memory
cfg_agilecfg_to_memory() {
    agileconfigsdir=${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig/
	get_conffiles "/tmp/conffiles.tmp"
	get_cfg_sync "/tmp/cfgsync.tmp"

    echo "Copy agileconfig to memory"
    #Configuration compatibility
    agile_cfg_dir=${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig
    for CFGSYNC in `cat /tmp/cfgsync.tmp`
    do
        chmod 0700 $CFGSYNC
        $CFGSYNC "/" "/tmp/userconfig/"
    done

    #copy jffs2 to conffiles
    for CONFFILES in `cat /tmp/conffiles.tmp`
    do
        isDiff=`diff ${agile_cfg_dir}${CONFFILES} ${CONFFILES} 2>/dev/null|wc -l`
        if [ ! -e "${CONFFILES}" ] || [ $isDiff -gt 0 ];then
            DIR=${CONFFILES%/*}
            #echo "mkdir -p ${DIR}"
            mkdir -p ${DIR}
            #DIR or FILE
            STRLEN=${#CONFFILES}
            let "STRLEN-=1"
            STRTYPE=${CONFFILES:$STRLEN}                           
            if [ "$STRTYPE" == "/" ];then
                PARENT_DIR=`dirname $CONFFILES`
                #echo "cp -a ${agile_cfg_dir}${CONFFILES} ${PARENT_DIR}"
                cp -a ${agile_cfg_dir}${CONFFILES} ${PARENT_DIR}
            else
                #echo "cp -a ${agile_cfg_dir}${CONFFILES} ${CONFFILES}"
                cp -a ${agile_cfg_dir}${CONFFILES} ${CONFFILES}
            fi                        
        fi
    done	
    echo "cfg_agilecfg_to_memory ok" > /dev/console
    rm /tmp/conffiles.tmp
    rm /tmp/cfgsync.tmp
}

cfg_accountmgnt_to_memory() {
    userconfigFlag=`cat ${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig/userpwd_default.tmp`
    pwdFlag=`cat ${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig/userconfig_default.tmp`
    if [ "$userconfigFlag" != "Modify" ] || [ "$pwdFlag" != "Modify" ] ; then
        cp -a /rom/etc/config/accountmgnt /etc/config/accountmgnt
	cp -a /rom/etc/passwd /etc/passwd
	cp -a /rom/etc/group /etc/group
	cp -a /rom/etc/shadow /etc/shadow
        echo "cfg_accountmgnt_to_memory:Default" > /dev/console
    fi
}

cfg_wireless_to_memory() {
    if [ -f "${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig/etc/config/wlancfg.json" ] ; then
        uclited -m ${TMP_AGILECONFIG_FILE_PATH}/tmp/userconfig/etc/config/wlancfg.json
        rm -rf /etc/config/wlancfg.json
        echo "cfg_wireless_to_memory ok" > /dev/console
    fi
}

rootfs_init_etc_ramfs() {
    [ ! -e /tmp/.default_config ] && mkdir -p /tmp/.default_config
    cp -rf /etc/config/* /tmp/.default_config/ 
    echo "=========================USERCONFIG INITIAL======================" > /dev/console
    if [ -f "/tmp/userconfig/userconfig_default.tmp" ] ; then
        cfg_userconfig_to_memory
    else
        cfg_dftcfg_to_usercfg
        [ -d /etc/config_special ] && cfg_specialcfg_to_usercfg
        agileconfig_mtd=`cat /proc/mtd | grep ${AGILECONFIG_PARTTTION_NAME} | head -n 1 | awk -F : '{print $1}'`
        if [ -n "$agileconfig_mtd" ] ; then
            echo "device-config exits" > /dev/console
            mount_agileconfig
            dec_and_untar_agilecfg
            if [ -f "/tmp/agileconfig_default.tmp" ];then
                cfg_agilecfg_to_memory
                cfg_wireless_to_memory
                cfg_accountmgnt_to_memory
                /lib/cfg_save/config.sh save
                set_agile_cfg_copy_success_flag
            else
                set_cfg_copy_success_flag
            fi
        else
            set_cfg_copy_success_flag
        fi
    fi

    touch /tmp/cfg_save.stat
    echo "Saved" > "/tmp/cfg_save.stat"
    echo "=========================USERCONFIG DONE======================"  > /dev/console
    if [ ! -s "/tmp/userconfig/etc/config/controller.conf" ] ; then
        cp /rom/etc/config/controller.conf /tmp/userconfig/etc/config/
	cp /rom/etc/config/controller.conf /etc/config/
	cd /tmp/userconfig/
	sync 
	echo "controller.conf is null!!!" > /dev/console
    fi
}
