#
# Copyright (C) 2008-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# 
# brief        ip mac binding module core_global shell scripts
# author     Huang Zhenwei
# version    1.0.0 
# date         06May2015
# history     06May2015    Huang Zhenwei    Create the file
#                22May2015    Huang Zhenwei    Modify 'zone' and 'interface'
#				4August2015  luopei   zone becomes the list of interfaces
. /lib/zone/zone_api.sh
	
IMB_PREFIX="imb_ipmac_"
IP_PREFIX="imb_ip_"
IMB_ARP_FILE="/tmp/.imb_arp"
IMB_IPSET_FILE="/tmp/.imb_ipset"
IMB_UCI_FILE="/tmp/.imb_uci"
IMB_PROC_FILE="/proc/garp_proc"
IMB_CHAIN="ip_mac_binding"
local imb_device=""

fw_config_get_global() {
	fw_config_get_section "$1" global { \
		string enable "off" \
	} || return
}

fw_config_get_imb_rule() {
	fw_config_get_section "$1" imb_rule { \
		string ipaddr "" \
		string mac "" \
		string zone "" \
		string enable "" \
		string description "" \
	} || return
}


fw_load_all() {
	fw_config_once fw_load_global global
}

fw_exit_all() {
	#fw_flush_arp
	
	/etc/init.d/arp_defense stop  #stop arp_defense firewall
	# local ZONES=$(zone_get_zonelist)	
	# for i in $ZONES; do
		# local EFF_DE=$(zone_get_effect_devices $i)
		# for j in $EFF_DE; do
			# ipset destroy "${IMB_PREFIX}${j}"
			# ipset destroy "${IP_PREFIX}${j}"
		# done
	# done
	while read LINE
	do
		local devname=$(echo $LINE | awk '{print $1}')
		local setname=$(echo $LINE | awk '{print $2}')
		local ipname=$(echo $LINE | awk '{print $3}')
		ipset list $ipname | sed -n '/^Members/,$p' | grep -v 'Members' >/tmp/.debind
		ipmac -d /tmp/.debind $devname
		echo "destroy $setname" >> /tmp/.imb_ipset_flush
		echo "destroy $ipname" >> /tmp/.imb_ipset_flush
	done < /tmp/state/imb
	
	ipset restore -file /tmp/.imb_ipset_flush
	rm /tmp/.imb_ipset_flush
	rm /tmp/state/imb
	imb_device=""
}

create_ipset_default()
{
	[ -f /tmp/state/imb ] && rm /tmp/state/imb
	local zone_list=$(zone_get_zonelist)
	local zone=""
	for zone in $zone_list;do
		#[ "$zone" == "VPN" ] && continue
		[ "${zone:0:3}" != "LAN" ] && continue		# Now only support LAN
		local iface=$(zone_get_wan_ifaces $zone)  # If it is PPPoE/L2TP/PPTP, we choose eth0.XX instead of pppoe/l2tp/pptp-wanX_poe to build the ipset rule.
		if [ -z $iface ];then
			continue
		fi
		local eth_dev=$(zone_get_eth_device $iface)
		if [ -z $eth_dev ];then
			continue
		fi
		echo "${eth_dev} ${IMB_PREFIX}${eth_dev} ${IP_PREFIX}${eth_dev}" >>/tmp/state/imb
		echo "create ${IMB_PREFIX}${eth_dev} hash:ip,mac  -exist" >> /tmp/.imb_ipset_add
		echo "create ${IP_PREFIX}${eth_dev}  hash:ip  -exist" >> /tmp/.imb_ipset_add
		if [ x"$imb_device" == "x" ];then
			imb_device=$eth_dev
		else
			imb_device="$imb_device,$eth_dev"
		fi
		local effect_device=$eth_dev
		local device_ip=$(ifconfig $effect_device | grep 'inet addr' | awk -F : '{print $2}' | awk '{print $1}')
		[ x"$device_ip" != "x" ] && {
				local device_mac=$(ifconfig $effect_device | grep 'HWaddr' | awk '{print $5}')
				if [ x"$device_mac" != "x" ];then
					arp -s -i $effect_device $device_ip $device_mac
					local inter=$(zone_get_effect_ifaces $zone)
					echo "$effect_device $device_ip $device_mac" >/tmp/state/$inter
					echo "add ${IMB_PREFIX}${effect_device} ${device_ip},${device_mac}" >> /tmp/.imb_ipset_add
					echo "add ${IP_PREFIX}${effect_device} ${device_ip}" >> /tmp/.imb_ipset_add
				fi
			}
	done
	ipset restore -file /tmp/.imb_ipset_add
	rm /tmp/.imb_ipset_add
}

create_ipset_interface(){
	local ZONES=$(zone_get_zonelist)
	create_ipset_default
	for i in $ZONES; do
		#local EFF_IF=$(zone_get_effect_devices $i)
		[ $i == "VPN" ] && continue
		local EFF_IF=$(zone_get_wan_devices $i)
		for j in $EFF_IF; do
			#local EFF_IF=$(zone_get_effect_ifaces NORMAL)
			#local effect_device=$(zone_get_device_byif $j)
			echo $j | grep -q '_poe$' && continue
			local effect_device=${j}
			ipset create "${IMB_PREFIX}${effect_device}" hash:ip,mac  -exist
			ipset create "${IP_PREFIX}${effect_device}"  hash:ip  -exist
			#echo "${effect_device} ${IMB_PREFIX}${effect_device} ${IP_PREFIX}${effect_device}" >>/tmp/state/imb
			if [ x"$imb_device" == 'x' ];then
				imb_device=$effect_device
			else
				imb_device="$imb_device $effect_device"
			fi
			local device_ip=$(ifconfig $effect_device | grep 'inet addr' | awk -F : '{print $2}' | awk '{print $1}')
			[ x"$device_ip" != "x" ] && {
				local device_mac=$(ifconfig $effect_device | grep 'HWaddr' | awk '{print $5}')
				if [ x"$device_mac" != "x" ];then
					arp -s -i $effect_device $device_ip $device_mac
					local inter=$(zone_get_effect_ifaces $i)
					echo "$effect_device $device_ip $device_mac" >/tmp/state/$inter
					ipset add "${IMB_PREFIX}${effect_device}" "${device_ip},${device_mac}" -exist
					ipset add "${IP_PREFIX}${effect_device}" "${device_ip}" -exist
				fi
			}
		done
	done
}

fw_load_global() {
	local global_enable=$(uci get arp_defense.settings.enable)
	if [ -z "`lsmod | grep arp_garp`" ]; then
		insmod /lib/modules/$(uname -r)/arp_garp.ko
	fi
	echo "off=1" > $IMB_PROC_FILE
	if [ $global_enable = 'on' ];then
		local imb_pass=$(uci get arp_defense.settings.imb_pass)
		local garp=$(uci get arp_defense.settings.garp)
		local time=$(uci get arp_defense.settings.time)
		create_ipset_default
		cd /usr/lib/lua && lua ./imb/imb_reload.lua
		[ $imb_pass = "on" ] && /etc/init.d/arp_defense start
		#[ x"$imb_device" == 'x' ] && imb_device="br-lan"
		#imb_device=${imb_device// /,}
		#imb_device="br-lan,eth0.10,eth0.20,eth0.30,eth0.40"
		#echo "$imb_device" >/dev/console
		if [ x$garp != 'xon' ];then
			echo "sendflag=0" > $IMB_PROC_FILE
			#insmod /lib/modules/$(uname -r)/arp_garp.ko sendflag=0 interval_time=$time  name=$devices
		else
			local devices=""
			local interfaces=$(uci get arp_defense.settings.interface)
			local inter=""
			for inter in $interfaces;do
				local dev=$(zone_get_effect_devices $inter)
				if [ x"$devices" == 'x' ];then
					devices=$dev
				else
					devices="$devices,$dev"
				fi
			done
			echo "sendflag=1" > $IMB_PROC_FILE
			[ x$time == 'x' ] && time=1000
			echo "interval_time=$time" > $IMB_PROC_FILE
			echo "name=$devices" > $IMB_PROC_FILE
			#insmod /lib/modules/$(uname -r)/arp_garp.ko sendflag=1 interval_time=$time  name=$devices
		fi
    fi
}

#$1: interface
#$2: ipaddr
#$3: mac
lua_load_imb_rule()
{
	local global_enable=$(uci get arp_defense.settings.enable)
	if [[ $global_enable = 'on' ]]; then
		ipmac -b $IMB_ARP_FILE
	fi
	#rm $IMB_ARP_FILE -f 2>/dev/null
	#rm $IMB_IPSET_FILE -f 2>/dev/null
}

fw_load_imb_rule() {
	fw_config_get_imb_rule $1
	
	case $imb_rule_enable in
		on )
		# need small changes to...
		local interface=$imb_rule_zone  
		#echo "imb_zone:$R_ZONE" >>/tmp/imb_log
		local R_IPADDR=$imb_rule_ipaddr
		local R_MAC=$(echo $imb_rule_mac | tr [a-z] [A-Z])
		
		local IF_IP
		
		local effect_devices=$(zone_get_wan_devices $interface)
		for effect_device in $effect_devices; do
			echo $effect_device | grep -q '_poe$' && continue

			ipset list ${IMB_PREFIX}$effect_device >/dev/null 2>&1 && {
				ipset add "${IMB_PREFIX}${effect_device}" "${R_IPADDR},${R_MAC//-/:}" -exist
				ipset add "${IP_PREFIX}${effect_device}" "${R_IPADDR}" -exist
				arp -s -i $effect_device $R_IPADDR ${R_MAC//-/:}
			}
		done

			;;
		off )		
			;;
	esac
}

fw_flush_arp() {
	local IFS_old=$IFS
	IFS=$'\n'
	local LINE=""
	for LINE in $(arp -n | sed '/Address/d' | awk '{ print }');
	do
		if [ $(echo $LINE | awk '{ print $2 }') != "(incomplete)" ]; then
			local IPADDR=$(echo $LINE | awk '{ print $1 }')
			local IFACE=$(echo $LINE | awk '{ print $5 }')
			arp -d -i $IFACE $IPADDR
		fi
	done
	IFS=$IFS_old
}

create_del_list()
{
	config_get zone "$1" "zone"
    if [ "$zone" == "$2" ]; then
        sed -i "1i delete imb.@imb_rule[$index]" $IMB_UCI_FILE
    fi
    index=$(($index+1))
}

imb_add_arp()
{
	local releated_zone=$2
	local releated_devices=$3
	local type=$4
	config_get imb_rule_zone "$1" "zone"
	config_get imb_rule_enable "$1" "enable"

	[ "on" == "$imb_rule_enable" -a "$imb_rule_zone" == "$releated_zone" ] && {
		config_get imb_rule_ipaddr "$1" "ipaddr"
		config_get imb_rule_mac "$1" "mac"
		local imb_rule_mac=$(echo $imb_rule_mac | tr [a-z] [A-Z])
		for device in $releated_devices; do
			echo $device | grep -q '_poe$' && continue
			arp -s -i $device $imb_rule_ipaddr ${imb_rule_mac//-/:} &
			[ "LAN" == "$type" ] && {
				echo "$imb_rule_ipaddr ${imb_rule_mac//-/:}" >> /tmp/state/$releated_zone
				# echo "$imb_rule_ipaddr ${imb_rule_mac//-/:}" > /dev/console
			}
		done
	}

}

fw_imb_event_do_interface()
{
	local releated_zone=$2
	local releated_devices=$3

	config_get imb_rule_zone "$1" "zone"
	config_get imb_rule_enable "$1" "enable"
	config_get imb_rule_mac "$1" "mac"
	config_get imb_rule_ipaddr "$1" "ipaddr"

	[ "on" == "$imb_rule_enable" -a "$imb_rule_zone" == "$releated_zone" ] && {
		local imb_rule_mac=$(echo $imb_rule_mac | tr [a-z] [A-Z])
		for device in $releated_devices; do
			echo $device | grep -q '_poe$' && continue
			vnete ipset add "${IMB_PREFIX}${device}" "${imb_rule_ipaddr},${imb_rule_mac//-/:}" -exist
			vnete ipset add "${IP_PREFIX}${device}" "${imb_rule_ipaddr}" -exist
			arp -s -i $device $imb_rule_ipaddr ${imb_rule_mac//-/:}
		done
	}		
}


fw_imb_event_interface()
{
	local iface=$1
	local action=$2
	local type=$3
	[ -z "$iface" -o -z "$action" ] && {
		return
	}

	local global_enable=$(uci get arp_defense.settings.enable)
	# todo zone_get_zone_byif较慢，若能改善应该都使用该接口
	if [ "$type" == "LAN" ]; then  
		zone=$(uci get vnetwork.$iface.zone)
	else
		zone=$(zone_get_zone_byif $iface)	
	fi
	[ -f "/tmp/state/$zone" ] && rm -f "/tmp/state/$zone"
	
	[ "add" == "$action" ] && {
		[ "on" == "$global_enable" ] && {
			local imb_pass=$(uci get arp_defense.settings.imb_pass)
			local devices=$(zone_get_wan_devices $iface)
			# 1. 添加ARP条目
			for device in $devices; do
				echo $device | grep -q '_poe$' && continue
				[ x"$device" != "x" ] && {
					local device_ip=$(ifconfig $device | grep 'inet addr' | awk -F : '{print $2}' | awk '{print $1}')
					[ x"$device_ip" != "x" ] && {
						local device_mac=$(ifconfig $device | grep 'HWaddr' | awk '{print $5}')
						if [ x"$device_mac" != "x" ];then
							arp -s -i $device $device_ip $device_mac &
							[ "$type" == "LAN" ] && {
								echo "$device_ip $device_mac" >> /tmp/state/$zone
								# echo "$device_ip $device_mac" > /dev/console
								# 2. 添加zone相关的ipset
								echo "${device} ${IMB_PREFIX}${device} ${IP_PREFIX}${device}" >> /tmp/state/imb &
								ipset create ${IMB_PREFIX}${device} hash:ip,mac -exist &
								ipset create ${IP_PREFIX}${device} hash:ip -exist &
								# 3. 添加iptables规则
								[ "on" == "$imb_pass" ] && {
									# iptables rules and ipset sets require strict sequential execution
									wait
									iptables -w -A "${IMB_CHAIN}" -i "${device}" -m set ! --match-set "${IMB_PREFIX}${device}" src,src -j DROP &
								}
							}
						fi
					}
				}
			done

			config_load imb
			config_foreach imb_add_arp imb_rule $zone $devices $type

			# 相关规则的添加
			[ "LAN" == "$type" ] && {
				# 添加相关ip(,mac)到ipset中
				# Ensure the zone file is written.
				wait
				while read LINE
				do
					local imb_ip=$(echo $LINE | awk '{ print $1 }')
					local imb_mac=$(echo $LINE | awk '{ print $2 }')
					ipset add "${IMB_PREFIX}${device}" "${imb_ip},${imb_mac}" -exist &
					ipset add "${IP_PREFIX}${device}" "${imb_ip}" -exist &
				done < /tmp/state/$zone
			}
		}
	}

	[ "del" == "$action" ] && {
		[ "on" == "$global_enable" ] && {
			# 相关规则的清理
			[ "LAN" == "$type" ] && {
				local imb_pass=$(uci get arp_defense.settings.imb_pass)
				local devices=$(zone_get_wan_devices $iface)
				for device in $devices; do
					echo $device | grep -q '_poe$' && continue
					[ x"$device" != 'x' ] && {
						[ "on" == "$imb_pass" ] && {
							ipset -q list "${IMB_PREFIX}${device}"
							[ x$? == x0 ] && {
								# iptables rules and ipset sets require strict sequential execution
								iptables -w -D "${IMB_CHAIN}" -i "${device}" -m set ! --match-set "${IMB_PREFIX}${device}" src,src -j DROP
							}
						}
						# 2. 将和zone相关的ipset删掉
						eval sed -i '/${device}/d' /tmp/state/imb &
						ipset destroy "${IMB_PREFIX}${device}" &
						ipset destroy "${IP_PREFIX}${device}" &
					}
				done
			}
		}
	}
}

imb_zone_config()
{
	local zone=$1
	# 将网卡相关的arp条目清除（会被自动删掉?）
	# 1. 如果GARP选项开启，需要维护LAN的列表
	local garp=$(uci get arp_defense.settings.garp)
	[ "on" == "$garp" ] && {
		uci del_list arp_defense.settings.interface="${zone}"
		uci commit
	}
	# 2. 删除ip-mac-binding中和该网卡相关的条目
	index=0
	echo "commit" > "${IMB_UCI_FILE}"
	config_load imb
	config_foreach create_del_list imb_rule "${zone}"
	uci batch < "${IMB_UCI_FILE}"
}
