#!/bin/sh
# Linux os-dependent shell code.

. /lib/functions.sh


delete_all_pd_route()
{
        local pd_list="$(uci get dhcp6s.pd_basic.id)"
	config_load vnetwork
	config_get gw_downstream6 "vwan" ipv6_addr ''
        [ -n "$pd_list" ] && {
                for id in ${pd_list};do
                        local prefix="$(uci get dhcp6s.pd_server${id}.new_prefix)"
                        ip -6 route del "${prefix}" 
                        echo "[set_downstream_routes_v6] ip -6 route del ${prefix}" >> /tmp/log/vnetc_log
                done
        }
}


delete_pd_route()
{
        config_load vnetwork
        config_get gw_downstream6 "vwan" ipv6_addr ''

        local pd_enable="$(uci get dhcp6s.pd_basic.switch)"
        [ "${pd_enable}" != "enable" ] && return

        local prefix="$1"
	[ ! -n "${prefix}" -o "${prefix}" == '::/' ] && return
        ip -6 route del "${prefix}"
        echo "[set_downstream_routes_v6] ip -6 route del ${prefix}" >> /tmp/log/vnetc_log
}


add_pd_route()
{
	local id=$1
	local prefix=$2
        config_load vnetwork
        config_get gw_downstream6 "vwan" ipv6_addr ''
	local pd_enable="$(uci get dhcp6s.pd_basic.switch)"
	[ "${pd_enable}" != "enable" ] && return
        local pd_list="$(uci get dhcp6s.pd_basic.id)"
       	if [ ! -n "${prefix}" ] ;then 
		prefix="$(uci get dhcp6s.pd_server${id}.new_prefix)"
	fi
	if [ ! -n "${prefix}" -o "${prefix}" == '::/' ];then
		return
	fi
        local link_addr="$(uci get dhcp6s.pd_server${id}.link_addr)"
	local pd_lanid="$(uci get dhcp6s.pd_server${id}.pd_lanid)"
	local pd_dev="$(uci get vnetwork.NETWORK${pd_lanid}.ifname)"
        ip -6 route add ${prefix} via ${link_addr} dev ${pd_dev}
	echo "[set_downstream_routes_v6] ip -6 route add ${prefix} via ${link_addr} dev ${pd_dev}" >> /tmp/log/vnetc_log
}

add_all_pd_route()
{
        config_load vnetwork
        config_get gw_downstream6 "vwan" ipv6_addr ''
	local pd_list="$(uci get dhcp6s.pd_basic.id)"

        [ -n "$pd_list" ] && {
                for id in ${pd_list};do
	        	local pd_enable="$(uci get dhcp6s.pd_basic.switch)"
		        local pd_list="$(uci get dhcp6s.pd_basic.id)"
		        local prefix="$(uci get dhcp6s.pd_server${id}.new_prefix)"
			[ ! -n "${prefix}" -o "${prefix}" == '::/' ] && return
		        local link_addr="$(uci get dhcp6s.pd_server${id}.link_addr)"
	        	local pd_lanid="$(uci get dhcp6s.pd_server${id}.pd_lanid)"
	        	local pd_dev="$(uci get vnetwork.NETWORK${pd_lanid}.ifname)"
		        ip -6 route add ${prefix} via ${link_addr} dev ${pd_dev}
			echo "[set_downstream_routes_v6] ip -6 route add ${prefix} via ${link_addr} dev ${pd_dev}" >> /tmp/log/vnetc_log
		done
	}
}



