#!/bin/sh
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file      nat_public.sh
# brief
# author    Chen chen
# version   1.1.0
# date      23Apr15
# history   arg 1.0.0, 23Apr15, Chen chen, Create the file.
#           arg 1.1.0, 21Dec17, Zhangzhongwei, Rewrite the file.

NAT_TRUE=0
NAT_FALSE=1
NAPT_DEVICE_MARK=0x80000000 # lan <-> wan
NAPT_DEVICE_MARK_LANIN=0x40000000 # lan -> wan
NAT_PREROUTING_MANGLE="NAT_PREROUTING_RULE"
NAPT_DEVICE_CHAIN="napt_netdevice_chain"
NAPT_DEVICE_OUTPUT_CHAIN="napt_netdevice_output_chain"
NAPT_DEVICE_CACHE="/tmp/.nat_napt_iface_list"

############### UCI Config Functions ###############
nat_config_append() # <package>
{ 
	CONFIG_APPEND=1 config_load "$@"
	unset CONFIG_APPEND
}

nat_config_load()
{
	package_is_load nat || config_load nat
}

nat_state_set_zones()
{
	local nat_zones="/var/run/nat_zones"
	local module="$1"

	shift
	[ ! -f "$nat_zones" ] && touch "$nat_zones"
	uci -c /var/run/ batch <<EOF
set nat_zones.${module}=${module}_state
set nat_zones.${module}.zones="$*"
EOF
}

nat_state_get_zones()
{
	local nat_zones="/var/run/nat_zones"
	local module="$1"

	[ ! -f "$nat_zones" ] && return

	uci -c /var/run/ get nat_zones.${module}.zones 2>/dev/null
}

nat_get_effect_ifaces()
{
	local __NZ__iface="CONFIG_NAT_ZONE_EFFECT_${1//[ -]/}"
	local __NZ__ret_v

	eval "__NZ__ret_v=\$$__NZ__iface"
	[ "$__NZ__ret_v" != "" ] || eval "$__NZ__iface=$(zone -F zone_get_effect_ifaces -P $1)"

	eval "$2=\$$__NZ__iface"
}

nat_get_device_byif()
{
	local __NZ__device="CONFIG_NAT_ZONE_DEV_BYIF_${1//[ -]/}"
	local __NZ__ret_v

	eval "__NZ__ret_v=\$$__NZ__device"
	[ "$__NZ__ret_v" != "" ] || eval "$__NZ__device=$(zone -F zone_get_device_byif -P $1)"

	eval "$2=\$$__NZ__device"
}

nat_config_once() # <function> <type>
{
	local func=$1
	local type=$2
	shift 2

	local config=cfg00nil
	nat_config__once()
	{
		config=$1
	}
	config_foreach nat_config__once "$type"

	$func $config "$@"
}

nat_config_get_section()  # <config> <prefix> <type> <name> <default> ...
{
	local config=$1
	local prefix=$2
	shift 2

	if [[ -z "$config" ]]; then
		return 1
	fi
	if [[ -n "$prefix" ]]; then
		prefix="${prefix}_"
		export ${NO_EXPORT:+-n} -- "${prefix}NAME"="${config}"
		config_get "${prefix}TYPE" "$config" TYPE
	fi

	local enabled
	config_get_bool enabled "$config" enabled 1
	[ $enabled -eq 1 ] || return 1

	[ "$1" == '{' ] && shift
	while [ $# -ge 3 ]; do
		local type=$1
		local name=$2
		local dflt=$3
		shift 3
		# TODO: Move handling of defaults to /lib/functions.sh
		# and get replace the case block with the following 
		# two lines:
		# type=${type#string}
		# config_get${type:+_${type}} "${prefix}${name}" "$config" "$name" "$dflt" || return
		case "$type" in
			string)
				local tmp
				config_get tmp "$config" "$name" || return
				[ -z "$tmp" ] && tmp=$dflt
				export ${NO_EXPORT:+-n} -- "${prefix}${name}=${tmp}"
				continue
			;;
			boolean)
				type=bool
			;;
		esac;

		local cmd=${prefix}config_get_${type}
		type $cmd > /dev/null || {
			cmd=config_get_${type} 
		}
		type $cmd > /dev/null || {
			echo "config type $type (for $name) not supported" >&2
			return 1
		}
		$cmd "${prefix}${name}" "$config" "$name" "$dflt" || return
	done
}

nat_get_rule_num()
{
	config_get enable $1 enable
	if [ -n "${enable}" -a -n "$3" -a "${enable}" == "on" ]; then
		config_get interfaces $1 interface
		for interface in $interfaces;do
			[ -z "${interface}" -o "${interface}" != "$3" ] && continue
			case $2 in
				rule_vs)
					let ruleNumVs++
					let ruleNum++
				;;
				rule_dmz)
					let ruleNumDmz++
					let ruleNum++
				;;
				rule_onenat)
					let ruleNumOnenat++
					let ruleNum++
				;;
				rule_pt)
					let ruleNumPt++
					let ruleNum++
				;;
			esac
			break
		done
	fi
}

nat_check_root()
{
	local table=$1
	local root=$2
	local parent=$3

	if ! iptables -w -t $table -C $root -j $parent 2>/dev/null; then
		nat_exec_and_print "iptables -w -t $table -I $root -j $parent"
	fi
}

nat_hotplug_check_root()
{
	local iface="$1"
	local zone="$2"
	local hook
	local chains="zone_${zone}_natfd zone_${zone}_natip"

	for chain in ${chains}; do
		case $chain in
		*natfd) hook=forward;;
		*natip) hook=input;;
		esac

		nat_check_root "filter" "zone_${iface}_${hook}" "$chain"
	done
}

nat_hotplug_send_sig()
{
	local nat_bg_pid

	read nat_bg_pid </var/run/nat_bg.pid
	if [ -z "$nat_bg_pid" ] || ! kill -USR1 "$nat_bg_pid" 2>/dev/null;then
		return 1
	fi

	return 0
}

nat_hotplug_check_rule()
{
	local iface="$1"
	local action="$2"
	local vpn="$3"
	local ruleNum=0
	local ruleNumVs=0
	local ruleNumPt=0
	local ruleNumDmz=0
	local ruleNumOnenat=0
	local func zone

	if [ "$action" = "ifup" ];then
		if [ "$vpn" = "1" ];then
			func=zone_dyn_vpn_name
		else
			func=zone_dyn_name
		fi

		$func "$iface" zone || {
			NAT_LOG_PRINT "act:$action get dyn zone fail for $iface"
			return
		}

		nat_config_load
		config_foreach nat_get_rule_num rule_vs "rule_vs" $zone
		config_foreach nat_get_rule_num rule_dmz "rule_dmz" $zone
		config_foreach nat_get_rule_num rule_onenat "rule_onenat" $zone
		config_foreach nat_get_rule_num rule_pt "rule_pt" $zone

		if [[ "$ruleNum" -gt 0 ]]; then
			nat_hotplug_check_root "$iface" "$zone"
		fi
	fi

	# mark hotplug missing when we send signal to nat_bg failed
	nat_hotplug_send_sig || {
		echo >"/var/run/nat_bg.hotplug.miss"
		# send again
		nat_hotplug_send_sig
	}
}

############### Common Functions ###############
nat_integer()
{
	if [[ -n "$(echo $1|sed -n "/^[0-9]\+$/p")" ]]; then
		return $NAT_TRUE
	fi
	return $NAT_FALSE
}

nat_port()
{
	if nat_integer $1 && [[ "$1" -ge 1 && "$1" -le 65535 ]]; then
		return $NAT_TRUE
	fi
	return $NAT_FALSE
}

nat_enable_network()
{
	iptables -w -D forward -j DROP
}

nat_disable_network()
{
	if ! iptables -w -C forward -j DROP 2>/dev/null; then
		iptables -w -I forward 1 -j DROP
	fi
}

nat_is_loaded()
{
	local bool=$(uci_get_state nat core loaded) 
	return $((! ${bool:-0}))
}


nat_napt_whole()
{
	local whole_napt

	config_get whole_napt global whole_napt
	[ "$whole_napt" = "" ] && whole_napt="$(uci get nat.global.whole_napt)"
	[ "$whole_napt" == "on" ]
}

nat_napt_lan_ipaddr()
{
	if ! $(nat_napt_whole); then
		echo $(uci get network.lan.ipaddr)
	else
		echo 0.0.0.0
	fi
}

nat_napt_lan_netmask()
{
	if ! $(nat_napt_whole); then
		echo $(uci get network.lan.netmask)
	else
		echo 0.0.0.0
	fi
}

nat_prerouting_mangle_hotplug()
{
	$(nat_napt_whole) || return
	[ "$2" != "br-lan" ] && return
	
	case "$1" in
		ifup)
			iptables -w -t mangle -L ${NAT_PREROUTING_MANGLE} >/dev/null 2>&1 && {
				iptables -w -t mangle -A ${NAT_PREROUTING_MANGLE} -m set --match-set lan_iface src -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK_LANIN}
				iptables -w -t mangle -A ${NAT_PREROUTING_MANGLE} -m set --match-set lan_iface src -j EXTMARK --or-mark ${NAPT_DEVICE_MARK_LANIN}
			}
		;;
		ifdown)
			iptables -w -t mangle -L ${NAT_PREROUTING_MANGLE} >/dev/null 2>&1 && {	
				iptables -w -t mangle -D ${NAT_PREROUTING_MANGLE} -m set --match-set lan_iface src -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK_LANIN}
				iptables -w -t mangle -D ${NAT_PREROUTING_MANGLE} -m set --match-set lan_iface src -j EXTMARK --or-mark ${NAPT_DEVICE_MARK_LANIN}
			}
		;;
	esac
}

nat_prerouting_mangle_create()
{
	$(nat_napt_whole) || return
	
	iptables -w -t mangle -L ${NAT_PREROUTING_MANGLE} >/dev/null 2>&1 || {	
		iptables -w -t mangle -N ${NAT_PREROUTING_MANGLE}
		iptables -w -t mangle -I PREROUTING 1 -j ${NAT_PREROUTING_MANGLE}
	}

	iptables -w -t mangle -F ${NAT_PREROUTING_MANGLE}
	iptables -w -t mangle -A ${NAT_PREROUTING_MANGLE} -m set --match-set lan_iface src -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK_LANIN}
	iptables -w -t mangle -A ${NAT_PREROUTING_MANGLE} -m set --match-set lan_iface src -j EXTMARK --or-mark ${NAPT_DEVICE_MARK_LANIN}
}

nat_prerouting_mangle_clean()
{
	$(nat_napt_whole) || return

	iptables -w -t mangle -L ${NAT_PREROUTING_MANGLE} >/dev/null 2>&1 && {
		iptables -w -t mangle -F ${NAT_PREROUTING_MANGLE}
	}
}

#1: action
#2: device
nat_napt_iface_list_hotplug()
{
	$(nat_napt_whole) || return

	[ -f ${NAPT_DEVICE_CACHE} ] || touch ${NAPT_DEVICE_CACHE}
	case "$1" in
		ifup)
			echo "$2" >> ${NAPT_DEVICE_CACHE}
			iptables -w -t mangle -L ${NAPT_DEVICE_CHAIN} >/dev/null 2>&1 && {
				if [ "$2" = "br-lan" ]; then
					iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface src -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface src -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface dst -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface dst -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
				else
					iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -i $2 -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -i $2 -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -o $2 -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -o $2 -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
				fi
			}
		;;
		ifdown)
			sed -i "/^${2}/d" ${NAPT_DEVICE_CACHE}
			iptables -w -t mangle -L ${NAPT_DEVICE_CHAIN} >/dev/null 2>&1 && {	
				if [ "$2" = "br-lan" ]; then
					iptables -w -t mangle -D ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface src -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -D ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface src -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -D ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface dst -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -D ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface dst -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
				else
					iptables -w -t mangle -D ${NAPT_DEVICE_CHAIN} -i $2 -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -D ${NAPT_DEVICE_CHAIN} -i $2 -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -D ${NAPT_DEVICE_CHAIN} -o $2 -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
					iptables -w -t mangle -D ${NAPT_DEVICE_CHAIN} -o $2 -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
				fi
			}
		;;
	esac
	
	nat_prerouting_mangle_hotplug $1 $2
}

nat_napt_iface_list_create()
{
	$(nat_napt_whole) || return

	iptables -w -t mangle -L ${NAPT_DEVICE_CHAIN} >/dev/null 2>&1 || {	
		iptables -w -t mangle -N ${NAPT_DEVICE_CHAIN}
		iptables -w -t mangle -I FORWARD 1 -j ${NAPT_DEVICE_CHAIN}
	}

	iptables -w -t mangle -L ${NAPT_DEVICE_OUTPUT_CHAIN} >/dev/null 2>&1 || {	
		iptables -w -t mangle -N ${NAPT_DEVICE_OUTPUT_CHAIN}
		iptables -w -t mangle -I OUTPUT 1 -j ${NAPT_DEVICE_OUTPUT_CHAIN}
	}

	iptables -w -t mangle -F ${NAPT_DEVICE_CHAIN}
	iptables -w -t mangle -F ${NAPT_DEVICE_OUTPUT_CHAIN}

	##add napt rule for local lan
	iptables -w -t mangle -A ${NAPT_DEVICE_OUTPUT_CHAIN} -m set --match-set VMe src -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
	iptables -w -t mangle -A ${NAPT_DEVICE_OUTPUT_CHAIN} -m set --match-set VMe src -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}

	for dev in $(cat ${NAPT_DEVICE_CACHE})
	do	
		if [ "$dev" = "br-lan" ]; then
			iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface src -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
			iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface src -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
			iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface dst -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
			iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -m set --match-set lan_iface dst -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
		else
			iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -i $dev -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
			iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -i $dev -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
			iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -o $dev -j CONNEXTMARK --or-mark ${NAPT_DEVICE_MARK}
			iptables -w -t mangle -A ${NAPT_DEVICE_CHAIN} -o $dev -j EXTMARK --or-mark ${NAPT_DEVICE_MARK}
		fi
	done
	
	nat_prerouting_mangle_create
}

nat_napt_iface_list_clean()
{
	$(nat_napt_whole) || return

	iptables -w -t mangle -L ${NAPT_DEVICE_CHAIN} >/dev/null 2>&1 && {
		iptables -w -t mangle -F ${NAPT_DEVICE_CHAIN}
	}
	
	iptables -w -t mangle -L ${NAPT_DEVICE_OUTPUT_CHAIN} >/dev/null 2>&1 && {
		iptables -w -t mangle -F ${NAPT_DEVICE_OUTPUT_CHAIN}
	}
	nat_prerouting_mangle_clean
}
