#!/bin/sh
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file      nat_mod_dmz.sh
# brief    
# author    Chen chen
# version   1.1.0
# date      23Apr15
# history   arg 1.0.0, 23Apr15, Chen chen, Create the file.
#           arg 1.1.0, 21Dec17, Zhangzhongwei, Rewrite the file.

DMZ_ENABLE=0

TCP_SERVICE_LIST="http https pptp ssh"
UDP_SERVICE_LIST="ike l2tp sae"

nat_service_to_port()
{ #$service
	local port http_list https_list ssh_list
	case "$1" in
	http|HTTP|Http)
		#port="$(uci get uhttpd.main.listen_http |cut -d : -f 5)"
		port="$(uci_get_state uhttpd main listen_http |cut -d : -f 5)"
		[ -z "$port" ] && port=80
		;;
	https|HTTPS|Https)
		#port="$(uci get uhttpd.main.listen_https |cut -d : -f 5)"
		port="$(uci_get_state uhttpd main listen_https |cut -d : -f 5)"
		[ -z "$port" ] && port=443
		;;
	pptp|Pptp|PPTP)
		port=1723  #default port
		;;
	l2tp|L2TP|L2tp)
		port=1701  #default port
		;;
	IKE|ike|Ike)
		port=500  #default port
		;;
	ssh|SSH|Ssh)
		port=$(uci_get administration port SSH)
		[ -z "$port" ] && port=22
		;;
	sae|SAE|Sae)
		port=4500  #default port
		;;
	*) 
		return 1
		;;
	esac

	echo "$port"
	return 0
}

nat_dmz_service_ports()
{
	local service_list=$@
	local ports port
	for ser in $service_list;do
		port=$(nat_service_to_port $ser)
		ports=${ports:+$ports,}${port:+$port}
	done
	echo $ports
	return 0
}

nat_config_dmz()
{
	nat_config_get_section "$1" rule_dmz { \
		string name "" \
		string enable "" \
		string ipaddr "" \
		string interface "" \
		string fromAddr "" \
		string natIndex "" \
		string wan_ip "" \
	}
}

nat_post_load_rule_dmz()
{
	if [[ "$DMZ_ENABLE" -eq 1 ]]; then
		nat_prerouting_add $MOD_DMZ "-I 1" "-p icmp -j RETURN"
	fi
	unset DMZ_ENABLE
}

nat_load_rule_dmz()
{
	nat_config_dmz "$1"

	[ "$rule_dmz_enable" != "on" ] && return

	for zone in $rule_dmz_interface; do
		local if_ip
		local flag

		if zone_dyn_name "$zone" dyn_zone || zone_dyn_vpn_name "$zone" dyn_zone;then
			list_contains mod_zones "$dyn_zone" || append mod_zones "$dyn_zone"
		else
			NAT_LOG_PRINT "dmz get dyn zone fail for $zone"
			continue
		fi

		if [[ -n "$rule_dmz_wan_ip" ]]; then		#添加替换字段
			for wan_ip in $rule_dmz_wan_ip; do	#循环添加别名ip规则
				ifname=${wan_ip%%-*}
				[ "$zone" != "$ifname" ] && continue

				flag="true"
				if_ip=${wan_ip##*-}
				[ -z "$if_ip" -o "$if_ip" = "0.0.0.0" ] && continue

				local ipgroup_src=$rule_dmz_fromAddr
				local index=$rule_dmz_natIndex

				if [[ "$ipgroup_src" != "" ]]; then
					# Build Filter Rules
					nat_filter_add $dyn_zone $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -m set --match-set $ipgroup_src src \
						-d $rule_dmz_ipaddr -m conntrack --ctstate DNAT -j ACCEPT"

					# No need to specify INPUT device, for DMZ loopback. Packets from other INPUT devices would be drop in the filter table.

					# Build prerouting Rules, for DNAT
					nat_prerouting_add $MOD_DMZ "-A" "-d $if_ip -m set --match-set ${dyn_zone}_ifname src -p 50 -m esp --espspi 0:0 -m set --match-set $ipgroup_src src -j RETURN"
					nat_prerouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d $if_ip -m set --match-set ${dyn_zone}_ifname src -m set --match-set $ipgroup_src src \
						-j DNAT --to-destination $rule_dmz_ipaddr ${index:+--nat-index $index}"
					#一体机执行如下打mark操作
					[ "x"`is_all_in_one` = "x1" ] && nat_prerouting_mark_add $MOD_DMZ "-I" "-m comment --comment ${rule_dmz_name},extmark:${ALL_IN_ONE_MARK} -d $if_ip -m set --match-set ${dyn_zone}_ifname src -m set --match-set $ipgroup_src src \
					-j EXTMARK --or-mark ${ALL_IN_ONE_MARK}"

					nat_prerouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d $if_ip -m set --match-set $ipgroup_src src \
						-m extmark --extmark ${NAPT_DEVICE_MARK_LANIN}/${NAPT_DEVICE_MARK_LANIN} -j DNAT --to-destination $rule_dmz_ipaddr ${index:+--nat-index $index}"

					# Build postrouting Rules, for Loopback
					nat_postrouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d ${rule_dmz_ipaddr}/32 -m set --match-set $ipgroup_src src \
						-m extmark --extmark ${NAPT_DEVICE_MARK_LANIN}/${NAPT_DEVICE_MARK_LANIN} -m conntrack --ctstate DNAT -j SNAT --to ${if_ip} ${index:+--nat-index $index}"
				else
					# Build Filter Rules
					nat_filter_add $dyn_zone $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d $rule_dmz_ipaddr \
						-m conntrack --ctstate DNAT -j ACCEPT"

					# No need to specify INPUT device, for DMZ loopback. Packets from other INPUT devices would be drop in the filter table.

					# Build prerouting Rules, for DNAT
					nat_prerouting_add $MOD_DMZ "-A" "-d $if_ip -m set --match-set ${dyn_zone}_ifname src -p 50 -m esp --espspi 0:0 -j RETURN"
					nat_prerouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d $if_ip -m set --match-set ${dyn_zone}_ifname src \
						-j DNAT --to-destination $rule_dmz_ipaddr ${index:+--nat-index $index}"
					#一体机执行如下打mark操作
					[ "x"`is_all_in_one` = "x1" ] && nat_prerouting_mark_add $MOD_DMZ "-I" "-m comment --comment ${rule_dmz_name},extmark:${ALL_IN_ONE_MARK} -d $if_ip -m set --match-set ${dyn_zone}_ifname src \
					-j EXTMARK --or-mark ${ALL_IN_ONE_MARK}"

					nat_prerouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d $if_ip -m extmark --extmark ${NAPT_DEVICE_MARK_LANIN}/${NAPT_DEVICE_MARK_LANIN} \
						-j DNAT --to-destination $rule_dmz_ipaddr ${index:+--nat-index $index}"

					# Build postrouting Rules, for Loopback
					nat_postrouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d ${rule_dmz_ipaddr}/32 \
						-m extmark --extmark ${NAPT_DEVICE_MARK_LANIN}/${NAPT_DEVICE_MARK_LANIN} -m conntrack --ctstate DNAT -j SNAT --to ${if_ip} ${index:+--nat-index $index}"
				fi
				DMZ_ENABLE=1
			done
		fi
		if [[ -z "$rule_dmz_wan_ip" || -z "$flag" ]]; then		#默认的ip添加规则
			local ipgroup_src=$rule_dmz_fromAddr
			local index=$rule_dmz_natIndex

			if [[ "$ipgroup_src" != "" ]]; then
				# Build Filter Rules
				nat_filter_add $dyn_zone $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -m set --match-set $ipgroup_src src \
					-d $rule_dmz_ipaddr -m conntrack --ctstate DNAT -j ACCEPT"

				# No need to specify INPUT device, for DMZ loopback. Packets from other INPUT devices would be drop in the filter table.

				# Build prerouting Rules, for DNAT
				nat_prerouting_add $MOD_DMZ "-A" "-m set --match-set ${dyn_zone}_ipv4 dst -m set --match-set ${dyn_zone}_ifname src -p 50 -m esp --espspi 0:0 -m set --match-set $ipgroup_src src -j RETURN"
				nat_prerouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -m set --match-set ${dyn_zone}_ipv4 dst -m set --match-set ${dyn_zone}_ifname src -m set --match-set $ipgroup_src src \
					-j DNAT --to-destination $rule_dmz_ipaddr ${index:+--nat-index $index}"
				nat_prerouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -m set --match-set ${dyn_zone}_ipv4 dst -m set --match-set $ipgroup_src src -m extmark --extmark ${NAPT_DEVICE_MARK_LANIN}/${NAPT_DEVICE_MARK_LANIN} \
					-j DNAT --to-destination $rule_dmz_ipaddr ${index:+--nat-index $index}"

				# Build postrouting Rules, for Loopback
				nat_postrouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d ${rule_dmz_ipaddr}/32 -m set --match-set $ipgroup_src src \
					-m extmark --extmark ${NAPT_DEVICE_MARK_LANIN}/${NAPT_DEVICE_MARK_LANIN} -m conntrack --ctstate DNAT -j NAT_SET --snat ${dyn_zone}_ipv4 --to-source 0.0.0.0 ${index:+--nat-index $index}"
			else
				# Build Filter Rules
				nat_filter_add $dyn_zone $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d $rule_dmz_ipaddr \
					-m conntrack --ctstate DNAT -j ACCEPT"

				# No need to specify INPUT device, for DMZ loopback. Packets from other INPUT devices would be drop in the filter table.

				# Build prerouting Rules, for DNAT
				nat_prerouting_add $MOD_DMZ "-A" "-m set --match-set ${dyn_zone}_ipv4 dst -m set --match-set ${dyn_zone}_ifname src -p 50 -m esp --espspi 0:0 -j RETURN"
				nat_prerouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -m set --match-set ${dyn_zone}_ipv4 dst -m set --match-set ${dyn_zone}_ifname src \
					-j DNAT --to-destination $rule_dmz_ipaddr ${index:+--nat-index $index}"
				nat_prerouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -m set --match-set ${dyn_zone}_ipv4 dst -m extmark --extmark ${NAPT_DEVICE_MARK_LANIN}/${NAPT_DEVICE_MARK_LANIN} \
					-j DNAT --to-destination $rule_dmz_ipaddr ${index:+--nat-index $index}"

				# Build postrouting Rules, for Loopback
				nat_postrouting_add $MOD_DMZ "-A" "-m comment --comment $rule_dmz_name -d ${rule_dmz_ipaddr}/32 \
					-m extmark --extmark ${NAPT_DEVICE_MARK_LANIN}/${NAPT_DEVICE_MARK_LANIN} -m conntrack --ctstate DNAT -j NAT_SET --snat ${dyn_zone}_ipv4 --to-source 0.0.0.0 ${index:+--nat-index $index}"
			fi
			DMZ_ENABLE=1
		fi
		unset flag
	done
}

nat_flush_rule_dmz()
{
	nat_prerouting_flush ${MOD_DMZ}
	nat_postrouting_flush ${MOD_DMZ}

	local zones=$(nat_state_get_zones ${MOD_DMZ})
	[ -z "$zones" ] && return

	for zone in $zones; do
		nat_filter_flush ${MOD_DMZ} $zone
	done
}

nat_rule_dmz_operation()
{
	local operation=$1

	mod_zones=""
	NAT_LOG_PRINT "@ enter DMZ"
	case $operation in
		start)
			NAT_LOG_PRINT "  building new DMZ rules"
			nat_config_load
			config_foreach nat_load_rule_dmz rule_dmz
			nat_post_load_rule_dmz
			nat_state_set_zones ${MOD_DMZ} "$mod_zones"
		;;
		stop)
			NAT_LOG_PRINT "  flusing old DMZ rules"
			nat_flush_rule_dmz
			nat_state_set_zones ${MOD_DMZ} ""
		;;
	esac
}

