#!/bin/sh
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file      nat_mod_napt.sh
# brief    
# author    Chen chen
# version   1.1.0
# date      23Apr15
# history   arg 1.0.0, 23Apr15, Chen chen, Create the file.
#           arg 1.1.0, 21Dec17, Zhangzhongwei, Rewrite the file.
. /lib/zone/zone_conf.sh

nat_config_rule_napt()
{
	nat_config_get_section "$1" rule_napt { \
		string name "" \
		string enable "" \
		string interface "" \
		string ipaddr "" \
		string mask "" \
		string comment "" \
	} || return
}

nat_set_no_nat()
{
	config_load nat

	set_no_nat()
	{
		nat_config_rule_napt "$1"
		if [[ "$rule_napt_enable" == "off" ]]; then
			for zone in $rule_napt_interface; do
				[ -z "$zone" ] && continue
				local device=$(zone_get_device "${zone}")
				local ifaces=$(zone_get_wan_ifaces "${zone}")
				for iface in $ifaces; do
					local device=$(zone_get_device_byif "${iface}")
					if [[ -n "$device" ]]; then	
						iptables -w -I postrouting_rule_multinat -t nat -o $device -j RETURN
					fi
				done
			done
		fi
	}
	config_foreach set_no_nat rule_napt
}

dnat_iptables_add()
{
	local status
	local ipgroup
	local ip_netmask

	config_get status     "$1" status
	config_get ipgroup    "$1" ipgroup
	config_get ip_netmask "$1" ip_netmask

	ipset create $ipgroup hash:net

	for item in $ip_netmask; do
		ipset add $ipgroup $item
	done

	if [[ "$status" == "on" ]]; then
		iptables -I postrouting_rule_multinat -t nat -m set --match-set $ipgroup src,src -j ACCEPT
	fi
}

nat_load_rule_napt()
{
	support_dpdk=$(uci -c /etc/profile.d/ get profile.@global[0].support_dpdk)
	if [ $? -eq 1 ]; then
		support_dpdk=1
	fi
	local mutli_nat=$(_zone_gname "$(global_wan_name_list)")
	if [ "$support_dpdk" == "1" ]; then
		nat_postrouting_add $MOD_NAPT "-A" "-m comment --comment $MOD_NAPT -p tcp -m set --match-set ${mutli_nat} dst \
			-m connextmark --mark ${NAPT_DEVICE_MARK}/${NAPT_DEVICE_MARK} -j MASQUERADE --to-ports 1025-19999"
	fi
	nat_postrouting_add $MOD_NAPT "-A" "-m comment --comment $MOD_NAPT -m set --match-set ${mutli_nat} dst \
		-m connextmark --mark ${NAPT_DEVICE_MARK}/${NAPT_DEVICE_MARK} -j MASQUERADE"
	nat_postrouting_add $MOD_NAPT "-A" "-m comment --comment $MOD_NAPT -m set --match-set ${mutli_nat} dst \
		-m extmark --extmark ${NAPT_DEVICE_MARK}/${NAPT_DEVICE_MARK} -j MASQUERADE"

	nat_filter_add lan $MOD_NAPT "-I 1" "-m comment --comment $MOD_NAPT -m set --match-set ${mutli_nat} dst \
		-m connextmark --mark ${NAPT_DEVICE_MARK}/${NAPT_DEVICE_MARK} -j ACCEPT" lan
	nat_filter_add lan $MOD_NAPT "-I 1" "-m comment --comment $MOD_NAPT -m set --match-set ${mutli_nat} dst \
		-m extmark --extmark ${NAPT_DEVICE_MARK}/${NAPT_DEVICE_MARK} -j ACCEPT" lan

	iptables -w -A zone_vpn_forward -m set --match-set ${mutli_nat} dst -m connextmark --mark ${NAPT_DEVICE_MARK}/${NAPT_DEVICE_MARK} -j ACCEPT
	iptables -w -A zone_vpn_forward -m set --match-set ${mutli_nat} dst -m extmark --extmark ${NAPT_DEVICE_MARK}/${NAPT_DEVICE_MARK} -j ACCEPT
	iptables -w -t nat -I postrouting_rule_multinat 1 -j "${chain_dslt}"
	## -- DNAT iptables START -- ##

	nat_config_load
	config_foreach dnat_iptables_add "rule_dnat"

	## -- DNAT iptables END   -- ##
}

nat_post_load_rule_napt()
{
	local device
	local dslt=`ifconfig | grep dslt`

	nat_get_device_byif lan device
	if [[ -n "$device" ]]; then
		if [ "$device" != "br-lan" ]; then
			nat_filter_add lan $MOD_NAPT "-A" "-o $device -j ACCEPT"
		else
			nat_filter_add lan $MOD_NAPT "-A" "-m set --match-set lan_iface dst -j ACCEPT"
		fi
	fi

	if [ -n "$dslt" ];then
		nat_filter_add lan $MOD_NAPT "-A" "-m set --match-set lan_iface src -j ACCEPT"
	fi

	nat_filter_add lan $MOD_NAPT "-A" "-j DROP"

	nat_napt_iface_list_create
	nat_bg_pid="$(cat /var/run/nat_bg.pid 2>/dev/null)" && kill -USR1 $nat_bg_pid
}

nat_flush_rule_napt()
{
	nat_postrouting_flush ${MOD_NAPT}
	nat_filter_flush ${MOD_NAPT} lan
	fw flush 4 f zone_vpn_forward

	nat_napt_iface_list_clean
}

nat_rule_napt_operation()
{
	local operation=$1
	NAT_LOG_PRINT "@ enter Multi-NAT"
	case $operation in
		start)
			NAT_LOG_PRINT "  building new Multi-NAT rules"
			nat_load_rule_napt
			nat_set_no_nat
			nat_post_load_rule_napt
		;;
		stop)
			NAT_LOG_PRINT "  flusing old Multi-NAT rules"
			nat_flush_rule_napt
		;;
	esac
}
