#!/bin/sh
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file      nat_mod_one.sh
# brief	
# author    Chen chen
# version   1.1.0
# date      23Apr15
# history   arg 1.0.0, 23Apr15, Chen chen, Create the file.
#           arg 1.1.0, 21Dec17, Zhangzhongwei, Rewrite the file.

nat_config_rule_onenat()
{
	nat_config_get_section "$1" rule_onenat { \
		string name "" \
		string enable "" \
		string interface "" \
		string internal_ip "" \
		string external_ip "" \
		string dmz "" \
		string comment "" \
	} || return
}

nat_load_rule_onenat()
{
	nat_config_rule_onenat "$1"

	[ "$rule_onenat_enable" != "on" ] && return

	[ "$ipsec_return_added" = "0" ] && {
		nat_postrouting_add $MOD_ONE "-A" "-m policy --dir out --pol ipsec --mode tunnel -j RETURN"
		ipsec_return_added=1
	}

	for iface in $rule_onenat_interface; do
		local effect_iface
		local device
		local zone

		if zone_dyn_name "$iface" zone || zone_dyn_vpn_name "$iface" zone;then
			list_contains mod_zones "$zone" || append mod_zones "$zone"
		else
			NAT_LOG_PRINT "onenat get dyn zone fail for $iface"
			continue
		fi

		nat_get_effect_ifaces "${iface}" effect_iface
		nat_get_device_byif "${effect_iface}" device
		[ -z "$device" ] && continue

		local netmask="255.255.255.255"
		ifconfig $device:$one_nat_id ${rule_onenat_external_ip} netmask $netmask up
		echo "ifconfig $device:$one_nat_id ${rule_onenat_external_ip} netmask $netmask up" >> /tmp/.nat/one_nat_id
		let one_nat_id++

		nat_postrouting_add $MOD_ONE "-A" "-m comment --comment $rule_onenat_name -m set --match-set ${zone}_ifname dst \
			-s $rule_onenat_internal_ip -j SNAT --to-source $rule_onenat_external_ip"

		[ "$rule_onenat_dmz" != "on" ] && continue
		# No need to specify INPUT device, for DMZ loopback. Packets from other INPUT devices would be drop in the filter table.
		nat_prerouting_add $MOD_ONE "-A" "-m comment --comment $rule_onenat_name \
			-d $rule_onenat_external_ip -j DNAT --to-destination $rule_onenat_internal_ip"

		# for One-to-one NAT Loopback
		nat_postrouting_add $MOD_ONE "-A" "-m comment --comment $rule_onenat_name -d $rule_onenat_internal_ip \
			-j SNAT --to-source $rule_onenat_external_ip -m extmark --extmark ${NAPT_DEVICE_MARK_LANIN}/${NAPT_DEVICE_MARK_LANIN}"

		nat_filter_add $zone $MOD_ONE "-A" "-m comment --comment $rule_onenat_name \
			-d $rule_onenat_internal_ip -m conntrack --ctstate DNAT  -j ACCEPT"
	done
}

nat_flush_rule_onenat()
{
	nat_prerouting_flush ${MOD_ONE}
	nat_postrouting_flush ${MOD_ONE}

	local zones=$(nat_state_get_zones ${MOD_ONE})
	[ -z "$zones" ] && return

	for zone in $zones; do
		nat_filter_flush ${MOD_ONE} $zone
	done
}

nat_rule_one_nat_operation()
{
	local operation=$1

	mod_zones=""
	ipsec_return_added=0
	NAT_LOG_PRINT "@ enter One-NAT"
	case $operation in
		start)
			NAT_LOG_PRINT "  building new One-NAT rules"

			# incase nat_bg access one_nat_id during we deleting one_nat_id
			[ -f "/tmp/.nat/one_nat_id" ] && {
				lock /var/.nat/one_nat_id.lock
				while read line; do ifconfig "$(echo $line|cut -d ' ' -f 2)" down; done < /tmp/.nat/one_nat_id
				rm -f /tmp/.nat/one_nat_id >/dev/null 2>&1
				lock -u /var/.nat/one_nat_id.lock
			}

			local one_nat_id=1
			nat_config_load
			config_foreach nat_load_rule_onenat rule_onenat
			nat_state_set_zones ${MOD_ONE} "$mod_zones"
		;;
		stop)
			NAT_LOG_PRINT "  flusing old One-NAT rules"
			nat_flush_rule_onenat
			nat_state_set_zones ${MOD_ONE} ""
		;;
	esac
}
