#!/bin/sh
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file      nat_mod_pt.sh
# brief    
# author    Chen chen
# version   1.1.0
# date      20Apr15
# history   arg 1.0.0, 21Apr15, Chen chen, Create the file.
#           arg 1.1.0, 21Dec17, Zhangzhongwei, Rewrite the file.

VALID_PT_ZONES=

nat_config_rule_pt()
{
	nat_config_get_section "$1" rule_pt { \
		string name "" \
		string enable "" \
		string interface "" \
		string ext_interface ""\
		string trigger_port "" \
		string trigger_protocol "" \
		string external_port "" \
		string external_protocol "" \
		string wan_ip "" \
	} || return
}

nat_load_rule_pt()
{
	local tri_proto="all tcp udp"
	local pub_proto="all tcp udp"

	nat_config_rule_pt "$1"
	
	if [[ -z "$rule_pt_interface" ]]; then
		NAT_LOG_PRINT "! interface is not set"
		return 1
	fi

	if [[ -n "$rule_pt_trigger_protocol" ]]; then
		rule_pt_trigger_protocol=$(echo $rule_pt_trigger_protocol |tr '[A-Z]' '[a-z]')
	fi

	if [[ -n "$rule_pt_external_protocol" ]]; then
		rule_pt_external_protocol=$(echo $rule_pt_external_protocol |tr '[A-Z]' '[a-z]')
	fi

	list_contains tri_proto ${rule_pt_trigger_protocol}  || return
	list_contains pub_proto ${rule_pt_external_protocol} || return

	nat__do_pt_rule ()
	{
		local tri_proto=$1
		local pub_proto=$2
		local zone=$3

		for tri_port in $rule_pt_trigger_port; do
			local tri_port_start=${tri_port%-*}
			local tri_port_end=${tri_port#*-}

			for ex_port in $rule_pt_external_port; do
				local ex_port_start=${ex_port%-*}
				local ex_port_end=${ex_port#*-}

				if [[ -n "$tri_port_start" && -n "$tri_port_end" && -n "$ex_port_start" && -n "$ex_port_end" ]]; then
					local ex_ports=${ex_port_start}-${ex_port_end}
					local tri_ports=${tri_port_start}-${tri_port_end}

					if [[ "${ex_port_start}" -gt "${ex_port_end}" ]]; then
						ex_ports=${ex_port_end}-${ex_port_start}
					fi
					if [[ "${tri_port_start}" -gt "${tri_port_end}" ]]; then
						tri_ports=${tri_port_end}-${tri_port_start}
					fi

					# Build Filter Rules
					local list=${tri_proto}

					[[ "$tri_proto" == "all" ]] && list="tcp udp"
					for pro in $list; do
						nat_filter_add lan $MOD_PT "-A" "-m comment --comment $rule_pt_name -p $pro -j TRIGGER \
							--trigger-type out --trigger-proto $pub_proto --trigger-match $tri_ports --trigger-relate $ex_ports"
					done

					##### save the valid zone list for PT-init-bottom half
					list_contains VALID_PT_ZONES $zone || append VALID_PT_ZONES $zone
				fi
			done
		done
	}

	[ "$rule_pt_enable" != "on" ] && return

	# Only for the first connection.
	echo "rule_pt_interface=$rule_pt_interface"
	list_contains mod_zones lan || append mod_zones lan
	for iface in $rule_pt_interface; do		#多个wan口时
		local flag
		local zone

		echo "rule_pt_wan_ip=$rule_pt_wan_ip"
		if zone_dyn_name "$iface" zone || zone_dyn_vpn_name "$iface" zone;then
			list_contains mod_zones "$zone" || append mod_zones "$zone"
		else
			NAT_LOG_PRINT "get dyn zone fail for $iface"
			continue
		fi

		if [[ -n "$rule_pt_wan_ip" ]]; then		#添加替换字段
			for wan_ip in $rule_pt_wan_ip; do	#循环添加规则
				ifname=${wan_ip%%-*}
				[ "$iface" != "$ifname" ] && continue

				flag="true"
				local if_ip=${wan_ip##*-}
				if [[ -n "$if_ip" ]]; then
					nat__do_pt_rule $rule_pt_trigger_protocol $rule_pt_external_protocol $zone
				fi
			done
		fi
		if [[ -z "$rule_pt_wan_ip" || -z "$flag" ]]; then		#使用默认ip
			nat__do_pt_rule $rule_pt_trigger_protocol $rule_pt_external_protocol $zone
		fi
		unset flag
	done
}

nat_post_load_rule_pt()
{
	nat_config_rule_pt "$1"

	echo "VALID_PT_ZONES=$VALID_PT_ZONES"
	for zone in $VALID_PT_ZONES; do
		echo "rule_pt_wan_ip=$rule_pt_wan_ip"
		local flag
		local dyn_zone

		if [[ -n "$rule_pt_wan_ip" ]]; then
			for wan_ip in $rule_pt_wan_ip; do	#循环添加规则
				ifname=${wan_ip%%-*}
				if zone_dyn_name "$ifname" dyn_zone || zone_dyn_vpn_name "$ifname" dyn_zone;then
					[ "$zone" != "$dyn_zone" ] && continue
				else
					NAT_LOG_PRINT "pt get dyn zone fail for $ifname"
					continue
				fi

				flag="true"
				local if_ip=${wan_ip##*-}
				if [[ -n "if_ip" && "$if_ip" != "0.0.0.0" ]]; then
					nat_filter_add $zone $MOD_PT "-A" "-j TRIGGER --trigger-type in"
					nat_prerouting_add $MOD_PT "-A" "-d $if_ip -j TRIGGER --trigger-type dnat"
				fi
			done
		fi
		if [[ -z "$rule_pt_wan_ip" || -z "$flag" ]]; then	#默认拨号ip添加规则
			nat_filter_add $zone $MOD_PT "-A" "-j TRIGGER --trigger-type in"
			nat_prerouting_add $MOD_PT "-A" "-m set --match-set ${zone}_ipv4 dst -j TRIGGER --trigger-type dnat"
		fi
		unset flag
	done

}

nat_flush_rule_pt()
{
	nat_prerouting_flush ${MOD_PT}

	local zones=$(nat_state_get_zones ${MOD_PT})
	[ -z "$zones" ] && return

	for zone in $zones; do
		nat_filter_flush ${MOD_PT} $zone
	done
}

nat_rule_pt_operation()
{
	local operation=$1

	mod_zones=""
	NAT_LOG_PRINT "@ enter Port Triggering"
	case $operation in
		start)
			NAT_LOG_PRINT "  building new Port Triggering rules"
			nat_config_load
			config_foreach nat_load_rule_pt rule_pt
			config_foreach nat_post_load_rule_pt rule_pt
			unset VALID_PT_ZONES
			nat_state_set_zones ${MOD_PT} "$mod_zones"
		;;
		stop)
			NAT_LOG_PRINT "  flusing old Port Triggering rules"
			nat_flush_rule_pt
			nat_state_set_zones ${MOD_PT} ""
		;;
	esac
}
