#!/bin/sh

. /lib/net_share.sh
. /lib/vnet/vnet_init.sh

[ -x /usr/sbin/dhcp6c ] || exit 0

dhcp6cdir="/tmp/dhcp6c"
conffile="$dhcp6cdir/dhcp6c.conf"
dhcp6cpid="$dhcp6cdir/dhcp6c.pid"

[ -n "$INCLUDE_ONLY" ] || {
	. /lib/functions.sh
	. ../netifd-proto.sh
	init_proto "$@"
}

proto_dhcp6c_init_config() {
	proto_config_add_string "dns"
	proto_config_add_string "ip_mode"
	proto_config_add_string "ip_config"
	proto_config_add_string "ifname"
	proto_config_add_string "dns_mode"
	proto_config_add_string "lanif"
}

proto_dhcp6c_create_conffile(){
	local ip_mode="$1"
	local dns_mode="$2"
	local ifname="$3"
	local lanif="$4"
	local ip_config="$5"
	#local id=`date +%s`

	#id=`expr $id % 1000000`

	#local id2=`expr $id + 1`
	
	local id=1
	local id2=$id

	rm -rf ${dhcp6cdir}
	mkdir -p ${dhcp6cdir}


	[ "$ip_mode" == "prefix" ] && {
		echo "interface $ifname {" > "${conffile}"

		[ "$ip_config" == "dhcpv6" ] && echo -e "\tsend ia-na $id;" >> "${conffile}"
		echo -e "\tsend ia-pd $id2;" >> "${conffile}"

		[ "$dns_mode" == "dynamic" ] && echo -e "\trequest domain-name-servers;" >> "${conffile}"
		echo -e "\tscript \"/lib/netifd/dhcp6c.script\";" >> "${conffile}"

		echo -e "};\n" >>  "${conffile}"

		echo -e "id-assoc na $id {};\n" >> "${conffile}"

		echo "id-assoc pd $id2 {" >> "${conffile}"
		echo -e "\tprefix-interface $lanif {" >> "${conffile}"
		echo -e "\t\tsla-id 1;" >> "${conffile}"
		echo -e "\t};" >>  "${conffile}"
		echo "};" >>  "${conffile}"
	}

	[ "$ip_mode" == "non_temp" ] && {
		echo "interface $ifname {" > "${conffile}"
		[ "$ip_config" == "dhcpv6" ] && echo -e "\tsend ia-na $id;" >> "${conffile}"

		[ "$dns_mode" == "dynamic" ] && echo -e "\trequest domain-name-servers;" >> "${conffile}"
		echo -e "\tscript \"/lib/netifd/dhcp6c.script\";" >> "${conffile}"

		echo "};" >>  "${conffile}"

		echo "id-assoc na $id {};" >> "${conffile}"
	}
}

proto_dhcp6c_create_config() {
	local config="$1"
	local ip_mode="$2"
	local dns_mode="$3"
	local ifname="$4"
	local lanif="$5"
	local ip_config="$6"

	uci_set dhcp6c $config enabled '0'
	uci_set dhcp6c $config interface $ifname
	if [ "$ip_config" == "dhcpv6" ]; then
		uci_set dhcp6c $config enabled '1'
		uci_set dhcp6c $config na '1'
	else
		uci_set dhcp6c $config na '0'
	fi
	if [ "$ip_mode" == "prefix" ]; then
		uci_set dhcp6c $config enabled '1'
		uci_set dhcp6c $config pd '1'
		uci_set dhcp6c lan lanif $lanif
		uci_set dhcp6c lan sla_id '1'
		uci_set dhcp6c lan enabled '1'
		$(multilan_on) && uci_set dhcp6c lan enabled '0'
	else
		uci_set dhcp6c $config pd '0'
	fi
	if [ "$dns_mode" == "dynamic" ]; then
		uci_set dhcp6c $config enabled '1'
		uci_set dhcp6c $config domain_name_servers '1'
	else
		uci_set dhcp6c $config domain_name_servers '0'
	fi
	uci_set dhcp6c $config script "/lib/netifd/dhcp6c.script"

	uci_commit dhcp6c
}

wait_for_mbit() {
	local ifname="$1"
	local mbit=0

	# wait for RS.
	defcnt=$(cat /proc/sys/net/ipv6/conf/$ifname/router_solicitations)
	echo 1024 > /proc/sys/net/ipv6/conf/$ifname/router_solicitations
	echo 1 > /proc/sys/net/ipv6/conf/$ifname/disable_ipv6
	echo 0 > /proc/sys/net/ipv6/conf/$ifname/disable_ipv6
	sleep 1
	mbit=`cat /proc/sys/net/ipv6/conf/$ifname/ndisc_mbit`
	while [ "$mbit" == "-1" ];
	do
		sleep 1
		mbit=`cat /proc/sys/net/ipv6/conf/$ifname/ndisc_mbit`
	done
	echo $defcnt > /proc/sys/net/ipv6/conf/$ifname/router_solicitations
	# end of RS.
}

# todo
update_lte_netifd_config() {
    local interface=$1 #wan7_v6
	local ifname=$2 #usb0

    local lte_params=$(ubus call mc.dial get_rt_param '{"ipv6": true}')
	json_load "$lte_params"
	json_get_var error error

    # echo "#####dhcp6c.sh,interface:$interface,ifname:$ifname" > /dev/console
    [ "$error" = "0" ] && {
        json_get_var IPADDRV6 IPADDRV6
        json_get_var PREFIXV6 PREFIXV6
        json_get_var PRIDNSV6 PRIDNSV6
        json_get_var SECDNSV6 SECDNSV6
        json_get_var SITPREFIXV6 SITPREFIXV6
        json_get_var GATEWAYV6 GATEWAYV6

        # echo "#####dhcp6c.sh,update_lte_netifd_config(),IPADDRV6:$IPADDRV6,PREFIXV6:$PREFIXV6,PRIDNSV6:$PRIDNSV6,SECDNSV6:$SECDNSV6,SITPREFIXV6:$SITPREFIXV6,GATEWAYV6:$GATEWAYV6" > /dev/console

        proto_init_update "$ifname" 1
        [ -z "$IPADDRV6" ] && {
			proto_notify_error "$interface" "NO ADDRESS"
			sleep 3
			proto_setup_failed "$interface"
			return
		}

        #[ -n "$IPADDRV6" ] && [ -n "$PREFIXV6" ] && {
        #    proto_add_ipv6_address "$IPADDRV6" "$PREFIXV6"
        #}
        [ -n "$PRIDNSV6" ] && {
            proto_add_dns_server "$PRIDNSV6"
        }
        [ -n "$SECDNSV6" ] && {
            proto_add_dns_server "$SECDNSV6"
        }

        [ -n "$GATEWAYV6" ] && {
            proto_add_ipv6_route "::" 0 "$GATEWAYV6" 1024
        }

        proto_send_update "$interface"

        sleep 10
        local gw_route=`ip -6 route|grep default`
        # echo "#####dhcp6c.sh,update_lte_netifd_config(),gw_route:$gw_route,GATEWAYV6:$GATEWAYV6" > /dev/console
        [-z "$gw_route" ] && [ -n "$GATEWAYV6" ] && {
            echo "$GATEWAYV6" > /proc/sys/net/ipv6/conf/$ifname/default_gateway
            ip -6 route add "::/0" via "$GATEWAYV6" dev "$ifname" metric 1024
        }
	}
}

append_config2radvd() {
    local radvdfile="$1"
    local ifname="$2"
    local prefix="$3"
    local dns1="$4"
    local dns2="$5"

    echo -e "interface $ifname " >> "$radvdfile"
    echo -e "{" >> "$radvdfile"
    echo -e "\tAdvSendAdvert on;" >> "$radvdfile"

    echo -e "\tAdvManagedFlag off;" >> "$radvdfile"
    echo -e "\tAdvOtherConfigFlag on;" >> "$radvdfile"
    echo -e "\tprefix $prefix" >> "$radvdfile"
    echo -e "\t{" >> "$radvdfile"
    echo -e "\t\tAdvOnLink on;" >> "$radvdfile"
    echo -e "\t\tAdvAutonomous on;" >> "$radvdfile"
    echo -e "\t\tAdvRouterAddr on;" >> "$radvdfile"
    echo -e "\t};" >> "$radvdfile"

    [ -n "$dns1" -o -n "$dns2" ] && {
        [ -z "$dns1" ] && dns1="$dns2"
        [ -z "$dns2" ] && dns2="$dns1"
        echo -e "\tRDNSS $dns1 $dns2" >> "$radvdfile"
        echo -e "\t{" >> "$radvdfile"
        echo -e "\t\tAdvRDNSSPreference 8;" >> "$radvdfile"
        echo -e "\t\tAdvRDNSSOpen off;" >> "$radvdfile"
        echo -e "\t};" >> "$radvdfile"
    }

    echo -e "};" >> "$radvdfile"
}

update_lte_lan_ndppd_radvd() {
	local config=$1 #wan7_v6
	local ifname=$2 #usb0
	# echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),config:$config,ifname:$ifname" > /dev/console
    local params=$(ubus call mc.dial get_rt_param '{"ipv6": true}')
	json_load "$params"
	json_get_var error error
    [ "$error" = "0" ] && {
        json_get_var IPADDRV6 IPADDRV6
        json_get_var PREFIXV6 PREFIXV6
        json_get_var PRIDNSV6 PRIDNSV6
        json_get_var SECDNSV6 SECDNSV6
        json_get_var SITPREFIXV6 SITPREFIXV6

        echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),IPADDRV6:$IPADDRV6,PREFIXV6:$PREFIXV6,SITPREFIXV6:$SITPREFIXV6" > /dev/console
        if [ -n "$IPADDRV6" ] && [ -n "$PREFIXV6" ] && [ -n "$SITPREFIXV6" ] ; then
            local vnetwork_config="/etc/config/vnetwork"
            local networks=$(sed -n "s/config vinterface '\(NETWORK\w\+\)'/\1/p" "$vnetwork_config")
            local ipv6_dnsMode,ipv6_prefix,ipv6_priDns,ipv6_sndDns,ipv6_prefixlen,vnet_ifname
            for network in $networks; do
                ipv6_ifacev6=$(uci get vnetwork."$network".ipv6_ifacev6 2>/dev/null)
                ipv6_enable=$(uci get vnetwork."$network".ipv6_enable 2>/dev/null)
                
                [ "$ipv6_ifacev6" = "$config" ] && [ "$ipv6_enable" = "on" ] && {
                    ipv6_dnsMode=$(uci get vnetwork."$network".ipv6_dnsMode 2>/dev/null)
                    ipv6_prefix=$(uci get vnetwork."$network".ipv6_prefix 2>/dev/null)
                    ipv6_priDns=$(uci get vnetwork."$network".ipv6_priDns 2>/dev/null)
                    ipv6_sndDns=$(uci get vnetwork."$network".ipv6_sndDns 2>/dev/null)
                    ipv6_prefixlen=$(uci get vnetwork."$network".ipv6_prefixlen 2>/dev/null)
                    vnet_ifname=$(uci get vnetwork."$network".ifname 2>/dev/null)
                    [ -z "$ipv6_sndDns" ] && [ -n "$ipv6_priDns" ] && {
                        ipv6_sndDns="$ipv6_priDns"
                    }
                    echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),ipv6_dnsMode:$ipv6_dnsMode,ipv6_prefix:$ipv6_prefix,ipv6_priDns:$ipv6_priDns,ipv6_sndDns:$ipv6_sndDns,ipv6_prefixlen:$ipv6_prefixlen,vnet_ifname:$vnet_ifname" > /dev/console
                }
            done

            # update /etc/config/ndppd.conf 's ipv6 prefix value,because the wan prefix may change
            local ndppd_conf="/etc/config/ndppd"
            [ -f "$ndppd_conf" ] && {
                local new_prefix="$SITPREFIXV6:/$PREFIXV6"
                local previous_prefix=$(grep '^rule' $ndppd_conf | awk '{print $2}' | head -n 1)
                
                # echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),ndppd new_prefix:$new_prefix,previous_prefix:$previous_prefix" > /dev/console
                [ -n "$previous_prefix" ] && [ "$new_prefix" != "$previous_prefix" ] && {
                    # transferred meaning of specific char(: /)
                    escaped_previous_prefix=$(echo "$previous_prefix" | sed 's/[\/&]/\\&/g')
                    escaped_new_prefix=$(echo "$new_prefix" | sed 's/[\/&]/\\&/g')

                    # sed replace
                    sed -i "s/${escaped_previous_prefix}/${escaped_new_prefix}/g" $ndppd_conf
                    # echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),ndppd restart" > /dev/console
                    /etc/init.d/ndppd restart
                }
            }

            # update /tmp/radvd-vnetwork/radvd.conf 's ipv6 prefix/dns value,because the wan prefix/dns may change
            local radvd_conf="/tmp/radvd-vnetwork/radvd.conf"
            local radvd_pid="/tmp/radvd-vnetwork/radvd.pid"
            local new_prefix="$SITPREFIXV6:/$PREFIXV6"
            local update_flag="0"
            if [ -f "$radvd_conf" ] ; then
                #update radvd.conf prefix
                local config_exist=$(grep 'interface $vnet_ifname' $radvd_conf)
                if [ -n "$vnet_ifname" ] && [ -z "$config_exist" ] && [ -z "$ipv6_prefix" ] ; then
                    echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),radvd interface $vnet_ifname config not exist,append it" > /dev/console
                    if [ "$ipv6_dnsMode" = "auto" ] ; then
                        append_config2radvd $radvd_conf $vnet_ifname $new_prefix $PRIDNSV6 $SECDNSV6
                    else
                        append_config2radvd $radvd_conf $vnet_ifname $new_prefix $ipv6_priDns $ipv6_sndDns
                    fi
                    update_flag="1"
                else
                    local previous_prefix="$ipv6_prefix/$ipv6_prefixlen"
                    echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),radvd new_prefix:$new_prefix,previous_prefix:$previous_prefix" > /dev/console
                    [ -n "$ipv6_prefix" ] && [ -n "$ipv6_prefixlen" ] && [ "$new_prefix" != "$previous_prefix" ] && {
                        # transferred meaning of specific char(: /)
                        escaped_previous_prefix=$(echo "$previous_prefix" | sed 's/[\/&]/\\&/g')
                        escaped_new_prefix=$(echo "$new_prefix" | sed 's/[\/&]/\\&/g')

                        # sed replace
                        sed -i "s/${escaped_previous_prefix}/${escaped_new_prefix}/g" $radvd_conf
                        update_flag="1"
                    }

                    #update radvd.conf dns
                    local previous_pridns=$ipv6_priDns
                    local previous_secdns=$ipv6_sndDns
                    echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),$radvd_conf,previous_pridns:$previous_pridns,previous_secdns:$previous_secdns,PRIDNSV6:$PRIDNSV6,SECDNSV6:$SECDNSV6" > /dev/console
                    [ -n "$PRIDNSV6" ] && [ -n "$SECDNSV6" ] && [ -n "$previous_pridns" ] && [ -n "$previous_secdns" ] && [ "$ipv6_dnsMode" = "auto" ] && {
                        [ "$PRIDNSV6" != "$previous_pridns" ] && {
                            escaped_previous_dns=$(echo "$previous_pridns" | sed 's/[\/&]/\\&/g')
                            escaped_new_dns=$(echo "$PRIDNSV6" | sed 's/[\/&]/\\&/g')
                            sed -i "s/${escaped_previous_dns}/${escaped_new_dns}/g" $radvd_conf
                            update_flag="1"
                        }

                        [ "$SECDNSV6" != "$previous_secdns" ] && {
                            escaped_previous_dns=$(echo "$previous_secdns" | sed 's/[\/&]/\\&/g')
                            escaped_new_dns=$(echo "$SECDNSV6" | sed 's/[\/&]/\\&/g')
                            sed -i "s/${escaped_previous_dns}/${escaped_new_dns}/g" $radvd_conf
                            update_flag="1"
                        }
                    }
                fi
            else
                echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),radvdfile not exist,add it,vnet_ifname:$vnet_ifname,new_prefix:$new_prefix" > /dev/console
                if [ "$ipv6_dnsMode" = "auto" ] ; then
                    append_config2radvd $radvd_conf $vnet_ifname $new_prefix $PRIDNSV6 $SECDNSV6
                else
                    append_config2radvd $radvd_conf $vnet_ifname $new_prefix $ipv6_priDns $ipv6_sndDns
                fi
                update_flag="1"
            fi

            [ "$update_flag" = "1" ] && {
                [ -f "$radvd_pid" ] && kill -15 `cat "$radvd_pid"`
                sleep 1
                echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),radvd restart" > /dev/console
                vnete /usr/sbin/radvd -C "$radvd_conf" -p "$radvd_pid"
            }

            # update /etc/config/vnetwork 's option ipv6_prefix/ipv6_priDns/ipv6_sndDns
            for network in $networks; do
                ipv6_ifacev6=$(uci get vnetwork."$network".ipv6_ifacev6 2>/dev/null)
                ipv6_enable=$(uci get vnetwork."$network".ipv6_enable 2>/dev/null)
                # echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),$vnetwork_config,network:$network,ipv6_ifacev6:$ipv6_ifacev6,config:$config,SITPREFIXV6:$SITPREFIXV6,PRIDNSV6:$PRIDNSV6,SECDNSV6:$SECDNSV6" > /dev/console

                [ "$ipv6_ifacev6" = "$config" ] && [ "$ipv6_enable" = "on" ] && {
                    local update_flag="0"
                    local previous_prefix=$(uci get vnetwork."$network".ipv6_prefix 2>/dev/null)
                    local previous_prefixlen=$(uci get vnetwork."$network".ipv6_prefixlen 2>/dev/null)
                    local new_prefix="$SITPREFIXV6:"

                    # echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),$vnetwork_config,previous_prefix:$previous_prefix,previous_prefixlen:$previous_prefixlen,new_prefix:$new_prefix,PREFIXV6:$PREFIXV6,PRIDNSV6:$PRIDNSV6,SECDNSV6:$SECDNSV6" > /dev/console
                    [ "$previous_prefix" != "$new_prefix" ] && {
                        uci set vnetwork."$network".ipv6_prefix="$new_prefix"
                        update_flag="1"
                    }
                    [ "$previous_prefixlen" != "$PREFIXV6" ] && {
                        uci set vnetwork."$network".ipv6_prefixlen="$PREFIXV6"
                        update_flag="1"
                    }
                    [ -n "$PRIDNSV6" ] && [ -n "$SECDNSV6" ] && [ "$ipv6_dnsMode" = "auto" ] && {
                        local previous_pridns=$(uci get vnetwork."$network".ipv6_priDns 2>/dev/null)
                        local previous_snddns=$(uci get vnetwork."$network".ipv6_sndDns 2>/dev/null)
                        [ "$previous_pridns" != "$PRIDNSV6" ] && {
                            uci set vnetwork."$network".ipv6_priDns="$PRIDNSV6"
                            update_flag="1"
                        }
                        [ "$previous_snddns" != "$SECDNSV6" ] && {
                            uci set vnetwork."$network".ipv6_sndDns="$SECDNSV6"
                            update_flag="1"
                        }
                    }

                    [ "$update_flag" = "1" ] && {
                        uci set vnetwork."$network".ipv6_enable="on"
                        uci set vnetwork."$network".ipv6_ndp_enable="on"
                        uci commit
                        # echo "#####dhcp6c.sh,update_lte_lan_ndppd_radvd(),$vnetwork_config,vnet reload" > /dev/console
                        clean_lan_interfaces_v6 "$network"
                        setup_lan_interfaces_v6 "$network"
                    }
                }
            done
 
        fi

    }
}

proto_dhcp6c_setup() {
	local config="$1"
	local ifname="$2"
	local mbit=0

    # echo "#####dhcp6c.sh,proto_dhcp6c_setup() enter" > /dev/console
	json_get_vars dns_mode ip_mode lanif ip_config

	echo 2 > /proc/sys/net/ipv6/conf/$ifname/accept_ra

	if [ "$ip_config" == "auto" -o "$ip_config" == "slaac" ]; then
		echo "1"  > /proc/sys/net/ipv6/conf/$ifname/accept_ra_pinfo
		wait_for_mbit $ifname
	else
		echo "0"  > /proc/sys/net/ipv6/conf/$ifname/accept_ra_pinfo
	fi

	if [ "$ip_config" == "auto" ]; then
		mbit=`cat /proc/sys/net/ipv6/conf/$ifname/ndisc_mbit`
		echo "mbit: $mbit" > /dev/console
		if [ $mbit -eq 1 ]; then
			ip_config="dhcpv6"
			echo "0"  > /proc/sys/net/ipv6/conf/$ifname/accept_ra_pinfo
			ip -6 addr flush $ifname scope 0 # flush the slaac ip addr   
		elif [ $mbit -eq 0 ]; then
			ip_config="slaac"
			echo "1"  > /proc/sys/net/ipv6/conf/$ifname/accept_ra_pinfo
		else
			sleep 3
			return
		fi
	else
		echo 1 > /proc/sys/net/ipv6/conf/$ifname/disable_ipv6
		echo 0 > /proc/sys/net/ipv6/conf/$ifname/disable_ipv6
	fi
	
	# [ "$ip_config" == "slaac" ] && {

	# }

	#proto_dhcp6c_create_conffile "$ip_mode" \
	#	"$dns_mode" "$ifname" "$lanif" "$ip_config"

	#proto_run_command "$config" /usr/sbin/dhcp6c -f \
	#	-p "$dhcp6cpid" \
	#	-c "$conffile" \
	#	-t "$config" \
	#	"$ifname"

	proto_dhcp6c_create_config "$config" "$ip_mode" \
		"$dns_mode" "$ifname" "$lanif" "$ip_config"	
    
    local lports_config="/etc/config/lports"
    local usbs=$(sed -n "s/config usb '\(usb\w\+\)'/\1/p" "$lports_config")
    local lte_port_flag="0"
    for usb in $usbs; do
                local pindex=$(uci get lports."$usb".pindex 2>/dev/null)
                local ptype=$(uci get lports."$usb".ptype 2>/dev/null)
                local wanid=$(echo "$config" | cut -c 4)
                [ "$ptype" = "5" ] && [ "$wanid" = "$pindex" ] && {
                    lte_port_flag="1"
                }
    done

    if [ "$lte_port_flag" == "1" ]; then
        # echo "#####dhcp6c.sh,config:ifname value is usb0,call lte config progress" > /dev/console
		uci_set dhcp6c $config enabled '0'
        update_lte_netifd_config $config $ifname
        update_lte_lan_ndppd_radvd $config $ifname
	elif [ "$(uci_get dhcp6c $config enabled)" == "1" ]; then
        # echo "#####dhcp6c.sh,config:enabled value is 1" > /dev/console
		/etc/init.d/dhcp6c start $config "/tmp/dhcp6c/proto_dhcp6c_cmd.$config"
		proto_run_command "$config" $(cat /tmp/dhcp6c/proto_dhcp6c_cmd.$config)
		sleep 3
		[  -f "/tmp/dhcp6c_duid_$ifname" ] && {
			local duid=`cat "/tmp/dhcp6c_duid_$ifname"`
                        [ ${#duid} -ne 0 ] && uci_set dhcp6c $config duid $duid
			uci_commit dhcp6c
		}
    
	else
		# sync exec for dhcp6c callback script.
		ifname="$ifname" interface="$config" new_dhcpc_state="3" \
		/lib/netifd/dhcp6c.script
	fi
}

proto_dhcp6c_teardown() {
	local interface="$1"
	local ifname="$2"

    # echo "#####dhcp6c.sh,proto_dhcp6c_teardown() enter" > /dev/console
    rm /tmp/dhcp6c/prefix.info
    rm /tmp/dhcp6c/prefix.info.${interface}
    rm /tmp/dhcp6c/prefixlen.info.${interface}

	case "$ERROR" in
		11|19)
			proto_notify_error "$interface" AUTH_FAILED
			proto_block_restart "$interface"
		;;
		2)
			proto_notify_error "$interface" INVALID_OPTIONS
			proto_block_restart "$interface"
		;;
	esac

	[ -n "$ifname" ] && {
		echo "1" > /proc/sys/net/ipv6/conf/$ifname/accept_ra
		ip -6 route del ::/0 dev "$ifname"
		echo 1 > /proc/sys/net/ipv6/conf/$ifname/disable_ipv6
		echo 0 > /proc/sys/net/ipv6/conf/$ifname/disable_ipv6
	}

	#proto_kill_command  "$interface" 15
	#sleep 1
	#rm -rf ${dhcp6cdir}

	echo 3 > /proc/sys/net/ipv6/conf/$ifname/router_solicitations

	if [ "$(uci_get dhcp6c $interface enabled)" == "1" ]; then
		local sigterm="$(kill -l SIGTERM)"
		[ -n "$sigterm" ] && proto_kill_command "$interface" $sigterm
		/etc/init.d/dhcp6c stop $interface "/tmp/dhcp6c/proto_dhcp6c_cmd.$interface"
	fi
	uci_set dhcp6c $interface enabled '0'         
	uci_commit dhcp6c
}

[ -n "$INCLUDE_ONLY" ] || {
	add_protocol dhcp6c
}
