local json = require "luci.json"
local dbg    = require "luci.tools.debug"
local controllerSettings = require "luci.controller.admin.controllerSettings"
local uci     = require "luci.model.uci"
local uci_r = uci.cursor()

function hex_to_string(hex_string)
    local bytes = {}
    for i = 1, #hex_string, 2 do
        if #hex_string < i + 1 then
            dbg("Error: The length of the hex string is singular")
            return
        else
            local byte = tonumber(hex_string:sub(i, i + 1), 16)
            if byte then
                table.insert(bytes, string.char(byte))
            else
                dbg("Error: The format of the hex string is invalid")
                return
            end
        end
    end
    return table.concat(bytes)
end

function hex_to_ip(hexStr)

    -- 将16进制字符串转换为两个16位的十进制数
    local high, low = tonumber(hexStr:sub(1, 4), 16), tonumber(hexStr:sub(5, 8), 16)

    -- 提取每个字节
    local byte1 = math.floor(high / 256)
    local byte2 = high % 256
    local byte3 = math.floor(low / 256)
    local byte4 = low % 256

    -- 格式化为IP地址字符串
    local ipStr = string.format("%d.%d.%d.%d", byte1, byte2, byte3, byte4)
    return ipStr
end

function is_controller_mode()
    local ecs_file = io.open("/etc/config/controller.conf", "r")
    if not ecs_file then
        return false
    end

    local ecs_config = ecs_file:read("*all")
    if not ecs_config or ecs_config == "" then
        return false
    end

    local ejson = json.decode(ecs_config)
    if not ejson or not ejson.csId then
        return false
    end

    return string.upper(ejson.csId) ~= "C21F969B5F03D33D43E04F8F136E7682"
end

local function send_option_43(opt)
	local http_form = {}
	local data_form = {}
    local url = ""

    -- ip地址转hex格式由于省略了"."只保留数字长度为8，如192.168.0.1转成hex格式为c0a80001；而string格式的ip或者url转hex格式未作处理所以长度均大于8。所以长度为8的opt为ip转换为的hex格式。
    if #opt == 8 then
        url = hex_to_ip(opt)
    else
        url = hex_to_string(opt)
    end

    if not url or url == "" then
        dbg("Invalid hex string")
        return
    end
    -- cwmp(tr069) acs url
    if url:sub(1, 4) == "http" then
        local cwmp_status = uci_r:get("cwmp", "info", "cwmp")
        if is_controller_mode() == false and cwmp_status ~= nil then
            local cwmp = require "luci.controller.admin.cwmp"
            cwmp.parse_option43(url)
        end
        return
    end
	-- -- prepare data
	-- data_form.hidden_txt = ""
	-- data_form.inform_url = url
	-- http_form.data = json.encode(
	-- 	{
	-- 		["method"] = "set",
	-- 		["params"] = data_form
	-- 	}
	-- )

	-- --print(http_form.data)
	-- --dbg("ip="..data_form.inform_url)

	-- -- call method
	-- ret = controllerSettings.cs_update_informUrl(http_form)
	-- if not ret then
	-- 	print("Setting csHost with DHCP option-43 failed")
	-- end
	-- --dbg("ret.url="..ret.inform_url)
end


if (arg[1]~=nil) then
	send_option_43(arg[1])
end