#!/bin/sh
# Copyright (C) 2011 OpenWrt.org

. /usr/share/libubox/jshn.sh
FDB_FILE="/tmp/.fdb.tmp"
find_config() {
	local device="$1"
	local ifdev ifl3dev ifobj
	for ifobj in `ubus list network.interface.\*`; do
		interface="${ifobj##network.interface.}"
		(
			json_load "$(ifstatus $interface)"
			json_get_var ifdev device
			json_get_var ifl3dev l3_device
			if [[ "$device" = "$ifdev" ]] || [[ "$device" = "$ifl3dev" ]]; then
				echo "$interface"
				exit 0
			else
				exit 1
			fi
		) && return
	done
}

unbridge() {
	return
}

ubus_call() {
	json_init
	local _data="$(ubus -S call "$1" "$2")"
	[ -z "$_data" ] && return 1
	json_load "$_data"
	return 0
}


fixup_interface() {
	local config="$1"
	local ifname type device l3dev

	config_get type "$config" type
	config_get ifname "$config" ifname
	config_get device "$config" device "$ifname"
	[ "bridge" = "$type" ] && ifname="br-$config"
	config_set "$config" device "$ifname"
	ubus_call "network.interface.$config" status || return 0
	json_get_var l3dev l3_device
	[ -n "$l3dev" ] && ifname="$l3dev"
	json_init
	config_set "$config" ifname "$ifname"
	config_set "$config" device "$device"
}

scan_interfaces() {
	config_load network
	config_foreach fixup_interface interface
}

prepare_interface_bridge() {
	local config="$1"

	[ -n "$config" ] || return 0
	ubus call network.interface."$config" prepare
}

setup_interface() {
	local iface="$1"
	local config="$2"

	[ -n "$config" ] || return 0
	ubus call network.interface."$config" add_device "{ \"name\": \"$iface\" }"
}

do_sysctl() {
	[ -n "$2" ] && \
		sysctl -n -e -w "$1=$2" >/dev/null || \
		sysctl -n -e "$1"
}

is_wan_phy()
{
        local wan_num="$1"
        local is_phy=`uci -q get lports.port${wan_num}.p_type`
        if [ -n "$is_phy" ] ;then
                echo $is_phy
        else
                echo 0
        fi
}


handle_bridge_fdb() {
	local act="$1" 
	local macaddr="$2"
	local vlanid="$3"
	[ "$act" != "add" -a "$act" != "delete" ] && return
	local vlan_filter=`uci -c /etc/profile.d get profile.@switch_info[0].vlan_filter`
	if [ -n "$macaddr" ];then
		if [ "$vlan_filter" == 1 -a -n "$vlanid" ];then
			bridge fdb $act $macaddr dev inf vlan $vlanid
		else
			bridge fdb $act $macaddr dev inf
		fi
	else
		wan_list=`uci -q get network.global.wan_numbers`
    	    if [ -n "$wan_list" ];then
				for wan in ${wan_list}; do
					is_phy=$(is_wan_phy ${wan})
					[ "$is_phy" -eq 0 ] && {
						macaddr=`uci -q get network.wan${wan}_eth.macaddr`
						[ -n "${macaddr}" ] && {
							[ "$vlan_filter" == 1 ] && {
								vlanid=`uci -q get lports.port${wan}.pvid`
								bridge fdb $act ${macaddr} dev inf vlan ${vlanid}
							} || bridge fdb $act ${macaddr} dev inf
						}	
					}
            	done
        	fi
		macaddr=`uci -q get network.lan.macaddr`
		[ -n "${macaddr}" ] && bridge fdb $act ${macaddr} dev inf
	fi
}



check_vlan()
{
	local wan_num=`uci get network.global.wan_numbers`
	_check_is_wan()
	{
		my_wan=$1
		for wan in $wan_num;do
			if [ "$my_wan" == "$wan" ];then
				is_wan="yes"
				break
			fi
		done
	}
	config_get users "$1" users
	[ "$users" == "wan" ] || return
	config_get vlan "$1" vlan
	config_get ports "$1" ports
	[ -n "$vlan" -a -n "$ports" ] || return
	local mac=""
	local vport=`uci get vlan0.vlan$vlan.ports`
	[ -z "$vport" ] && {
		count=0
		ports=${ports//t/}   #remove tag flag 't'
		for i in $ports;do
			local is_wan=""
			_check_is_wan "$i"
			[ -z "$is_wan" ] && continue
			mac=`uci get network.wan${i}_eth.macaddr`
			[ -n "$mac" ] || continue
			[ -n "$update_vlan_cmd" ] && update_vlan_cmd=${update_vlan_cmd}";"
			update_vlan_cmd=${update_vlan_cmd}"bridge fdb add ${mac} dev inf vlan ${vlan}"
		done
	}

}

check_child_vlan()
{
	local wan_num=`uci get network.global.wan_numbers`
	_check_is_wan()
	{
		my_wan=$1
		for wan in $wan_num;do
			if [ "$my_wan" == "$wan" ];then
				is_wan="yes"
				break
			fi
		done
	}

	config_get vlan "$1" vlan
	config_get ports "$1" ports
	[ -n "$vlan" -a -n "$ports" ] || return
	local mac=""
	local vport=`uci get vlan0.vlan$vlan.ports`
	[ -z "$vport" ] && {
		count=0
		ports=${ports//t/}   #remove tag flag 't'
		for i in $ports;do
			local is_wan=""
			_check_is_wan "$i"
			[ -z "$is_wan" ] && continue
			mac=`uci get network.wan${i}_eth.macaddr`
			[ -n "$mac" ] || continue
			[ -n "$update_vlan_cmd" ] && update_vlan_cmd=${update_vlan_cmd}";"
			update_vlan_cmd=${update_vlan_cmd}"bridge fdb add ${mac} dev inf vlan ${vlan}"
		done
	}
}

update_fdb_delay()
{
	local update_vlan_cmd=""
	. /lib/functions.sh
	config_load lports
	config_foreach check_vlan switch_vlan
	config_foreach check_child_vlan switch_child_vlan
	[ -n "$update_vlan_cmd" ] && echo -n ${update_vlan_cmd} > ${FDB_FILE}

}

