#!/bin/sh

ONLINE_STATE_DIR="/tmp/online/"
ONLINE_STATE_FILE="/tmp/online/state"
ONLINE_DEV_FILE="/tmp/online/zone"
ONLINE_LATENCY_FILE="/tmp/online/latency"
ONLINE_BLOCK_FILE="/tmp/online/block"
ONLINE_MODE_ID=26
#MSG(13901, WNG, "[OMADA]%255 The online detection result of [WAN%1] is offline")
#MSG(13902, WNG, "[OMADA]%255 The online detection result of [WAN%1] is online")
LINKBACKUP_MODE_ID=29
#MSG(14235, NTC, "[OMADA]%255 Link Backup %1 took effect.")
#MSG(14236, NTC, "[OMADA]%255 Link Backup %1 was down.")
UP_LOG_ID=14235
DOWM_LOG_ID=14236

. /lib/zone/zone_api.sh
. /lib/functions/network.sh 
. /lib/functions.sh 

# for fit logical WAN
# WANx --> (x-1)*4 + 1
# WANx_y --> (x-1)*4+y + 1
get_wan_id()
{
    zone_name=$1
    list=`echo $zone_name | grep -o "[0-9]*"`

    num=0
    res=1
    tmp=0
    for number in $list; do
        num=$((num+1))
        if [ $num -gt 2 ]; then
            echo 0
            return
        fi

        if [ $num -eq 1 ]; then
            tmp=$((number-1))
            tmp=$((tmp*4))
            res=$((res+tmp))
        fi

        if [ $num -eq 2 ]; then
            res=$((res+number))
        fi
    done

    echo $res
}

# for US lte-wan
check_lte_wan_info()
{
	lte_dev=`uci get online.settings.lte_dev`
	lte_interval=`uci get online.settings.lte_interval`
	[ -z $lte_dev ] && {
		uci set online.settings.lte_dev="usb0"
	}
	[ -z $lte_interval ] && {
		uci set online.settings.lte_interval="300"
	}
	
	uci commit online
}

wanmax()
{
	wan_num=0
	tmp_wan_num=`uci get lports.global.pwanmax 2>/dev/null`
	if [ -n "$tmp_wan_num" ]; then
		wan_num=$((tmp_wan_num))
	fi
	usbs=`uci get lports.usb_list.usb_name 2>/dev/null`
	if [ -n "$usbs" ]; then
		for usb in $usbs; do
			pmax=`uci get lports.$usb.pwanmax 2>/dev/null`
			if [ -n "$pmax" ]; then
				wan_num=$(($wan_num+$((pmax))))
			fi
		done
	fi

	if [ $wan_num -eq 0 ]; then
		echo -n "4"
	else
		echo -n "$wan_num"
	fi
}

get_weight()
{
	local iface="$1"

	up_bps=`uci get load_balance.weight.$iface 2>/dev/null`
	[ -z "$up_bps" ] && {
		up_bps=`uci get qos.$iface.uplink 2>/dev/null`
	}
	[ -z "$up_bps" ] && up_bps="0"

	#down_bps=`qos_iface_get_downlink_bandwidth $iface`
	down_bps=`uci get load_balance.weight.$iface 2>/dev/null`
	[ -z "$down_bps" ] && {
		down_bps=`uci get qos.$iface.downlink 2>/dev/null`
	}
	[ -z "$down_bps" ] && down_bps="0"
	up_bps=$((up_bps * 2))
	down_bps=$((down_bps * 2))
	echo "up_bps: $up_bps" > /dev/console
	echo "down_bps: $down_bps" > /dev/console
	echo -n $(((${up_bps} + ${down_bps}) / 2))
}

# $1:global state       "on"/"off"
# $2:balance state      "on"/"off"
# $3:sepcial state      "on"/"off"
# $4:debug flag       	"0"--细粒度分流/不打印 "1"--细粒度分流/打印 "2"--粗粒度分流/不打印 "3"--粗粒度分流/打印
# $5:weight array       e.g.-->"["1","2","3","4"]"
# call type:
#               [1] update_balance_state_info "on" "on" "on" "0" "[\"1\",\"2\",\"3\",\"4\"]"
#               [2] update_balance_state_info "off" "on" "on" "0"
update_balance_state_info()
{
    [ -z "$1" -o -z "$2" -o -z "$3" ] && return

    [ "$1" == "on" -o "$1" == "off" ] || {
        echo "error input[1]: global_state=\"on\" or \"off\""
        return
    }

    [ "$2" == "on" -o "$2" == "off" ] || {
        echo "error input[2]: balance_state=\"on\" or \"off\""
        return
    }

    [ "$3" == "on" -o "$3" == "off" ] || {
        echo "error input[3]: special_state=\"on\" or \"off\""
        return
    }

    # 0和2中选择，默认不打印debug信息
    debug_flag=0
    [ -n "$4" ] && {
        debug_flag=$4
    }

    # 没有输入weight才读取本地数据
    wan_num=`wanmax`
	
    # for fit logical WAN
    wan_num=$((wan_num * 4))

    w=""
    sub=1
    for i in $(seq 1 1 $wan_num); do
        # default weight equal 1
        w="$w$sub"
        if [ $i -lt $wan_num ];then
            w="$w "
        fi
    done

    if [ -z "$5" ]; then
        # read use_iface
        ifaces=`uci get load_balance.balance.use_iface 2>/dev/null`
        for inface in $ifaces; do
            # weight init
            zone_weight=`get_weight "$inface"`
            [ -z "$zone_weight" ] && continue

            # parse WANx WANx_y
            index=`get_wan_id $inface`
            n=0
            tmp=""
            for i in $w; do
                n=$(echo "$(($n+1))")
                if [ $index -eq $n ];then
                    tmp="$tmp$zone_weight"
                else
                    tmp="$tmp$i"
                fi
                if [ $n -lt $wan_num ];then
                    tmp="$tmp "
                fi
            done
            w="$tmp"
        done

        # build weight
        tmp1="\""
        tmp2=","
        weight="["
        idx=0
        for i in $w; do
            idx=$(echo "$(($idx+1))")
            weight="$weight$tmp1$i$tmp1"
            if [ $idx -lt $wan_num ]; then
                weight="$weight$tmp2"
            fi
        done
        weight="$weight]"
    fi

    # call ubus
    if [ -n "$5" ]; then
        echo "ubus call online update_balance_state {\"global_state\":\"$1\",\"balance_state\":\"$2\",\"special_state\":\"$3\",\"debug\":$debug_flag,\"weight\":$5}" > /dev/console
        ubus call online update_balance_state "{\"global_state\":\"$1\",\"balance_state\":\"$2\",\"special_state\":\"$3\",\"debug\":$debug_flag,\"weight\":$5}"
    else
        echo "ubus call online update_balance_state {\"global_state\":\"$1\",\"balance_state\":\"$2\",\"special_state\":\"$3\",\"debug\":$debug_flag,\"weight\":$weight}" > /dev/console
        ubus call online update_balance_state "{\"global_state\":\"$1\",\"balance_state\":\"$2\",\"special_state\":\"$3\",\"debug\":$debug_flag,\"weight\":$weight}"
    fi
}

start_balance()
{
	local global_state="$(uci get mwan3.global.state)"
	local special_state="$(uci get mwan3.special_rule.state)"
	local loadbalance_state="$(uci get load_balance.balance.state)"
	[ -z "$loadbalance_state"] && loadbalance_state="off"
	#echo "global_state: $global_state, special_state: $special_state, loadbalance_state: $loadbalance_state" > /dev/console
	update_balance_state_info $global_state $loadbalance_state $special_state 
}

change_default_route()
{
	local dev="$1"
	local default=`ip route | awk 'FNR==1 {print $1}'`
	local dev_1=`ip route | awk 'FNR==1 {print $5}'`
	[ "$default" == "default" ] && [ "$dev_1" == "$dev" ] && {
		local dev_on=`grep "=on.*" "$ONLINE_STATE_FILE" | awk 'FNR==1' | awk -F "=" '{print $1}'` 
		[ -n "$dev_on" ] && {
			local gw=`ip route | grep default | grep $dev_on | awk -F ' ' '{printf $3}' `
			route del default gw $gw
			route add default gw $gw
		}
	}
}

online_set_state()
{
	local dev="$1"
	#[ -n "$key" ] || { echo "set online state failed! can not get interface by device ${1}" > /dev/console }
	local cont="on"
	if [ "$2" = "0" ]; then
		cont="off"
		change_default_route $1 $2
	fi
	
	[ ! -e $ONLINE_STATE_DIR ] && mkdir -p $ONLINE_STATE_DIR
	[ ! -f $ONLINE_STATE_FILE ] && touch $ONLINE_STATE_FILE

	[ -n "$3" ] && {
		local device=""
		local i=""
		global_wanmode="$(uci get network.global.wanmode)"
		for i in `seq 1 $global_wanmode`
		do
			network_get_device device wan${i}_eth
			[ "$device" == "$dev" ] && break
			network_get_device device wan${i}_poe
			[ "$device" == "$dev" ] && break
		done
		[ "$device" == "$dev" ] && echo -e "$3" > ${ONLINE_LATENCY_FILE}${i}
		echo -e "$3" > ${ONLINE_LATENCY_FILE}_${dev}
	}

	local iface=`zone_get_iface_bydev $dev`
	local wan_num=`zone_get_zone_byif $iface`
	grep "^${dev}=.*" "$ONLINE_STATE_FILE" 2>&1 >/dev/null || {
		echo -e "${dev}=${cont}" >> "$ONLINE_STATE_FILE" 2>/dev/null
		if [ "$cont" = "on" ]; then
			logger_reg  $ONLINE_MODE_ID 13902 $wan_num
		elif [ "$cont" = "off" ]; then
			logger_reg  $ONLINE_MODE_ID 13901 $wan_num
		fi 
 		return 0
	}

	local old=""
	old=`grep "^${dev}=.*" "$ONLINE_STATE_FILE"`
	if [ "$old" != "${dev}=${cont}" ]; then
		if [ "$cont" = "on" ]; then
			logger_reg  $ONLINE_MODE_ID 13902 $wan_num
		elif [ "$cont" = "off" ]; then
			logger_reg  $ONLINE_MODE_ID 13901 $wan_num
		fi 
	fi
	sed -i "s/^${dev}=.*/${dev}=${cont}/" $ONLINE_STATE_FILE	
}

online_del_state()
{
	#[ -n "$1" ] || { echo "delete online state failed! no device ${1}" > /dev/console }
	[ -z ${1} ] && return
	local dev="$1"
	#[ -n "$key" ] || { echo "delete online state failed! can not get interface by device ${1}" > /dev/console }
	
	grep "^${dev}=.*" $ONLINE_STATE_FILE 2>&1 >/dev/null && sed "/^${dev}=.*/d" -i $ONLINE_STATE_FILE || {
		echo "online_del_state(iface:${dev})  failed!" >> /tmp/online_wanhook.log  
	}
}

online_clear_state()
{
	[ ! -e $ONLINE_STATE_DIR ] && return 
	[ ! -f $ONLINE_STATE_FILE ] && return 
	echo >$ONLINE_STATE_FILE 
}

device_to_iface()
{
	iface=`zone_get_iface_bydev $1`
	zone=`zone_get_zone_byif $iface`

	if [ "$zone" == "VPN" ];then
		zone=$iface
	fi

	echo "$zone"
}

add_online_check()
{
	[ -n "$1" ] || return
	local zone_or_iface="$1"
	for cnt in $(seq 1 300); do
		dev=`zone_get_effect_devices $zone_or_iface`
		if [ -z "$dev" ]; then
		    dev=`zone_get_devices $zone_or_iface`
		fi

		if [ -n "$dev" ]; then
			break
		fi	
		sleep 1
	done
	#[ -n "$dev" ] || continue

	zone=`zone_get_iface_bydev $dev`
	[ -n "$zone" ] || continue

	iface=`device_to_iface $dev`
	[ -n "$iface" ] || continue

	#ubus call online add_info "{\"dev\":\"$dev\"}"
	ubus call online add_info "{\"dev\":\"$dev\",\"zone\":\"$zone\",\"iface\":\"$iface\"}"
	add_online_dev_state ${zone_or_iface} ${dev}
}

add_online_dev_state()
{
	[ -z ${1} ] && return
	[ -z ${2} ] && return 
	local zone_or_iface="$1"
	local dev="$2"

	[ ! -e $ONLINE_STATE_DIR ] && mkdir -p $ONLINE_STATE_DIR
	[ ! -f $ONLINE_DEV_FILE ] && touch $ONLINE_DEV_FILE

	grep "^${zone_or_iface}=.*" "$ONLINE_DEV_FILE" 2>&1 >/dev/null || {
		echo -e "${zone_or_iface}=${dev}" >> "$ONLINE_DEV_FILE" 2>/dev/null
		return 0
	}
	sed -i "s/^${zone_or_iface}=.*/${zone_or_iface}=${dev}/" $ONLINE_DEV_FILE

}

del_online_check()
{
	[ -z "$1" ] && return
	local zone_or_iface="$1"
	local dev="`cat $ONLINE_DEV_FILE |egrep "^${zone_or_iface}=.*" |cut -d '=' -f 2`"
	[ -z "$dev" ] && return

	ubus call online delete_info "{\"dev\":\"$dev\"}"
	del_online_dev_state ${zone_or_iface}
}

del_online_dev_state()
{
	[ -z ${1} ] && return
	local zone_or_iface="$1"
	local zone_tmp="`cat $ONLINE_DEV_FILE |egrep "^${zone_or_iface}=.*" |cut -d '=' -f 1`"

	if [ "x${zone_tmp}" != "x" ]; then
		grep "^${zone_tmp}=.*" $ONLINE_DEV_FILE 2>&1 >/dev/null && sed "/^${zone_tmp}=.*/d" -i $ONLINE_DEV_FILE
	else
		echo "del_online_dev_state(iface_or_zone:${zone_or_iface})  failed" >> /tmp/online_wanhook.log  
	fi
}

flush_online_check()
{
	. /usr/share/libubox/jshn.sh

	jdata=`ubus call online get_info`
	json_load "$jdata"
	json_get_keys jkeys
	for jkey in $jkeys
	do
		json_select "$jkey"
		json_get_var jval dev
		ubus call online delete_info "{\"dev\":\"$jval\"}"
		json_select ..
	done
}

update_online_manual_check()
{
	[ -n "$1" ] || [ -n "$2" ] || [ -n "$3" ] || [ -n "$4" ] || return

	. /lib/zone/zone_api.sh
	dev=`zone_get_effect_devices $1`
	if [ -z "$dev" ]; then
	    dev=`zone_get_devices $1`
	fi

	[ -n "$dev" ] || return

	zone=`zone_get_iface_bydev $dev`
	[ -n "$zone" ] || continue

	iface=`device_to_iface $dev`
	[ -n "$iface" ] || continue

	#ubus call online update_manual_info "{\"dev\":\"$dev\",\"mode\":\"$2\",\"manual_gw\":\"$3\",\"manual_dns\":\"$4\"}"
	ubus call online update_manual_info "{\"dev\":\"$dev\",\"zone\":\"$zone\",\"iface\":\"$iface\",\"mode\":\"$2\",\"manual_gw\":\"$3\",\"manual_dns\":\"$4\"}"
}

#$1: ifaces
del_ifaces_online_check()
{
	[ -n "$1" ] || return

	for i in $1;do
		del_online_check $i
	done
}

#$1: interval
update_online_interval()
{
	[ -z "$1" ] && return
	ubus call online update_interval "{\"interval\":$1}"
}

cmpslave()
{
        config_get slave_iface $1 slave_iface
        local slave_dev=`zone_get_devices $slave_iface`

        # shell内调用for循环效率较低，不推荐；但此处slave_dev最多包含两个值，影响较小
        for sdev in $slave_dev
        do
                if [ "$sdev" == "$dev" ]; then
                        if [ "$blocked" == "no" ]; then
                                ubus call online refresh "{\"dev\":\"$dev\"}"
                                logger_reg $LINKBACKUP_MODE_ID $DOWM_LOG_ID "Backup" $slave_iface
                        elif [ "$blocked" == "yes" ]; then
                                ubus call online refresh "{\"dev\":\"$dev\"}"
                                logger_reg $LINKBACKUP_MODE_ID $UP_LOG_ID "Backup" $slave_iface
                        fi
                        slave_flag=1
                fi
        done
}

#$1: zone name
#$2: block status
#return old status
update_online_dev_blocked()
{
	. /lib/zone/zone_api.sh
	devs=`zone_get_devices $1`
	slave_flag=0

	blocked="no"
	for dev in $devs
	do
		if [ -e ${ONLINE_BLOCK_FILE}"_${dev}" ];
		then
			blocked="yes"
		fi
	done

	if [ "$2" != "0" ];
	then
		for dev in $devs
		do
			echo -n "$1" > ${ONLINE_BLOCK_FILE}"_${dev}"
			if [ "$blocked" == "no" ]; then 
				config_load line_backup
				config_foreach cmpslave rule
				if [ "$slave_flag" == "0" ]; then
					ubus call online refresh "{\"dev\":\"$dev\"}"
					logger_reg $LINKBACKUP_MODE_ID $DOWM_LOG_ID "Primary" $1
				else
					slave_flag=0
				fi
			fi
		done
	else
		for devfile in `ls ${ONLINE_BLOCK_FILE}_*`
		do
			if [ "$1" == "`cat ${devfile}`" ];
			then
				rm -f ${devfile}
				dev="${devfile##${ONLINE_BLOCK_FILE}_}"
				iface=`zone_get_iface_bydev $dev`
				if [ "x$iface" != "x" -a "$blocked" == "yes" ]; then
					config_load line_backup
					config_foreach cmpslave rule
					if [ "$slave_flag" == "0" ]; then
						ubus call online refresh "{\"dev\":\"$dev\"}"
						logger_reg $LINKBACKUP_MODE_ID $UP_LOG_ID "Primary" $1
					else
						slave_flag=0
					fi
				fi
			fi
		done
	fi
}
