#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
. /lib/firewall/core_tpfirewall.sh

VPN_RULE_NUM=25
OPENVPN_FWMARK=0x20000
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK
OPENVPN_INPUT_RULE=openvpn_input_rule

IFNAME=$1

openvpn_server_down_hook()
{
	local indexname=
	indexname=$(echo $IFNAME | tr -cd "[0-9]")
	echo "openvpn_server_down $IFNAME" >  /dev/console

	uci set openvpn.server${indexname}.vlocalip=""
	uci commit openvpn

        #update V zone groups
        . /lib/zone/zone_api.sh
        zone_delete_vgroup "$IFNAME"

	#add ifname into dnsproxy
	[ -n "$IFNAME" ] && {
		/lib/dnsproxy/api.sh delete "$IFNAME"
	}

	#删除网络LAN代理
	do_service_vpn_down

	#删除dmz相关规则
	config_load 'openvpn'
	config_get port "server$indexname" port
	config_get proto "server$indexname" proto
	config_get wans "server$indexname" wan
	/lib/nat/nat_dmz_bypass.sh  ovpn_reload

	#update hwnat_ipstat interface
	if [ -e "/etc/init.d/hwnat_ipstat" ];then
                echo "### update hwnat_ipstat" >/dev/console
                hwnat_id=`ps -w | grep hwnat_ipstat | awk '/\/usr\/sbin\/hwnat_ipstat/ {print $1}'`
                [ "$hwnat_id" -gt 0 ] && kill -10 ${hwnat_id} 2 >/dev/null
        fi
	mssfix_down "$IFNAME"

	nums=$(iptables -w -nvL openvpn_forwarding_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1 | tac)
	for num in $nums;
	do
		iptables -w -D openvpn_forwarding_rule ${num}
	done

	nums=$(iptables -w -t nat -nvL openvpn_prerouting_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1 | tac)
	for num in $nums;
	do
		iptables -w -D openvpn_prerouting_rule ${num}
	done

	nums=$(iptables -w -nvL zone_wan_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1 | tac)
	for num in $nums;
	do
		iptables -w -D zone_wan_openvpn ${num}
	done

	nums=$(iptables -w -t nat -nvL postrouting_rule_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1 | tac)
	for num in $nums;
	do
		iptables -w -t nat -D postrouting_rule_openvpn ${num}
	done

	#CHAIN:manage_devs
	fw_event_manage_dev "${IFNAME}:down"
	nbptc down ${IFNAME} "ovpn"

	tables="${OPENVPN_FORWARD_TABLE} ${OPENVPN_ROUTING_TABLE}"
	for tbl in $tables
	do
		nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1 | tac)
		for num in $nums;
		do
			iptables -w -t mangle -D ${tbl} ${num}
		done
	done

	##del filter rules
	nums=$(iptables -w -t filter -nL ${OPENVPN_INPUT_RULE} --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1 | tac)
	for num in $nums;
	do
		iptables -w -t filter -D ${OPENVPN_INPUT_RULE} ${num}
	done

	ippool_set="ovpn_${IFNAME}"
	ipset destroy $ippool_set

}

openvpn_server_down_hook
hnat_up
/etc/init.d/zone restart
exit 0
