#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
. /lib/nat/nat_core.sh
. /lib/firewall/core_tpfirewall.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
VPN_RULE_NUM=25
OPENVPN_FWMARK=0x20000
ANTI_OPENVPN_FWMARK=0xfffdffff
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK
OPENVPN_INPUT_RULE=openvpn_input_rule
OVPN_RULE_NUM=24

IFNAME=$1
VLOCALIP=$4

route_table_init()
{
	ret=`ip rule list | grep $VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		iptables -w -t mangle -F ${OPENVPN_FORWARD_TABLE}
		iptables -w -t mangle -F ${OPENVPN_ROUTING_TABLE}
		ip rule add pref $VPN_RULE_NUM fwmark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} table $VPN_RULE_NUM
	fi

	ret=`ip rule list | grep $OVPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		ip rule add pref $OVPN_RULE_NUM  table $OVPN_RULE_NUM
	fi

	ret=`vnete ip rule list | grep $VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		vnete ip rule add pref $VPN_RULE_NUM table $VPN_RULE_NUM
	fi
}

openvpn_server_up_hook()
{
	local indexname=
    local proto=
    local port=
	local local_network=
	local ippool=
	local pool_network=
	local pool_prefix=
	local wans=
	local ifname=
	local device=

	indexname=$(echo $IFNAME | tr -cd "[0-9]")
	echo "openvpn_server_up ${IFNAME} ${indexname}" >  /dev/console

	uci set openvpn.server${indexname}.vlocalip=${VLOCALIP}
	uci commit openvpn

	config_load 'openvpn'
	config_get port "server$indexname" port
	config_get proto "server$indexname" proto
	config_get local_network "server$indexname" local_network
	config_get ippool "server$indexname" server
	config_get wans "server$indexname" wan
	config_get dns1 "server$indexname" dns1
	config_get dns2 "server$indexname" dns2
	config_get full_mode "server$indexname" full_mode
    config_get ifname "server$indexname" dev
	if [ "x$full_mode" = "xon" ] && [ -z "$dns1" ] && [ -z "$dns2" ]; then
		config_load vnetwork
		config_foreach find_lanip vinterface
		if [ "$lanip" != "0.0.0.0" ]; then
			dns1=$lanip
			dns2=$PUBLIC_DNS_SERVER
		fi
	fi

    #update V zone groups
    for wan in ${wans}
	do
		zone_add_vgroup "$IFNAME" "$wan"
        mssfix_up "$wan" "$IFNAME"
		local devices=$(zone_get_wan_devices $wan)
		for device in ${devices}
		do
			fw add 4 f zone_wan_openvpn ACCEPT { "-i $device -p $proto -m $proto --dport $port -m comment --comment $IFNAME" }
		done
	done
	# add ifname into dnsproxy
	[ -n "$IFNAME" ] && {
		/lib/dnsproxy/api.sh add "$IFNAME"
	}

	#update hwnat_ipstat interface
	if [ -e "/etc/init.d/hwnat_ipstat" ];then
                echo "### update hwnat_ipstat" >/dev/console
                hwnat_id=`ps -w | grep hwnat_ipstat | awk '/\/usr\/sbin\/hwnat_ipstat/ {print $1}'`
                [ "$hwnat_id" -gt 0 ] && kill -10 ${hwnat_id} 2 >/dev/null
        fi

	#create ipset to reduce iptables rules
	ipset_local="ovpn_server${indexname}_local"
	ipset create $ipset_local hash:net
	ipset flush $ipset_local
	for local in ${local_network}
	do
		ipset add $ipset_local $local
	done

	#FORWORDING
	fw add 4 n openvpn_prerouting_rule ACCEPT { "-p $proto -m $proto --dport $port -m comment --comment $IFNAME" }
    fw add 4 f openvpn_forwarding_rule ACCEPT { "-o $IFNAME -m conntrack --ctstate RELATED,ESTABLISHED" }
	fw add 4 f openvpn_forwarding_rule ACCEPT { "-m set --match-set lan_iface src -m set --match-set $ipset_local src -o $IFNAME" }
	fw add 4 f openvpn_forwarding_rule ACCEPT { "-m set --match-set lan_iface dst -m set --match-set $ipset_local dst -i $IFNAME" }

	#CHAIN:manage_devs
	fw_event_manage_dev "${IFNAME}:up"
	nbptc up ${IFNAME} "ovpn"


#	export > /dev/console
#	vnete sysctl -w /proc/sys/net/ipv4/conf/all/proxy_arp=1

	pool_network=$(ipcalc -n $ippool | cut -d '=' -f2)
	pool_prefix=$(ipcalc -p $ippool | cut -d '=' -f2)
	ippool=${pool_network}/${pool_prefix}

	ippool_set="ovpn_${IFNAME}"
	ipset create $ippool_set hash:net
	ipset flush $ippool_set
	iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set lan_iface src -m set --match-set $ipset_local src -m set --match-set ${ippool_set} dst -m comment --comment "if=${IFNAME}" -j MARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set lan_iface src -m set --match-set $ipset_local src -m set --match-set ${ippool_set} dst -m comment --comment "if=${IFNAME}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A ${OPENVPN_ROUTING_TABLE} -i "${IFNAME}" -m set --match-set ${ippool_set} src -m set --match-set $(zone_allwan_group) dst -m comment --comment "if=${IFNAME},vnet_set=${ipset_local}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} -m comment --comment "if=${IFNAME},vnet_set=${ipset_local}" -j MARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t filter -A ${OPENVPN_INPUT_RULE} -i ${IFNAME} -m set --match-set VMe dst -m comment --comment "if=${IFNAME}" -j ACCEPT
	if [ -z "$(echo " ${local_network} " | grep " 0.0.0.0/0 ")" ]; then # don't add if 0.0.0.0/0
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $ipset_local src  -m set ! --match-set lan_iface src -m conntrack --ctdir REPLY -m comment --comment "if=${IFNAME}" -j MARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $ipset_local src  -m set ! --match-set ${ippool_set} dst -m policy --dir in --pol ipsec -m set ! --match-set lan_iface src -m conntrack --ctdir REPLY -m comment --comment "if=${IFNAME}" -j MARK --and-mark ${ANTI_OPENVPN_FWMARK}
	fi

	#添加网络代理功能
	nat_postrouting_add openvpn "-A" "-m set --match-set ${ippool_set} src -m set --match-set lan_iface dst -m comment --comment "if=${IFNAME}" -j RETURN"
	nat_postrouting_add openvpn "-A" "-m set --match-set ${ippool_set} src -m comment --comment "if=${IFNAME}" -j MASQUERADE"

	#添加网络LAN代理
	do_service_vpn_up ${VLOCALIP} ${dns1} ${dns2}

	#添加dmz相关规则
	/lib/nat/nat_dmz_bypass.sh ovpn_reload

}



route_table_init
openvpn_server_up_hook
hnat_down
/etc/init.d/zone restart
exit 0
