
local ctl = require "luci.model.controller"
local uci = require "luci.model.uci"
local sys = require "luci.sys"
local form = require "luci.tools.form"
local gdb = require "luci.tools.debug"
local err = require "luci.tools.error"
local utl = require "luci.util"
local ifs = require "luci.model.interfaces".ifs()
local ifaces = require "luci.model.interfaces"
local qos = require "luci.model.qos"
local userconfig   = require "luci.model.userconfig"
local dtype = require "luci.tools.datatypes"
local pppox = require "luci.controller.admin.pppox"
local ippoolref = require "luci.model.reference"

local uci_r = uci.cursor()
form = form.Form(uci_r, {"confname"})
ippoolref = ippoolref.REF_INST()

local server_list_max = 200
local client_list_max = 32
local max_session = 100

local user_configname = "user-secrets"

function get_max_sessions()
    local ret

    if uci_r:get_profile("pptp", "tunnel_max") then
        ret = uci_r:get_profile("pptp", "tunnel_max") > max_session and max_session or uci_r:get_profile("pptp", "tunnel_max")
    else
        ret = max_session
    end
    return ret
end

function insert_client_list(http_form)
    local cmd = {}
    local new = luci.json.decode(http_form)

    local une = new.username
    une = string.gsub(une, "\\", "\\\\")
    une = string.gsub(une, '([%$"`])',"\\%1")
    local psd = new.password
    psd = string.gsub(psd, "\\", "\\\\")
    psd = string.gsub(psd, '([%$"`])',"\\%1")

    if(new.local_network ~= nil) then
        cmd[#cmd+1] = string.format("/lib/pptp/pptp-client-add.sh %s \"%s\" \"%s\" %s %s %s %s %s %s %s %s",dtype.safeSpecialString(new.confname),une,psd,dtype.safeSpecialString(new.pns),dtype.safeSpecialString(new.workmode),dtype.safeSpecialString(new.remotesubnet),dtype.safeSpecialString(new.outif),dtype.safeSpecialString(new.enable),dtype.safeSpecialString(new.mppeencryption),dtype.safeSpecialString(new.tunnelname),dtype.safeSpecialString(new.local_network))
    else
        cmd[#cmd+1] = string.format("/lib/pptp/pptp-client-add.sh %s \"%s\" \"%s\" %s %s %s %s %s %s %s ",dtype.safeSpecialString(new.confname),une,psd,dtype.safeSpecialString(new.pns),dtype.safeSpecialString(new.workmode),dtype.safeSpecialString(new.remotesubnet),dtype.safeSpecialString(new.outif),dtype.safeSpecialString(new.enable),dtype.safeSpecialString(new.mppeencryption),dtype.safeSpecialString(new.tunnelname))
    end
    cmd[#cmd+1] = "/etc/init.d/qos-tplink start"

    -- add interface info into qos model
    if not qos.add_interface(new.tunnelname, tonumber(new.uplink), tonumber(new.downlink)) then
        return false, err.ERR_COM_TABLE_ITEM_UCI_ADD
    end

    ifs.update_if_reference(new.outif,1)
    userconfig.cfg_modify();
    sys.call("/etc/init.d/zone restart")

    sys.fork_call(cmd[1])
    sys.fork_call(cmd[2])
    return ret
end

function delete_client_list(http_form)
    local content = luci.json.decode(http_form)

    local index = content.index
    local path = content.path
    local outif = content.outif
    local tunnelnamex = content.tunnelnamex

    index = (type(index)=="table") and index or {index}
    path = (type(path)=="table") and path or {path}
    local list = {}
    local confname = {}
    local cmd = {}


    for _,j in ipairs(path) do
        cmd[#cmd+1] = string.format("/lib/pptp/pptp-client-delete.sh %s 1",j)
    end

    cmd[#cmd+1] = "/etc/init.d/qos-tplink start"

    local refifs = { }
    local tnifs = { }
    tnifs[#tnifs + 1] = tunnelnamex
    refifs[#refifs + 1] = outif

    local infaces = ""
    local qoss = ""
    if next(refifs) ~= nil then
        infaces = table.concat(refifs, " ")   
    end

    if next(tnifs) ~= nil then
        qoss = table.concat(tnifs, " ")
    end

    sys.call("/etc/init.d/zone restart")
    --del the reference of interface etc
    sys.fork_call(". /lib/pppox/api.sh && deal_with_qos_ref %s %s" % {dtype.safeSpecialString(infaces), dtype.safeSpecialString(qoss)})

    userconfig.cfg_modify();

    for _,j in ipairs(cmd) do
        sys.fork_call(j)
    end

    return rets
end

--index is the service to be deleted,zero based. format is like '0,2,3,5'
function delete_server_service(http_form)
    local content = luci.json.decode(http_form)
    gdb.dumptable(content)

    local index = utl.split(content.index,',')
    index = (type(index)=="table") and index or {index}

    local servername = utl.split(content.name,',')
    servername = (type(servername)=="table") and servername or {servername}

    local old_bindif = utl.split(content.old_bindif,',')
    old_bindif = (type(old_bindif)=="table") and old_bindif or {old_bindif}

    local ret = {}
    local count=0
    
    local result = {}

    local refifs = { }
    for i,j in ipairs(servername) do
        local cmd = string.format("/lib/pptp/delete-service.sh %s",j)
        sys.fork_call(cmd)
--		ifs.update_if_reference(old_bindif[j.index+1],0)
        refifs[#refifs + 1] = old_bindif[i]
    end
    local infaces = ""
    if next(refifs) ~= nil then
        infaces = table.concat(refifs, " ")
    end
    --del the reference of interface etc
    sys.fork_call(". /lib/pppox/api.sh && deal_with_qos_ref \"%s\"" % {infaces})

    uci_r:foreach(user_configname,"user",
                    function(section)
                        local tmp = uci_r:get_all(user_configname,section[".name"])
                        if string.find(tmp["username"],"@ldap",1)  then
                            for _,v in ipairs(servername) do
                                if tmp["username"] == ("@ldap-pptp-"..v) then
                                    ippoolref:update(0,tmp["ippool"])
                                    uci_r:delete(user_configname,section[".name"])
                                end
                            end
                        end
                    end
    )
    uci_r:commit(user_configname)
    userconfig.cfg_modify();
    return true

end

-- warning,the webserver should make sure that server is a unique name
function add_server_service(http_form)
    local content = luci.json.decode(http_form)

    local new = {}
    local ldap_password = "@secret"
    local ldap_type = "pptp"
    local cmd = {}
    local index = 0
    local c_flag = 0
    local ms_dns1=""
    local ms_dns2=""

    if content.cid ~= nil then
        c_flag = 1
    end

    uci_r:foreach("pptp-server","pns",
            function(section)
                index = index + 1
            end
        )

    --directly pass those params to shell
    if content.ms_dns1 ~=nil then
        ms_dns1=content.ms_dns1
    end
    if content.ms_dns2 ~=nil then
        ms_dns2=content.ms_dns2
    end

    local ldap_name = "@ldap-pptp".."-"..content.server
    local id = tonumber(string.sub(content.server, 7, #(content.server)))
    local localip = "172.31."..tostring(math.floor(id/255)).."."..tostring(id % 255)

    if (content.authtype == "ldap") then
        if not content.remotesubnet then
            content.remotesubnet = "--"
        end
        if not  content.maxsessions then
            content.maxsessions = "1"
        end
        if c_flag == 0 then --standalone
            cmd = string.format(". /lib/pptp/add-service.sh %s %s %s %s %s %s %s",content.server,content.bindif,content.enable,content.mppeencryption,content.authtype,content.ldap_profile,ms_dns1,ms_dns2)
            new =
            {
                ["username"] = ldap_name,
                ["password"] = ldap_password,
                ["type"] = ldap_type,
                ["localip"] = localip,
                ["ippool"] = content.ippool,
                ["ms_dns1"] = content.ms_dns1,
                ["ms_dns2"] = content.ms_dns2,
                ["netmode"] = content.netmode,
                ["local_network"] =content.local_network,
                ["remotesubnet"] = content.remotesubnet,
                ["maxsessions"] = content.maxsessions,
                ["networktype"] = "",
            }
        else --controller
            cmd = string.format(". /lib/pptp/add-service.sh %s %s %s %s %s %s %s",content.server,content.bindif,content.enable,content.mppeencryption,content.authtype,content.ldap_profile,ms_dns1,ms_dns2)
            new =
            {
                ["username"] = ldap_name,
                ["password"] = ldap_password,
                ["type"] = ldap_type,
                ["localip"] = localip,
                ["ippool"] = content.ippool,
                ["ms_dns1"] = content.ms_dns1,
                ["ms_dns2"] = content.ms_dns1,
                ["netmode"] = "client2lan",
                ["local_network"] =content.local_network,
                --["remotesubnet"] = content.remotesubnet,
                ["maxsessions"] = tostring(get_max_sessions()),
                ["networktype"] = "",
            }
        end

    else
        cmd = string.format(". /lib/pptp/add-service.sh %s %s %s %s %s %s %s",content.server,content.bindif,content.enable,content.mppeencryption,content.authtype,ms_dns1,ms_dns2)
    end
    --local cmd = string.format(". /lib/pptp/add-service.sh %s %s %s %s %s",content.server,content.bindif,content.enable,content.mppeencryption,content.authtype)
    --sys.fork_exec(cmd)
    sys.fork_call(cmd)
    if content.authtype == "ldap" then
        local add_form = {}
        add_form.data= luci.json.encode(
            {
                ["method"] = "add",
                ["params"] =
                {
                    ["index"] = index,
                    ["old"] = "add",
                    ["new"] = new,
                    ["key"] = "add",
                }
            }
        )
        add_form.form = "usermanager"
        local ret = pppox.dispatch(add_form)
        if (not ret) or (ret.error_code ~= "0") then
            gdb.print_tty(error_print.check_error(ret))
            return "FAILED"
        end
    end
    ifs.update_if_reference(content.bindif,1)

    --add ldap ref
    if (content.authtype == "ldap") then
        local ref = 0
        ref = tonumber(uci_r:get("ldap",content.ldap_profile,"ref"))
        ref = ref + 1
        uci_r:set("ldap", content.ldap_profile, "ref", tostring(ref))
        if not uci_r:commit("ldap") then
            return false, err.ERR_COM_UCI_COMMIT
        end
    end
    local controller_mode = io.open("/tmp/pppox.controller.mode","r")
    if controller_mode then
        cmd = ". /lib/pppox/pppox-functions.sh && pppox_load_user" .. " pptp " .. content.server
        sys.fork_call(cmd)
        cmd = ". /lib/pppox/pppox-remote-management.sh; refresh_vpn_remote_management;"
        sys.fork_call(cmd)
        controller_mode:close()
    end
    

    userconfig.cfg_modify()
    return true
end

local type = arg[1]
local action = arg[2]
local http = arg[3]

if type == "client" then
    if action == "add" then
        insert_client_list(http)
    elseif action == "del" then
        delete_client_list(http)
    end
elseif type == "server" then
    if action == "add" then
        add_server_service(http)
    elseif action == "del" then
        delete_server_service(http)
    end
end