#!/bin/sh

. /lib/functions.sh

#修改配置文件总入口
pre_setting_config() {
    dynanmic_setting_loadbalance
    mdns_config_sync
	qos_config_sync
    set_wan_poe_mss_auto
	set_lan_ipv6_ra_auto
    pre_network_synchronous_ipv6group
	check_lte_wan
	ospf_rip_config_sync
	service_sync_icmpv6
	ddns_set_global_state
    update_hosts
	set_fqdn_config
	set_network_reqopts
	update_uhttpd_port
    update_pseudo_interface
	update_dpi_network2zone
}

#根据wan口数量和用户配置动态修改load balance总开关
dynanmic_setting_loadbalance() {
    local wan_numbers=$(uci get network.global.wan_numbers)
    local flag=$(uci get mwan3.global.flag)
    local count=`echo $wan_numbers | wc -w`  
    if [ "x${flag}" == "x0" ]; then
        if [ $count -gt 1 ]; then
            local state="on"       
        else
            local state="off"    
        fi
        uci set mwan3.global.state=${state} 
        uci commit
    else
        echo "do not adjust load balance"
    fi
}

MDNS_CONF="/etc/config/avahi-daemon"

avahi_config_rule_name_modify()
{
	#替换config forward_rule '为config forward_rule 'rule_
	sed -i "s/config forward_rule '/config forward_rule 'rule_/g" ${MDNS_CONF}
}

avahi_config_service_name_modify()
{
	sed -i "s/config service_type '/config service_type 'service_/g" ${MDNS_CONF}
}

avahi_config_list_rule_modify()
{
	sed -i "s/list rule '/list rule 'rule_/g" ${MDNS_CONF}
}

avahi_config_list_service_modify()
{
	sed -i "s/list service_type '/list service_type 'service_/g" ${MDNS_CONF}
}

mdns_config_sync()
{
	local mdnsSync=$(uci -c /etc/profile.d get profile.@global[0].mdns_sync)
	if [ -n "$mdnsSync" -a "$mdnsSync" == "1" ]; then
		local ifsync=`uci -q get avahi-daemon.avahi.sectionNameSync`
		if [ -z "$ifsync" -o "$ifsync" != "1" ]; then

			avahi_config_list_rule_modify
			avahi_config_list_service_modify
			avahi_config_rule_name_modify
			avahi_config_service_name_modify

			uci set avahi-daemon.avahi.sectionNameSync=1
			uci commit avahi-daemon
		fi
	fi
}

qos_config_sync()
{
	lua /lib/qos_ctl/qos_config_sync.lua
}

set_wan_poe_mss_auto()
{
	mss_support=`uci get network.global.mss_support`
	[ "x${mss_support}" == "x1" ] && return
	major_wanipv4=`uci -c /etc/profile.d get profile.@ecs_mod_major_ver[0].wanIpv4`
	minor_wanipv4=`uci -c /etc/profile.d get profile.@ecs_mod_minor_ver[0].wanIpv4`
	version="${major_wanipv4}${minor_wanipv4}"
	[ $version -lt 13 ] && return
	uci set network.global.mss_support="1"
	local wan_nums=`uci get network.global.wan_numbers`
	for wan_id in ${wan_nums}; do
		local poe_enable=`uci get network.wan${wan_id}_poe.t_poe_enable`
		local poe_mss=`uci get network.wan${wan_id}_poe.mss_clamping_type`
		[ "x${poe_enable}" == "x1" ] && [ -z "${poe_mss}" ] && uci set network.wan${wan_id}_poe.mss_clamping_type="auto"
	done
	uci commit network
}

set_lan_ipv6_ra()
{
	ipv6_enable=`uci get vnetwork.$1.ipv6_enable`
	[ "$1" == "vwan" ] && return
	if [ "$ipv6_enable" == "on" ]; then
		ra_enable=`uci get vnetwork.$1.ra_enable`
		[ "$ra_enable" == "on" ] && return
		uci set vnetwork.$1.ra_enable="on"
		uci set vnetwork.$1.ra_preference="medium"
		uci set vnetwork.$1.ra_validLifetime="86400"
		uci set vnetwork.$1.ra_preferredLifetime="14400"
	fi
}

#解决从5.8出货版升级到5.13版本后，5.11新增的lan ipv6 ra相关配置不会自动下发的问题
set_lan_ipv6_ra_auto()
{
	major_network=`uci -c /etc/profile.d get profile.@ecs_mod_major_ver[0].network`
	minor_network=`uci -c /etc/profile.d get profile.@ecs_mod_minor_ver[0].network`
        if [ -z $major_network ] || [ -z $minor_network ]; then
        	return
    	fi
	version="${major_network}${minor_network}"
	[ $version -lt 12 ] && return
	config_load vnetwork
	config_foreach set_lan_ipv6_ra vinterface
	uci commit vnetwork
}

#解决5.8版本升级到5.13版本后，在5.8版本配置的lan network不会同步生成ipv6group的问题
pre_network_synchronous_ipv6group()
{
	. /lib/ipv6group/config.sh
	network_synchronous_ipv6group	
}

set_reqopts_43_option()
{
        local reqopts
        reqopts=`uci get network.$1.reqopts`
        [ -z "$reqopts" ] && return

        for req in $reqopts
        do
                [ "x$req" == "x43" ] && return
        done

        reqopts="${reqopts} 43"
        uci set network.$1.reqopts="$reqopts"
}

# 解决DHCP option 43获取controller地址的升降级问题
set_network_reqopts()
{
	support_cwmp=`uci get cwmp.info.cwmp`
	[ -z "$support_cwmp" ] && return
	config_load network
	config_foreach set_reqopts_43_option interface
	uci commit network
}

#for US: 检测online配置文件的lte_dev和lte_interval字段是否存在
# for US lte-wan
pre_check_lte_wan_info()
{
	lte_dev=`uci get online.settings.lte_dev`
	lte_interval=`uci get online.settings.lte_interval`
	[ -z "$lte_dev" ] && {
		uci set online.settings.lte_dev="usb0"
	}
	[ -z "$lte_interval" ] && {
		uci set online.settings.lte_interval="300"
	}
	
	uci commit online
}

check_lte_wan() {
	local lte_flag="$(uci -c /etc/profile.d get profile.@global[0].lte_interval_low_threshold 2>/dev/null)"
	[ -z "$lte_flag" ] && return
	[ "$lte_flag" != "1" ] && return
	pre_check_lte_wan_info
}

#解决0207出货版本升级到0325提测版本后，基于pppoe拨号配置MD5/Simple认证的接口不带认证，导致邻居无法建立的问题
ospf_rip_config_sync() {
	local ospf_rip_sync=$(uci -c /etc/profile.d get profile.@global[0].ospf_rip_sync)
	if [ -n "$ospf_rip_sync" -a "$ospf_rip_sync" == "1" ]; then
		local ifsync=`uci -q get ospf.global.sectionNameSync`
		if [ -z "$ifsync" -o "$ifsync" != "1" ]; then
			lua -e "require 'luci.controller.admin.ospf'.config_interface_sync(\"ospf\")"
			lua -e "require 'luci.controller.admin.ospf'.config_interface_sync(\"rip\")"
			uci set ospf.global.sectionNameSync=1
			uci commit ospf
		fi
	fi
}
# 解决从0207出货版本升级到0325提测版本后，service没有icmpv6的默认配置问题
service_sync_icmpv6() {
	local exist_flag="$(uci get service.ICMPv6 2>/dev/null)"
	[ -n "$exist_flag" ] && return 

	uci set service.ICMPv6='service'
	uci set service.ICMPv6.name='ICMPv6'
	uci set service.ICMPv6.proto='icmpv6'
	uci set service.ICMPv6.sport='0-65535'
	uci set service.ICMPv6.dport='0-65535'
	uci set service.ICMPv6.type='255'
	uci set service.ICMPv6.code='255'
	uci set service.ICMPv6.comment='icmpv6'
	uci set service.ICMPv6.flag='0'
	uci set service.ICMPv6.ref='0'
	uci commit
	/lib/cfg_save/config.sh "save"
}
#eapcontroller需要在hosts中添加机型名称以缩短启动时间
update_hosts(){
	local one=`cat /etc/profile.d/profile |grep all_in_one | awk -F " " '{print $3}' |sed "s/'//g"`
	if [ "$one" == "1" ]; then
		local name=`grep -n "product_name" /tmp/device-info.json | awk -F ":" '{print $3}' | sed -e "s/\"//g;s/,//g;s/ //g"`
		local num=`grep -n "127.0.0.1 localhost $name" /etc/hosts | awk -F ":" '{print $1}'`
		if [ -z $num ]; then
			num=`grep -n "127.0.0.1 localhost" /etc/hosts | awk -F ":" '{print $1}'`
			if [ -n $num ]; then
			sed -i "s/127.0.0.1 localhost/127.0.0.1 localhost $name/g" /etc/hosts
			fi
		fi
		local str=`cat /etc/nsswitch.conf |grep -n "hosts" | awk -F " " '{print $4}'`
		if [ "x${str}" == "xfiles" ]; then
			sed -i "/hosts:/c hosts: files dns \[\!UNAVAIL=return\]" /etc/nsswitch.conf
		fi
    fi
}

ddns_set_global_state() {
	ddns_status=`uci get ddns.global.status`
	if [ -z "$ddns_status" ]; then
		. /lib/functions.sh
		ddns_providers="dynddns noipddns customddns cmxddns phddns"
		count=0
		get_rule_num()
		{
			let count++
		}
		for provider in $ddns_providers; do
			config_load $provider
			config_foreach  get_rule_num $provider
			if [ ${count} -gt 0 ]; then
				break
			fi
		done
		if [ $count -gt 0 ]; then 
			uci set ddns.global="status"
			uci set ddns.global.status="on"
		else
			uci set ddns.global="status"
			uci set ddns.global.status="off"
		fi
		uci commit ddns
		/lib/cfg_save/config.sh "save"
	fi
}

set_fqdn_config() {
	local count="$(uci show fqdn | grep domain_any_fqdngroup | wc -l)"
	if [ $count -eq 0 ]; then
		uci set fqdn.domain_any_fqdngroup=fqdn_group
		uci set fqdn.domain_any_fqdngroup.name='domain_any'
		uci set fqdn.domain_any_fqdngroup.domains='domain_any'
		uci set fqdn.domain_any_fqdngroup.ref=0
		uci set fqdn.domain_any_fqdngroup.flag='system'
		uci commit fqdn
		/lib/cfg_save/config.sh "save"
	fi
}

#eapcontroller新增开机进度条优化功能,需要把uhttpd默认监听的port 80/443修改为8143/8188,低版本升级高版本不用reset样机,可以生效新监听port
update_uhttpd_port(){
	local all_in_one=`cat /etc/profile.d/profile |grep all_in_one | awk -F " " '{print $3}' |sed "s/'//g"`
	if [ "$all_in_one" == "1" ]; then
		uci delete uhttpd.main.listen_http
		uci add_list uhttpd.main.listen_http='0.0.0.0:8143'
		uci add_list uhttpd.main.listen_http='[::]:8143'

		uci delete uhttpd.main.listen_https
		uci add_list uhttpd.main.listen_https='0.0.0.0:8188'
		uci add_list uhttpd.main.listen_https='[::]:8188'

		uci commit
	fi
}

FIX_PSEUDO="0"
#参考interface_wan.lua配置pseudo ip
fix_pseudo_proto(){
	local s="$1"
	if echo "$s" |grep -q "pseudo"; then
		config_get proto "$s" "proto"
		if [ x"$proto" = "xnone" ]; then
			config_get parent "$s" parent
			local parent_proto=$(uci get network.$parent.proto)
			if [ x"$parent_proto" = "xnone" ]; then
				local parent_mac=$(uci get network.$parent.macaddr) 
				echo "parent_mac $parent_mac" 
				local mac5=`echo "$parent_mac"| awk '{split($1,arr,":"); print arr[5]}'`
				local mac6=`echo "$parent_mac"| awk '{split($1,arr,":"); print arr[6]}'`
				local dec_mac5=$(printf "%d" "0x$mac5")
				local dec_mac6=$(printf "%d" "0x$mac6")
				local ip3=$(expr $dec_mac5 % 255)
				let ip3+=1
				local ip4=$(expr $dec_mac6 % 255)
				let ip4+=1
				local new_ip="168.254.$ip3.$ip4"
				echo "setting $s new_ip $new_ip"
				uci set network.$s.ipaddr=$new_ip
				uci set network.$s.proto='static'
				FIX_PSEUDO="1"
			fi
		fi
	fi

}
#修复pppoe拨号无法走sfe的问题
update_pseudo_interface(){
	local region=$(uci get tddp.region.region)
	if [ -n "$region"  ] && [ $region != "GB" ]; then
		config_load network
		config_foreach  fix_pseudo_proto interface
		if [ $FIX_PSEUDO = "1" ]; then
			uci commit network
			/lib/cfg_save/config.sh "save"
		fi
	fi
}
dpi_update_config(){
	local section="$1"
	config_get network $section network
	if [ -n "$network" ]; then
		local zone=`uci get vnetwork.$network.zone`
		uci set dpi.$section.zone="$zone"
		uci delete dpi.$section.network
		FIX_PSEUDO="1"
	fi
}
#修复Bug935678，添加hotplug机制，使用zone ipset进行匹配
update_dpi_network2zone(){
	FIX_PSEUDO="0"
	config_load dpi
	config_foreach dpi_update_config restriction
	if [ $FIX_PSEUDO = "1" ]; then
		uci commit dpi
		/lib/cfg_save/config.sh "save"
	fi
}