#!/bin/sh
#Copyright (C) 2015 tplink.com


# include dependence
. /lib/qos-tplink/qos_public.sh
. /lib/qos-tplink/qos_core.sh
. /lib/qos-tplink/qos_dpdk.sh

bw_rule="/tmp/.qos/bw_rule"

local RULE_OPT_LIST="enable if_ping if_pong rate_max ip_group mode time"
[ -z $QOS_CONFIG_ER ] && RULE_OPT_LIST="$RULE_OPT_LIST rate_max_mate" || RULE_OPT_LIST="$RULE_OPT_LIST ip_type"


# destroy all qos cfgs
qos_destroy()
{
	rm -rf $QOS_READY 2>/dev/null
	QOS_STATE clear

	# destroy all nf qos chains
	qos_nf_destroy

	# delete time object

	ret=$(timeobj_api_deleteAll ${THIS_MODULE})
	ret=$(timeobj_api_commit ${THIS_MODULE})

	rm -f ${QOS_TMP_DIR}/stop_flag 2>/dev/null

	# toggle sfe_qos according to the fact that whether netfilter(iptables) qos rule exist
	qos_nf_sfe_toggle
	#rmmod xt_tplimit here
	[ -d /sys/module/xt_tplimit ] && rmmod xt_tplimit.ko
	QOS_LOG_PRINT "qos_destroy, flush all rules!"
}

# get rule_list with the callback $1, which define the match method
qos_rulelist_get()
{
	local cb="$1"
	local list=""
	config_foreach $cb $STYPE_RULE
	QOS_RET "$list"
}

#############################################################################
# qos module UCI rule parse handle: set up rule leafs
	# UCI rule leaf -> netfilter
qos_rules_parse()
{
	local rule_list="$1"
	local flag=0  # IF have time obj reg
	local rule_num=$(QOS_STATE get rule_num)

	# 1. get evary rule within a big big big loop
	echo "*mangle" > "$bw_rule"
	for rule_name in $rule_list; do
		[ -f ${QOS_TMP_DIR}/stop_flag ] && break

		QOS_WRITE_LOG "qos_rule, parsing rule $rule_name ..."

		[ "$rule_num" -ge "$QOS_RULE_SPEC" ] && {
			QOS_LOG_PRINT " error, qos_rule_parse, user rule num exceed the spec: $QOS_RULE_SPEC !"
			return 1
		}
		let rule_num++

		# 1. check rule should be to install
		# IF rule is enable?
		config_get enable "$rule_name" "enable"
		[ "$enable" != "on" ] && QOS_WRITE_LOG " info, rule disable, skip install !" && continue

		# IF rule has option time, reg rule to time manager module, then parse next rule
		config_get timeobj "$rule_name" "time"
		[ "$timeobj" = "" ] && timeobj="Any"
		[ "$timeobj" != "Any" ] && {
			QOS_WRITE_LOG " info, rule reg to tmngtd, timeobj=$timeobj, rule=$rule_name ! "
			ret=$(timeobj_api_add $timeobj $THIS_MODULE $rule_name)
			flag=$((flag + 1))
			continue
		}

		qos_rule_install $rule_name "restore" || {
			QOS_LOG_PRINT " error, qos_rule_parse, install rule $rule_name fail!" && break
		}
	done
	echo "COMMIT" >> "$bw_rule"
	iptables-restore -w -n < "$bw_rule"
	QOS_STATE set rule_num "$rule_num"
	[ $flag -gt 0 ] && ret=$(timeobj_api_commit $THIS_MODULE)

	# flush conntracks, for nfmark may vary on every qos_start
	#conntrack -F >/dev/null 2>&1
	return 0
}

# qos start entry
	# complete hard init qos in TC & nf
qos_start()
{

	[ -z $QOS_CONFIG_INVALID ] || {
		QOS_WRITE_LOG "  NO UCI config, exit the QOS process ! "
		return
	}

	# 1. global init root
	[ $(qos_global_init) -ne 0 ] && return
	
	# 2. insmod xt_tplimit.ko here
        local version=`uname -r`
        insmod /lib/modules/$version/xt_tplimit.ko

	# 2. extract all user rules from UCI config file
	#local rule_list=`awk '/rule/ {print $3}' $QOS_UCI | sed s/\'*//g`
	rule_get()
	{
		[ -z "$list" ] && list="$1" || list="$list $1"
	}
	local rule_list=$(qos_rulelist_get rule_get)
	if [ ${#rule_list} -eq 0 ]; then
		QOS_LOG_PRINT " info, NO user rule find!"
	else
		# get current UCI rules list for rule order
		# local rules_order="${QOS_TMP_DIR}/rules_order"
		# echo -e "$rule_list" | sed 's/[ ]/\n/g' | awk '{print NR,$0}' > $rules_order

		QOS_LOG_PRINT " qos_start, rule list: $rule_list"
		# store all rules info as qos_state
		QOS_STATE set rule_list "$rule_list "
		QOS_STATE set rule_num "0"

		lua /lib/qos-tplink/state_gen.lua

		# for rule in $rule_list; do
		# 	for opt in $RULE_OPT_LIST; do
		# 		QOS_STATE set ${rule}.${opt} "$(config_get $rule $opt)"
		# 	done
		# done

		qos_rules_parse "$rule_list"
	fi

	# toggle sfe_qos according to the fact that whether netfilter(iptables) qos rule exist
	qos_nf_sfe_toggle

	# set READY flag
	touch $QOS_READY
}

# handle time object response
	# ACTIVE
qos_time_handle()
{
	[ ! -f $QOS_READY ] && QOS_LOG_PRINT " info, qos not config, NO need time handle ! " && return

	local mode="$1"  # active or expire or reset
	[ "$mode" = "RESET" ] && {
		QOS_LOG_PRINT " info, time objects RESET, STUB now ! "
		#qos_start
		return
	}
	local symbol="$2"  # time object name

	# reserve rule_list support
	[ "$mode" = ACTIVE ] && cb=qos_rule_install || cb=qos_rule_uninstall
	for rule in $symbol; do
		[ -f ${QOS_TMP_DIR}/stop_flag ] && break

		# check IF rule exist in UCI?
		[ "`uci get ${THIS_MODULE}.${rule} 2>/dev/null`" != "rule" ]&& {
			QOS_LOG_PRINT " exception qos_time_handle, rule $rule not exist in UCI "
			continue
		}

		$cb $rule || {
			QOS_WRITE_LOG " error, qos_time_handle process $cb $rule fail!"
			break
		}
	done

	# toggle sfe_qos according to the fact that whether netfilter(iptables) qos rule exist
	qos_nf_sfe_toggle

	## [ "$mode" = ACTIVE ] && conntrack -F >/dev/null 2>&1
	return 0
}

qos_ipgroup_handle()
{
	[ ! -f $QOS_READY ] && QOS_LOG_PRINT " info, qos not config, NO need ipgroup handle ! " && return
	local mode="$1"  # UPDATE or DELETE
	local ipobj="$2"

	[ "$mode" != "update" ] && QOS_LOG_PRINT " info, ipgroup handle mode != update ! " && return

	# compose the rule_list match the ipgroup obj
	rule_get()
	{
		local rule="$1"
		[ "$(config_get "$rule" ip_group)" = "$ipobj" ] && list="$list $rule"
	}
	local rule_list=$(qos_rulelist_get rule_get)
	[ ${#rule_list} -eq 0 ] && {
		QOS_LOG_PRINT " info, No rule match ipgroup $ipobj "
		return
	}

	QOS_LOG_PRINT " info, ipgroup $ipobj apply by rule_list:$rule_list "
	for rule in $rule_list; do
		[ -f ${QOS_TMP_DIR}/stop_flag ] && break
	done
	#conntrack -F >/dev/null 2>&1

	# toggle sfe_qos according to the fact that whether netfilter(iptables) qos rule exist
	qos_nf_sfe_toggle
}

qos_relaod_rule()
{
    local rule_name=$1
	local timeobj
	local timeobj_bk

	QOS_WRITE_LOG "**************：rule_name=$rule_name"
	local timeobj_bk=$(QOS_STATE get "${rule_name}.time")
	config_get timeobj "$rule_name" "time"
	QOS_WRITE_LOG "timeobj=$timeobj,timeobj_bk=$timeobj_bk"

	[ "$timeobj" = "" ] && timeobj="Any"

	[ "$timeobj_bk" != "Any" ] && {
		QOS_WRITE_LOG " info, rule del from tmngtd, timeobj_bk=$timeobj_bk, rule=$rule_name ! "
		ret=$(timeobj_api_delete $timeobj_bk $THIS_MODULE $rule_name)
		ret=$(timeobj_api_commit $THIS_MODULE)
	}

	[ "$timeobj" != "Any" ] && {
		QOS_WRITE_LOG " info, rule reg to tmngtd, timeobj=$timeobj, rule=$rule_name ! "
		ret=$(timeobj_api_add $timeobj $THIS_MODULE $rule_name)
		ret=$(timeobj_api_commit $THIS_MODULE)
		return
	}


	qos_rule_install $rule_name || {
		QOS_LOG_PRINT " error, qos_rule_parse, install rule $rule_name fail!"
	}
}

# qos warm change adapt
	# for now, only support rule-add warm init
	        #  rule delete MUST re start
qos_reload()
{
	local op="$1"
	shift
	[ "$op" = "update" ] && {
		local update_list="$@"
	}
	[ "$op" = "add" ] && {
		local add_list="$@"
	}
	[ "$op" = "delete" ] && {
		local del_list="$@"
	}

	# if no qos.ready, proc total start
	[ ! -f $QOS_READY ] && { QOS_LOG_PRINT "qos_reload, NO effective config, run hard start!"
		qos_start
		return
	}

	# if global qos_enable changed, ONLY happen enable on->off
	local enable_cur=$(config_get "$SNAME_GLOBAL" qos_enable)
	local enable_bak=$(QOS_STATE get qos_enable)
	[ "$enable_cur" != "$enable_bak" ] && { QOS_LOG_PRINT "qos_reload, qos_enable->off! run stop"
		#echo 0 > /sys/qoslimit/g_natfwd_qoslimit_switch
		#echo "enable_cur = " $enable_cur > /dev/console
		dpdk_on_off "off"
		qos_destroy
		return
	}

	###############################################
	# IF list interface change ?
		# tip: IFACE_LIST always the latest UCI ifaces
	iface_list_bak=$(QOS_STATE get iface_list)
	[ "`echo $iface_list_bak | wc -w`" != "`echo $IFACE_LIST | wc -w`" ] && {
		QOS_LOG_PRINT "qos_reload, interface varied, run hard start"
		qos_start
		return
	}

	for iface in $IFACE_LIST; do
		# IF iface exist in bak file
		`echo $iface_list_bak | grep -w -q "${iface}" ` || {
			QOS_LOG_PRINT "qos_reload, ${iface} is new, run hard start!"
			qos_start
			return
		}

		# up/down link rate of the iface change?
		for item in uplink downlink; do
			local cur=$(config_get "$iface" $item)
			local bak=$(QOS_STATE get "${iface}.${item}")
			[ "$cur" != "$bak" ] && {
				QOS_LOG_PRINT "qos_reload, ${iface} $item changed, run hard start!"
				qos_start
				return
			}
		done
	done

	##############################################
	# get user rule change list
		# current UCI and bak-UCI  diff_rule list
	local rules_cur="${QOS_TMP_DIR}/rules_cur"
	local rules_bak="${QOS_TMP_DIR}/rules_bak"
	# local rules_update="${QOS_TMP_DIR}/rules_update"
	rule_get()
	{
		[ -z "$list" ] && list="$1" || list="$list $1"
	}

	local rule_list=$(qos_rulelist_get rule_get)

	# get current UCI rules list for rule order
	local rules_order="${QOS_TMP_DIR}/rules_order"
	cp ${rules_order} ${rules_order}_bak
	echo -e "$rule_list" | sed 's/[ ]/\n/g' | awk '{print NR,$0}' > $rules_order

	# this process order is important : delete -> add -> update
	# proc dellist
	[ 0 -ne ${#del_list} ] && {
		local rule_num=$(QOS_STATE get rule_num)
		for rule in $del_list; do
			[ -f ${QOS_TMP_DIR}/stop_flag ] && break

			QOS_LOG_PRINT "qos_reload, delete rules: ${rule}"
			[ "$(QOS_STATE get ${rule}.enable)" = "on" ] && qos_rule_uninstall $rule

			tobj=$(QOS_STATE get ${rule}.time)
			if [ ${#tobj} -ne 0 -a $tobj != "Any" ]; then
				timeobj_api_delete $tobj $THIS_MODULE $rule && timeobj_api_commit $THIS_MODULE
			fi
			QOS_STATE del "$rule"
			let rule_num--
		done
		QOS_STATE set rule_list "$rule_list "
		[ $rule_num -lt 0 ] && rule_num=0
		QOS_STATE set rule_num "$rule_num"
	}

	# proc addlist
	[ 0 -ne ${#add_list} ] && {
		for rule in $add_list; do
			QOS_LOG_PRINT "qos_reload, add new rules: ${rule}"
			for opt in $RULE_OPT_LIST; do
				QOS_STATE set ${rule}.${opt} "$(config_get $rule $opt)"
			done
		done
		QOS_STATE set rule_list "$rule_list "

		qos_rules_parse "$add_list"
		#conntrack -F >/dev/null 2>&1
	}

	# confirm rules in comlist is changed ?
	if [ 0 -ne ${#update_list} ]; then
		for rule in $update_list; do
			qos_rule_uninstall $rule
		done

		for rule in $update_list; do
			[ "$(config_get $rule enable)" == "on" ] && {
				 qos_relaod_rule $rule || {
					QOS_LOG_PRINT " error, update rule $rule fail! "
					return;
				}
			}

			[ "$(config_get $rule enable)" == "off" ] && {
			    local timeobj_del=$(QOS_STATE get "${rule}.time")
			    [ "$timeobj_del" != "Any" ] && {
            		ret=$(timeobj_api_delete $timeobj_del $THIS_MODULE $rule)
            		ret=$(timeobj_api_commit $THIS_MODULE)
            		QOS_WRITE_LOG " info, rule del from tmngtd, timeobj_del=$timeobj_del, rule=$rule ! "
        		}
			}

			# update rule info in qos_state
			for haha in $RULE_OPT_LIST; do
				QOS_STATE set "${rule}.${haha}" "$(config_get $rule $haha)"
			done

		done
		QOS_STATE set rule_list "$rule_list "
	fi

	qos_nf_sfe_toggle

}

##################################################################################
# qos module root command
if [ "$1" = "stop" ]; then
	touch ${QOS_TMP_DIR}/stop_flag
fi

{
    flock -x 25
    local args="$@"
    #QOS_LOG_PRINT "$0 $args. begin @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ ! "
    if [ "$1" = "start" ]; then
    	qos_start

    elif [ "$1" = "stop" ]; then
	dpdk_on_off "off"
    	qos_destroy

    elif [ "$1" = "reload" ]; then
        shift
        local op="$1"
        shift
        local op_list="$@"
    	qos_reload "$op" "$op_list"

    elif [ "$1" = "ifdown" ]; then
    	local intf="$2"
    	qos_start

    elif [ "$1" = "ifup" ]; then
    	local intf="$2"
    	qos_start

    elif [ "$1" = "time" ]; then
    	local mode="$2"
    	shift 2
    	local symbol="$@"  # precise rule_name regged by timeobj_add_api
    	qos_time_handle "$mode" "$symbol"

    elif [ "$1" = "ipgroup" ]; then
    	local mode="$2"
    	local obj="$3"
    	qos_ipgroup_handle $mode $obj

    else
    	QOS_INFO_PRINT "$0 NO support arg $args "
    fi

    #QOS_LOG_PRINT "$0 $args. done !\n"
    flock -u 25
} 25<> /tmp/.qos/qos.lock
