#!/bin/sh
#Copyright (C) 2015 tplink.com

[ "$INC_QOS_CORE" != "" ] && return
export INC_QOS_CORE="INC"

. /lib/qos-tplink/qos_public.sh
. /lib/qos-tplink/qos_nf.sh
. /lib/qos-tplink/qos_dpdk.sh

############################################################################
# get the whole option from section
qos_config_get_section()
{
	local cfg=$1
	local sect=$2;
	echo "###debug, get sect $sect of cfg $cfg!"
}

# create and nf root for zone&dev, which has NOT init before
	# for which was init, just skip.
# This is A reenterable handle
qos_global_init()
{
	local qos_enable=""

	# 1. if QOS module is enabled?
	config_get qos_enable "$SNAME_GLOBAL" "qos_enable"
	[ "$qos_enable" != "on" ] && QOS_WRITE_LOG " QOS disable, NO need config, just return! " &&  QOS_RET 1 && return
	QOS_WRITE_LOG "qos_global_init, begin ..."

	dpdk_on_off "on"

	# 2. prepare dependence for QOS  - kmods , ipt-chains
	[ ${#IFACE_LIST} -eq 0 ] && {
		QOS_WRITE_LOG " info, NO user interface find, can not config QOS!"
		QOS_RET 2
		return
	}
	QOS_STATE set qos_enable "on"
	QOS_STATE set iface_list "$IFACE_LIST"
	for iface in $IFACE_LIST; do
		# MUST set the dev txqueuelen NON-zero, it affect the pfifo_fast performance
		#ifconfig "$(indirect_var_get ${iface}_dev)" txqueuelen 500 2>/dev/null

		QOS_STATE set ${iface}.t_name $iface
		QOS_STATE set ${iface}.uplink "$(config_get $iface  uplink)"
		QOS_STATE set ${iface}.downlink "$(config_get $iface  downlink)"
	done

	QOS_WRITE_LOG "qos_global_init, global ok!"
	QOS_RET 0
}


# config one rule for qos module
    # ret: 0 ok,  1 err.  (err condition is only SPEC exceed!)
qos_rule_install()
{
	local rule_name="$1"
	local restore="false"
	[ "$2" == "restore" ] && restore="true"	
	QOS_WRITE_LOG " install rule $rule_name ... "
	[ "$(QOS_STATE get ${rule_name}.install)" = "true" ] && {
		QOS_WRITE_LOG " info, rule $rule_name has alreay install, skip !" && QOS_RET 0 && return 1
	} 

	# IF iface is valid ?
	# Temp fix now!! if we add the invalid iface rule in iptables, it doesn't matter.
	# Here we have lots of ifaces and if one down, the whole rule install will be skipped.
	# So the codes below to skip rule install must be comment out !! Until we found more effect way to due with this problem.
	config_get if_ping "$rule_name" "if_ping"
	[ "$if_ping" = "WAN_ALL" ] && {
		list=""
		if_ping=""
		config_get list "$SNAME_GLOBAL" "$STYPE_IFACE"
		for iface in $list;do
			[ $iface != "LAN" ] && {
				[ -z "$if_ping" ] && if_ping="$iface" || if_ping="$if_ping $iface"
			}
		done
	}
	[ ${#if_ping} -eq 0 ] && QOS_WRITE_LOG " info, inf_ping not set, skip !" && QOS_RET 0 && return 1
	config_get if_pong "$rule_name" "if_pong"
	[ "$if_pong" = "WAN_ALL" ] && {
		list=""
		if_pong=""
		config_get list "$SNAME_GLOBAL" "$STYPE_IFACE"
		for iface in $list;do
			[ $iface != "LAN" ] && {
				[ -z "$if_pong" ] && if_pong="$iface" || if_pong="$if_pong $iface"
			}
		done
	}
	[ ${#if_pong} -eq 0 ] && QOS_WRITE_LOG " info, if_pong not set, skip !" && QOS_RET 0 && return 1
	[ "$if_ping" = "$if_pong" ] && QOS_WRITE_LOG " info, rule's iface src = dst invalid, skip !" && QOS_RET 0 && return 1

	# 2. get all options from a rule section
	config_get rate "$rule_name" "rate_max"
	[ "$rate" = "" ] && rate="0kbps"
	rate="${rate/[^0-9]*/}kbps"
	[ "$rate" = "0kbps" -a "$downrate" = "0kbps" ] && QOS_WRITE_LOG " info, rule bandwidth not assigned! skip install" && QOS_RET 0 && return 1

	config_get ip_group "$rule_name" "ip_group"
	if [ "$ip_group" != "" ]; then
		# add adapt for IPGROUP_ANY, this means NO consider ipaddr
		[ "$ip_group" != "IPGROUP_ANY" ] && {
			[ `ipset list -n | grep -cw "$ip_group"` -eq 0 ] && QOS_WRITE_LOG " error, rule ip_group $ip_group NOT exist in system! " && QOS_RET 0 && return 1
		}
	else
		QOS_WRITE_LOG " error, ip & ip_group NOT found! next" && QOS_RET 0 && return 1
	fi

	# 4. uplink and downlink config
	local flag=""
	local ip_set=""
	if [ "$ip_group" != "" -a "$ip_group" != "IPGROUP_ANY" ]; then
		local flag="ip_group"
		config_get mode "$rule_name" "mode"
		[ "$mode" != "share" -a "$mode" != "priv" ] && mode="share"
		flag="ipset_share"
		[ "$mode" = "priv" ] && flag="ipset_priv"
		QOS_WRITE_LOG " rule is $ip_group $flag limit mode"
		ip_set="$ip_group"
	elif [ "$ip_group" == "IPGROUP_ANY" ]; then
		# IPGROUP_ANY
		flag="ipset_any"
		ip_set="$ip_group"
	else
		QOS_WRITE_LOG "  error, rule $rule_name illegal, ip & ip_group! next"
	fi

	if [ 0 -eq ${rate/[^0-9]*/} ]; then
                QOS_WRITE_LOG "  qos_rule_install_raw, NO limit for rule $rule_name on $if_ping -> $if_pong, skip "
                QOS_RET 0
        fi

        qos_nf_add_rule "$if_ping" "$if_pong" "$rule_name" "$rate" "$flag" "$ip_set" "$restore"|| {
                QOS_WRITE_LOG "  qos_rule_install, add rule $rule_name fail! "
                QOS_RET 1
                return 1
	}

	QOS_STATE set ${rule_name}.install true

	return 0
}

# support siface or giface rule uninstall
qos_rule_uninstall()
{
	local rule="$1"

	[ "$(QOS_STATE get ${rule}.install)" != "true" ]  && QOS_WRITE_LOG "  info, rule $rule NOT install" && QOS_RET 0 && return 1

	QOS_WRITE_LOG " uninstall qos rule $rule ... "
        ret=$(qos_nf_del_rule "$rule")
	QOS_STATE set ${rule}.install false

	return 0
}



