#!/bin/sh
#Copyright (C) 2015 tplink.com

[ "$INC_QOS_NF" != "" ] && return
export INC_QOS_NF="INC"

. /lib/qos-tplink/qos_public.sh
. /lib/qos-tplink/qos_dpdk.sh

IPT=iptables
TABLE=mangle
IPT_PREFIX="$IPT -w -t $TABLE"
QOS_CHAIN="qos_chain_default"
QOS_SFE_CHAIN="sfe_mark"
QOS_HWNAT_CHAIN="hwnat_mark"
NFMARK_MUTEX="$QOS_NFMARK_MUTEX"
NFMARK_MUTEX_MASK="$QOS_NFMARK_MUTEX_MASK"
dpdk_file="/etc/init.d/dpdk"
bw_rule="/tmp/.qos/bw_rule"

qos_nf_sfe_toggle()
{
	#Set the value of sfe_fastpath_nat_switch according to the state of qos
	local qos_enable=`uci get qos.setting.qos_enable`
	if [ $qos_enable = "on" ]; then
		echo 1 >/proc/net/sfe_fastpath_nat_switch
	else
		echo 0 >/proc/net/sfe_fastpath_nat_switch
	fi

	local chain="$QOS_CHAIN"
	local isExist=`$IPT_PREFIX -L FORWARD --line | grep -w "$chain" | awk '{print $1}'`

	# should clean hwnat/sfe connections after iptables rules be renewed
	echo qos_nf_sfe_toggle >> /dev/console

	if fastpath_sw_real_on; then
		local sfe_qos="`uci get sfe.module.qos`"
		if [ ${#isExist} -ne 0 ]; then
			# real qos rule exist, "real" means rule is installed in netfilter(iptables)
			# notify sfe that qos is on when in need
			[ $sfe_qos = "off" ] && {
				uci set "sfe.module.qos=on"
				uci commit sfe
				# `/etc/init.d/sfe restart` 2>/dev/null
				xfwdm -F
			}
		else
			# notify sfe that qos is off when in need
			[ $sfe_qos = "on" ] && {
				uci set "sfe.module.qos=off"
				uci commit sfe
				# `/etc/init.d/sfe restart` 2>/dev/null
				xfwdm -F
			}
		fi
	fi

	if fastpath_sw_real_on; then
		xfwdm -F
	fi
	if fastpath_hw_real_on; then
		echo 1 > /sys/kernel/debug/ecm/ecm_db/defunct_all
		[ -e "/etc/init.d/hwnat" ] && hwnat -y
	fi
	if fastpath_sw_real_on; then
		xfwdm -F
	fi
}

# flush all qos chains
#   unmount the target to qos-chains
qos_nf_destroy()
{
	local chain="$QOS_CHAIN"

	# qos single iface chain flush
	local ln=`$IPT_PREFIX -L FORWARD --line | grep -w "$chain" | awk '{print $1}'`
	[ ${#ln} -ne 0 ] && $IPT_PREFIX -D FORWARD $ln
	$IPT_PREFIX -F $chain 2>/dev/null
	$IPT_PREFIX -X $chain 2>/dev/null

	# extra flush, those not mount in FORWARD
	for chain in `$IPT_PREFIX -S | awk '/-N[ ]qos_.*/ {print $2}'`; do
		$IPT_PREFIX -F $chain 2>/dev/null
		$IPT_PREFIX -X $chain 2>/dev/null
	done

	#QOS_LOG_PRINT "qos_nf_destroy, flush and umount all qos-chains in $TABLE"
}

qos_nf_create_default_chain()
{
	QOS_WRITE_LOG " info, create qos default chain $chain"
	QOS_WRITE_LOG "\t$IPT_PREFIX -N $chain 2>/dev/null"
	$IPT_PREFIX -N $chain 2>/dev/null

	# anti-reentry mechanism
	QOS_WRITE_LOG "\t$IPT_PREFIX -A $chain -m mark ! --mark 0/${NFMARK_MUTEX_MASK} -j RETURN"
	$IPT_PREFIX -A $chain -m mark ! --mark 0/${NFMARK_MUTEX_MASK} -j RETURN

	# global fastpath mark.
	if fastpath_sw_real_on; then
		$IPT_PREFIX -A $chain -m tplimit --rate 0x7FFFFFFF --maskbit $QOS_NFMARK_MUTEX_START --info global --setting_num $setting_num --setting_index $setting_index -j DROP
	fi

	ln=`$IPT_PREFIX  -L FORWARD --line | awk '/zone.*MSSFIX/ {print $1}' | awk 'NR==1'`
	[ ${#ln} -ne 0 ] && plus="-I FORWARD $ln" || plus="-A FORWARD"
	QOS_WRITE_LOG "\t$IPT_PREFIX $plus -j $chain"
	$IPT_PREFIX $plus -j $chain

}

# setup a new handle to add rule
qos_nf_add_rule()
{
	local if_src="$1"
	local if_dst="$2"
	local rule="$3"
	local rate="$4"
	local flag="$5"
	local iporset="$6"
	local restore="$7"
	local is_group="false"
	config_get setting_num "$SNAME_GLOBAL" "qos_setting_number"
	config_get setting_index "$rule" "setting_index"

	if [ `echo "$if_src" | wc -w` -gt 1 -o `echo "$if_dst" | wc -w` -gt 1 ]; then
                is_group="true"
	fi

	local chain="$QOS_CHAIN"

	if [ ! -n "$setting_num" ]; then
		setting_num=1
	fi

	if [ ! -n "$setting_index" ]; then
		setting_index=0
	fi

	# create qos_default target to
	$IPT_PREFIX  -L FORWARD --line | grep -q -w "$chain" || qos_nf_create_default_chain
	local mt_netdev=""
	dst_zone_group=$(zone_group ${if_dst})
	src_zone_group=$(zone_group ${if_src})
	if [ "$dst_zone_group" = "" -a "$src_zone_group" = "" ];then
		mt_netdev="-i $(indirect_var_get ${if_src}_dev) -o $(indirect_var_get ${if_dst}_dev)"
	elif [ "$dst_zone_group" = "" ];then
		[ "$if_src" = "LAN" ] && src_zone_group="lan_iface"
		mt_netdev="-o $(indirect_var_get ${if_dst}_dev) -m set --match-set ${src_zone_group} src"
	elif [ "$src_zone_group" = "" ];then
		[ "$if_dst" = "LAN" ] && dst_zone_group="lan_iface"
		mt_netdev="-i $(indirect_var_get ${if_src}_dev) -m set --match-set ${dst_zone_group} dst"
	else
		[ "$if_src" = "LAN" ] && src_zone_group="lan_iface"
		[ "$if_dst" = "LAN" ] && dst_zone_group="lan_iface"
		mt_netdev="-m set --match-set ${src_zone_group} src -m set --match-set ${dst_zone_group} dst"
	fi
	
	local ip_type=$(config_get $rule "ip_type")
	[ "$ip_type" != "src" -a "$ip_type" != "dst" ] && ip_type="src"
	local mt_user=""
	local mt_limit="-m tplimit --rate ${rate/kbps/} --type share --maskbit $QOS_NFMARK_MUTEX_START --info $rule --setting_num $setting_num --setting_index $setting_index"
	if [ "$flag" = "ipset_priv" ]; then
		# in tplimit priv mode, setup the tplimit to priv mode by --type
		mt_user="-m set --match-set ${iporset} $ip_type "
		mt_limit="-m tplimit --rate ${rate/kbps/} --type $ip_type --maskbit $QOS_NFMARK_MUTEX_START --info $rule --setting_num $setting_num --setting_index $setting_index"
		dpdk_qos="dpdk_add_rule ${rate/kbps/} $ip_type $setting_num $setting_index $rule $QOS_NFMARK_MUTEX_START"
	elif [ "$flag" = "ipset_share" ]; then
		mt_user="-m set --match-set ${iporset} $ip_type"
		dpdk_qos="dpdk_add_rule ${rate/kbps/} share $setting_num $setting_index $rule $QOS_NFMARK_MUTEX_START"
	else
		# IPGROUP_ANY, mt_user no need
		QOS_WRITE_LOG " CAUTION, $rule is a IPGROUP_ANY rule!"

		# even though is IPGROUP_ANY, allow to be priv mode now
		# be careful that if there is too many ip and over xt_tplimit cell_mempool, ip that overflow could not be qos
		local mode=$(config_get $rule "mode")
		[ "$mode" = "share" ] && dpdk_qos="dpdk_add_rule ${rate/kbps/} share $setting_num $setting_index $rule $QOS_NFMARK_MUTEX_START"
		[ "$mode" != "share" -a "$mode" != "priv" ] && mode="share"
		[ "$mode" = "priv" ] && {
			dpdk_qos="dpdk_add_rule ${rate/kbps/} $ip_type $setting_num $setting_index $rule $QOS_NFMARK_MUTEX_START"
			mt_limit="-m tplimit --rate ${rate/kbps/} --type $ip_type --maskbit $QOS_NFMARK_MUTEX_START --info $rule --setting_num $setting_num --setting_index $setting_index"
		}
	fi

	local insert="-A $chain"
	if [ -f "$dpdk_file" ]; then
		$IPT_PREFIX -nL $chain --line | sed -e '1d' | grep -w "rate.*type.*maskbit.*, info" | awk '{print $NF}' > ${QOS_TMP_DIR}/installed_file 2>/dev/null
	else
		$IPT_PREFIX -nL $chain --line | sed -e '1,4d' | grep -w "rate.*type.*maskbit.*, info" | awk '{print $NF}' > ${QOS_TMP_DIR}/installed_file 2>/dev/null
	fi

	lua /lib/qos-tplink/find_index.lua ${rule}
	local insert_index=`cat ${QOS_TMP_DIR}/insert_index`
	[ ${insert_index} -gt 0 ] && {
		if [ -f "$dpdk_file" ]; then
			let insert_index++
		else
			insert_index=$((insert_index-1))
			insert_index=$((insert_index*2))
			let insert_index++
			let insert_index++
			let insert_index++
		fi
		insert="-I $chain $insert_index"
	}

	if fastpath_sw_on; then
		if [ "$restore" == "true" ];then
			echo "$insert $mt_netdev $mt_user -m comment --comment $rule -j $QOS_SFE_CHAIN" >> "$bw_rule"
		else
			$IPT_PREFIX $insert $mt_netdev $mt_user -m comment --comment $rule -j $QOS_SFE_CHAIN
		fi
	elif fastpath_hw_on; then
		if [ "$restore" == "true" ];then
				echo "$insert $mt_netdev $mt_user -m comment --comment $rule -j $QOS_HWNAT_CHAIN" >> "$bw_rule"
		else
				$IPT_PREFIX $insert $mt_netdev $mt_user -m comment --comment $rule -j $QOS_HWNAT_CHAIN
		fi
		
	fi
	if [ "$restore" == "true" ];then
		echo "$insert $mt_netdev $mt_user $mt_limit -j DROP" >> "$bw_rule"
	else
		$IPT_PREFIX $insert $mt_netdev $mt_user $mt_limit -j DROP
	fi
	QOS_WRITE_LOG "\t$IPT_PREFIX $insert $mt_netdev $mt_user $mt_limit -j DROP"

####### dpdk config ##########

	[ -f "$dpdk_file" ] && {
		$dpdk_qos
		if_flush_conntrack=$(echo $rule | grep "_mate$")
		if [[ "$if_flush_conntrack" != "" ]]
		then
			dpdk_flush_conntrack
		fi
	}
############ dpdk config end ########
	return 0
}

# delete the item defined by the nfmark from nf qos chain
	# nfmark is get from mark.data with arg rule_name
qos_nf_del_rule()
{
	local rule="$1"
	local chain="$QOS_CHAIN"
	local rate=`$IPT_PREFIX -nvL $chain --line | sed -e '1d' | grep -w "info[ ]$rule" | awk -F 'rate' '{print $2}' | awk '{print $1}'`
	local type=`$IPT_PREFIX -nvL $chain --line | sed -e '1d' | grep -w "info[ ]$rule" | awk -F 'type' '{print $2}' | awk '{print $1}'`
	local maskbit=`$IPT_PREFIX -nvL $chain --line | sed -e '1d' | grep -w "info[ ]$rule" | awk -F 'maskbit' '{print $2}' | awk '{print $1}'`
	local setting_num=`$IPT_PREFIX -nvL $chain --line | sed -e '1d' | grep -w "info[ ]$rule" | awk -F 'setting_num' '{print $2}' | awk '{print $1}'`
	local setting_index=`$IPT_PREFIX -nvL $chain --line | sed -e '1d' | grep -w "info[ ]$rule" | awk -F 'setting_index' '{print $2}' | awk '{print $1}'`
	dpdk_del_rule ${rate/kbps,/} ${type/,/} ${setting_num/,/} ${setting_index/,/} ${rule/,/} ${maskbit/,/}
	

	local idx=`$IPT_PREFIX -nvL $chain --line | sed -e '1d' | grep -w "info[ ]$rule" | awk '{print $1}'`
	[ ${#idx} -ne 0 ] && {
		$IPT_PREFIX -D $chain $idx

		# delete sfe xfwdm mark rule
		( fastpath_sw_on || fastpath_hw_on ) && {	
			idx=`$IPT_PREFIX -nvL $chain --line | sed -e '1,4d' |  grep -w "/\* $rule \*/" | awk '{print $1}'`
			[ ${#idx} -ne 0 ] && $IPT_PREFIX -D $chain $idx
		}

		# IF $QOS_CHAIN is empty, smash it and delete target from FORWARD
		[ `$IPT_PREFIX -S $chain | awk 'END{print NR}'` = 2 ] && {
			QOS_WRITE_LOG " info, $chain is empty, should destroy! "
			local i=`$IPT_PREFIX -L FORWARD --line | grep -w "$chain" | awk '{print $1}'`
			[ ${#i} -ne 0 ] && $IPT_PREFIX -D FORWARD $i
			$IPT_PREFIX -F $chain
			$IPT_PREFIX -X $chain
		}
	}

	QOS_RET 0 && return 0
}


