#!/bin/sh
# public defines and handles \ adapt for QOS

[ "$INC_QOS_COM" != "" ] && return
export INC_QOS_COM="INC"

#################################################################################################
#include extern lib
. /lib/functions.sh
. /lib/zone/zone_api.sh
. /lib/time_mngt/timeobj_api.sh

#################################################################################################
# qos  module defines
export THIS_MODULE="qos"
export STYPE_RULE="rule"
export STYPE_GLOBAL="global"
export SNAME_GLOBAL="setting"
export STYPE_IFACE="interface"

export LINERATE=1000000kbps

#########################
# get qos spec from /etc/profile.d/profile
QOS_RULE_SPEC=`uci -c /etc/profile.d show profile | grep "@qos.*" | awk -F'=' '/rule_spec/ {print $2}'`
[ -z $QOS_RULE_SPEC ] && QOS_RULE_SPEC=64
export QOS_RULE_SPEC=$(($QOS_RULE_SPEC * 2))
export QOS_CELL_SPEC=`uci -c /etc/profile.d show profile | grep "@qos.*" | awk -F'=' '/cell_spec/ {print $2}'`
[ -z $QOS_CELL_SPEC ] && QOS_CELL_SPEC=1024

# nfmark for ifgroup rule
	# one ifgroup rule cost ONE mark,

export QOS_NFMARK_MUTEX_START=28
export QOS_NFMARK_MUTEX=`awk "BEGIN{f=lshift(1, $QOS_NFMARK_MUTEX_START); print f}"`
QOS_NFMARK_MUTEX=`printf "0x%x" $QOS_NFMARK_MUTEX`
export QOS_NFMARK_MUTEX_MASK=$QOS_NFMARK_MUTEX

#########################
QOS_CONFIG_DIR="/etc/config"
QOS_LIB_DIR="/lib/qos-tplink"
QOS_UCI="${QOS_CONFIG_DIR}/${THIS_MODULE}"
QOS_TMP_DIR="/tmp/.qos"
QOS_LOG="${QOS_TMP_DIR}/qos-tplink.log"
QOS_READY="${QOS_TMP_DIR}/.ready"
export QOS_POLLING_FILE="${QOS_TMP_DIR}/.polling"
# extract uci-config without comments
 # link qos uci config
[ -f $QOS_UCI ] || {
    touch $QOS_UCI
    export QOS_CONFIG_INVALID="true"
    return
}

#sed '/^[ \t]*#.*/d' -i $QOS_UCI
ln -sf $QOS_UCI $QOS_LIB_DIR/uci

 # load my configs
config_load "$THIS_MODULE"
 # backup effective UCI config, for config reload changes get

#########################
# setup tmp dir and log file
[ ! -d $QOS_TMP_DIR ] && mkdir $QOS_TMP_DIR
[ ! -d ${QOS_TMP_DIR}/log ] && mkdir ${QOS_TMP_DIR}/log
[ ! -f $QOS_LOG ] && echo -e "time\t\t\tlog_info" > "$QOS_LOG" && echo -e " create qos.log file"
[ ! -f ${QOS_LIB_DIR}/log ] && ln -sf $QOS_LOG ${QOS_LIB_DIR}/log

#################################################################################################
# general-use base API
QOS_RET()
{
	echo "$1"
}

QOS_INFO_PRINT()
{
	echo -e "$1"
}

QOS_WRITE_FILE()
{
	echo -e "$1" >> "$2" 2>/dev/null
}

QOS_WRITE_LOG()
{
	local log="$1"
#return
	local file=$QOS_LOG
	local time=`date +%Y%m%d-%H.%M.%S`
	local item="${time}    ${log}"
	# if log size > 150K, save and recreate
	[ `ls -l "$file" | awk '{print $5}'` -gt 100000 ] && {
		# keep count of logfile below 10
		[ `ls ${QOS_TMP_DIR}/log/ | wc -l` -ge 10 ] && {
			rm ${QOS_TMP_DIR}/log/`ls ${QOS_TMP_DIR}/log | awk 'NR==1 {print $1}'`
		}
		mv "$file" "${QOS_TMP_DIR}/log/${time}_qos.log"  2>/dev/null
		QOS_WRITE_FILE "time\t\t\tlog_info" "$file"
	}
	echo -e "$item" >> "$file" 2>/dev/null
}

QOS_LOG_PRINT()
{
	local item="$1"
#return
	QOS_WRITE_LOG "$item"
	echo -e "$item"
}

# get val from indirect var
# aaa=123;bbb=aaa; eval ccc=\${$bbb};
# echo ccc        stdout is 123
indirect_var_get()
{
	local raw=$1
	#variable name can not include char like '.' and '-', so change to '_'
	local raw=`echo ${raw//"."/"_"}`
	local raw=`echo ${raw//"-"/"_"}`
	eval val=\${$raw}
	QOS_RET "$val"
}

# ser val to a indirect var
# r_1=0; n=1 ;a=5; eval "r_$n=$a"
indirect_var_set()
{
	local raw="$1"
	#variable name can not include char like '.' and '-', so change to '_'
	local raw=`echo ${raw//"."/"_"}`
	local raw=`echo ${raw//"-"/"_"}`
	local val="$2"
	eval "$raw=\"$val\""
}


#################################################################################################
# QOS module state maintain
# awk field seprate with "="
local QOS_STATE_FILE="${QOS_TMP_DIR}/state"
[ ! -f $QOS_STATE_FILE ] && touch $QOS_STATE_FILE
ln -sf $QOS_STATE_FILE $QOS_LIB_DIR/state
qos_get_state()
{
	local key="$1"
	grep "^${key}.*" "$QOS_STATE_FILE" 2>&1 >/dev/null || {
		return 1
	}
	QOS_RET "`awk -F= "/^${key}/" $QOS_STATE_FILE | awk -F= '{print $2}'`"
}
qos_set_state()
{
	local key="$1"
	local cont="$2"
	grep "^${key}.*" "$QOS_STATE_FILE" 2>&1 >/dev/null || {
		QOS_WRITE_FILE "${key}=${cont}" "$QOS_STATE_FILE"
		return 0
	}
	sed "/^${key}/ s/=.*$/=${cont}/" -i "$QOS_STATE_FILE"
}
qos_delete_state()
{
	local key="$1"
	# CAUTION: Actually, iface and rule need call this handle. IN CASE of name-include(r1<r11), MUST set the pattern with '.'
	grep "^${key}\..*" $QOS_STATE_FILE 2>&1 >/dev/null && sed "/^${key}\..*/d" -i $QOS_STATE_FILE
}
qos_clear_state()
{
	echo >$QOS_STATE_FILE
}
## qos state operate handle
QOS_STATE()
{
	local cmd="$1"
	shift
	case "$cmd" in
		new)
			qos_set_state "$1"  "$1_state"
		;;
		clear)
			qos_clear_state
		;;
		set)
			qos_set_state "$1" "$2"
		;;
		get)
			QOS_RET "$(qos_get_state "$1")"
		;;
		del)
			qos_delete_state "$1"
		;;
	esac;
}

export QOS_THRESHOLD=$(config_get "$SNAME_GLOBAL" qos_threshold)
[ -z $QOS_THRESHOLD ] && QOS_THRESHOLD=0
export SUPPORT_QOS_THRESHOLD="true"

#################################################################################################
# qos filter condition for users, all rules enabled should add themselves to this two set

# get user-defined interface(zone) from UCI global setting
	# return the valid user interface
qos_user_ifaces_get()
{
	local list=""
	config_get list "$SNAME_GLOBAL" "$STYPE_IFACE"
	# verify the iface in list is defined?
	for iface in $list;do
		grep -q "config.*${STYPE_IFACE}.*${iface}" $QOS_UCI || {
			#echo -e "   iface not defined, delete from global list"
			local nr=`awk "/(list|option).*${STYPE_IFACE}.*${iface}/ {print NR}" $QOS_UCI`
			#sed "${nr}d" -i $QOS_UCI
			list=`echo $list | sed "s/${iface}//g"`
		}
	done
	QOS_RET "$list"
}

## local iface only defined for 450G, named "LAN" constly
export IFACE_LIST=$(qos_user_ifaces_get)
[ ${#IFACE_LIST} -eq 0 ] && QOS_CONFIG_INVALID="true"
export IFACE_NUM=0
export DEV_LIST=""
export ZONE_LIST="$LAN_IFACE_NAME $IFACE_LIST"  # all ifaces participate in qos
cnt=1
for iface in $ZONE_LIST; do
	IFACE_NUM=$((IFACE_NUM + 1))
	local dev=$(zone_get_wan_devices $iface)
	indirect_var_set ${iface}_dev "${dev// /,}"
	DEV_LIST="$DEV_LIST $dev "
	indirect_var_set ${iface}_rootid $cnt
	indirect_var_set ${iface}_linerate $LINERATE
	cnt=$((cnt+1))
done


#FASTPATH: SFE/HWNAT
fastpath_sw_on()
{
	return 1
}

fastpath_hw_on()
{
	dpdk_init_file="/etc/init.d/dpdk"
	if [ -f "$dpdk_init_file" ]; then
		return 1
	else
		return 0
	fi
}

fastpath_sw_real_on()
{
	local sfe_switch=`uci -q get sfe.setting.switch`
	[ "$sfe_switch" == "on" ]
}

fastpath_hw_real_on()
{
	local nss_switch=`uci -q get ecm.global.enable`
	local hwnat_switch=`uci -q get hwnat.global.enabled`
	[ "$nss_switch" == "on" -o "$hwnat_switch" == "1" ]
}

