#!/bin/sh /etc/rc.common

START=98


ROUTE_STATE_DIR="/tmp/"
ROUTE_STATE_FILE="/tmp/.route"

ROUTE_LOG_FILE=/tmp/static_route.log
ROUTE_LOG_FILE_BAK=/tmp/static_route.log.bak
ROUTE_LOG_ENABLE=1

route_check_log_file_size() {
        # if file size > 100k, save and recreate
        if [ `ls -l ${ROUTE_LOG_FILE} | awk '{print $5}'` -gt 102400 ]; then
                mv ${ROUTE_LOG_FILE} ${ROUTE_LOG_FILE_BAK}
        fi
}
route_add_log()
{
        if [ "$ROUTE_LOG_ENABLE" -eq 1 ]; then
                local item="`date +%Y%m%d-%H:%M:%S` $1"
                echo -e "$item" >> ${ROUTE_LOG_FILE}
        fi
}

route_get_state()
{
	local key="$1"
	grep "^${key}=.*" "$ROUTE_STATE_FILE" 2>&1 >/dev/null || {
		return 1
	}
	
	echo "`cat $ROUTE_STATE_FILE |grep "\<${key}\>" |cut -d '=' -f 2`"
}

route_set_state()
{
	local key="$1"
	local cont="$2"

	[ ! -e $ROUTE_STATE_DIR ] && mkdir -p $ROUTE_STATE_DIR
	[ ! -f $ROUTE_STATE_FILE ] && touch $ROUTE_STATE_FILE

	grep "^${key}=.*" "$ROUTE_STATE_FILE" 2>&1 >/dev/null || {
		echo -e "${key}=${cont}" >> "$ROUTE_STATE_FILE" 2>/dev/null
		return 0
	}

	sed -i "s/^${key}=.*/${key}=${cont}/" $ROUTE_STATE_FILE
}

route_del_state()
{
	local key="$1"
	
	grep "^${key}=.*" $ROUTE_STATE_FILE 2>&1 >/dev/null && sed "/^${key}=.*/d" -i $ROUTE_STATE_FILE
}

route_clear_state()
{
	echo >$ROUTE_STATE_FILE 
}

#$1: device
route_get_iface()
{
	. /lib/zone/zone_api.sh

	iface=`zone_get_iface_bydev $1`
	if [ -z "$iface" ];then
		echo "$1"
		return 1
	fi

	zone=`zone_get_zone_byif $iface`
	if [ -z "$zone" ];then
		echo "$iface"
		return 1
	fi

	if [ "$zone" == "VPN" ];then
		zone=$iface
	fi

	echo "$zone"
}

add_lan_default_route()
{
	local iface="LAN"
	local metric=0
	local ip netmask dev target

	ip=`uci get network.lan.ipaddr`
	[ -n "$ip" ] || return

	netmask=`uci get network.lan.netmask`
	[ -n "$netmask" ] || return

	dev=`zone_get_effect_devices $iface`
	[ -n "$dev" ] || return 

	tmp=`ipcalc -n $ip $netmask`
	target=`echo $tmp |cut -d "=" -f 2`

	[ -n "$target" ] || return

	ip route append $target/$netmask via $ip dev $dev metric $metric table $ROUTE_NUM 2>&1
}


get_metric_from_iface()
{
	# iface format as: wan3_eth or wan3_eth_1
	local iface="$1"
	[ -n "$iface" ] || {
		echo "0"
		return
	}
	local logywan_num="$(uci -c /etc/profile.d get profile.@global[0].logywan_num 2>/dev/null)"
	[ -z "$logywan_num" ] && logywan_num="0"
	local sub_wannum=${iface##*_}
	[ -z "$sub_wannum" ] && sub_wannum="0"
	local wannum=${iface%%_*}
	wannum=${wannum#*wan}
	[ -n "$wannum" ] || {
		echo "0"
		return 
	}
	echo $((($wannum * ($logywan_num + 1)) + $sub_wannum))
}

check_dns_ip_same_subnet()
{
	local ipaddr="$1"
	local mask="$2"
	local dns_s="$3"
	[ -n "$dns_s" -a -n "$ipaddr" -a -n "$mask" ] || {
		echo "check_dns_ip_same_subnet: invalid parameters: ipaddr:$ipaddr dns:${dns_s} mask:${mask}" >/dev/console
		echo "no"	
		return
	}
	local wan_sub_max=`ipcalc -b ${ipaddr}/${mask}|awk -F 'BROADCAST=' '{print $2}'`
	local wan_sub_min=`ipcalc -n ${ipaddr}/${mask}|awk -F 'NETWORK=' '{print $2}'`
	local sub_ip_min=`echo $wan_sub_min|awk -F"." '{printf"%.0f\n",$1*256*256*256+$2*256*256+$3*256+$4}'`
	local sub_ip_max=`echo $wan_sub_max|awk -F"." '{printf"%.0f\n",$1*256*256*256+$2*256*256+$3*256+$4}'`
	local dns_ip_value=`echo $dns_s|awk -F"." '{printf"%.0f\n",$1*256*256*256+$2*256*256+$3*256+$4}'`
	if [ "$dns_ip_value" -ge "$sub_ip_min" ] && [ "$dns_ip_value" -le "$sub_ip_max" ]
	then
		echo "yes"
	else
		echo "no"
	fi
}

