local sys    = require "luci.sys"
local uci    = require "luci.model.uci"
local ipm    = require "luci.ip"

local uci_r  = uci.cursor()

function ipCompare(ipaddr, gateway, mask)
	if not ipaddr or not gateway or not mask then
		return false
	end
	local ipentry = string.split(tostring(ipaddr), ".")
	local gwentry = string.split(tostring(gateway), ".")
	local maskentry = string.split(tostring(mask), ".")
	local ip1_bi = tp_and(ipentry[1],maskentry[1])
	local ip2_bi = tp_and(ipentry[2],maskentry[2])
	local ip3_bi = tp_and(ipentry[3],maskentry[3])
	local ip4_bi = tp_and(ipentry[4],maskentry[4])

	local gw1_bi = tp_and(gwentry[1],maskentry[1])
	local gw2_bi = tp_and(gwentry[2],maskentry[2])
	local gw3_bi = tp_and(gwentry[3],maskentry[3])
	local gw4_bi = tp_and(gwentry[4],maskentry[4])

	if ip1_bi == gw1_bi and ip2_bi == gw2_bi and ip3_bi == gw3_bi and ip4_bi == gw4_bi then
		return true
	else
		return false
	end
end

function tp_and(num1,num2)
	local tmp1 = num1
	local tmp2 = num2
	local str = ""
	repeat
		local s1 = tmp1 % 2
		local s2 = tmp2 % 2
		if s1 == s2 then
			if s1 == 1 then
				str = "1"..str
			else
				str = "0"..str
			end
		else
			str = "0"..str
		end
		tmp1 = math.modf(tmp1/2)
		tmp2 = math.modf(tmp2/2)
	until(tmp1 == 0 and tmp2 == 0)
	return tonumber(str,2)
end

local function delete_rule(zone)
	local change = false
	uci_r:foreach("static_route", "route",
		function(section)
				local interface = uci_r:get("static_route", section[".name"], "interface") or ""
				if tostring(zone) == tostring(interface) then
					uci_r:delete("static_route", section[".name"])
					change = true
				end
		end)
	if change == true then
		uci_r:commit("static_route")
	end

	return true
end

local function add_rule(zone, iface, dev)
	uci_r:foreach("static_route", "route",
		function(section)
			local enable = uci_r:get("static_route", section[".name"], "enable") or "0"
			if tostring(enable) == "1" then
				local gateway = uci_r:get("static_route", section[".name"], "gateway") or ""
				local interface = uci_r:get("static_route", section[".name"], "interface") or ""
				local target = uci_r:get_list("static_route", section[".name"], "target") or {}
				local target_netmask = uci_r:get("static_route", section[".name"], "netmask") --controller下无target_netmask,掩码作为prefix携带在target中
				local metric = uci_r:get("static_route", section[".name"], "metric") or ""
				if tostring(gateway) == "NA" then
					if dev and tostring(interface) == tostring(zone) then
						target = (type(target) == "table") and target or {target}
						for _,v in ipairs(target) do
							local obj = ipm.IPv4(v, target_netmask)
							local netip = obj:network()
							local prefix = obj:prefix() --如果target_netmask为nil，则会取target中的prifex
							sys.fork_exec("ip route append table 30 " .. netip:string() .. "/" .. prefix .. " dev " .. dev .. " metric " .. metric)
						end
					end
				else
					if tostring(interface) == "NA" or tostring(interface) == tostring(zone) then
						local ipaddr = uci_r:get("vnetwork", iface, "ipaddr") or ""
						local netmask = uci_r:get("vnetwork", iface, "netmask") or ""
						if ipCompare(ipaddr, gateway, netmask) == true then
							target = (type(target) == "table") and target or {target}
							for _,v in ipairs(target) do
								local obj = ipm.IPv4(v, target_netmask)
								local netip = obj:network()
								local prefix = obj:prefix() --如果target_netmask为nil，则会取target中的prifex
								sys.fork_exec("ip route append table 30 " .. netip:string() .. "/" .. prefix .. " via " .. gateway .. " metric " .. metric)
							end
						end
					end
				end
			end
		end)
	uci_r:commit("static_route")
end

local op = arg[1]
local zone = arg[2]
local iface = arg[3]
local dev = arg[4]

if op == "delete" then
	delete_rule(zone)
elseif op == "add" then
	add_rule(zone, iface, dev)
end
