local uci  = require "luci.model.uci"
local dbg = require "luci.tools.debug"
local sys  = require "luci.sys"
local uci_r = uci.cursor()

-- local function readUptime()
--     local file = io.open("/proc/uptime", "r")
--     if not file then
--         error("无法打开 /proc/uptime 文件")
--     end
--     local content = file:read("*all")
--     file:close()

--     local uptime = tonumber(string.match(content, "^([%d.]+)"))
--     return uptime
-- end

local function file_exists(path)  
	local file = io.open(path, "rb")  
	if file then 
		file:close()
	end

	return file ~= nil
end

local function get_phy_index(port)
    local phy_index = nil
    local tatget_switch_config = nil
    if file_exists("/etc/config/switch_new") then
        tatget_switch_config = "switch_new"
    else
        tatget_switch_config = "switch"
    end
    phy_index = uci_r:get(tatget_switch_config, "port" .. port, "phy_index")

    return phy_index
end

local function get_phy_ports(ports)
    if type(ports) ~= "string" or #ports == 0 then
        return nil
    end
    local vlan_port = ""
    for k in string.gmatch(ports, "%d[%w]*") do
        local len = #k
        local tag = k:sub(len, len)

        if tag == "t" then
            k = k:sub(1, len - 1)
        end

        local phy_port = get_phy_index(k)
        if tag == "t" then
            phy_port = phy_port .. "t"
        end

        if vlan_port == "" then
            vlan_port = phy_port
        else
            vlan_port = vlan_port .. " " .. phy_port
        end
    end
    return vlan_port
end

local function _add_vlan(switch_index, section)
    local vlan_config = {}
    vlan_config["name"] = section["name"]
    vlan_config["vlan"] = section["vlan"]
    vlan_config["desc"] = section["desc"]
    vlan_config["users"] = section["users"]
    if section["users"] == "wan" then
        sys.exec("swconfig dev switch0 set vlan_wan ".. section["vlan"])
    end
    vlan_config["device"] = "switch" .. switch_index

    local phy_ports = get_phy_ports(section["ports"])
    if section["vlan"] ~= "0" then
        if phy_ports ~= nil and phy_ports ~= "" then
            -- dbg.fprint(nil, string.format("Valid phy_ports:%s, vlan:%s", phy_ports, section["vlan"]))
            vlan_config["ports"] = phy_ports
        end
    end

    uci_r:section("vlan" .. switch_index, "switch_vlan", section["name"], vlan_config)
end

local function _add_child_vlan(switch_index, section)
    local child_vlan_config = {}
    child_vlan_config["name"] = section["name"]
    child_vlan_config["vlan"] = section["vlan"]
    child_vlan_config["parent"] = section["parent"]
    child_vlan_config["device"] = "switch" .. switch_index
    sys.exec("swconfig dev switch0 set vlan_wan ".. section["vlan"])

    local phy_ports = get_phy_ports(section["ports"])
    if section["vlan"] ~= "0" then
        if phy_ports ~= nil and phy_ports ~= "" then
            -- dbg.fprint(nil, string.format("Valid phy_ports:%s, vlan:%s", phy_ports, section["vlan"]))
            child_vlan_config["ports"] = phy_ports
        end
    end

    uci_r:section("vlan" .. switch_index, "switch_child_vlan", section["name"], child_vlan_config)
end


local function _vlan_config_sync(switch_type)
	local switch_num = uci_r:get("switch", "global", "switch_num")
	if switch_num == nil or switch_num == "" then
		return
	end
	-- dbg.fprint(nil, "switch num " .. switch_num)
	for i = 1, tonumber(switch_num) do
	    local port_list = nil
        local index = nil
        if switch_type ~= nil and switch_type == "8411c" then
            index = 0
        else
            index = i - 1
        end
        local vlan_config_name = "vlan" .. index
        -- dbg.fprint(nil, string.format("vlan_config_name: %s, real_index %d", vlan_config_name, index))
        local vlan_new_exist = file_exists("/etc/config/vlan_new")
        if vlan_new_exist then
            port_list = uci_r:get("vlan_new", "switch0", "port_list")
        else
            port_list = uci_r:get(vlan_config_name, "switch" .. index, "port_list")
        end
        if type(port_list) ~= "string" then
            -- dbg.fprint(nil, string.format("Test Invalid port list"))
            return
        end
        -- dbg.fprint(nil, string.format("Test Port list %s;", port_list))

        for j in string.gmatch(port_list, "%S+") do
            local lport = nil
            if vlan_new_exist then
                lport = uci_r:get("vlan_new", "port" .. j, "logical")
            else
                lport = uci_r:get(vlan_config_name, "port" .. j, "logical")
                if lport == "none" then
                    break
                end
            end
            local pvid = uci_r:get("lports", "port" .. lport, "pvid")
            uci_r:set(vlan_config_name, "port" .. j, "pvid", pvid)
        end

        -- delete old section
		uci_r:foreach(vlan_config_name, "switch_vlan",
		function(section)
			if section['.name'] ~= nil then
				uci_r:delete(vlan_config_name, section['.name'])
			end
		end)
		uci_r:foreach(vlan_config_name, "switch_child_vlan",
		function(section)
			if section['.name'] ~= nil then
				uci_r:delete(vlan_config_name, section['.name'])
			end
		end)

		sys.exec("swconfig dev switch0 set clear_vlan_wan 1")
		-- create new vlan section
		uci_r:foreach("lports", "switch_vlan",
		function(section)
			_add_vlan(index, section)
		end)

        if switch_type == nil or switch_type ~= "8411c" then
            uci_r:foreach("lports", "switch_child_vlan",
            function(section)
                _add_child_vlan(index, section)
            end)
        end

		uci_r:commit(vlan_config_name)
	end
end

local function vlan_config_sync()
    _vlan_config_sync()
end

local function vlan_config_sync_8411c()
    _vlan_config_sync("8411c")
end

local function_map = {
    vlan_config_sync = vlan_config_sync,
    vlan_config_sync_8411c = vlan_config_sync_8411c
}

if #arg > 0 and function_map[arg[1]] then
    function_map[arg[1]]()
else
    print("USAGE: lua switch_functions.lua function_name\n")
    for key, _ in pairs(function_map) do
        print(string.format("-- Valid function name:%s\n", key))
    end
end
