local uci       = require "luci.model.uci"
local ACL       = require 'luci.controller.admin.access_ctl'
local config    = require "luci.model.userconfig"
local json_m    = require "luci.json"
local debug     = require "luci.tools.debug"
local form      = require "luci.tools.form"
local utl       = require "luci.util"
local err       = require "luci.tools.error"
local ifaces    = require "luci.model.interfaces"
local sys       = require "luci.sys"

local FIT_SSID_LIST_PATH = "/tmp/ssid_list"
local SSID_MAX_CNT = 64
local uci_r = uci.cursor()
form = form.Form(uci_r, {"ssid_list"})

local module_name = "ssid_list"
local SSID_KEY = {
    ssid_info = {"name", "ifname", "rid", "ssid", "id", "portalEnable", "httpsRedirectEnable"}
}
local ACTION_ADD    = 1
local ACTION_DELETE = 2
local ACTION_UPDATE = 3
local ACTION_KEY = {
    [ACTION_ADD] = "ADD", 
    [ACTION_DELETE] = "DELETE",
    [ACTION_UPDATE] = "UPDATE",
}
function get_ssid_max_count()
    return { ["max_ssid"] = uci_r:get_profile("ssid_list", "ssid_max") or SSID_MAX_CNT }
end
local function get_ssid_cur_num()
	local count=0
	uci_r:foreach('ssid_list','rule_ssid_info',function(section)
		count=count+1
	end)
	return count
end

local function wlan_uci_get(mod, cfg)
	local data={}
	local data_flag={}
	local uci = uci_r
	
	uci:foreach(mod, cfg,
		function(section)
			data[#data+1] = uci:get_all(mod, section[".name"])
		end
	)
	return data
end

function wireless_get_ssid_index(ifname)
	local index = -1
	local rules = {}
	
	rules = wlan_uci_get("ssid_list", "rule_ssid_info")
	
	if not rules then
		return false, err.ERR_COM_TABLE_ITEM_UCI_GET
	end

	for i,r in ipairs(rules) do
		if tostring(r.ifname) == tostring(ifname) then
            index = i
            return index
        end
	end
	return index	
end

function ssid_info_add(http_form)
    local ret = nil
	local new = luci.json.decode(http_form.data)
    new = new.params.new
    local max = get_ssid_max_count()    
	if get_ssid_cur_num() >= max.max_ssid then
		return false,err.ERR_COM_TABLE_ITEM_FULL
	end
    new.name = string.gsub(new.name,"%.","%_")
    ret = form:insert(module_name, "rule_ssid_info", new, SSID_KEY.ssid_info, new.ifname)
	if not ret then
		return false, err.ERR_COM_TABLE_ITEM_UCI_ADD
	end
    ret = uci_r:commit(module_name)
	if not ret then
		return false, err.ERR_COM_UCI_COMMIT
	end
    ret = true
    return ret
end

function ssid_info_del(http_form)
	local new = luci.json.decode(http_form.data)
    new = new.params.new
    local index = wireless_get_ssid_index(new.ifname)
    local list = {}
    uci_r:foreach(module_name, "rule_ssid_info",
				  function(section)
					list[#list+1] = section[".name"]
				  end
	)
    local ret = uci_r:delete(module_name,list[index]) and true or false
    if not ret then
        return false,err.ERR_COM_TABLE_ITEM_UCI_DEL
	end
    ret = uci_r:commit(module_name)
	if not ret then
		return false, err.ERR_COM_UCI_COMMIT
	end
    return ret
end

local function wlan_hotplug_call(action, ssid, ifname, rid)
    cmd = string.format("env -i ACTION=\"%s\" SSID=\"%s\" INTERFACE=\"%s\" RID=\"%s\" /sbin/hotplug-call wlan", action, ssid, ifname, rid)
    debug(cmd)
    sys.fork_call(cmd)
end

local function ssid_add(http_form)
	local new = luci.json.decode(http_form.data)
    new = new.params.new
    local ret = ssid_info_add(http_form)
    if not ret then
        return false
    end
    wlan_hotplug_call(ACTION_KEY[ACTION_ADD], new.ssid, new.ifname, new.rid)
    return ret
end

local function ssid_del(http_form)
	local new = luci.json.decode(http_form.data)
    new = new.params.new
    local ret = ssid_info_del(http_form)
    if not ret then
        debug("ssid_info_del failed")
        return false
    end
    wlan_hotplug_call(ACTION_KEY[ACTION_DELETE], new.ssid, new.ifname, new.rid)
    return ret
end

local function ssid_update(http_form)
	local new = luci.json.decode(http_form.data)
    new = new.params.new
    local ret = ssid_info_del(http_form)
    if not ret then
        return false
    end
    ret = ssid_info_add(http_form)
    if not ret then
        return false
    end
    wlan_hotplug_call(ACTION_KEY[ACTION_UPDATE], new.ssid, new.ifname, new.rid)
    --update access_ctl 
    ACL.access_ctl_update(new.ifname,new.ssid)
    return ret
end

function add_ssid_list()
    local file = io.open(FIT_SSID_LIST_PATH, "r")
	local ssid_list = {
       ["ssid"] = {},
       ["ifname"] = {},
       ["rid"] = {},
       ["id"] = {},
       ["portalEnable"] = {},
       ["httpsRedirectEnable"] = {},
       ["vlan"]={},
       ["count"] = 0
    }
	local i = 0
	local count = 0
	for line in file:lines() do
		local tmp = utl.split(line,",")
        ssid_list.count = ssid_list.count + 1
        count = count + 1
		ssid_list.ssid[count] = tmp[1]
		ssid_list.ifname[count] = tmp[2]
		ssid_list.rid[count] = tmp[3]	 
        ssid_list.id[count] = tmp[4]
        ssid_list.portalEnable[count] = tmp[5]
        ssid_list.httpsRedirectEnable[count] = tmp[6]
        ssid_list.vlan[count]= tmp[7]
	end
    sys.fork_call("echo > /etc/config/ssid_list")
	for i = 1,count do
        local ret
        local hdata = {["data"] = luci.json.encode({["params"] = {["new"] = {
            ["name"] = ssid_list.ssid[i],
            ["ssid"] = ssid_list.ssid[i],
            ["ifname"] = ssid_list.ifname[i],
            ["rid"] = ssid_list.rid[i],
            ["id"] = ssid_list.id[i],
            ["portalEnable"] = ssid_list.portalEnable[i],
            ["httpsRedirectEnable"] = ssid_list.httpsRedirectEnable[i],
            ["vlan"] = ssid_list.vlan[i],
        }}})}
        ret = ssid_add(hdata)
        if not ret then
            debug("add a ssid_info failed!")
            return nil
        end
        
    end
    return true
end

function set_ssid()
    local file = io.open(FIT_SSID_LIST_PATH, "r")
	local ssid_list = {
       ["ssid"] = "",
       ["ifname"] = "",
       ["rid"] = "",
       ["varyType"] = "",
       ["id"] = "",
       ["portalEnable"] = "",
       ["httpsRedirectEnable"] = "",
       ["vlan"] = "",
    }
	local tmp = {}
	local i = 0
	local count = 0
	for line in file:lines() do
		local tmp = utl.split(line,",")
        debug.dumptable(tmp)
        ssid_list.varyType = tmp[1]
		ssid_list.ssid = tmp[2]
		ssid_list.ifname = tmp[3]
		ssid_list.rid = tmp[4]	
        ssid_list.id = tmp[5]
        ssid_list.portalEnable = tmp[6]
        ssid_list.httpsRedirectEnable = tmp[7]
        ssid_list.vlan = tmp[8]
	end
    local ret
  
    local hdata = {["data"] = luci.json.encode({["params"] = {["new"] = {
        ["name"] = ssid_list.ssid,
        ["ssid"] = ssid_list.ssid,
        ["ifname"] = ssid_list.ifname,
        ["rid"] = ssid_list.rid,
        ["id"] = ssid_list.id,
        ["portalEnable"] = ssid_list.portalEnable,
        ["httpsRedirectEnable"] = ssid_list.httpsRedirectEnable,
        ["vlan"] = ssid_list.vlan,
    }}})}
    if ssid_list.varyType == "0" then
        ret = ssid_add(hdata)
    elseif ssid_list.varyType == "1" then
        ret = ssid_del(hdata)
    elseif ssid_list.varyType == "2" then
        ret = ssid_update(hdata)
    end   
    if not ret then
        debug("set_ssid failed!")
        return nil
    end
    return true
end

if arg[1] == "add_ssid_list" then
    add_ssid_list()
elseif  arg[1] == "set_ssid" then
    set_ssid()
else
    debug("unknown operation")
end