# Copyight(c) 2008-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file     zone_conf.sh
# bief     Config the zone info
# author   Yuan Fengjia
# vesion   1.0.0
# history  1.0.0, 13May15, Yuanfengjia, Ceate the file

. /usr/share/libubox/jshn.sh
. /lib/net_share.sh

ZONE_STATE=${ZONE_STATE:=zone}
ZONE_REBUILD_LOCK="/tmp/.zone_rebuild_lock"

_zone_group_zones_init() {
	[ -z "$GROUP_ZONES" ] && {
		#group of zones which includes the vpn interfaces of each WAN
		GROUP_VZONES="$(global_wan_name_list)"
		GROUP_ZONES="LAN VPN $GROUP_VZONES"
	}
}

# Notice: max length of ipset name is 32 chars.
_zone_gname() {
	short_names="${@//WAN/W}"
	short_names="${short_names//LAN/L}"
	short_names="${short_names//VPN/V}"
	echo -n "Z_${short_names// /}"
}

_zone_vgname() {
    short_vnames="${@//WAN/W}"
    short_vnames="${short_vnames//LAN/L}"
    short_vnames="${short_vnames//VPN/V}"
    echo -n "ZV_${short_vnames// /}"    #ZV means zone with VPN
}

zone_vgname() {
    _zone_vgname $@
}

# 新增ipset动态zone表(wan第一连接或者lan)，根据接口up/down状态动态添加接口以及IP地址成员
# 1 假设当前动态/静态拨号 输入WAN1/wan1_eth 输出WAN1,输入wan1_poe,返回错误
# 2 假设当前pppoe/l2tp/pptp拨号 输入WAN1/wan1_poe 输出WAN1,输入wan1_eth,返回错误
# 3 输入lan/LAN 输出lan
zone_dyn_name() {
	local zone_num
	local iface
	local if_eth
	local if_1

	[ -z "$1" ] && return 1
	zone_num="$(echo "$1" | tr 'A-Z' 'a-z')"
	if [ "${zone_num/lan/}" != "${zone_num}" ];then
		eval "$2=lan"
	else
		# 跳过wan的alias接口
		[ "${zone_num/alias/}" != "${zone_num}" ] && return 1
		# 跳过lan和loopback
		[ "${zone_num/loopback/}" != "${zone_num}" ] && return 1

		zone_num="${zone_num#*wan}"
		zone_num="${zone_num%%[^0-9]*}"

		[ -z "$zone_num" ] && return 1

		package_is_load zone || {
			CONFIG_APPEND=1
			{
				flock -s 71
				config_load zone
				flock -u 71
			} 71<>${ZONE_REBUILD_LOCK}
			unset CONFIG_APPEND
		}

		config_get if_1 WAN${zone_num} if_1
		config_get if_eth WAN${zone_num} if_eth

		[ -n "$if_1" -a "$if_eth" = "$1" ] && return 1
		[ -z "$if_1" -a -z "$if_eth" ] && return 1

		eval "$2=WAN${zone_num}"
	fi

	return 0
}

# 新增vpn ipset动态zone表，每个vpn实例一个单独的表，根据接口up/down状态动态添加接口以及IP地址成员
zone_dyn_vpn_name() {
	local zone_num

	[ -z "$1" ] && return 1
	zone_num="$(echo "$1" | tr 'A-Z' 'a-z')"
	# 跳过wan接口
	[ "${zone_num/wan/}" != "${zone_num}" ] && return 1
	# 跳过lan
	[ "${zone_num/lan/}" != "${zone_num}" ] && return 1
	# 跳过lan和loopback
	[ "${zone_num/loopback/}" != "${zone_num}" ] && return 1

	eval "$2=$1"
	return 0
}


_zone_create_group() {
    local zone

    gname="$(_zone_gname $1)"
    if [ "$1" == "${1// /}" ]; then
        ipset create "$gname" hash:iface
        zone_dyn_name "$1" zone && {
            ipset -! create "${zone}_ifname" hash:iface maxelem 64
            ipset -! create "${zone}_ipv4" hash:ip maxelem 64
        }
    else
        ipset create "$gname" list:set
        [ $? -eq 0 ] && {
            for zone in ${1}
            do
                ipset add "$gname" "$(_zone_gname $zone)"
            done
        }
    fi
    echo -n "$gname"
}

_zone_create_vgroup() {
    vgname="$(_zone_vgname $1)"
    if [ "$1" == "${1// /}" ]; then
        ipset create "$vgname" hash:iface
        ipset flush "$vgname"
        #devs=$(uci_get_state $ZONE_STATE $1 effect_devs)
        #注释原因：zone_create_group在zone start之前，此时uci_get_state无法获取值。
        #修改方式：按照_zone_state_config中设置effect_devs的逻辑，通过uci_get得到effect_devs的值。
        local ifname_eth=$(uci_get zone "$1" if_eth)
        local devname_eth=$(uci_get zone "$1" dev_eth)
        local devname_x=$(uci_get zone "$1" dev_1)
        local proto=$(uci_get network "$ifname_eth" proto)
        local devs=${devname_eth}
        [ "none" == "$proto" ] && {
            devs=""
        }
        append devs $devname_x

        for dev in $devs
        do	
            ipset add "$vgname" "$dev"
            ipset_add_for_poe_parent "$vgname" "$dev"
        done
    else
        ipset create "$vgname" list:set
        ipset flush "$vgname"
                [ $? -eq 0 ] && {
                        for zone in ${1}
                        do
                                ipset add "$vgname" "$(_zone_vgname $zone)"
                        done
                }
    fi
        echo -n "$vgname"
}

zone_create_group() {
    # create wan group
    _zone_create_group "" 1>/dev/null 2>&1

	_zone_group_zones_init
    for wname in ${GROUP_ZONES}
    do
        _zone_create_group "$wname" 1>/dev/null 2>&1
    done

    for wname in ${GROUP_VZONES}
    do
         _zone_create_vgroup "$wname" 1>/dev/null 2>&1
    done	
	# create all wan group
    _zone_create_group "$(global_wan_name_list)"
    _zone_create_vgroup "$(global_wan_name_list)"
}

#$1:VPN iface name $2:WAN
zone_add_vgroup() {
    [ ! -n "$1" -o ! -n "$2" ] && return
    local vgroup="$(_zone_vgname $2)"
    ipset add "$vgroup" "$1" 
}

zone_delete_vgroup() {
	_zone_group_zones_init
    local wan_list="${GROUP_VZONES}"
    [ -n "$2" ] && wan_list="$2"
    [ ! -n "$1" ] && return
    for wan in $wan_list ;do
        local vgroup="$(_zone_vgname $wan)"
        ipset del "$vgroup" "$1" --exist
    done
}

zone_rebuild_group() {
	_zone_group_zones_init
    local is_virtual_wan=`echo $1 |grep -E "^WAN[0-9]+_[0-9]+$"`
    
    #echo "$1 is_virtual_wan $is_virtual_wan" >/dev/console
    
    [ -z $is_virtual_wan ] && echo $GROUP_ZONES | grep -vqE "(^| )${1}( |$)" && return

    #create zone group
    gname="$(_zone_gname $1)"

    ipset flush "$gname"

    devs=$(uci_get_state $ZONE_STATE $1 effect_devs)

    for dev in $devs
    do
        ipset add "$gname" "$dev"
    done
}

#Used for Bug 730739 - ipv4 pppoe拨号，ipv6静态拨号，会导致相应WAN的QoS规则对ipv6流量不生效
#对于pppoe等拨号生成的poe网卡，将其原生网卡一并添加至ipset之中
ipset_add_for_poe_parent() {
    local vgname=$1
    local dev=$2
    local postfix=${dev#*_}
    if [ "$postfix"x = "poe"x ]; then
		local dev_wan_poe=${dev#*-}
        local poe_parent=$(uci get network.$dev_wan_poe.parent)
        local parent_ifname=$(uci get network.$poe_parent.ifname)
        if [ $parent_ifname ]; then
            ipset add "$vgname" "$parent_ifname"
        fi
	fi
}

zone_rebuild_vgroup() {
	_zone_group_zones_init
    echo $GROUP_VZONES | grep -vqE "(^| )${1}( |$)" && return
    #create zone group
    vgname="$(_zone_vgname $1)"

    local v6_devs=`ipset list $vgname | grep v6`
    ipset flush "$vgname"
    if [ -n "$v6_devs" ]; then
        for v6_dev in $v6_devs
        do
            ipset add "$vgname" "$v6_dev"
        done
    fi

    devs=$(uci_get_state $ZONE_STATE $1 effect_devs)

    for dev in $devs
    do
        ipset add "$vgname" "$dev"
        ipset_add_for_poe_parent "$vgname" "$dev"
    done
}

zone_group() {
    local zones=""

	_zone_group_zones_init
    for gzone in $GROUP_ZONES
    do
        for zone in $@
        do
            if [ "$gzone" == "$zone" ]; then
				if [ -z "$zones" ]; then
                    zones=$gzone
                else
                    zones=$zones" "$gzone
                fi
            fi
        done
    done

    [ -n "$zones" ] && {
		_zone_create_group "$zones"
	}
}

zone_vgroup() {
    local zones=""

	_zone_group_zones_init
    for vgzone in $GROUP_VZONES
    do
        for zone in $@
        do
            if [ "$vgzone" == "$zone" ]; then
                if [ -z "$zones" ]; then
                    zones=$vgzone
                else
                    zones=$zones" "$vgzone
                fi
           fi
        done
    done

    [ -n "$zones" ] && {
        _zone_create_vgroup "$zones"
    }
}


zone_allwan_group() {
	zone_group "$(global_wan_name_list)"
}

_config_vpn_iface() {
    local vpn_iface=""
    local name="" 
    local count=""
    
    local iface=""
    local device="" 
    name=$(uci_get zone "$1")
    [ -n "$name"  ] && {
        for i in $(uci_get_state $ZONE_STATE VPN ifaces); do
            local j=${i#}
            append iface $j
        done
        
        local tmp=$(uci_get zone "$1" iface)
        append iface $tmp
        #echo "ifaces=$iface"
        uci_toggle_state $ZONE_STATE VPN ifaces "$iface"
        uci_toggle_state $ZONE_STATE VPN effect_ifs "$iface"
        
        for m in $(uci_get_state $ZONE_STATE VPN devices); do
            local n=${m#}                          
            append device $n                           
        done
        #echo "devices_old=$device"
        tmp=""
        tmp=$(uci_get zone "$1" dev)
        append device $tmp
        #echo "devices=$device"                                         
        uci_toggle_state $ZONE_STATE VPN devices "$device"
        uci_toggle_state $ZONE_STATE VPN effect_devs "$device"
        
        iface=""
        device=""
        iface=$(uci_get zone "$1" iface)
        device=$(uci_get zone "$1" dev)
        #echo "iface=$iface"
        uci_set_state $ZONE_STATE $iface "" ${iface}_state
        uci_set_state $ZONE_STATE $iface devname $device
        device=${device/-/_}
 
        uci_set_state $ZONE_STATE $device "" ${device}_state
        uci_set_state $ZONE_STATE $device ifname $iface
    }
}

_config_normal_iface() {
    local normal_iface=""
    local name="" 
    local count=""
    
    local iface=""
    local device="" 
    name=$(uci_get zone "$1")
    [ -n "$name"  ] && {
        for i in $(uci_get_state $ZONE_STATE NORMAL ifaces); do
            local j=${i#}
            append iface $j
        done
        
        local tmp=$(uci_get zone "$1" iface)
        append iface $tmp
        #echo "ifaces=$iface"
        uci_toggle_state $ZONE_STATE NORMAL ifaces "$iface"
        uci_toggle_state $ZONE_STATE NORMAL effect_ifs "$iface"
        
        for m in $(uci_get_state $ZONE_STATE NORMAL devices); do
            local n=${m#}                          
            append device $n                           
        done
        tmp=""
        tmp=$(uci_get zone "$1" dev)
        append device $tmp
        #echo "devices=$device"                                         
        uci_toggle_state $ZONE_STATE NORMAL devices "$device"
        uci_toggle_state $ZONE_STATE NORMAL effect_devs "$device"
        
        iface=""
        device=""
        iface=$(uci_get zone "$1" iface)
        device=$(uci_get zone "$1" dev)
        #echo "iface=$iface"
        uci_set_state $ZONE_STATE $iface "" ${iface}_state
        uci_set_state $ZONE_STATE $iface devname $device
        device=${device/./_}
        device=${device/-/_}
 
        uci_set_state $ZONE_STATE $device "" ${device}_state
        uci_set_state $ZONE_STATE $device ifname $iface
    }
}

_zone_del_state_config() {
    local zone_name=$1
    
    local ifname_eth
    local devname_eth
    local zone_old
    #echo "zone_name=$zone_name" > /dev/console
    [ -n "$zone_name" ] && {
        #echo "del zones" > /dev/console
        for zs in $(uci_get_state $ZONE_STATE core zones); do
                local i=${zs#*}
                if [ "$i" != "$zone_name" ]; then
                    append zone_old $i
                fi
        done                                                              
        #echo "zone_old=$zone_old" > /dev/console
        ifname_eth=`uci get zone.$zone_name.if_eth`
        devname_eth=`uci get zone.$zone_name.dev_eth`
        #echo "ifname_eth=$ifname_eth" > /dev/console
        #echo "devname_eth=$devname_eth" > /dev/console
        uci_toggle_state $ZONE_STATE core zones "$zone_old"
        [ -n "$zone_name" ] && uci_revert_state $ZONE_STATE $zone_name
        [ -n "$ifname_eth" ] && uci_revert_state $ZONE_STATE $ifname_eth
        devname_eth=${devname_eth//./_}
        #echo "devname_eth=$devname_eth" > /dev/console
        [ -n "$devname_eth" ] && uci_revert_state $ZONE_STATE $devname_eth
        zone_rebuild_group "$zone_name"
    }  
}

_zone_state_config() {
	local zone_name=$1

	local ifname_eth
	local ifname_x
	local devname_eth
	local devname_x
	local proto
	#echo "zone_name=$zone_name"
	[ -n "$zone_name" ] && {
		#echo "set zones"
		append zone_core $zone_name  

		ifname_eth=$(uci_get zone "$zone_name" if_eth)
		ifname_x=$(uci_get zone "$zone_name" if_1)
		devname_eth=$(uci_get zone "$zone_name" dev_eth)
		devname_x=$(uci_get zone "$zone_name" dev_1)

		[ -n "$ifname_eth" -o -n "$ifname_x" -o -n "$devname_eth" -o -n "$devname_x" ] && {
			#echo "set interfaces"
			#echo "ifname_eth=$ifname_eth"
			uci_set_state $ZONE_STATE $zone_name ifname_eth "$ifname_eth"

			#echo "ifname_x=$ifname_x"
			[ -n "$ifname_x" ] && {
				uci_set_state $ZONE_STATE $zone_name ifname_1 "$ifname_x"
			}

			#echo "set devices"
			#echo "devname_eth=$devname_eth"
			uci_set_state $ZONE_STATE $zone_name devname_eth "$devname_eth"

			#echo "devname_x=$devname_x"
			[ -n "$devname_x" ] && {
				uci_set_state $ZONE_STATE $zone_name devname_1 "$devname_x"
			}

			local ifname_eth_bak=${ifname_eth}
			local devname_eth_bak=${devname_eth}

			#get the proto
			proto=$(uci_get network "$ifname_eth" proto)
			[ "none" == "$proto" ] && {
				ifname_eth_bak=""
				devname_eth_bak=""
			}

			#echo "set effect interfaces"
			append ifname_eth_bak $ifname_x
			#echo "effect_ifs=$ifname_eth"
			uci_set_state $ZONE_STATE $zone_name effect_ifs "$ifname_eth_bak"

			#echo "set effect devices"
			append devname_eth_bak $devname_x
			#echo "effect_devs=$devname_eth"
			uci_set_state $ZONE_STATE $zone_name effect_devs "$devname_eth_bak"

			#echo "set interfaces devices map"
			#echo "ifname_eth=$ifname_eth,devname_eth=$devname_eth"
			uci_set_state $ZONE_STATE $ifname_eth "" ${ifname_eth}_state
			uci_set_state $ZONE_STATE $ifname_eth devname "$devname_eth"

			#echo "devname_eth=$devname_eth"
			devname_eth=${devname_eth//./_}
			devname_eth=${devname_eth//-/_}
			#echo "devname_eth=$devname_eth" 
            if [ "$devname_eth" != "$zone_name" ]; then
			    uci_set_state $ZONE_STATE $devname_eth "" ${devname_eth}_state
            fi
			uci_set_state $ZONE_STATE $devname_eth ifname "$ifname_eth" 

			#echo "ifname_x=$ifname_x,devname_x=$devname_x"
			[ -n "$ifname_x" ] && [ -n "$devname_x" ] && {
				uci_set_state $ZONE_STATE $ifname_x "" ${ifname_x}_state
				uci_set_state $ZONE_STATE $ifname_x devname "$devname_x"

				devname_x=${devname_x/./_}
				devname_x=${devname_x/-/_}
				uci_set_state $ZONE_STATE $devname_x "" ${devname_x}_state
				uci_set_state $ZONE_STATE $devname_x ifname "$ifname_x" 
			}
			zone_rebuild_group "$zone_name"
		}

		#VPN
		[ "$zone_name" == "VPN" ] && {
			#config_foreach _config_vpn_iface
			local vpn_iface
			local count

			config_get vpn_iface "$zone_name" iface
			#echo "vpn_iface=$vpn_iface"
			let count=0
			for i in $vpn_iface
			do
				zone_dyn_vpn_name "$i" zone && {
					ipset -! create "${zone}_ifname" hash:iface maxelem 64
					ipset -! create "${zone}_ipv4" hash:ip maxelem 64
				}

				let count=$count+1
				#echo "i=$i"
				#config_foreach _config_vpn_iface iface $i $count
				_config_vpn_iface $i
			done
			zone_rebuild_group "$zone_name"       
		} 
		#zone_rebuild_vgroup "$zone_name"
	}
}

zone_config() {
	local zone_core
	config_load zone
	config_foreach _zone_state_config zone
	uci_toggle_state $ZONE_STATE core zones "$zone_core"
}

__zone_del_vpnif_state() {
    local vpn_iface="$1"
    local name=
    name=$(uci_get zone "$vpn_iface")
    [ -n "$name"  ] && {
        local iface=$(uci_get_state $ZONE_STATE VPN ifaces)
        local tmp=$(uci_get zone "$vpn_iface" iface)
        list_remove iface $tmp
        uci_toggle_state $ZONE_STATE VPN ifaces "$iface"
        uci_toggle_state $ZONE_STATE VPN effect_ifs "$iface"
        
        local device=$(uci_get_state $ZONE_STATE VPN devices)
        tmp=$(uci_get zone "$vpn_iface" dev)
        list_remove device $tmp
        uci_toggle_state $ZONE_STATE VPN devices "$device"
        uci_toggle_state $ZONE_STATE VPN effect_devs "$device"
        
        iface=""
        device=""
        iface=$(uci_get zone "$vpn_iface" iface)
        device=$(uci_get zone "$vpn_iface" dev)
        device=${device/-/_}
        [ -n "$device" ] && uci_revert_state $ZONE_STATE $device
        [ -n "$iface" ] && uci_revert_state $ZONE_STATE $iface

        zone_dyn_vpn_name "$vpn_iface" zone && {
            ipset -q destroy "${zone}_ifname"
            ipset -q destroy "${zone}_ipv4"
        }
    }
}

__zone_del_zone_state() {
    local zone_name="$1"
    local ifname_eth
    local devname_eth
    local ifname_x
    local devname_x

    ifname_eth=$(uci_get zone "$zone_name" if_eth)
    ifname_x=$(uci_get zone "$zone_name" if_1)
    devname_eth=$(uci_get zone "$zone_name" dev_eth)
    devname_x=$(uci_get zone "$zone_name" dev_1)

    uci_revert_state $ZONE_STATE $zone_name
    [ -n "$ifname_eth" ] && uci_revert_state $ZONE_STATE $ifname_eth
    [ -n "$ifname_x" ] && uci_revert_state $ZONE_STATE $ifname_x
    devname_eth=${devname_eth//./_}
    devname_eth=${devname_eth//-/_}
    devname_x=${devname_x/./_}
    devname_x=${devname_x/-/_}
    [ -n "$devname_eth" ] && uci_revert_state $ZONE_STATE $devname_eth
    [ -n "$devname_x" ] && uci_revert_state $ZONE_STATE $devname_x
}

# arg1: zone_name
## 根据zone的/etc/config/zone配置删除/var/state/zone临时状态文件中的内容等
## NOTE: 删除指定zone时，需要先调用该接口，才能删除zone的config配置
zone_del_zone_state() {
    local zone_name="$1"
    [ -z "$zone_name" ] && return

    local zone_core_all=$(uci_get_state $ZONE_STATE core zones)
    [ "${zone_core_all}" != "${zone_core_all#*$zone_name}" ] && {
        list_remove zone_core_all "$zone_name"
        uci_toggle_state $ZONE_STATE core zones "$zone_core_all"
    }

    config_load zone
    [ "$zone_name" == "VPN" ] && {
        local i=
        local vpn_iface=
        config_get vpn_iface "$zone_name" iface
        for i in $vpn_iface
        do
            __zone_del_vpnif_state "$i"
        done
    } || {
        __zone_del_zone_state "$zone_name"
    }
    zone_rebuild_group "$zone_name"
}

# arg1: zone_name
## 根据zone的config配置生成临时状态文件的内容等
zone_add_zone_state() {
    local zone_name="$1"
    [ -z "$zone_name" ] && return

    local zone_core_all=$(uci_get_state $ZONE_STATE core zones)
    [ "${zone_core_all}" == "${zone_core_all#*$zone_name}" ] && {
        append zone_core_all "$zone_name"
        uci_toggle_state $ZONE_STATE core zones "$zone_core_all"
    }

    config_load zone
    _zone_state_config "$zone_name"
}