# NUM=0
# STR=""

#avoid dns packets in dpdk
fqdn_mark_rule_start()
{
    local support_dpdk=`uci -c /etc/profile.d/ get profile.@global[0].support_dpdk`
    local ns="";vnet=`ip netns list | grep vnet`;if [ -n "$vnet" ];then ns="vnete"; fi

    [ "$support_dpdk" == "1" ] && {
        $ns iptables -w -t mangle -C FORWARD -j fqdn_mark 2> /dev/null
        if [ "$?" != "0" ];then
            $ns iptables -w -t mangle -N fqdn_mark
            $ns iptables -w -t mangle -I FORWARD -j fqdn_mark
        else
            $ns iptables -w -t mangle -F fqdn_mark
        fi

        $ns iptables -w -t mangle -I fqdn_mark -j CONNEXTMARK --set-xmark 0x400000c/0x400000c -p udp --dport 53
        $ns iptables -w -t mangle -I fqdn_mark -j CONNEXTMARK --set-xmark 0x400000c/0x400000c -p tcp --dport 53
    }
}

fqdn_mark_rule_stop()
{
    local support_dpdk=`uci -c /etc/profile.d/ get profile.@global[0].support_dpdk`
    local ns="";vnet=`ip netns list | grep vnet`;if [ -n "$vnet" ];then ns="vnete"; fi

    [ "$support_dpdk" == "1" ] && {
        $ns iptables -w -t mangle -C FORWARD -j fqdn_mark 2> /dev/null
        if [ "$?" == "0" ];then
            $ns iptables -w -t mangle -D FORWARD -j fqdn_mark
            $ns iptables -w -t mangle -F fqdn_mark
            $ns iptables -w -t mangle -X fqdn_mark
        fi
    }
}

fqdn_add_domain_foreach() {
    local cfg="$1"
    config_get name "$cfg" name
    # let NUM++
    # STR="$STR,$name"
    if [ "${name:0:1}" == "*" ]; then
        ipset create $name hash:ip timeout 900 maxelem 1024
    else
        ipset create $name hash:ip timeout 900 maxelem 256
    fi
    echo "1 1 $name" >/proc/fqdn
}

fqdn_add_domainName_foreach() {
    local cfg="$1"
    config_get name "$cfg" domainName
    if [ "${name:0:1}" == "*" ]; then
        ipset create $name hash:ip timeout 900 maxelem 1024
    else
        ipset create $name hash:ip timeout 900 maxelem 256
    fi
    echo "1 1 $name" >/proc/fqdn
}

fqdn_add_fqdn_group_foreach() {
    local cfg="$1"
    config_get id "$cfg" id
    config_get option_nums "$cfg" option_nums
    config_get name_1  "$cfg" "name_1"
    [ $name_1 == $id ] && {
        ipset create $id hash:net maxelem 256
        ipset add $id 0.0.0.0/1
        ipset add $id 128.0.0.0/1
    } 
    # || {
    #     ipset create $id list:set
    #     for i in $(seq 1 $option_nums)
    #     do
    #         # echo "name_$i"
    #         config_get name  "$cfg" "name_$i"
    #         ipset add $id $name
    #     done
    # }

    if [ "$id" == "" ]; then
        if [ "$cfg" == "domain_any_fqdngroup" ]; then
            ipset create $cfg hash:net maxelem 256
            ipset add $cfg 0.0.0.0/1
            ipset add $cfg 128.0.0.0/1
        else
            ipset create $cfg list:set
            config_get domains "$cfg" domains
            for domain in $domains
            do
                config_get domainName "$domain""_domain" domainName
                ipset add $cfg $domainName
            done
        fi
    fi
}

fqdn_add_merged_group_foreach() {
    local cfg="$1"
    ipset create $cfg list:set
    config_get domains "$cfg" domains
    for domain in $domains
    do
        config_get domainName "$domain""_domain" domainName
        ipset add $cfg $domainName
    done
}

fqdn_del_domain_foreach() {
    local cfg="$1"
    config_get name "$cfg" name
    ipset destroy $name
}

fqdn_del_domainName_foreach() {
    local cfg="$1"
    config_get name "$cfg" domainName
    ipset destroy $name
}

fqdn_del_fqdn_group_foreach() {
    local cfg="$1"
    config_get id "$cfg" id
    if [ "$id" == "" ]; then
        id=$cfg
    fi
    ipset destroy $id 2>/dev/null
}

fqdn_del_merged_group_foreach() {
    local id="$1"
    ipset destroy $id 2>/dev/null
}

fqdn_start() {
    echo "fqdn start."
    fqdn_mark_rule_start
    config_load fqdn
    config_foreach fqdn_add_domain_foreach domain
    config_foreach fqdn_add_domainName_foreach domain_Name
    # echo "1 $NUM ${STR:1}" >/proc/fqdn
    # echo "------------------------"
    # echo $NUM
    # echo ${STR:1}
    # echo "------------------------"
    config_foreach fqdn_add_fqdn_group_foreach fqdn_group
    config_foreach fqdn_add_merged_group_foreach merged_group
}

fqdn_stop() {
    echo "fqdn stop."
    fqdn_mark_rule_stop
    config_load fqdn
    config_foreach fqdn_del_fqdn_group_foreach fqdn_group
    config_foreach fqdn_del_domainName_foreach domain_Name
    config_foreach fqdn_del_domain_foreach domain
    config_foreach fqdn_del_merged_group_foreach merged_group
}