#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
. /lib/firewall/core_tpfirewall.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
VPN_RULE_NUM=25
OPENVPN_FWMARK=0x20000
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK
IFNAME=$1

if [ ! -n "$7" ] ;then
        echo "no config file name"
        CONFIGFILENAME=${IFNAME}
else
        CONFIGFILENAME=$7
fi

openvpn_client_down_hook()
{
	local indexname=
	indexname=$(echo $CONFIGFILENAME | tr -cd "[0-9]")
	echo "openvpn_client_down $IFNAME $indexname" >  /dev/console
	local dns_len=0

	config_load 'openvpn'
	config_get dns_len "client$indexname" dns_len
	config_get wan "client$indexname" wan
        #update V zone groups
        . /lib/zone/zone_api.sh
        zone_delete_vgroup "$IFNAME"

	#update hwnat_ipstat interface
        if [ -e "/etc/init.d/hwnat_ipstat" ];then
                echo "### update hwnat_ipstat" >/dev/console
                hwnat_id=`ps -w | grep hwnat_ipstat | awk '/\/usr\/sbin\/hwnat_ipstat/ {print $1}'`
                [ "$hwnat_id" -gt 0 ] && kill -10 ${hwnat_id} 2 >/dev/null
        fi

	#fixme:dns没有值
	mssfix_down "$IFNAME"

	for i in seq $dns
	do
		uci set openvpn.client${indexname}.dns$i=""
	done
	# prevent the recurrence of the same interface
	uci set openvpn.client${indexname}.dev=""
	uci set openvpn.client${indexname}.uptime=""
	uci set openvpn.client${indexname}.vlocalip=""
	uci set openvpn.client${indexname}.vremoteip=""
	uci commit openvpn

	nums=$(iptables -w -nvL openvpn_forwarding_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1 | tac)
	for num in $nums;
	do
		iptables -w -D openvpn_forwarding_rule ${num}
	done

	nums=$(iptables -w -nvL zone_wan_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1 | tac)
	for num in $nums;
	do
		iptables -w -D zone_wan_openvpn ${num}
	done

	nums=$(iptables -w -t nat -nvL postrouting_rule_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1 | tac)
	for num in $nums;
	do
		iptables -w -t nat -D postrouting_rule_openvpn ${num}
	done

	nbptc down ${IFNAME} "ovpn"

	tables="${OPENVPN_FORWARD_TABLE} ${OPENVPN_ROUTING_TABLE} OPENVPN_OUTPUT_RULE"
	for tbl in $tables
	do
		nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${dev}\b" | cut -d ' ' -f 1 | tac)
		for num in $nums;
		do
			iptables -w -t mangle -D ${tbl} ${num}
		done
	done
	#CHAIN:manage_devs
	fw_event_manage_dev "${IFNAME}:down"

    local conf_line=$(grep -n "${IFNAME}" /tmp/resolv.conf.auto | cut -d ":" -f 1)
    local ovpn_conf_line=$(grep -n "${IFNAME}" /tmp/resolv.conf.ovpn | cut -d ":" -f 1)
    if [ -n "${conf_line}" ]; then
        echo "### begin delete openvpn client$indexname dns_server" >/dev/console
        {
            flock -x 199
            if [ -n "${conf_line}" ]; then
                let conf_line2=${conf_line}+2
                echo delete "${conf_line},${conf_line2}" >/dev/console
                sed -i "${conf_line},${conf_line2}d" /tmp/resolv.conf.auto
            fi
            if [ -n "${ovpn_conf_line}" ]; then
                let ovpn_conf_line2=${ovpn_conf_line}+2
                echo delete "${ovpn_conf_line},${ovpn_conf_line2}" >/dev/console
                sed -i "${ovpn_conf_line},${ovpn_conf_line2}d" /tmp/resolv.conf.ovpn
            fi
        } 199> /tmp/resolv.conf.l2tp.pptp.lock
        echo "### finish delete openvpn client$indexname dns_server" >/dev/console
    fi

	[ -x /etc/init.d/dnsproxy ] && /etc/init.d/dnsproxy restart
	echo "### dnsproxy restart finish!" >/dev/console
#	ip route del $remote_network via $route_vpn_gateway dev $dev table $VPN_RULE_NUM

	ipset_local="ovpn_client${indexname}_local"
	ipset_remote="ovpn_client${indexname}_remote"
	ipset destroy ${ipset_local}
	ipset destroy ${ipset_remote}
}


{
	flock -x -w 30 200
	openvpn_client_down_hook
	flock -u 200
} 200> /tmp/ovpn_up_down_${IFNAME}.lock
hnat_up
/etc/init.d/zone restart
exit 0
