#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh

VPN_RULE_NUM=25
OPENVPN_FWMARK=0x20000
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK

local indexname=
local local_network=

indexname=$(echo $config | tr -cd "[0-9]")
#echo "openvpn_client_routeup $dev $indexname" >  /dev/console

#create ipset to reduce iptables rules
ipset_local="ovpn_client${indexname}_local"
ipset_remote="ovpn_client${indexname}_remote"
ipset create $ipset_local hash:net
ipset create $ipset_remote hash:net
ipset flush $ipset_local
ipset flush $ipset_remote

config_load 'openvpn'
config_get local_network "client$indexname" local_network

if [ $redirect_gateway -eq 0 ]; then

	local netnum=0
	for i in $(seq 1 129)
	do
		local tempnet=$(eval echo '$'route_network_${i})
		if [ -n "$tempnet" ]; then
			netnum=$(($netnum+1))
		else
			break
		fi
	done

	for i in $(seq 1 $netnum)
	do
		local remote_network=
		local remote_net=
		local remote_mask=

		remote_net=$(eval echo '$'route_network_${i})
		remote_mask=$(eval echo '$'route_netmask_${i})
		remote_mask=$(ipcalc -p ${remote_net} ${remote_mask} | cut -d '=' -f2)
		remote_network=${remote_net}/${remote_mask}

		ipset add $ipset_remote $remote_network
	done
	for local in ${local_network}
	do
		ipset add $ipset_local $local
	done

	iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $ipset_local src -m set --match-set $ipset_remote dst -m comment --comment "tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $ipset_local src -m set --match-set $ipset_remote dst -m comment --comment "tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A ${OPENVPN_ROUTING_TABLE} -i "${dev}" -m set --match-set $ipset_remote src -m set --match-set $(zone_allwan_group) dst -m comment --comment "vnet_ipset=${ipset_local},tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} -m comment --comment "vnet_ipset=${ipset_local},tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
	#MARK for controller fixBug 931045 - OpenVPN远程收养无法发现样机
	iptables -w -t mangle -A OPENVPN_OUTPUT_RULE -m set --match-set $ipset_local src -m set --match-set $ipset_remote dst -m comment --comment "tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A OPENVPN_OUTPUT_RULE -m set --match-set $ipset_local src -m set --match-set $ipset_remote dst -m comment --comment "tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
else
	for local in ${local_network}
	do
		ipset add $ipset_local $local
	done
	iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $ipset_local src -d "0.0.0.0/0" -m comment --comment "tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $ipset_local src -d "0.0.0.0/0" -m comment --comment "tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A ${OPENVPN_ROUTING_TABLE} -i "${dev}" -s "0.0.0.0/0" -m set --match-set $(zone_allwan_group) dst -m comment --comment "vnet_ipset=${ipset_local},tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} -m comment --comment "vnet_ipset=${ipset_local},tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A OPENVPN_OUTPUT_RULE -m set --match-set $ipset_local src -d "0.0.0.0/0" -m comment --comment "tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
	iptables -w -t mangle -A OPENVPN_OUTPUT_RULE -m set --match-set $ipset_local src -d "0.0.0.0/0" -m comment --comment "tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
fi

/etc/init.d/zone restart
