#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
. /lib/nat/nat_core.sh
. /lib/firewall/core_tpfirewall.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
VPN_RULE_NUM=25
VPN_RULE_NUM_FULLMODE=221
OPENVPN_FWMARK=0x20000
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK
IFNAME=$1
VLOCALIP=$4
VREMOTEIP=$route_vpn_gateway
if [ ! -n "$7" ] ;then
        echo "no config file name"
        CONFIGFILENAME=${IFNAME}
else
        CONFIGFILENAME=$7
fi

# init variables
i=1
fopt=
nns=0
nl='
'

while true; do
  eval fopt=\$foreign_option_${i}
  [ -z "${fopt}" ] && break

  case ${fopt} in
		dhcp-option\ DNS\ *)
           nns=$((nns + 1))
           if [ $nns -le 3 ]; then
             dns="${dns}${dns:+$nl} ${fopt#dhcp-option DNS }"
           else
             printf "%s\n" "Too many nameservers - ignoring after third" >&2
           fi
           ;;
        *)
           printf "%s\n" "Unknown option \"${fopt}\" - ignored" >&2
           ;;
	esac
  i=$((i + 1))
done



route_table_init()
{
	ret=`ip rule list | grep $VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		iptables -w -t mangle -F ${OPENVPN_FORWARD_TABLE}
		iptables -w -t mangle -F ${OPENVPN_ROUTING_TABLE}
		ip rule add pref $VPN_RULE_NUM fwmark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} table $VPN_RULE_NUM
	fi
    ret=`ip rule list | grep $VPN_RULE_NUM_FULLMODE:`
    if [ -z "$ret" ]; then
        ip rule add pref $VPN_RULE_NUM_FULLMODE fwmark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} table $VPN_RULE_NUM_FULLMODE
    fi
}

openvpn_client_up_hook()
{
	local indexname=
    local proto=
    local port=
	local local_network=
	local dns_num=1
	#local wans=
	local dns_1=
	local dns_2=
    local line=1
	local line_ovpn_resolv_conf=1
    local null_file=1
	local null_file_ovpn_resolv_conf=1
	local dns_iface_flag=1

	indexname=$(echo $CONFIGFILENAME | tr -cd "[0-9]")
	echo "openvpn_client_up $IFNAME $indexname" >  /dev/console

    if [ -n "${dns}" ]; then
        echo "### begin add openvpn client dns_server" >/dev/console
        {
            flock -x 199
            test -s /tmp/resolv.conf.auto && null_file=0
            test -s /tmp/resolv.conf.ovpn && null_file_ovpn_resolv_conf=0
            #添加了127.0.0.1到resolv.conf文件，直接将openvpn dns加到第一行会影响lan dns代理，将openvpn dns加到127.0.0.1之后，即将openvpn设备作为文件中首个interface
            if [ $null_file == 0 ]; then
                local interface_line=`awk '/#Interface/{print NR; exit}' /tmp/resolv.conf.auto`
                [ -n "${interface_line}" ] && line=${interface_line} && dns_iface_flag=0
            fi
            #sed不能操作空文件，sed直接操作resolv.conf会破坏软连接，于是空文件采用echo到resolv.conf.auto，非空sed操作/tmp/resolv.conf.auto
            [ $null_file == 0 -a $dns_iface_flag == 0 ] && sed -i "${line}i #Interface ${IFNAME}" /tmp/resolv.conf.auto || echo "#Interface ${IFNAME}" >>/tmp/resolv.conf
            [ $null_file_ovpn_resolv_conf == 0 ] && sed -i "${line_ovpn_resolv_conf}i #Interface ${IFNAME}" /tmp/resolv.conf.ovpn || echo "#Interface ${IFNAME}" >>/tmp/resolv.conf.ovpn
            for nameserver in $dns
            do
                if [ $dns_num == 1 ]; then
                    dns_1=${nameserver}
                fi
                if [ $dns_num == 2 ]; then
                    dns_2=${nameserver}
                fi

                let line+=1
                let line_ovpn_resolv_conf+=1
                [ $null_file == 0 -a $dns_iface_flag == 0 ] && sed -i "${line}i nameserver ${nameserver}" /tmp/resolv.conf.auto || echo "nameserver ${nameserver}" >>/tmp/resolv.conf
                [ $null_file_ovpn_resolv_conf == 0 ] && sed -i "${line_ovpn_resolv_conf}i nameserver ${nameserver}" /tmp/resolv.conf.ovpn || echo "nameserver ${nameserver}" >>/tmp/resolv.conf.ovpn
                uci set openvpn.client${indexname}.dns$dns_num=${nameserver}
                let dns_num+=1
            done
            if [ -z ${dns_2} ]; then
                let line+=1
                let line_ovpn_resolv_conf+=1
                [ $null_file == 0 -a $dns_iface_flag == 0 ] && sed -i "${line}i nameserver ${dns_1}" /tmp/resolv.conf.auto || echo "nameserver ${dns_1}" >>/tmp/resolv.conf
                [ $null_file_ovpn_resolv_conf == 0 ] && sed -i "${line_ovpn_resolv_conf}i nameserver ${dns_1}" /tmp/resolv.conf.ovpn || echo "nameserver ${dns_1}" >>/tmp/resolv.conf.ovpn
                dns_2=${dns_1}
            fi
        } 199> /tmp/resolv.conf.l2tp.pptp.lock
        echo "### finish add openvpn client dns_server" >/dev/console
    fi


	uci set openvpn.client${indexname}.dns_len=$nns
	uci set openvpn.client${indexname}.dev=${IFNAME}
	uci set openvpn.client${indexname}.uptime=$(date +%s)
	uci set openvpn.client${indexname}.vlocalip=${VLOCALIP}
	uci set openvpn.client${indexname}.vremoteip=${VREMOTEIP}
	uci commit openvpn

	config_load 'openvpn'
	config_get port "client$indexname" port
	config_get proto "client$indexname" proto
	config_get local_network "client$indexname" local_network
	config_get wans "client$indexname" wan

	#zone_wan_openvpn forword
	for wan in ${wans}
	do
		devices=$(zone_get_wan_devices $wan)
		for device in ${devices}
		do
			echo "zone_wan_openvpn ${wan} ${device}" > /dev/console
			fw add 4 f zone_wan_openvpn ACCEPT { "-i $device -p $proto -m $proto --sport $port -m comment --comment $IFNAME" }
		done
	done

	#INPUT
	#for wan in ${wans}
	#do
	#	device=$(zone_get_wan_devices $wan)
	#	fw add 4 f zone_wan_openvpn ACCEPT { "-i $device -p $proto -m $proto --dport $port" }
	#done
	local authname=`grep "^auth " $CONFIGFILENAME |awk '{print $2}'`
	echo "authname $authname " >/dev/console
	#update V zone groups
	. /lib/zone/zone_api.sh
	for wan in ${wans}
        do
		zone_add_vgroup "$IFNAME" "$wan"
		#TCPMSS
		mssfix_up "$wan" "$IFNAME" "$authname"
        done

	#update hwnat_ipstat interface
        if [ -e "/etc/init.d/hwnat_ipstat" ];then
                echo "### update hwnat_ipstat" >/dev/console
                hwnat_id=`ps -w | grep hwnat_ipstat | awk '/\/usr\/sbin\/hwnat_ipstat/ {print $1}'`
                [ "$hwnat_id" -gt 0 ] && kill -10 ${hwnat_id} 2 >/dev/null
        fi

	#create ipset to reduce iptables rules
	ipset_local="ovpn_client${indexname}_local"
	ipset create $ipset_local hash:net
	ipset flush $ipset_local
	for local in ${local_network}
	do
		ipset add $ipset_local $local
	done

	#FORWORDING
    fw add 4 f openvpn_forwarding_rule ACCEPT { "-o $IFNAME -m conntrack --ctstate RELATED,ESTABLISHED" }
	fw add 4 f openvpn_forwarding_rule ACCEPT { "-m set --match-set lan_iface src -m set --match-set $ipset_local src -o $IFNAME" }
	fw add 4 f openvpn_forwarding_rule ACCEPT { "-m set --match-set lan_iface dst -m set --match-set $ipset_local dst -i $IFNAME" }
	nat_postrouting_add openvpn "-A" "-m set --match-set $ipset_local src -o $IFNAME -j MASQUERADE"

	add_dnsserver ${IFNAME} ${dns_1} ${dns_2}
	[ -x /etc/init.d/dnsproxy ] && /etc/init.d/dnsproxy restart
	echo "### dnsproxy restart finish!" >/dev/console

	#CHAIN:manage_devs
	fw_event_manage_dev "${IFNAME}:up"

	nbptc up ${IFNAME} "ovpn"
}

route_table_init
{
	flock -x -w 30 200
	openvpn_client_up_hook
	flock -u 200
} 200> /tmp/ovpn_up_down_${IFNAME}.lock
hnat_down
/etc/init.d/zone restart
exit 0
