local utl = require "luci.util"
local ip   		= require "luci.ip"
local lan    = require "luci.controller.admin.lan"
local sys      = require "luci.sys"
local http     = require "luci.http"
local ctypes   = require "luci.model.checktypes"
local dbg    = require "luci.tools.debug"
local ctl      = require "luci.model.controller"
local form = require "luci.tools.form"
local userconfig   = require "luci.model.userconfig"
local err      = require "luci.tools.error"
local ifs = require "luci.model.interfaces".ifs()
local infs = require "luci.model.interfaces"
local openvpn_tool = require "luci.controller.admin.openvpn"
local uci     = require "luci.model.uci"
local uci_r = uci.cursor()
form = form.Form(uci_r, {"confname"})

local server_list_max = 4
local client_list_max = 10

local uci_config = "openvpn"
local uci_section_server = "server"
local uci_section_client = "client"

local startnetbios = "/etc/init.d/netbios_passthrough start"

local FILE_TMP_PATH = "/www/luci-static/resources/authserver/tmpfile"

local OPENVPN_PEM_DIR = "/etc/openvpn/dh1024.pem"

local SSL_VPN_CONFIG= "ssl_vpn_config"
local SSL_VPN_SECNAME= "sslvpn_server_config"

local hnat_no_accl_mark = "0x4"


-- 1.  Server
--index is the service to be deleted,zero based. format is like '0,2,3,5'
function openvpn_delete_server(json_form)
    local content = luci.json.decode(json_form)
    local result = content.result


    if result.enabled == "on" then
        local name = "server" .. result.indexname
        local cmd = string.format("/lib/openvpn/openvpn-instance.sh delserver %s",name)

        ---disconnect all openvpn clients
        openvpn_tool.openvpn_disconn_all(name)
        sys.fork_call(cmd)

        local pname = "ovpn_pool"..result.indexname
        local remote_mgmt_cmd = ". /lib/openvpn/openvpn-remote-management.sh; rm_openvpn_remote_management"
        remote_mgmt_cmd = remote_mgmt_cmd.." "..pname
        sys.fork_call(remote_mgmt_cmd)
    end

    openvpn_tool.flush_hnat_extmark(0, content.port, content.proto, nil, nil)

    userconfig.cfg_modify();
    return result

end

-- 1.  Server
-- Set OpenVPN UCI config
-- @param N/A
-- @return table
function openvpn_add_server(json_form)
    local content = luci.json.decode(json_form)
    local section_name = "server"..content.indexname
    --netbios start
    sys.fork_exec(startnetbios)
    if content.enabled == "on" then
    -- check cert
        dbg("on:")
        if not openvpn_tool.openvpn_check_cert(uci_section_server,content.indexname) then
            dbg("============generate certificate...")
            --local rebuild_cmd = string.format("/usr/sbin/build-ovpn-crt %d rebuild",content.indexname)
            --sys.fork_exec(rebuild_cmd)

            local cmd = string.format("/lib/openvpn/openvpn-instance.sh addserver %s rebuild",section_name)

            local openvpn_ca = io.open(OPENVPN_PEM_DIR,"r")
			if openvpn_ca then
				sys.fork_call(cmd)
			else
				sys.fork_exec(cmd)
			end	
        else
            dbg("addserver:")
            local cmd = string.format("/lib/openvpn/openvpn-instance.sh addserver %s",section_name)
            local openvpn_ca = io.open(OPENVPN_PEM_DIR,"r")
			if openvpn_ca then 
				sys.fork_call(cmd)
			else
				sys.fork_exec(cmd)
			end
        end
    elseif content.enabled == "off" then
        -- disconnect all openvpn clients
        -- openvpn_disconn_all()
    end
    dbg("cfg_modify:")
    userconfig.cfg_modify();

    --添加相关的remote rule
    if content.enabled == "on" then
        local start_ip
        local end_ip
        local pname
        start_ip = ip.IPv4(content.serverip, content.mask):host():network():string()
        if poolmask == "255.255.255.255" then
            end_ip = start_ip
        else
            end_ip = ip.IPv4(content.serverip, content.mask):broadcast():network():string()
        end
        pname = "ovpn_pool"..tostring(content.indexname)
        local remote_mgmt_cmd = ". /lib/openvpn/openvpn-remote-management.sh; add_openvpn_remote_management"
        remote_mgmt_cmd = remote_mgmt_cmd.." "..start_ip
        remote_mgmt_cmd = remote_mgmt_cmd.." "..end_ip
        remote_mgmt_cmd = remote_mgmt_cmd.." "..pname
        sys.fork_call(remote_mgmt_cmd)
    end
    return ret
end

-- 2.  Client
--index is the service to be deleted,zero based. format is like '0,2,3,5'
function openvpn_delete_client(json_form)
    local content = luci.json.decode(json_form)
    result = content.result

    if result.enabled == "on" then
        local name = "client" .. result.indexname
        local cmd = string.format("/lib/openvpn/openvpn-instance.sh delclient %s",name)
        sys.fork_call(cmd)
    end
    --delete all cert key
    openvpn_tool.openvpn_delete_cert(uci_section_client,result.indexname)
    userconfig.cfg_modify();

    openvpn_tool.flush_hnat_extmark(1, content.port, content.proto, nil, nil)
    return result

end

-- 2.  Client
-- Set OpenVPN UCI config
-- @param N/A
-- @return table
function openvpn_add_client(json_form)
    local content = luci.json.decode(json_form)
    local section_name = "client"..content.indexname

    dbg("import certificate.")
    local certfrom= "/tmp/" .. string.format("cert%d.ovpn",content.cid)
    if not openvpn_tool.file_exists(certfrom) then
        dbg("not have cert.")
        return false, "don't have certfrom"
    end
    local certto= "/etc/openvpn/" .. string.format("client_client%d.ovpn",content.indexname)
    os.execute("cp -f " .. certfrom .. " " .. certto .. "  >/dev/null 2>&1")
    os.execute("rm -f " .. certfrom .. " >/dev/null 2>&1")

    if content.enabled == "on" then
        local cmd = string.format("/lib/openvpn/openvpn-instance.sh addclient %s",section_name)
        sys.fork_call(cmd)
        --sys.fork_call("fw openvpn_access " .. content.indexname .. " " .. content.proto .. " " .. content.port)
    elseif form.enabled == "off" then
        -- disconnect all openvpn clients
        -- vpn.vpn_disconn_all("openvpn")
        -- openvpn_disconn_all()
    end

    userconfig.cfg_modify();
    return ret
end

local type = arg[1]
local action = arg[2]
local http = arg[3]

if type == "client" then
    if action == "add" then
        openvpn_add_client(http)
    elseif action == "del" then
        openvpn_delete_client(http)
    end
elseif type == "server" then
    if action == "add" then
        openvpn_add_server(http)
    elseif action == "del" then
        openvpn_delete_server(http)
    end
end